Package: sem
Version: 3.1-16
Date: 2024-08-25
Title: Structural Equation Models
Authors@R: c(person("John", "Fox", role = "aut", email = "jfox@mcmaster.ca"),
	person("Zhenghua", "Nie", role = c("aut", "cre"), email="zhenghua.nie@gmail.com"),
    person("Jarrett", "Byrnes", role = "aut", email="byrnes@nceas.ucsb.edu"),
    person("Michael", "Culbertson", role = "ctb"),
    person("Saikat", "DebRoy", role = "ctb"),
    person("Michael", "Friendly", role = "ctb"),
    person("Benjamin", "Goodrich", role = "ctb"),
    person(given="Richard H.", family="Jones", role = "ctb"),
    person("Adam", "Kramer", role = "ctb"),
    person("Georges", "Monette", role = "ctb"),
    person("Frederick", "Novomestky", role = "ctb"),
    person("R-Core", role = "ctb")
	)
Depends: R (>= 3.5.0), stats
Imports: MASS, boot, mi (>= 0.9-99), utils
Suggests: polycor, DiagrammeR (>= 1.0.9)
LazyLoad: yes
LazyData: yes
ByteCompile: yes
Description: Functions for fitting general linear structural
    equation models (with observed and latent variables) using the RAM approach, 
    and for fitting structural equations in observed-variable models by two-stage least squares.
License: GPL (>= 2)
URL: https://CRAN.R-project.org/package=sem
NeedsCompilation: yes
Packaged: 2024-08-28 01:40:20 UTC; niez
Author: John Fox [aut],
  Zhenghua Nie [aut, cre],
  Jarrett Byrnes [aut],
  Michael Culbertson [ctb],
  Saikat DebRoy [ctb],
  Michael Friendly [ctb],
  Benjamin Goodrich [ctb],
  Richard H. Jones [ctb],
  Adam Kramer [ctb],
  Georges Monette [ctb],
  Frederick Novomestky [ctb],
  R-Core [ctb]
Maintainer: Zhenghua Nie <zhenghua.nie@gmail.com>
Repository: CRAN
Date/Publication: 2024-08-28 08:10:04 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 16:09:36 UTC; unix
Archs: sem.so.dSYM

# last modified 2020-06-02 by J. Fox

useDynLib(sem)
useDynLib(sem, csemSolve)

import(stats)

# importFrom(matrixcalc, duplication.matrix, vech)
importFrom(MASS, ginv)
importFrom(boot, boot.ci)
importFrom(mi, mi, complete)
#importFrom(DiagrammeR, DiagrammeR)
importFrom(utils, combn, edit, setTxtProgressBar, txtProgressBar)

export(ram, standardizedResiduals, normalizedResiduals, readMoments, 
    startvalues, startvalues2,
    standardizedCoefficients, stdCoef, tsls, sem, modIndices, pathDiagram,
    specifyModel, specifyEquations, cfa, math, bootSem, rawMoments, cov2raw, fscores,
    AICc, CAIC,
    classifyVariables, combineModels, removeRedundantPaths,
    optimizerNlm, optimizerOptim, optimizerNlminb,
    objectiveML2, objectiveGLS2, objectiveFIML2,
	optimizerSem,  
	objectiveML, objectiveGLS, objectiveFIML,   
	multigroupModel, msemObjectiveML, msemObjectiveGLS, msemOptimizerNlm,
	# msemObjectiveFIML, 
	msemObjectiveML2, optimizerMsem,  
    miSem,
    # deprecated:
    standardized.residuals, normalized.residuals, read.moments,
    standardized.coefficients, std.coef, mod.indices, path.diagram,
    specify.model, boot.sem, raw.moments
    )

S3method(effects, sem)
S3method(logLik, objectiveML)
S3method(logLik, objectiveFIML)
S3method(miSem, semmod)
S3method(miSem, semmodList)
S3method(bootSem, sem)
S3method(bootSem, msem)
S3method(residuals, sem)
S3method(residuals, objectiveFIML)
S3method(standardizedCoefficients, sem)
S3method(standardizedResiduals, sem)
S3method(standardizedResiduals, objectiveFIML)
S3method(normalizedResiduals, objectiveML)
S3method(normalizedResiduals, objectiveGLS)
S3method(normalizedResiduals, objectiveFIML)
S3method(tsls, default)
S3method(tsls, formula)
S3method(print, tsls)
S3method(summary, tsls)
S3method(print, summary.tsls)
S3method(residuals, tsls) 
S3method(coef, tsls)
S3method(coef, sem)
S3method(fitted, tsls)
S3method(anova, tsls)
S3method(anova, objectiveML)
S3method(anova, objectiveFIML)
S3method(vcov, tsls)
S3method(vcov, sem)
S3method(sem, semmod)
S3method(sem, default)
S3method(summary, objectiveML)
S3method(summary, objectiveGLS)
S3method(summary, objectiveFIML)
S3method(print, summary.objectiveML)
S3method(summary, modIndices)
S3method(summary, msemModIndices)
S3method(print, modIndices)
S3method(print, msemModIndices)
S3method(modIndices, objectiveML)
S3method(modIndices, msemObjectiveML)
S3method(pathDiagram, sem)
S3method(pathDiagram, semmod)
S3method(print, semeffects)     
S3method(print, semmod)
S3method(print, bootsem)
S3method(print, objectiveML)
S3method(print, objectiveGLS)
S3method(print, objectiveFIML)
S3method(summary, bootsem)
S3method(print, summary.bootsem)
S3method(print, miSem)
S3method(summary, miSem)
S3method(print, summary.miSem)
S3method(rawMoments, formula)
S3method(rawMoments, default)
S3method(print, rawmoments)
S3method(deviance, objectiveML)
S3method(deviance, objectiveFIML)
S3method(df.residual, sem)
S3method(coef, sem)
S3method(vcov, sem)
S3method(fscores, sem)
S3method(AIC, objectiveML)
S3method(AIC, objectiveFIML)
S3method(AICc, objectiveML)
S3method(AICc, objectiveFIML)
S3method(AICc, objectiveGLS)
S3method(CAIC, objectiveML)
S3method(CAIC, objectiveGLS)
S3method(CAIC, objectiveFIML)
S3method(BIC, objectiveML)
S3method(BIC, objectiveFIML)
S3method(combineModels, semmod)
S3method(update, semmod)
S3method(edit, semmod)
S3method(print, semmodList)
S3method(sem, semmodList)
S3method(sem, msemmod)
S3method(print, msemObjectiveML)
S3method(summary, msemObjectiveML)
S3method(print, msemObjectiveGLS)
S3method(summary, msemObjectiveGLS)
S3method(print, msemObjectiveFIML)
S3method(summary, msemObjectiveFIML)
S3method(deviance, msemObjectiveML)
S3method(AIC, msemObjectiveML)
S3method(AICc, msemObjectiveML)
S3method(BIC, msemObjectiveML)
S3method(residuals, msem)
S3method(coef, msem)
S3method(vcov, msem)
S3method(df.residual, msem)
S3method(anova, msemObjectiveML)
S3method(logLik, msemObjectiveML)
S3method(effects, msem)
S3method(print, semeffectsList)
S3method(standardizedCoefficients, msem)
S3method(standardizedResiduals, msem)
S3method(normalizedResiduals, msemObjectiveML)
S3method(fscores, msem)
S3method(vcov, msem)

<p align="center">
  <img src="res/logo.png" width="200">
</p>

# A Simulation Execution Manager for ns-3 #

[![Binder](https://mybinder.org/badge_logo.svg)](https://mybinder.org/v2/gh/signetlabdei/sem/develop?urlpath=lab)

This is a Python library to perform multiple ns-3 script executions, manage the
results and collect them in processing-friendly data structures.

# How does this work? #

For complete step-by-step usage and installation instructions, check out
[our documentation][docs].

# How to cite us #

If you used SEM for your ns-3 analysis, please cite the following paper, both to
provide a reference and help others find out about this tool:

Davide Magrin, Dizhi Zhou, and Michele Zorzi. 2019. A Simulation Execution
Manager for ns-3: Encouraging reproducibility and simplifying statistical
analysis of ns-3 simulations. In Proceedings of the 22nd International ACM
Conference on Modeling, Analysis and Simulation of Wireless and Mobile Systems
(MSWIM '19). ACM, New York, NY, USA, 121-125. DOI:
https://doi.org/10.1145/3345768.3355942

# Contributing #

This section contains information on how to contribute to the project. If you
are only interested in using SEM, check out the [documentation][docs].

If you want to contribute to sem development, first of all you'll need
an installation that allows you to modify the code, immediately see
the results and run tests.

## Building the module from scratch ##

This module is developed using
[`poetry`](https://python-poetry.org/docs/): in order to correctly
manage virtual environments and install dependencies, make sure it is installed.
Typically, the following is enough:

```bash
curl -sSL https://install.python-poetry.org | python3 -
```

Note that, if poetry's installer does not add poetry's path to your shell's startup file properly, you may need to add
`source $HOME/.poetry/env` to your startup file. You can tell that you need to add it if your shell cannot find the poetry command the next time you open a terminal window.

Then, clone the repo (or your fork, by changing the url in the following
command), also getting the `ns-3` installations that are used for running
examples and tests:

```bash
git clone https://github.com/signetlabdei/sem
cd sem
git submodule update --init --recursive
```

From the project root, you can then install the package and the
requirements with the following:

```bash
poetry install
```

This will also get you a set of tools such as `sphinx`, `pygments` and `pytest`
that handle documentation and tests.

Finally, you can spawn a sub-shell using the new virtual environment by calling:

```bash
poetry shell
```

Now, you can start a python REPL to use the library interactively, issue the
bash `sem` program, run tests and compile the documentation of your local copy
of sem.

## Running tests ##

This project uses the [`pytest`](https://docs.pytest.org/en/latest/) framework
for running tests. Tests can be run, from the project root, using:

```bash
python -m pytest --doctest-glob='*.rst' docs/
python -m pytest -x -n 3 --doctest-modules --cov-report term --cov=sem/ ./tests
```

These two commands will run, respectively, all code contained in the `docs/`
folder and all tests, also measuring coverage and outputting it to the terminal.

Since we are mainly testing integration with ns-3, tests require frequent
copying and pasting of folders, ns-3 compilations and simulation running.
Furthermore, documentation tests run all the examples in the documentation to
make sure the output is as expected. Because of this, full tests are far from
instantaneous. Single test files can be targeted, to achieve faster execution
times, by substituting `./tests` in the second command with the path to the test
file that needs to be run.

## Building the documentation ##

Documentation can be built locally using the makefile's `docs` target:

```bash
make docs
```

## Running examples ##

The scripts in `examples/` can be directly run:

```bash
python examples/wifi_example.py
```

## Installing SEM in pip's editable mode ##

`pip` currently requires a `setup.py` file to install projects in editable mode.

As explained [here](https://github.com/python-poetry/poetry/issues/761), poetry
actually already generates a `setup.py`. After building the project, you can
extract the file from the archive using the following command:

``` bash
tar -xvf dist/*.tar.gz --wildcards --no-anchored '*/setup.py' --strip=1
```

After this step, it becomes possible to install SEM in editable mode.


## Authors ##

Davide Magrin

[docs]: https://signetlabdei.github.io/sem

# last modified 2015-06-09 by J. Fox
# some changes by Benjamin K Goodrich 2015-01-20

miSem <- function(model, ...){
    UseMethod("miSem")
}

miSem.semmod <- function(model, ..., data, formula = ~., raw=FALSE, fixed.x=NULL, objective=objectiveML,
                  n.imp=5, n.chains=n.imp, n.iter=30, seed=sample(1e6, 1), mi.args=list(), show.progress=TRUE){
    cls <- gsub("\\.", "", deparse(substitute(objective)))
    cls <- gsub("2", "", cls)
    cls <- c(cls, "sem")
    warn <- options(warn=-1)
    on.exit(options(warn))
    initial.fit <- sem(model, ..., data=data, formula=formula, raw=raw, fixed.x=fixed.x,
                       objective = if (raw) objectiveFIML else objective)
    options(warn)
    class(initial.fit) <- if (raw) c("objectiveFIML", "sem") else cls
    coefficients <- coefficients(initial.fit)
    coef.names <- names(coefficients)
    var.names <- initial.fit$var.names 
    ram <- initial.fit$ram 
    ram[coef.names, "start value"] <- coefficients 
    N <- nrow(data)
    if (!is.null(fixed.x)) fixed.x <- apply(outer(var.names, fixed.x, "=="), 2, which)
    mi.args$n.chains <- n.chains
    mi.args$n.iter <- n.iter
    mi.args$seed <- seed
    mi.args$y <- data
    if (show.progress) cat("\n Beginning", n.imp, "imputations\n")
    mi.data <- do.call("mi", mi.args)
    if (show.progress) cat("\n Imputations complete\n")
#     has.tcltk <- require("tcltk")
# 	  if (has.tcltk) pb <- tkProgressBar("Fitting", "Imputation no.: ", 0, n.imp)
    if (show.progress) {
        cat("\n Fitting model to imputations:\n")
        pb <- txtProgressBar(min=0, max=n.imp, style=3)
    }
    fits <- complete(mi.data, m = n.imp, include_missing = FALSE)
    for (i in seq_along(fits)) {
#        if (has.tcltk) setTkProgressBar(pb, i, label=sprintf("Imputation no.: %d", i))
        if (show.progress) setTxtProgressBar(pb, i)
        data.i <- model.frame(formula, data=fits[[i]])
        data.i <- model.matrix(formula, data=fits[[i]])
        colnames(data.i)[colnames(data.i) == "(Intercept)"] <- "Intercept"
    	  S <- if (raw) rawMoments(data.i) else {
			    data.i <-  data.i[, colnames(data.i) != "Intercept"]
			    cov(data.i)
		    }
        fit <- sem(ram, S=S, N=N, data=data.i, raw=raw, param.names=coef.names, var.names=var.names, fixed.x=fixed.x,
                              optimizer=initial.fit$optimizer, objective=objective, ...)
        class(fit) <- cls   
	      fits[[i]] <- fit
    }
#    if (has.tcltk) close(pb)
    if (show.progress) close(pb)
    result <- list(initial.fit=initial.fit, mi.fits=fits, imputations=mi.data, seed=seed, mi.data=mi.data)
    class(result) <- "miSem"
    result
}

miSem.semmodList <- function(model, ..., data, formula = ~., group, raw=FALSE, 
        fixed.x=NULL, objective=msemObjectiveML,
        n.imp=5, n.chains=n.imp, n.iter=30, seed=sample(1e6, 1), mi.args=list(),
        show.progress=TRUE){
    if (missing(formula)) formula <- as.formula(paste("~ . -", group))
    warn <- options(warn=-1)
    on.exit(options(warn))
    initial.fit <- sem(model, ..., data=na.omit(data), formula=formula, group=group, raw=raw, fixed.x=fixed.x,
                       objective = objective)
    options(warn)
    coefficients <- coefficients(initial.fit)
    coef.names <- names(coefficients)
    var.names <- initial.fit$var.names 
    ram <- initial.fit$ram 
    groups <- initial.fit$groups
    group <- initial.fit$group
    G <- length(groups)
    for (g in 1:G){
        pars <- ram[[g]][, "parameter"]
        free <- pars != 0
        ram[[g]][free, "start value"] <- coefficients[pars[free]]
    }
    mi.args$n.chains <- n.chains
    mi.args$n.iter <- n.iter
    mi.args$seed <- seed
    mi.args$y <- data
    if (show.progress) cat("\n Beginning", n.imp, "imputations\n")
    mi.data <- do.call("mi", mi.args)
    if (show.progress) cat("\n Imputations complete\n")
    fits <- complete(mi.data, m = n.imp, include_missing = FALSE)
#     has.tcltk <- require("tcltk")
#     if (has.tcltk) pb <- tkProgressBar("Fitting", "Imputation no.: ", 0, n.imp)
    if (show.progress) {
        cat("\n Fitting model to imputations:\n")
        pb <- txtProgressBar(min=0, max=n.imp, style=3)
    }
    for (i in 1:n.imp){
#        if (has.tcltk) setTkProgressBar(pb, i, label=sprintf("Imputation no.: %d", i))
        if (show.progress) setTxtProgressBar(pb, i)
        data.i <- fits[[i]]
        group.i <- data.i[, group]
        data.i <- model.frame(formula, data=data.i)
        data.i <- model.matrix(formula, data=data.i)
        colnames(data.i)[colnames(data.i) == "(Intercept)"] <- "Intercept"
        S <- data.out <- vector(G, mode="list")
        N <- numeric(G)
        for (g in 1:G){
            data.g <- data.i[group.i == groups[g], ]
            data.out[[g]] <- data.g
            N[g] <- nrow(data.g)
            S[[g]] <- if (raw) rawMoments(data.g) else {
    			  data.g <-  data.g[, colnames(data.g) != "Intercept"]
    			  cov(data.g)
    		}
        }
        fit <- sem(ram, S=S, N=N, group=group, groups=groups, raw=raw, data=data.out, 
                  fixed.x=initial.fit$fixed.x, param.names=coef.names, var.names=var.names,
                  optimizer=initial.fit$optimizer, objective=objective, ...)
	    fits[[i]] <- fit
    }
#    if (has.tcltk) close(pb)
    if (show.progress) close(pb)
    result <- list(initial.fit=initial.fit, mi.fits=fits, imputations=mi.data, seed=seed, mi.data=mi.data)
    class(result) <- "miSem"
    result
}

print.miSem <- function(x, ...){
    coefs <- sapply(x$mi.fits, coef)
    vars <- sapply(x$mi.fits, function(x) diag(vcov(x)))
    table <- matrix(0, NROW(coefs), 4)
    table[, 1] <- rowMeans(coefs)
    ses <- sqrt(rowMeans(vars) + apply(coefs, 1, var) * (1 + 1/NCOL(coefs)))
    table[, 2] <- ses
    table[, 3] <- table[, 1]/table[, 2]
    table[, 4] <- 2*pnorm(abs(table[, 3]), lower.tail=FALSE)
    rownames(table) <- rownames(coefs)
    cat("\nCoefficients:\n")
    colnames(table) <- c("Estimate", "Std. Error", "z value", "Pr(>|z|)")
    printCoefmat(table, ...)
    invisible(x)
}

summary.miSem <- function(object, digits=max(3, getOption("digits") - 2), ...){
    coefs <- lapply(object$mi.fits, coef)
    table <- do.call("cbind", coefs)
    rownames(table) <- names(coefs[[1]])
    table <- cbind(table, rowMeans(table), coef(object$initial.fit))
    colnames(table) <- c(paste("Imputation", 1:length(coefs)), "Averaged", "Initial Fit")
    result <- list(object=object, mi.results=table, digits=digits)
    class(result) <- "summary.miSem"
    result
}

print.summary.miSem <- function(x, ...){
    cat("\nCoefficients by imputation:\n")
    print(x$mi.results, digits=x$digits, ...)
    print(x$object, digits=x$digits, ...)
    invisible(x)
}

# with contributions by Adam Kramer and Michael Friendly (originally by J. Fox)
# last modified 2015-06-09 by J. Fox

globalVariables("dot")

Greek <- read.table(system.file("etc/GreekLetters.txt", package="sem"), as.is=TRUE)

math <- function(text, html.only=FALSE, hat=FALSE){
    if (length(text) > 1) {
        result <- sapply(text, math, html=html.only, hat=hat)
        names(result) <- names(text)
        return(result)
    }
#     subscripts <- c("&#8320;", "&#8321;", "&#8322;", "&#8323;", "&#8324;", "&#8325;", "&#8326;",
#         "&#8327;", "&#8328;", "&#8329;")
    subscripts <- c("&#x2080;", "&#x2081;", "&#x2082;", "&#x2083;", "&#x2084;", "&#x2085;", "&#x2086;",
        "&#x2087;", "&#x2088;", "&#x2089;")
    superscripts <- c("&#x2070;", "&sup1;", "&sup2;", "&sup3;", "&#x2074;", "&#x2075;", 
        "&#x2076;", "&#x2077;", "&#x2078;", "&#x2079;")
    names(subscripts) <- names(superscripts) <- 0:9
    hat <- if (hat) "&#770;" else ""
    text <- gsub(" ", "", text)
    symbol <- regexpr("^[a-zA-Z]+", text)
    if (symbol != 1) stop("text does not start with an alphabetic symbol name")
    symbol <- if (html.only) {
        paste0("&", substring(text, 1, attr(symbol, "match.length")), ";")
    }
    else{
        s <- substring(text, 1, attr(symbol, "match.length"))
        s <- Greek[s, "decimal"]
        if (is.na(s)) stop(s, " is not a Greek letter")
        s
    }
    subscript <- regexpr("_\\{", text)
    subscript <- if (subscript >= 1){
        subscript <- substring(text, subscript + 2)
        endbrace <- regexpr("\\}", subscript)
        if (endbrace < 1) stop("unmatched closing brace in ", text)
        substring(subscript, 1, endbrace - 1)
    }
    else ""
    if (subscript != ""){
        subscript <- unlist(strsplit(subscript, split=""))
        subscript <- subscripts[subscript]
        if (any(is.na(subscript))) stop ("invalid non-numeral subscript")
        subscript <- paste(subscript, collapse="")
    }
    superscript <- regexpr("\\^\\{", text)
    superscript <- if (superscript >= 1){
        superscript <- substring(text, superscript + 2)
        endbrace <- regexpr("\\}", superscript)
        if (endbrace < 1) stop("unmatched closing brace in ", text)
        substring(superscript, 1, endbrace - 1)
    }
    else ""
    if (superscript != ""){
        superscript <- unlist(strsplit(superscript, split=""))
        superscript <- superscripts[superscript]
        if (any(is.na(superscript))) stop ("invalid non-numeral superscript")
        superscript <- paste(superscript, collapse="")
    }
    paste0(symbol, hat, subscript, superscript)
}

path.diagram <- function(...) {
    .Deprecated("pathDiagram", package = "sem")
    pathDiagram(...)
}

pathDiagram <- function (model, ...)
{
    UseMethod("pathDiagram")
}

pathDiagram.semmod <- function(model, obs.variables, ...) {
    parse.path <-
        function(path) {
            path.1 <- gsub("-", "", gsub(" ","", path))
            direction <- if (regexpr("<>", path.1) > 0)
                2
            else if (regexpr("<", path.1) > 0)
                - 1
            else if (regexpr(">", path.1) > 0)
                1
            else
                stop(paste("ill-formed path:", path))
            path.1 <- strsplit(path.1, "[<>]")[[1]]
            list(first = path.1[1], second = path.1[length(path.1)], direction =
                    direction)
        }
    if ((!is.matrix(model)) |
            ncol(model) != 3)
        stop ("model argument must be a 3-column matrix")
    startvalues <- as.numeric(model[,3])
    par.names <- model[,2]
    n.paths <- length(par.names)
    heads <- from <- to <- rep(0, n.paths)
    for (p in 1:n.paths) {
        path <- parse.path(model[p,1])
        heads[p] <- abs(path$direction)
        to[p] <- path$second
        from[p] <- path$first
        if (path$direction == -1) {
            to[p] <- path$first
            from[p] <- path$second
        }
    }
    ram <- matrix(0, p, 5)
    all.vars <- unique(c(to, from))
    latent.vars <- setdiff(all.vars, obs.variables)
    vars <- c(obs.variables, latent.vars)
    pars <- na.omit(unique(par.names))
    ram[,1] <- heads
    ram[,2] <- apply(outer(vars, to, "=="), 2, which)
    ram[,3] <- apply(outer(vars, from, "=="), 2, which)
    par.nos <- apply(outer(pars, par.names, "=="), 2, which)
    if (length(par.nos) > 0)
        ram[,4] <-
        unlist(lapply(par.nos, function(x)
            if (length(x) == 0)
                0
            else
                x))
    ram[,5] <- startvalues
    colnames(ram) <-
        c("heads", "to", "from", "parameter", "start value")
    pars <- unique(na.omit(par.names))
    coeff <- rep(0, length(pars))
    names(coeff) <- pars
    fake.sem <-
        list(
            ram = ram, n = length(obs.variables), var.names = vars, coeff = coeff,
            semmod = model
        )
    class(fake.sem) <- "sem"
    pathDiagram(fake.sem, ...)
}


pathDiagram.sem <-
    function (model, file = "pathDiagram", style = c("ram", "traditional"),
        output.type = c("html", "graphics", "dot"), graphics.fmt = "pdf", dot.options = NULL,
        size = c(8, 8), node.font = c("Helvetica", 14),
        edge.font = c("Helvetica", 10), digits = 2, rank.direction = c("LR", "TB"),
        min.rank = NULL, max.rank = NULL, same.rank = NULL,
        variables = model$var.names, var.labels, parameters, par.labels,
        ignore.double = TRUE, ignore.self = FALSE, error.nodes = TRUE,
        edge.labels = c("names", "values", "both"),  edge.colors = c("black", "black"),
        edge.weight = c("fixed", "proportional"),
        node.colors = c("transparent", "transparent", "transparent"),
        standardize = FALSE, ...) {
            Dot <- function(..., semicolon = TRUE, newline = TRUE) {
                cat(file = handle, paste0(..., if (semicolon)
                    ";"
                    else
                        "",
                    if (newline)
                        "\n"
                    else
                        ""))
            }
            style <- match.arg(style)
            output.type <- match.arg(output.type)
            edge.labels <- match.arg(edge.labels)
            edge.weight <- match.arg(edge.weight)
            rank.direction <- match.arg(rank.direction)
            if (output.type == "html") {
                handle <- textConnection("dot", "w")
            }
            else {
                dot.file <- paste0(file, ".dot")
                handle <- file(dot.file, "w")
                if (output.type == "graphics")
                    graph.file <- paste0(file, ".", graphics.fmt)
            }
            on.exit(close(handle))
            Dot("digraph \"", deparse(substitute(model)), "\" {", semicolon = FALSE)
            Dot("  rankdir=", rank.direction)
            Dot("  size=\"", size[1], ",", size[2], "\"")
            Dot(
                "  node [fontname=\"", node.font[1],
                "\" fontsize=", node.font[2], " fillcolor=\"", node.colors[1],
                "\" shape=box style=filled]"
            )
            Dot("  edge [fontname=\"", edge.font[1],
                "\" fontsize=", edge.font[2], "]")
            Dot("  center=1")
            if (!is.null(min.rank)) {
                min.rank <- paste0("\"", min.rank, "\"")
                min.rank <- gsub(",", "\" \"", gsub(" ", "", min.rank))
                Dot("  {rank=min ", min.rank, "}", semicolon = FALSE)
            }
            if (!is.null(max.rank)) {
                max.rank <- paste0("\"", max.rank, "\"")
                max.rank <- gsub(",", "\" \"", gsub(" ", "", max.rank))
                Dot("  {rank=max ", max.rank, "}", semicolon = FALSE)
            }
            if (!is.null(same.rank)) {
                for (s in 1:length(same.rank)) {
                    same <- paste0("\"", same.rank[s], "\"")
                    same <- gsub(",", "\" \"", gsub(" ", "", same))
                    Dot("  {rank=same ", same, "}", semicolon = FALSE)
                }
            }
            latent <- variables[-(1:model$n)]
            for (lat in latent) {
                Dot("  \"", lat, "\" [shape=ellipse]", semicolon = FALSE)
            }
            endogenous <- classifyVariables(model$semmod)$endogenous
            endogenous <-
                variables[apply(outer(endogenous, model$var.names, "=="), 1, which)]
            if (style == "traditional") {
                variables <- c(variables, paste0(endogenous, ".error"))
                error.color <-
                    if (length(node.colors) < 3)
                        node.colors[1]
                else
                    node.colors[3]
            }
            for (endog in endogenous) {
                Dot("  \"", endog, "\" [fillcolor=\"", node.colors[2], "\"]", semicolon =
                        FALSE)
                if (style == "traditional") {
                    if (error.nodes) Dot(
                        "  \"", endog, ".error\" [shape=ellipse] [fillcolor=\"", error.color, "\"]",
                        semicolon = FALSE
                    )
                    else Dot(
                        "  \"", endog, 
                        ".error\" [shape=ellipse width=0 height=0 fixedsize=true label=\"\"] [fillcolor=\"", 
                        error.color, "\"]",
                        semicolon = FALSE
                    )
                }
            }
            ram <- model$ram
            if (missing(parameters)) {
                par.names <- names(coef(model))
                rownames(ram)[ram[, "parameter"] != 0] <-
                    par.names[ram[, "parameter"]]
                rownames(ram)[ram[, "parameter"] == 0] <-
                    ram[ram[, "parameter"] == 0, "start value"]
                parameters <- rownames(ram)
            }
            if (standardize)
                ram[, 5] <- stdCoef(model)[, 2]
            else
                ram[names(model$coeff), 5] <- model$coeff
            coefs <- ram[, 5]  # handle equality constraints, if any
            na.coefs <- is.na(coefs)
            if (any(na.coefs)) {
                for (coef in which(na.coefs)) {
                    ram[coef, 5] <-
                        (ram[ram[coef, 4] == ram[, 4], 5])[1] # paste in the estimate
                }
            }
            values <- round(ram[, 5], digits)
            heads <- ram[, 1]
            to <- ram[, 2]
            from <- ram[, 3]
            if (!missing(par.labels)) {
                check <- names(par.labels) %in% parameters
                if (any(!check)) {
                    msg <- if (sum(!check) > 1)
                        paste(
                            "The following parameters do not appear in the model:", paste(names(par.labels)[!check], collapse =
                                    ", ")
                        )
                    else
                        paste("The following parameter does not appear in the model:", names(par.labels)[!check])
                    warning(msg)
                    par.labels <- par.labels[check]
                }
                names(parameters) <- parameters
                parameters[names(par.labels)] <- par.labels
            }
            labels <- if (edge.labels == "names")
                parameters
            else if (edge.labels == "values")
                values
            else
                paste(parameters, values, sep = "=")
            colors <- ifelse(values > 0, edge.colors[1], edge.colors[2])
            direction <- ifelse((heads == 2), " dir=both", "")
            lineweight <- rep(1, nrow(ram))
            if (edge.weight == "proportional") {
                lineweight <- abs(values) / mean(values)
                if (!standardize)
                    warning("proportional edge weights requested for an unstandardized model")
            }
            if (style == "ram") {
                for (par in 1:nrow(ram)) {
                    if ((!ignore.double) || (heads[par] == 1)) {
                        if (ignore.self && to[par] == from[par]) next
                        Dot(
                            "  \"", variables[from[par]],
                            "\" -> \"", variables[to[par]], "\" [label=\"",
                            labels[par], "\"", direction[par], " color=", colors[par],
                            " penwidth=", round(lineweight[par] + 0.001, 3), "]"
                        )
                    }
                }
            }
            else
                for (par in 1:nrow(ram)) {
                    # style == "traditional"
                    if (heads[par] == 1) {
                        Dot(
                            "  \"", variables[from[par]],
                            "\" -> \"", variables[to[par]], "\" [label=\"",
                            labels[par], "\"", direction[par], " color=", colors[par],
                            " penwidth=", round(lineweight[par] + 0.001, 3), "]"
                        )
                    }
                    else if (variables[to[par]] %in% endogenous) {
                        if (variables[to[par]] == variables[from[par]]) {
                            # convert self-arrow to residual path
                            lab <- labels[par]
                            val <-
                                round(sqrt(values[par]), digits = digits)
                            lab <-
                                if (edge.labels == "names")
                                    paste0("sqrt(", lab, ")")
                            else if (edge.labels == "values")
                                val
                            else
                                paste0("sqrt(", parameters[par], ")=", val)
                            Dot(
                                "  \"", variables[to[par]], ".error\" -> \"",
                                variables[to[par]], "\" [color=", edge.colors[1], " label=\"", lab,
                                "\" penwidth=", round(sqrt(lineweight[par]) + 0.001, 3)," ]"
                            )
                        }
                        else{
                            # convert endogenous covariance to error covariance
                            Dot(
                                "  \"", variables[to[par]], ".error\" -> \"",
                                variables[from[par]], ".error\" [dir=both label=\"", labels[par],
                                "\" color=", colors[par],
                                " penwidth=", round(lineweight[par] + 0.001, 3), "]"
                            )
                        }
                    }
                    else if (!ignore.double &&
                            (variables[to[par]] != variables[from[par]])) {
                                Dot(
                                    "  \"", variables[from[par]],
                                    "\" -> \"", variables[to[par]], "\" [label=\"",
                                    labels[par], "\"", direction[par], " color=", colors[par],
                                    " penwidth=", round(lineweight[par] + 0.001, 3), "]"
                                )
                            }
                }
            if (!missing(var.labels)) {
                check <- names(var.labels) %in% variables
                if (any(!check)) {
                    msg <- if (sum(!check) > 1)
                        paste(
                            "The following variables do not appear in the model:", paste(names(var.labels)[!check], collapse =
                                    ", ")
                        )
                    else
                        paste("The following variable does not appear in the model:", names(var.labels)[!check])
                    warning(msg)
                    var.labels <- var.labels[check]
                }
                Dot("  // variable labels: ", semicolon = FALSE)
                lines <-
                    paste0('    "', names(var.labels), '" [label="', var.labels, '"];\n')
                Dot(paste(lines, collapse = ""), semicolon = FALSE, newline = FALSE)
            }
            Dot("}", semicolon = FALSE)
            if (output.type == "graphics") {
                cmd <-
                    paste0("dot -T", graphics.fmt, " -o ", graph.file,  " -Gcharset=latin1 ",
                        dot.options, " ", dot.file)
                cat("Running ", cmd, "\n")
                result <- try(system(cmd))
            }
            if (output.type == "html" && requireNamespace("DiagrammeR")) {
                print(DiagrammeR::DiagrammeR(textConnection(dot), type = "grViz"))
            }
            result <-
                if (output.type == "html")
                    dot
            else
                readLines(dot.file)
            invisible(result)
        }

# last modified 2012-01-06 by J. Fox
# Modified for Compiled code in C/C++ by Zhenghua Nie.

objectiveML <- function(gradient=TRUE, hessian=FALSE){
	result <- list(
		objective = function(par, model.description){
			with(model.description, {

					res <- CompiledObjective(par=par, model.description=model.description, objective="objectiveML",gradient=gradient, hessian=hessian) 

					f <- res$f
					C <- res$C
					A <- res$A
					P <- res$P

					grad <- NULL
					if(gradient)
							grad <- res$gradient
					hess <- NULL
					if(hessian) 
							hess <- res$hessian
					attributes(f) <- list(C=C, A=A, P=P, gradient=grad, hessian=hess)
					f
}
)
		}
)
if (gradient)
		result$gradient <- function(par, model.description){
				with(model.description, {

					res <- CompiledObjective(par=par, model.description=model.description, objective="objectiveML",hessian=hessian) 

				  A <- res$A
					P <- res$P
					C <- res$C
					grad <- res$gradient

					attributes(grad) <- list(C=C, A=A, P=P, gradient=grad)
					grad
				}
			)
		}
if (hessian)
		result$hessian <- function(par, model.description){
				with(model.description, {

					res <- CompiledObjective(par=par, model.description=model.description, objective="objectiveML",hessian=hessian) 

				  A <- res$A
					P <- res$P
					C <- res$C
					hess <- res$hessian

					attributes(grad) <- list(C=C, A=A, P=P, hessian=hess)
					grad
				}
			)
		}
	class(result) <- "semObjective"
	result
}

# last modified 2011-11-04 by J. Fox

residuals.sem <- function(object, ...){
    object$S - object$C
    }
	
standardized.residuals <- function(...){
	.Deprecated("standardizedResiduals", package="sem")
	standardizedResiduals(...)
}

standardizedResiduals <- function(object, ...){
    UseMethod("standardizedResiduals")
    }

standardizedResiduals.sem <- function(object, ...){
    res <- residuals(object)
    s <- diag(object$S)
    res/sqrt(outer(s, s))
    }

normalized.residuals <- function(...){
	.Deprecated("normalizedResiduals", package="sem")
	normalizedResiduals(...)
    }
	
normalizedResiduals <- function(object, ...){
	UseMethod("normalizedResiduals")
}
    
normalizedResiduals.objectiveML <- function(object, ...){
    res <- residuals(object)
    N <- object$N - (!object$raw)
    C <- object$C
    c <- diag(C)
    res/sqrt((outer(c,c) + C^2)/N)
    }
	
normalizedResiduals.objectiveGLS <- function(object, ...){
	res <- residuals(object)
	N <- object$N - (!object$raw)
	S <- object$S
	s <- diag(S)
	res/sqrt((outer(s,s) + S^2)/N)
}
	

# last modified 2015-05-14 by J. Fox

specify.model <- function(...){
	.Deprecated("specifyModel", package="sem")
	specifyModel(...)
}

specifyModel <- function(file="", text, exog.variances=FALSE, endog.variances=TRUE, covs, suffix="", quiet=FALSE){
    add.variances <- function () {
        if (!quiet) message("NOTE: it is generally simpler to use specifyEquations() or cfa()\n",
                "      see ?specifyEquations")
        variables <- need.variance()
        nvars <- length(variables)
        if (nvars == 0) return(model)
        message("NOTE: adding ", nvars, " variances to the model")
        paths <- character(nvars)
        par.names <- character(nvars)
        for (i in 1:nvars) {
            paths[i] <- paste(variables[i], "<->", variables[i])
            par.names[i] <- paste("V[", variables[i], "]", sep = "")
        }
        model.2 <- cbind(c(model[, 1], paths), c(model[, 2], par.names), 
            c(model[, 3], rep(NA, length(paths))))
        class(model.2) <- "semmod"
        model.2
    }
    need.variance <- function () {
        all.vars <- classifyVariables(model)
        exo.vars <- all.vars$exogenous
        end.vars <- all.vars$endogenous
        variables <- logical(0)
        for (paths in model[, 1]) {
            vars <- strip.white(paths)
            vars <- sub("-*>", "->", sub("<-*", "<-", vars))
            vars <- sub("<->|<-", "->", vars)
            vars <- strsplit(vars, "->")[[1]]
            if (vars[1] != vars[2]) {
                for (a.variable in vars) {
                    if (is.na(variables[a.variable])) variables[a.variable] <- TRUE
                }
            }
            else {
                variables[vars[1]] <- FALSE
            }
        }
        if (!exog.variances && length(exo.vars) > 0) variables[exo.vars] <- FALSE
        if (!endog.variances && length(end.vars) > 0) variables[end.vars] <- FALSE
        names(variables)[variables]
    }
    model <- if (!missing(text)) scan(text=text, what=list(path="", par="", start=1, dump=""), sep=",", 
                                      strip.white=TRUE, comment.char="#", fill=TRUE, quiet=quiet) 
      else scan(file=file, what=list(path="", par="", start=1, dump=""), sep=",", 
        strip.white=TRUE, comment.char="#", fill=TRUE, quiet=quiet) 
    # dump permits comma at line end
    model$path <- gsub("\\t", " ", model$path)
    model$par[model$par == ""] <- NA
    model <- cbind(model$path, model$par, model$start)
    if (!(missing(covs))){
        for (cov in covs){
            vars <- strsplit(cov, "[ ,]+")[[1]]
            nvar <- length(vars)
            for (i in 1:nvar){
                for (j in i:nvar){
                    row <- c(paste(vars[i], "<->", vars[j]), 
                        if (i == j) paste("V[", vars[i], "]", sep="") else paste("C[", vars[i], ",", vars[j], "]", sep=""),
                        NA)
                    if (row[2] %in% model[,2]) next
                    model <- rbind(model, row)
                }
            }
        }
    }
    model <- removeRedundantPaths(model, warn=FALSE)
    result <- add.variances()
    which.pars <- !is.na(result[, 2])
    result[which.pars, 2] <- paste(result[which.pars, 2], suffix, sep="")
    result
}

print.semmod <- function(x, ...){
	path <- x[,1]
	parameter <- x[,2]
	parameter[is.na(parameter)] <- "<fixed>"
	startvalue <- as.numeric(x[,3])
	startvalue[is.na(startvalue)] <- " "
	if (all(startvalue == " "))  print(data.frame(Path=path, Parameter=parameter),
				right=FALSE)
	else print(data.frame(Path=path, Parameter=parameter, StartValue=startvalue),
				right=FALSE)
	invisible(x)
}

classifyVariables <- function(model) {
	variables <- logical(0)
	for (paths in model[, 1]) {
		vars <- strip.white(paths)
		vars <- sub("-*>", "->", sub("<-*", "<-", vars))
		if (grepl("<->", vars)){
			vars <- strsplit(vars, "<->")[[1]]
			if (is.na(variables[vars[1]])) variables[vars[1]] <- FALSE
			if (is.na(variables[vars[2]])) variables[vars[2]] <- FALSE
		}
		else if (grepl("->", vars)){
			vars <- strsplit(vars, "->")[[1]]
			if (is.na(variables[vars[1]])) variables[vars[1]] <- FALSE
			variables[vars[2]] <- TRUE
		}
		else if (grepl("<-", vars)){
			vars <- strsplit(vars, "<-")[[1]]
			if (is.na(variables[vars[2]])) variables[vars[2]] <- FALSE
			variables[vars[1]] <- TRUE
		}
		else stop("incorrectly specified model")
	}
	list(endogenous=names(variables[variables]), exogenous=names(variables[!variables]))
}

strip.white <- function(x) gsub(' ', '', x)

removeRedundantPaths <- function(model, warn=TRUE){
    paths <- model[, 1]
    paths <- strip.white(paths)
    paths <- sub("-*>", "->", sub("<-*", "<-", paths))
    start <- regexpr("<->|<-|->", paths)
    end <- start + attr(start, "match.length") - 1
    arrows <- substr(paths, start, end)
    vars <- matrix(unlist(strsplit(paths, "<->|<-|->")), ncol=2, byrow=TRUE)
    for (i in 1:length(arrows)){
        if (arrows[i] == "<-"){
            arrows[i] <- "->"
            vars[i, ] <- vars[i, 2:1]
        }
    }
    vars <- cbind(vars, arrows)
    dupl.paths <- duplicated(vars)
    if (warn && any(dupl.paths)){
        warning("the following duplicated paths were removed: ", paste(model[dupl.paths, 1], collapse=", "))
    }
    model <- model[!dupl.paths, , drop=FALSE]
    class(model) <- "semmod"
    model
}

specifyEquations <- function(file="", text, ...){
    par.start <- function(coef, eq){
        if (length(grep("\\(", coef)) == 0){
            return(c(coef, "NA"))
        }
        par.start <- strsplit(coef, "\\(")[[1]]
        if (length(par.start) != 2) stop("Parse error in equation: ", eq,
            '\n  Start values must be given in the form "parameter(value)".')
        par <- par.start[[1]]
        start <- par.start[[2]]
        if (length(grep("\\)$", start)) == 0) stop("Parse error in equation: ", eq,
            "\n  Unbalanced parentheses.")
        start <- sub("\\)", "", start)
        return(c(par, start))  
    }
    parseEquation <- function(eqn){
        eq <- eqn
        eqn <- gsub("\\s*", "", eqn)
        eqn <- strsplit(eqn, "=")[[1]]
        if (length(eqn) != 2) stop("Parse error in equation: ", eq,
            "\n  An equation must have a left- and right-hand side separated by =.")
        lhs <- eqn[1]
        rhs <- eqn[2]
        if (length(grep("^[cC]\\(", lhs)) > 0){
            if (length(grep("\\)$", lhs)) == 0) stop("Parse error in equation: ", eq,
                "\n  Unbalanced parentheses.")
            lhs <- sub("[cC]\\(", "", lhs)
            lhs <- sub("\\)", "", lhs)
            variables <- strsplit(lhs, ",")[[1]]
            if (length(variables) != 2) stop("Parse error in equation: ", eq,
                "\n  A covariance must be in the form C(var1, var2) = cov12")
            if (not.number(rhs)){
                par.start <- par.start(rhs, eq)
                if (not.number(par.start[2]) && (par.start[2] != "NA")) 
                    stop("Parse error in equation: ", eq,
                        "\n  Start values must be numeric constants.")
                ram <- paste(variables[1], " <-> ", variables[2], ", ", par.start[1], ", ", par.start[2], sep="")
            }
            else{
                ram <- paste(variables[1], " <-> ", variables[2], ", NA, ", rhs, sep="")
            }
        }
        else if (length(grep("^[vV]\\(", lhs)) > 0){
            lhs <- sub("[vV]\\(", "", lhs)
            if (length(grep("\\)$", lhs)) == 0) stop("Parse error in equation: ", eq,
                "\n  Unbalanced parentheses.")
            lhs <- sub("\\)", "", lhs)
            if (not.number(rhs)){
                par.start <- par.start(rhs, eq)
                if (not.number(par.start[2]) && (par.start[2] != "NA")) 
                    stop("Parse error in equation: ", eq,
                        "\n  Start values must be numeric constants.")
                ram <- paste(lhs, " <-> ", lhs, ", ", par.start[1], ", ", par.start[2], sep="")
            }
            else{
                ram <- paste(lhs, " <-> ", lhs, ", NA, ", rhs, sep="")
            }
        }
        else{
            terms <- strsplit(rhs, "\\+")[[1]]
            terms <- strsplit(terms, "\\*")
            ram <- character(length(terms))
            for (term in 1:length(terms)){
                trm <- terms[[term]]
                if (length(trm) != 2) stop("Parse error in equation: ", eq,
                    '\n  The term  "', trm, '" is malformed.',
                    '\n  Each term on the right-hand side of a structural equation must be of the form "parameter*variable".')
                coef <-  trm[1]
                if (not.number(coef)){
                    par.start <- par.start(coef, eq)
                    if (not.number(par.start[2]) && (par.start[2] != "NA")) 
                        stop("Parse error in equation: ", eq,
                            "\n  Start values must be numeric constants.")
                    ram[term] <- paste(trm[2], " -> ", lhs, ", ", par.start[1], ", ", par.start[2], sep="")
                }
                else{
                    ram[term] <- paste(trm[2], " -> ", lhs, ", NA, ", coef, sep="")
                }
            }
        }
        ram
    }
    equations <- if (!missing(text)) scan(text=text, what="", sep=";", strip.white=TRUE, comment.char="#")
      else scan(file=file, what="", sep=";", strip.white=TRUE, comment.char="#")
    equations2 <- character(0)
    eqn <- 0
    skip <- FALSE
    for(equation in equations){
        eqn <- eqn + 1
        if (skip){
            skip <- FALSE
            next
        }
        if (substring(equation, 1, 1) == "+"){
            equations2[length(equations2)] <- paste(equations2[length(equations2)], equation)
        }
        else if (substring(equation, nchar(equation)) == "+"){
            equations2 <- c(equations2, paste(equation, equations[eqn + 1]))
            skip <- TRUE
        }
        else equations2 <- c(equations2, equation)
    }
    ram <- unlist(lapply(equations2, parseEquation))
    specifyModel(text=ram, ..., quiet=TRUE)
}

# the following function (not exported) checks whether a text string can be converted into a number

not.number <- function(constant){
	save <- options(warn = -1)
	on.exit(save)
	is.na(as.numeric(constant))
}

# last modified 2012-07-29 by J. Fox

raw.moments <- function(...){
	.Deprecated("rawMoments", package="sem")
	rawMoments(...)
}

rawMoments <- function(object, ...) UseMethod("rawMoments")

rawMoments.formula <- function(formula, data, subset, na.action, 
		contrasts = NULL, ...) {
	if (missing(na.action))
		na.action <- options()$na.action
	m <- match.call(expand.dots = FALSE)
	if (is.matrix(eval(m$data, sys.frame(sys.parent()))))
		m$data <- as.data.frame(data)
	m$instruments <- m$contrasts <- NULL
	m[[1]] <- as.name("model.frame")
	mf <- eval(m, sys.frame(sys.parent()))
	response <- attr(attr(mf, "terms"), "response")
	if (response) stop("formula cannot have a response")
	na.act <- attr(mf, "na.action")
	X <- model.matrix(formula, data = mf, contrasts)
	nms <- colnames(X)
	if ("(Intercept)" %in% nms) colnames(X)[nms == "(Intercept)"] <- "Intercept"
	rawMoments(X)
}

rawMoments.default <- function(object, na.rm=FALSE, ...){
	object <- as.matrix(object)
	if (na.rm) object <- na.omit(object)
	N <- nrow(object)
	result <- crossprod(object, object)/N
	attr(result, "N") <- N
	class(result) <- "rawmoments"
	result
}

print.rawmoments <- function(x, ...){
	xx <- unclass(x)
	attr(xx, "N") <- NULL
	cat("\nRaw Moments\n")
	print(xx, ...)
	cat("\nN = ", attr(x, "N"), "\n")
	invisible(x)
}

cov2raw <- function(cov, mean, N, sd){
	if (all(1 == diag(cov)) && !missing(sd))
		cov <- cov * outer(sd, sd)
	raw <- ((N - 1)*cov + N*outer(mean, mean))/N
	colnames(raw) <- rownames(raw) <- rownames(cov)
	raw <- rbind(c(1, mean), cbind(mean, raw))
	if (!("Intercept" %in% rownames(raw)))
		colnames(raw)[1] <- rownames(raw)[1] <- "Intercept"
	attr(raw, "N") <- N
	class(raw) <- "rawmoments"
	raw
} 

# last modified 2012-01-06 by J. Fox

objectiveGLS2 <- function (gradient = FALSE) 
{
    result <- list(objective = function(par, model.description) {
        with(model.description, {
            A <- P <- matrix(0, m, m)
            val <- ifelse(fixed, ram[, 5], par[sel.free])
            A[arrows.1] <- val[one.head]
            P[arrows.2t] <- P[arrows.2] <- val[!one.head]
            I.Ainv <- solve(diag(m) - A)
            C <- J %*% I.Ainv %*% P %*% t(I.Ainv) %*% t(J)
#            Cinv <- solve(C)
            SS <- invS %*% (S - C)
            f <- 0.5 * sum(diag(SS %*% SS))
            attributes(f) <- list(C = C, A = A, P = P)
            f
        })
    })
    class(result) <- "semObjective"
    result
}

# last modified 2012-01-06 by J. Fox
# Modified for Compiled code in C/C++ by Zhenghua Nie.

objectiveFIML <- function (gradient = TRUE, hessian=FALSE) 
{
    result <- list(objective = function(par, model.description) {
        with(model.description, {
						 
						 res <- CompiledObjective(par=par, model.description=model.description, objective="objectiveFIML", gradient=gradient, hessian=hessian) 
				
						 f <- res$f
				     C <- res$C
				     A <- res$A
				     P <- res$P

						 grad <- NULL
						 if(gradient)
								 grad <- res$gradient
						 hess <- NULL
						 if(hessian) 
								 hess <- res$hessian

						 attributes(f) <- list(C = C, A = A, P = P, gradient=grad, hessian=hess)
						 f
}
				)
}
		)
		class(result) <- "semObjective"
		result
}

#objectivelogLik <- function(gradient=FALSE, hessian=FALSE)
#{
#		result<-list(objective = function(par, object){
#								 with(object, {

#											res <- CompiledObjective(par=par, model.description=object, objective="objectivelogLik", gradient=gradient, hessian=hessian)

#		f <- res$f
#		grad <- NULL
#		if(gradient)
#				grad <- res$gradient
#		hess <- NULL
#		if(hessian)
#				hess <- res$hessian
#		attributes(f) <- list(gradient=grad, hessian=hess)
#		f
#				}
#		)
#		}
#		)
#		result
#}


logLik.objectiveFIML <- function(object, saturated=FALSE, intercept="Intercept", iterlim=1000, ...){
		logLikSaturated <- function(object, iterlim, ...){
		#objective <- function(par)
		#{

		#		res <- CompiledObjective(par=par, model.description=object, objective="objectivelogLik", gradient=FALSE, hessian=FALSE)

		#		f <- res$f
		#		f
		#}

		data <- object$data
		valid <- !is.na(data)
		valid.pattern <- apply(valid, 1, function(row) paste(row, collapse="."))
		unique.patterns <- unique(valid.pattern)
		pattern.number <- apply(outer(valid.pattern, unique.patterns, `==`), 1, which)
		valid.data.patterns <- t(sapply(strsplit(unique.patterns, "\\."), as.logical))
		n.pat <- nrow(valid.data.patterns) 
		log.2pi <- log(2*pi)
		n <- ncol(data)
		N <- nrow(data)
		C <- object$C
		tri <- lower.tri(C, diag=TRUE)

		posn.intercept <- which(rownames(C) == intercept)

		tri[posn.intercept, posn.intercept] <- FALSE
		start <- C[tri]
		opt <- options(warn=-1)
		on.exit(options(opt))
		object$posn.intercept <- posn.intercept
		object$intercept <- intercept
		object$pattern.number <- pattern.number
		object$valid.data.patterns <- valid.data.patterns
		object$tri <- tri

		#res <- nlm(objective, start, iterlim=iterlim)
		res <- CompiledSolve(model.description=object, start=start, objective="objectivelogLik",  maxiter=iterlim)

		logL <- - res$minimum/2
		C <- matrix(0, n, n)
		C[tri] <- res$estimate
		C <- C + t(C) - diag(diag(C))
		C[posn.intercept, posn.intercept] <- 1
		list(logL=logL, C=C, code=res$code)
		}
		if (saturated) {
				res <- logLikSaturated(object, iterlim=iterlim)
				if (res$code > 3) warning("nlm return code = ", res$code)
				logL <- res$logL
				attr(logL, "C") <- res$C
				return(logL)
		}
		else return(- object$criterion*object$N/2)
}

# last modified 2019-11-15 by J. Fox

sem <- function(model, ...){
	if (is.character(model)) class(model) <- "semmod"
	UseMethod("sem", model)
}

sem.semmod <- function(model, S, N, data, raw=identical(na.action, na.pass), obs.variables=rownames(S), 
		fixed.x=NULL, formula= ~ ., na.action=na.omit, robust=!missing(data), debug=FALSE, 
		optimizer=optimizerSem, objective=objectiveML, ...){
	parse.path <- function(path) {                                           
		path.1 <- gsub("-", "", gsub(" ","", path))
		direction <- if (regexpr("<>", path.1) > 0) 2 
				else if (regexpr("<", path.1) > 0) -1
				else if (regexpr(">", path.1) > 0) 1
				else stop(paste("ill-formed path:", path))
		path.1 <- strsplit(path.1, "[<>]")[[1]]
		list(first=path.1[1], second=path.1[length(path.1)], direction=direction)
	}
	any.NA <- FALSE
	unique.patterns <- valid.pattern <- valid <- pattern.number <- valid.data.patterns <- NULL
	if (missing(S)){
		if (missing(data)) stop("S and data cannot both be missing")
		N.all <- nrow(data)
		data <- model.frame(formula, data=data, na.action=na.action)
		data <- model.matrix(formula, data=data)
		colnames(data)[colnames(data) == "(Intercept)"] <- "Intercept"
		S <- if (raw) rawMoments(data, na.rm=TRUE) else {
					data <-  data[, colnames(data) != "Intercept"]
					cov(data, use="complete.obs")
				}
		N <- nrow(data)
		if (N < N.all) warning(N - N.all, " observations removed due to missingness")        
		if (identical(na.action, na.pass) && any(is.na(data))){
				any.NA <- TRUE
				valid <- !is.na(data)
				colnames(valid) <- colnames(data)
			}
			else {
					valid <- matrix(TRUE, nrow(data), ncol(data))
					colnames(valid) <- colnames(data)
			}
		}
	if ((!is.matrix(model)) | ncol(model) != 3) stop ("model argument must be a 3-column matrix")
	startvalues <- as.numeric(model[,3])
	par.names <- model[,2]
	n.paths <- length(par.names)
	heads <- from <- to <- rep(0, n.paths)
	for (p in 1:n.paths){
		path <- parse.path(model[p,1])
		heads[p] <- abs(path$direction)
		to[p] <- path$second
		from[p] <- path$first
		if (path$direction == -1) {
			to[p] <- path$first
			from[p] <- path$second
		}
	}
	ram <- matrix(0, p, 5)
	all.vars <- unique(c(to, from))
	latent.vars <- setdiff(all.vars, obs.variables)
	not.used <- setdiff(obs.variables, all.vars)
	if (length(not.used) > 0){
		rownames(S) <- colnames(S) <- obs.variables
		obs.variables <- setdiff(obs.variables, not.used)
		S <- S[obs.variables, obs.variables]
		warning("The following observed variables are in the input covariance or raw-moment matrix ",
				"but do not appear in the model:\n",
				paste(not.used, collapse=", "), "\n")
	}
	vars <- c(obs.variables, latent.vars)
	pars <- na.omit(unique(par.names))
	ram[,1] <- heads
	ram[,2] <- apply(outer(vars, to, "=="), 2, which)
	ram[,3] <- apply(outer(vars, from, "=="), 2, which)   
	par.nos <- apply(outer(pars, par.names, "=="), 2, which)
	if (length(par.nos) > 0)
		ram[,4] <- unlist(lapply(par.nos, function(x) if (length(x) == 0) 0 else x))
	ram[,5]<- startvalues
	colnames(ram) <- c("heads", "to", "from", "parameter", "start")
	if (!is.null(fixed.x)) fixed.x <- apply(outer(vars, fixed.x, "=="), 2, which)
	n <- length(obs.variables)
	m <- length(all.vars)
	t <- length(pars)
	if (debug) {
		cat("\n observed variables:\n") 
		print(paste(paste(1:n,":", sep=""), obs.variables, sep=""))
		cat("\n")
		if (m > n){ 
			cat("\n latent variables:\n")
			print(paste(paste((n+1):m,":", sep=""), latent.vars, sep=""))
			cat("\n")
		}
		cat("\n parameters:\n") 
		print(paste(paste(1:t,":", sep=""), pars, sep=""))
		cat("\n\n RAM:\n")
		print(ram)
	}
	if (missing(data)) data <- NULL 
	else {
		data <- data[, obs.variables]
		if (!is.null(valid)) {
			valid <- valid[, obs.variables]
			valid.pattern <- apply(valid, 1, function(row) paste(row, collapse="."))
			unique.patterns <- unique(valid.pattern)
			pattern.number <- apply(outer(valid.pattern, unique.patterns, `==`), 1, which)
			valid.data.patterns <- t(sapply(strsplit(unique.patterns, "\\."), as.logical))
		}
	}
	if (identical(objective, objectiveFIML2) || identical(objective, objectiveFIML)){
		message("NOTE: start values computed from preliminary ML fit")
        opt <- options(warn=-1)
        on.exit(options(opt)) # in case of error
		prelim.fit <- sem(ram, S=S, N=N, raw=raw, data=na.omit(data), valid=valid, param.names=pars, var.names=vars, fixed.x=fixed.x,
				semmod=model, robust=robust, debug=debug, ...)
        if (!prelim.fit$convergence) message("NOTE: preliminary ML fit may not have converged")
        options(opt)
		message("NOTE: preliminary iterations, ", prelim.fit$iterations)
		message("NOTE: iterations reported for final fit are post preliminary fit")
		coeff <- coef(prelim.fit)
		rownames(ram) <- rownames(prelim.fit$ram)[1:nrow(ram)]
		ram[names(coeff), 5] <- coeff
	}
	cls <- gsub("\\.", "", deparse(substitute(objective)))
	cls <- gsub("2", "", cls)
	result <- sem(ram, S=S, N=N, raw=raw, data=data, 
			pattern.number=pattern.number, valid.data.patterns=valid.data.patterns,
			param.names=pars, var.names=vars, fixed.x=fixed.x,
			semmod=model, robust=robust, debug=debug, optimizer=optimizer, objective=objective, cls=cls, ...)
	class(result) <- c(cls, "sem")
	result
}

sem.default <- function(model, S, N, raw=FALSE, data=NULL, start.fn=startvalues,
		pattern.number=NULL, valid.data.patterns=NULL,
		use.means=TRUE, param.names, 
		var.names, fixed.x=NULL, robust=!is.null(data), semmod=NULL, debug=FALSE,
		analytic.gradient=!identical(objective, objectiveFIML), warn=FALSE, maxiter=1000, par.size=c("ones", "startvalues"), 
		start.tol=1E-6, optimizer=optimizerSem, objective=objectiveML, cls, ...){
	ord <- function(x) 1 + apply(outer(unique(x), x, "<"), 2, sum)
	is.triangular <- function(X) {
		is.matrix(X) && (nrow(X) == ncol(X)) && 
				(all(0 == X[upper.tri(X)])) || (all(0 == X[lower.tri(X)]))
	} 
	ram <- model
	S <- unclass(S) # in case S is a rawmoment object
	if (nrow(S) > 1 && is.triangular(S)) S <- S + t(S) - diag(diag(S))
	if (!isSymmetric(S)) stop("S must be a square triangular or symmetric matrix")
	if (qr(S)$rank < ncol(S)) warning("S is numerically singular: expect problems")
	if (any(eigen(S, symmetric=TRUE, only.values=TRUE)$values <= 0)) 
		warning("S is not positive-definite: expect problems")
	if ((!is.matrix(ram)) | ncol(ram) != 5 | (!is.numeric(ram)))
		stop ("ram argument must be a 5-column numeric matrix")
	par.size <- if (missing(par.size)) {
				range <- range(diag(S))
				if (range[2]/range[1] > 100) "startvalues" else "ones"
			}
			else match.arg(par.size)
	n <- nrow(S)
	observed <- 1:n
	n.fix <- length(fixed.x)
	if (!is.null(fixed.x)){
		for (i in 1:n.fix){
			for (j in 1:i){
				ram <- rbind(ram, c(2, fixed.x[i], fixed.x[j], 
								0, S[fixed.x[i], fixed.x[j]]))
			}
		}
	}
	m <- max(ram[,c(2,3)])
	missing.variances <- setdiff(1:m, ram[,2][ram[,2] == ram[,3]])
	if (length(missing.variances) > 0) warning(paste("The following variables have no variance or error-variance parameter (double-headed arrow):\n",
						paste(var.names[missing.variances], collapse=", "), 
						"\nThe model is almost surely misspecified; check also for missing covariances.\n"))
	t <- max(ram[,4])
	df <- n*(n + 1)/2 - t - n.fix*(n.fix + 1)/2
	if (df < 0) stop(paste("The model has negative degrees of freedom =", df))
	J <- matrix(0, n, m)
	correct <- matrix(2, m, m)
	diag(correct) <- 1
	J[cbind(1:n, observed)]<-1
	par.posn <-  sapply(1:t, function(i) which(ram[,4] == i)[1])
	colnames(ram)<-c("heads", "to", "from", "parameter", "start value")
	rownames(ram)<-rep("",nrow(ram))
	if (length(param.names) > 0) rownames(ram)[par.posn]<-param.names
	fixed <- ram[,4] == 0
	sel.free <- ram[,4]
	sel.free[fixed] <- 1
	one.head <- ram[,1] == 1
	one.free <- which( (!fixed) & one.head )
	two.free <- which( (!fixed) & (!one.head) )
	arrows.1 <- ram[one.head, c(2,3), drop=FALSE]
	arrows.2 <- ram[!one.head, c(2,3), drop=FALSE]
	arrows.2t <- ram[!one.head, c(3,2), drop=FALSE]
	arrows.1.free <- ram[one.free,c(2,3), drop=FALSE]
	arrows.2.free <- ram[two.free,c(2,3), drop=FALSE]
	sel.free.1 <- sel.free[one.free]
	sel.free.2 <- sel.free[two.free]
	unique.free.1 <- unique(sel.free.1)
	unique.free.2 <- unique(sel.free.2)
	rownames(S) <- colnames(S) <- var.names[observed]
	result <- list(var.names=var.names, ram=ram, S=S, J=J, n.fix=n.fix, n=n, N=N, m=m, t=t, raw=raw,
			data=data, semmod=semmod, optimizer=optimizer, objective=objective,
			# remaining values to be supplied after optimization
			coeff=NULL, vcov=NULL, par.posn=NULL, convergence=NULL, iterations=NULL, criterion=NULL, C=NULL, A=NULL, P=NULL,
			adj.obj=NULL, robust.vcov=NULL)
	if (length(param.names)== 0){
		warning("there are no free parameters in the model")
	}
	else {
		if (!is.null(data) && raw && use.means){
			to <- ram[, 2]
			from <- ram[, 3]
			rows <- (from == which(var.names == "Intercept")) & (ram[, 1] == 1) & (ram[, 4] != 0) & (to <= n) & is.na(ram[, 5])
			ram[rows, 5] <- colMeans(data, na.rm=TRUE)[var.names[to[rows]]]
		}
		start <- if (any(is.na(ram[,5][par.posn])))
					start.fn(S, ram, debug=debug, tol=start.tol)
				else ram[,5][par.posn]
		typsize <- if (par.size == "startvalues") abs(start) else rep(1,t)
		model.description <- list(data=data, 
				pattern.number=pattern.number, valid.data.patterns=valid.data.patterns,
				S=S, logdetS=log(det(S)), invS=solve(S), N=N, m=m, n=n, t=t, 
				fixed=fixed, ram=ram, sel.free=sel.free, arrows.1=arrows.1, arrows.1.free=arrows.1.free,
				one.head=one.head, arrows.2t=arrows.2t, arrows.2=arrows.2, arrows.2.free=arrows.2.free, 
				unique.free.1=unique.free.1, unique.free.2=unique.free.2,
				J=J, correct=correct, param.names=param.names, var.names=var.names, observed=observed, raw=raw)
		res <- optimizer(start=start, 
				objective=objective, gradient=analytic.gradient, maxiter=maxiter, debug=debug, par.size=par.size, 
				model.description=model.description, warn=warn, ...)
		ram[par.posn, 5] <- start
		par.code <- paste(var.names[ram[,2]], c("<---", "<-->")[ram[,1]],
				var.names[ram[,3]])
		result$coeff <- res$par
		result$vcov <- res$vcov
		result$par.posn <- par.posn
		result$convergence <- res$convergence
		result$iterations <- res$iterations
		result$criterion <-  res$criterion
		result$C <- res$C
		result$A <- res$A
		result$P <- res$P
		if (!is.na(result$iterations)) if(result$iterations >= maxiter) warning("maximum iterations exceeded")
	}
	if (missing(cls)){
		cls <- gsub("\\.", "", deparse(substitute(objective)))
		cls <- gsub("2", "", cls)
	}
#	if(cls == "objectiveCompiledGLS") 
#			cls <- c(cls, "objectiveGLS")
#	else if(cls == "objectiveCompiledML") 
#			cls <- c(cls, "objectiveML")
	class(result) <- c(cls, "sem")
	if (robust && !is.null(data) && inherits(result, "objectiveML")){
		result$adj.obj <- sbchisq(result, na.omit(data))
		result$robust.vcov <- robustVcov(result, adj.obj=result$adj.obj)
	}
	result
}

vcov.sem <- function(object, robust=FALSE, analytic=inherits(object, "objectiveML") && object$t <= 500, ...) {
	if (robust) return(object$robust.vcov)
	if (!analytic) return(object$vcov)
	if (!inherits(object, "objectiveML")) stop("analytic coefficient covariance matrix unavailable")
	hessian <- function(model){
#		accumulate <- function(A, B, C, D, d) {
#				res <- matrix(0, d^2, d^2)
#			B[1:d, 1:d] %x% A[1:d, 1:d] + matrix(rep(rep(t(C[1:d, 1:d]), 1, each=d), d), d^2, d^2, byrow=TRUE) * matrix(rep(rep((D[1:d, 1:d]), 1, each=d), d), d^2, d^2)
#		}    
		A <- model$A
		P <- model$P
		S <- model$S
		C <- model$C
		J <- model$J
		m <- model$m
		t <- model$t
		I.Ainv <- solve(diag(m) - A) 
		Cinv <- solve(C)    
		AA <- t(I.Ainv) %*% t(J)
		BB <- J %*% I.Ainv %*% P %*% t(I.Ainv)
		CC <- t(I.Ainv) %*% t(J)
		DD <- J %*% I.Ainv
		dF.dBdB <- accumulate(AA %*% Cinv %*% t(AA), t(BB) %*% Cinv %*% BB,
				AA %*% Cinv %*% BB, t(BB) %*% Cinv %*% t(AA), m)                
		dF.dPdP <- accumulate(CC %*% Cinv %*% t(CC), t(DD) %*% Cinv %*% DD,
				CC %*% Cinv %*% DD, t(DD) %*% Cinv %*% t(CC), m)                
		dF.dBdP <- accumulate(AA %*% Cinv %*% t(CC), t(BB) %*% Cinv %*% DD,
				AA %*% Cinv %*% DD, t(BB) %*% Cinv %*% t(CC), m)               
		ram <- model$ram    
		fixed <- ram[, 4] == 0
		sel.free <- ram[, 4]
		sel.free[fixed] <- 0
		one.head <- ram[, 1] == 1
		one.free <- which( (!fixed) & one.head )
		two.free <- which( (!fixed) & (!one.head) )
		two.free.cov <- which((!fixed) & (!one.head) & (ram[, 2] != ram[, 3]))
		arrows.1 <- ram[one.head, c(2, 3), drop=FALSE]
		arrows.2 <- ram[!one.head, c(2, 3), drop=FALSE]
		arrows.2t <- ram[!one.head, c(3, 2), drop=FALSE]
		arrows.1.free <- ram[one.free, c(2, 3), drop=FALSE]
		arrows.2.free <- ram[two.free, c(2, 3), drop=FALSE]
		sel.free.1 <- sel.free[one.free]
		sel.free.2 <- sel.free[two.free]
		unique.free.1 <- unique(sel.free.1)
		unique.free.2 <- unique(sel.free.2)    
		posn.matrix <- matrix(1:(m^2), m, m)    
		posn.free <- c(posn.matrix[arrows.1.free], 
				(m^2) + posn.matrix[arrows.2.free])        
		DBB <- dF.dBdB[posn.matrix[arrows.1.free], 
				posn.matrix[arrows.1.free], drop=FALSE]
		DPP <- dF.dPdP[posn.matrix[arrows.2.free], 
				posn.matrix[arrows.2.free], drop=FALSE]
		DBP <- dF.dBdP[posn.matrix[arrows.1.free],
				posn.matrix[arrows.2.free], drop=FALSE]
#    browser()
		hessian <- rbind( cbind(DBB,    DBP),
				cbind(t(DBP), DPP))    
		n1 <- length(one.free)
		n2 <- length(two.free)
		nn <- rep(c(sqrt(2), sqrt(2)/2), c(n1, n2))
		nn[c(one.free, two.free) %in% two.free.cov] <- sqrt(2)
#    browser()
		hessian <- hessian * outer(nn, nn)
		pars <- ram[, 4][!fixed]
		Z <- outer(1:t, pars, function(x, y) as.numeric(x == y))
		hessian <- Z %*% hessian %*% t(Z)
		par.names <- c(names(one.free), names(two.free))
		par.names <- par.names[par.names != ""]
		rownames(hessian) <- colnames(hessian) <- par.names
		nms <- names(coef(object))
		hessian[nms, nms]
	}
	h <- hessian(object)
	t <- object$t
	N <- object$N
	raw <- object$raw
	param.names <- rownames(h)
	vcov <- matrix(NA, t, t)
	qr.hess <- try(qr(h), silent=TRUE)
	if (inherits(qr.hess, "try-error")){
		warning("Could not compute QR decomposition of Hessian.\nOptimization probably did not converge.\n")
	}
	else if (qr.hess$rank < t){
		warning(" singular Hessian: model is probably underidentified.\n")
		which.aliased <- qr.hess$pivot[-(1:qr.hess$rank)]
		attr(vcov, "aliased") <- param.names[which.aliased]
	}
	else {
		vcov <- (2/(N - (!raw))) * solve(h)
		if (any(diag(vcov) < 0)) {
			attr(vcov, "aliased") <- param.names[diag(vcov) < 0]
			warning("Negative parameter variances.\nModel may be underidentified.\n")
		}
	}
	vcov
}

coef.sem <- function(object, standardized=FALSE, ...){
	if (!standardized) return(object$coeff)
	sc <- stdCoef(object, ...)
	names <- as.character(sc[, 1])
	which <- names != " "
	coef <- sc[which, 2]
	names(coef) <- names[which]
	coef
}

# the following auxiliary function is for computing Hessians

accumulate <- function(A, B, C, D, d) {
	B[1:d, 1:d] %x% A[1:d, 1:d] + matrix(rep(rep(t(C[1:d, 1:d]), 1, each=d), d), d^2, d^2, byrow=TRUE) * matrix(rep(rep((D[1:d, 1:d]), 1, each=d), d), d^2, d^2)
}    

# last modified 2019-11-15 by J. Fox
# Modified for Compiled Objective and nlm in C/C++ by Zhenghua Nie.

optimizerSem <- function(start, objective=objectiveML,  
	gradient=TRUE, maxiter, debug, par.size, model.description, warn, ...){
	with(model.description, {
			obj <- objective(gradient=gradient)$objective
			typsize <- if (par.size == 'startvalues') abs(start) else rep(1, t) #we move typesize in csem.cpp.

#			objectiveCompiled <- "objectiveML"
#			if(identical(objective, objectiveCompiledML) || identical(objective, objectiveML)) 
#					objectiveCompiled <- "objectiveML"
#			if(identical(objective, objectiveCompiledGLS) || identical(objective, objectiveGLS))
#					objectiveCompiled <- "objectiveGLS"
				
#			objectiveCompiled <- deparse(substitute(objective))
#			if (!objectiveCompiled %in% c("objectiveML", "objectiveGLS")) stop("optimizerSem requires the objectiveML or objectiveGLS objective function")
#			objectiveCompiled <- "objectiveML"
				
			if(identical(objective, objectiveML)) objectiveCompiled <- "objectiveML"
			else if (identical(objective, objectiveGLS)) objectiveCompiled <- "objectiveGLS"
			else if (identical(objective, objectiveFIML)) objectiveCompiled <- "objectiveFIML"
			else stop("optimizerSem requires the objectiveML or objectiveGLS or objectiveFIML objective function")
			
			if (!warn) save.warn <- options(warn=-1)

			res <- CompiledSolve(model.description=model.description, start=start, objective=objectiveCompiled, gradient=gradient, typsize=typsize, debug=debug, maxiter=maxiter)

			if (!warn) options(save.warn)
			result <- list()
			result$convergence <- res$code <= 2
			result$iterations <- res$iterations
			par <- res$estimate
			names(par) <- param.names
			result$par <- par
			if (!result$convergence)
				warning(paste('Optimization may not have converged; nlm return code = ',
						res$code, '. Consult ?nlm.\n', sep=""))
			vcov <- matrix(NA, t, t)
			qr.hess <- try(qr(res$hessian), silent=TRUE)
			if (inherits(qr.hess, "try-error")){
				warning("Could not compute QR decomposition of Hessian.\nOptimization probably did not converge.\n")
			}
			else if (qr.hess$rank < t){
				warning(' singular Hessian: model is probably underidentified.\n')
				which.aliased <- qr.hess$pivot[-(1:qr.hess$rank)]
				result$aliased <- param.names[which.aliased]
			}
			else {
				vcov <- (2/(N - (!raw))) * solve(res$hessian)
				if (any(diag(vcov) < 0)) {
					result$aliased <- param.names[diag(vcov) < 0]
					warning("Negative parameter variances.\nModel may be underidentified.\n")
				}
			}
			colnames(vcov) <- rownames(vcov) <- param.names
			result$vcov <- vcov
			result$criterion <- res$minimum # c(result$obj) - n - log(det(S))
#			if(identical(objective, objectiveML) || identical(objective, objectiveGLS)){
					C <- res$C
					A <- res$A
					P <- res$P
#			} else {
#					obj <- obj(par, model.description)  
#					C <- attr(obj, "C")
#					A <- attr(obj, "A")
#					P <- attr(obj, "P")
#			}

			rownames(C) <- colnames(C) <- var.names[observed]
			result$C <- C
			rownames(A) <- colnames(A) <- var.names
			result$A <- A
			rownames(P) <- colnames(P) <- var.names
			result$P <- P
			class(result) <- "semResult"
			result
	}
)
}

# these functions originally by Jarrett Byrnes

# last modified 2019-11-15 by J. Fox

robustVcov <- function(sem.obj, adj.obj, data.obj, use.fit=FALSE, use.ginv=FALSE){
	if (missing(adj.obj) && missing(data.obj)) 
		stop("Need a data or sbchisq object")
	if (missing(adj.obj)) {
		adj.obj <- sbchisq(sem.obj, data.obj, use.fit = use.fit)
	}
	ncases <- sem.obj$N
	hes <- semHessian(adj.obj$w_mat, adj.obj$p_deriv_mat)
	info_m <- try(solve(hes), silent = TRUE)
	if (inherits(info_m, "try-error") && isTRUE(use.ginv)) {
		info_m <- ginv(hes)
		ginvFlag <- TRUE
	}
	acov <- info_m %*% (t(adj.obj$p_deriv_mat) %*% adj.obj$w_mat %*% 
			adj.obj$w_adf %*% adj.obj$w_mat %*% adj.obj$p_deriv_mat) %*% 
		info_m
	acov <- acov/(ncases - 1)
	rownames(acov) <- colnames(acov) <- colnames(adj.obj$p_deriv_mat)
	acov
}

sbchisq <- function(sem.obj, sem.data, adj=1e-04, use.fit=FALSE, use.ginv=FALSE){
	props <- semProps(sem.obj)
	chisq <- props$chisq
	df <- props$df
	w_adf <- adfWmat(sem.data)
	w_mat <- mlWmat(sem.obj, use.fit = use.fit)
	p_deriv_mat <- deltaMatrix(sem.obj)
	ginvFlag <- FALSE
	invMat <- try(solve(t(p_deriv_mat) %*% w_mat %*% p_deriv_mat), 
		silent = TRUE)
	if (inherits(invMat, "try-error") && isTRUE(use.ginv)) {
		invMat <- ginv(t(p_deriv_mat) %*% w_mat %*% p_deriv_mat)
		ginvFlag <- TRUE
	}
	res_u <- w_mat - (w_mat %*% p_deriv_mat %*% invMat %*% t(p_deriv_mat) %*% 
			w_mat)
	ug <- res_u %*% w_adf
	scale_stat <- sum(diag(ug))/df
	chisq.scaled <- chisq/scale_stat
	p.old <- pchisq(chisq, df, lower.tail=FALSE)
	p <- pchisq(chisq.scaled, df, lower.tail=FALSE)
	ret <- list(chisq = chisq, t = sem.obj$t, df = df, p.old = p.old, 
		c = scale_stat, chisq.scaled = chisq.scaled, p = p, w_mat = w_mat, 
		p_deriv_mat = p_deriv_mat, w_adf = w_adf, res_u = res_u, 
		N = length(sem.data[, 1]), ginvFlag = ginvFlag)
	class(ret) <- "adjchisq"
	return(ret)
}

adfWmat <- function(rawdata) {
	names <- colnames(rawdata)
	n <- nrow(rawdata)
	n.col <- ncol(rawdata)
	nc.star <- n.col * (n.col + 1)/2
	nc2 <- n.col^2
	z <- scale(rawdata, center=TRUE, scale=FALSE)
	sc <- vector(nc.star, mode="list")
	outnames <- vector(nc.star, mode="character")
	ind <- combn(n.col + 1, 2)
	ind[2, ] <- ind[2, ] - 1
	for (q in 1:nc.star) {
		i <- ind[1, q]
		j <- ind[2, q]
		outnames[q] <- paste(names[i], names[j], sep="_")
		sc[[q]] <- z[, i] * z[, j]
	}
	names(sc) <- outnames
	adf_mat <- var(data.frame(sc)) * (n - 1)/n
	return(adf_mat)
}

mlWmat <- function(sem.obj, use.fit=FALSE) {
	p <- nrow(sem.obj$C) 
	if (use.fit) {
		An <- sem.obj$C
	}
	else {
		An <- sem.obj$S
	}
	Dp <- Ktrans(p)
	An.inv <- solve(An)
	w_mat <- 0.5 * t(Dp) %*% kronecker(An.inv, An.inv) %*% Dp
	rownames(w_mat) <- vech(matrixNames(sem.obj$C))
	colnames(w_mat) <- rownames(w_mat)
	return(w_mat)
}

deltaMatrix <- function(sem.object, adj=1e-04) {
	p.star <- sem.object$n * (sem.object$n + 1)/2
	pars <- names(sem.object$coeff)
	nparms <- length(pars)
	delta.mat <- matrix(0, nparms, p.star)
	rownames(delta.mat) <- pars
	vars <- sem.object$var.names
	J <- sem.object$J
	m <- sem.object$m
	for (j in 1:nparms) {
		A <- sem.object$A
		P <- sem.object$P
		i <- which(rownames(sem.object$ram) == pars[j])
		from <- sem.object$ram[i, 2]
		to <- sem.object$ram[i, 3]
		path_type <- sem.object$ram[i, 1][1]
		if (path_type == 1){
			AA <- A[cbind(from, to)][1]
			adjust <- abs(AA) * adj
			A[cbind(from, to)] <- AA + adjust
		}
		else {
			PP <- P[cbind(to, from)][1]
			adjust <- PP * adj
			P[cbind(from, to)] <- P[cbind(to, from)] <- PP + adjust
		}
		I.Ainv <- solve(diag(m) - A)
		C <- J %*% I.Ainv %*% P %*% t(I.Ainv) %*% t(J)
		delta.mat[j, ] <- (vech(C) - vech(sem.object$C))/adjust
	}
	t(delta.mat)
}

semProps <- function(object){
	N <- object$N
	n <- object$n
	t <- object$t
	n.fix <- object$n.fix
	list(N=N, n=n, t=t, n.fix=n.fix, df=n*(n + 1)/2 - t - n.fix*(n.fix + 1)/2, 
			chisq=object$criterion*(N - (!object$raw)))
}

Ktrans <- function(num.vars){
	duplication.matrix(num.vars)
}

matrixNames <- function(mat, sep="_"){
	outer(rownames(mat), colnames(mat), function(x, y) paste(x, y, sep=sep))
}

semHessian <- function(w.mat, delta.mat){
	ret <- t(delta.mat) %*% w.mat %*% delta.mat
	return(ret)
}

# last modified 2012-10-25 by J. Fox

## this is the straightforward approach summing over observations:
# objectiveFIML2 <- function(gradient=FALSE){
#   result <- list(
#     objective = function(par, model.description){
#       with(model.description, {
#         A <- P <- matrix(0, m, m)
#         val <- ifelse (fixed, ram[,5], par[sel.free])
#         A[arrows.1] <- val[one.head]
#         P[arrows.2t] <- P[arrows.2] <- val[!one.head]
#         I.Ainv <- solve(diag(m) - A)
#         C <- J %*% I.Ainv %*% P %*% t(I.Ainv) %*% t(J)
#         f <- 0
#         log.2pi <- log(2*pi)
#         for (i in 1:nrow(data)){
#           sel <- valid[i, ]
#           x <- data[i, sel]
#           CC <- C[sel, sel]
#           f <- f + log(det(CC)) + as.vector(x %*% solve(CC) %*% x) + log.2pi*(sum(sel))
#         }
#         f <- f/N
#         attributes(f) <- list(C=C, A=A, P=P)
#         f
#       })
#     }
#   )
#   class(result) <- "semObjective"
#   result
# }

## this avoids duplication by using the svd for the inverse and determinant but is actually slower in R
# objectiveFIML2 <- function(gradient=FALSE){
#   result <- list(
#     objective = function(par, model.description){
#       with(model.description, {
#         A <- P <- matrix(0, m, m)
#         val <- ifelse (fixed, ram[,5], par[sel.free])
#         A[arrows.1] <- val[one.head]
#         P[arrows.2t] <- P[arrows.2] <- val[!one.head]
#         I.Ainv <- solve(diag(m) - A)
#         C <- J %*% I.Ainv %*% P %*% t(I.Ainv) %*% t(J)
#         f <- 0
#         log.2pi <- log(2*pi)
#         for (i in 1:nrow(data)){
#           sel <- valid[i, ]
#           x <- data[i, sel]
#           CC <- C[sel, sel]
#           svdCC <-svd(CC)
#           f <- f + log(prod(svdCC$d)) + as.vector(x %*% svdCC$v %*% diag(1/svdCC$d) %*% t(svdCC$u) %*% x) + log.2pi*(sum(sel))
#         }
#         f <- f/N
#         attributes(f) <- list(C=C, A=A, P=P)
#         f
#       })
#     }
#   )
#   class(result) <- "semObjective"
#   result
# }

## this caches results for distinct valid-data patterns but still sums over observations:
# objectiveFIML2 <- function(gradient=FALSE){
#   result <- list(
#     objective = function(par, model.description){
#       with(model.description, {
#         A <- P <- matrix(0, m, m)
#         val <- ifelse (fixed, ram[,5], par[sel.free])
#         A[arrows.1] <- val[one.head]
#         P[arrows.2t] <- P[arrows.2] <- val[!one.head]
#         I.Ainv <- solve(diag(m) - A)
#         C <- J %*% I.Ainv %*% P %*% t(I.Ainv) %*% t(J)
#         f <- 0
#         log.2pi <- log(2*pi)
#         CC <- vector(length(unique.patterns), mode="list")
#         names(CC) <- unique.patterns
#         log.det.plus <- CC
#         for (i in 1:nrow(data)){
#           sel <- valid[i, ]
#           if (is.null(log.det.plus[[valid.pattern[i]]])){
#             log.det.plus[[valid.pattern[i]]] <- log(det(C[sel, sel])) + log.2pi*(sum(sel))
#             CC[[valid.pattern[i]]] <- solve(C[sel, sel])
#           }
#           x <- data[i, sel]
#           f <- f + as.vector(x %*% CC[[valid.pattern[i]]] %*% x) + log.det.plus[[valid.pattern[i]]]
#         }
#         f <- f/N
#         attributes(f) <- list(C=C, A=A, P=P)
#         f
#       })
#     }
#   )
#   class(result) <- "semObjective"
#   result
# }

## this sums over distinct valid data patterns rather than observations:
objectiveFIML2 <- function(gradient=TRUE, hessian=FALSE){

## the following two functions are commented since right now we require the package matriccalc when loading sem.		
#				commutation <- function( m, n )
#				{
						# this function return a permutation matrix T_(m, n) which
						# have the following definition:
						# T_(m, n)vec(A) = vec(A^T)
						# where A is a m-by-n matrix.
						# T_(m, n) is an orthogonal matrix. T_(m, n)=T(n, m)^T
						# Magnus,  J. R. and H. Neudecker (1979). The commutation matrix: some properties and applica- tions,  The Annals of Statistics,  7(2),  381-394.
						# a square mn-by-mn matrix partioned into mn submatrices of n-by-m such that the i, jth matrix has a 1 in its j, ith position,  others are zeros. 

#						p <- m * n
#						C <- matrix( 0, nrow=p, ncol=p )
#						r <- 0
#						for ( i in 1:m ) {
#								c <- i # the ith row submatrix
#								for ( j in 1:n ) { # the ith row,  jth column submatrix
#										r <- r + 1 
#										C[r,c] <- 1  # the (j, i) enttry
#										c <- c + m # for the next submatrix
#								}
#						}
#						return( C )
#				}

#				vec <- function( x )
#				{
            ## a matrix (m, n) to a vec matrix  (1*(m*n))
#						return( t( t( as.vector( x ) ) ) )
#				}

				extendMatrix <- function(selA, sel)
				{
						nsel <- length(sel);
						A <- matrix(data=0, nrow=nsel, ncol=nsel);
						B <- matrix(data=0, nrow=nsel, ncol=ncol(selA));
						iselA <- 1
						#extend row
						for(i in (1:nsel)) {
								if(sel[i])
								{
										B[i, ] = selA[iselA, ];
										iselA <- iselA + 1;
								}
						}
						#extend column
						iselA <- 1;
						for(i in (1:nsel))
						{
								if(sel[i])
								{
										A[, i]=B[, iselA];
										iselA <- iselA + 1;
								}
						}
						A
				}


				result <- list(
											 objective = function(par, model.description){
													 with(model.description, {
																A <- P <- matrix(0, m, m)
													 val <- ifelse (fixed, ram[,5], par[sel.free])
													 A[arrows.1] <- val[one.head]
													 P[arrows.2t] <- P[arrows.2] <- val[!one.head]
													 I.Ainv <- solve(diag(m) - A)
													 C <- J %*% I.Ainv %*% P %*% t(I.Ainv) %*% t(J)
													 f <- 0
													 grad <- NULL


													 dfdC <- matrix(data=0.0, nrow=nrow(C), ncol=ncol(C))

													 log.2pi <- log(2*pi)
													 n.pat <- nrow(valid.data.patterns)
													 for (i in 1:n.pat){
															 sel <- valid.data.patterns[i, ]
															 X <- data[pattern.number == i, sel, drop=FALSE]
															 Ci <- C[sel, sel]
															 Cinv <- solve(Ci)
															 # note: in the matrix product X %*% solve(C[sel, sel]) %*% t(X))
															 #       only the trace is required and so it isn't necessary to form the whole product
															 #	 f <- f + sum(diag(X %*% Cinv %*% t(X))) + nrow(X)*(log.2pi + log(det(Ci)))
															 for (j in 1:nrow(X)){
																	 f <- f + as.vector(X[j, ] %*% Cinv %*% X[j, ])
															 }
															 f <- f + nrow(X)*(log.2pi + log(det(Ci)))

															 #gradient
															 if(gradient)
															 {
																	 dfdCi <- nrow(X)*t(vec(t(Cinv))) - t(vec(t(X))) %*% (X %x% diag(nrow(Cinv))) %*% (t(Cinv) %x% Cinv)
																	 dfdCiExtend <- extendMatrix(matrix(dfdCi, nrow(Cinv), ncol(Cinv)), sel);
																	 dfdC <- dfdC + dfdCiExtend;
															 }
													 }
													 f <- f/N
													 if(gradient)
													 {
															 dfdC <- t(vec(dfdC/N))   #dfdC is a 1*(n^2) matrix.

															 dCdP <- (J %*% I.Ainv) %x% (J %*% I.Ainv)

															 B <- J %*% I.Ainv
															 dBdA <- (diag(m) %x% J) %*% (t(I.Ainv) %x% I.Ainv)
															 Tmn <- commutation.matrix(nrow(B),ncol(B))
															 dCdA <- (((B %*% t(P)) %x% diag(nrow(B)) )+(diag(nrow(B)) %x% B) %*% Tmn %*% (P %x% diag(nrow(B)))) %*% dBdA
															 dfdP <- t(vec(correct)) * (dfdC %*% dCdP)
															 dfdA <- dfdC %*% dCdA

															 grad <- rep(0,t)
															 myarrows.1.free <- rep(0,nrow(arrows.1.free))
															 myarrows.2.free <- rep(0,nrow(arrows.2.free))

															 for(i in 1:nrow(arrows.1.free)) myarrows.1.free[i] <- (arrows.1.free[i,][2]-1)*m+arrows.1.free[i,][1]
															 for(i in 1:nrow(arrows.2.free)) myarrows.2.free[i] <- (arrows.2.free[i,][2]-1)*m+arrows.2.free[i,][1]


															 grad[sort(unique.free.1)] <- tapply(t(dfdA)[myarrows.1.free],ram[ram[,1]==1 & ram[,4]!=0, 4], sum)
															 grad[sort(unique.free.2)] <- tapply(t(dfdP)[myarrows.2.free],ram[ram[,1]==2 & ram[,4]!=0, 4], sum)
													 }

													 attributes(f) <- list(C=C, A=A, P=P, gradient=grad)
													 f
})
											 }
				)
				if (gradient)
						result$gradient <- function(par, model.description){
								with(model.description, {
										 A <- P <- matrix(0, m, m)
								val <- ifelse (fixed, ram[,5], par[sel.free])
								A[arrows.1] <- val[one.head]
								P[arrows.2t] <- P[arrows.2] <- val[!one.head]
								I.Ainv <- solve(diag(m) - A)
								C <- J %*% I.Ainv %*% P %*% t(I.Ainv) %*% t(J)

								dfdC <- matrix(data=0.0, nrow=nrow(C), ncol=ncol(C))

								n.pat <- nrow(valid.data.patterns)
								for (i in 1:n.pat){
										sel <- valid.data.patterns[i, ]
										X <- data[pattern.number == i, sel, drop=FALSE]
										Ci <- C[sel, sel]
										Cinv <- solve(Ci)

										dfdCi <- nrow(X)*t(vec(t(Cinv))) - t(vec(t(X))) %*% (X %x% diag(nrow(Cinv))) %*% (t(Cinv) %x% Cinv)
										dfdCiExtend <- extendMatrix(matrix(dfdCi, nrow(Cinv), ncol(Cinv)), sel);
										dfdC <- dfdC + dfdCiExtend;

								}

								dfdC <- t(vec(dfdC/N))   #dfdC is a 1*(n^2) matrix.

								dCdP <- (J %*% I.Ainv) %x% (J %*% I.Ainv)

								B <- J %*% I.Ainv
								dBdA <- (diag(nrow(A)) %x% J) %*% (t(I.Ainv) %x% I.Ainv)
								Tmn <- commutation.matrix(nrow(B),ncol(B))
								dCdA <- (((B %*% t(P)) %x% diag(nrow(B)) )+(diag(nrow(B)) %x% B) %*% Tmn %*% (P %x% diag(nrow(B)))) %*% dBdA
								dfdP <- dfdC %*% dCdP
								dfdA <- dfdC %*% dCdA

								grad <- rep(0,t)
								myarrows.1.free <- rep(0,nrow(arrows.1.free))
								myarrows.2.free <- rep(0,nrow(arrows.2.free))

								for(i in 1:nrow(arrows.1.free)) myarrows.1.free[i] <- (arrows.1.free[i,][2]-1)*9+arrows.1.free[i,][1]
								for(i in 1:nrow(arrows.2.free)) myarrows.2.free[i] <- (arrows.2.free[i,][2]-1)*9+arrows.2.free[i,][1]


								grad[sort(unique.free.1)] <- tapply(t(dfdA)[myarrows.1.free],ram[ram[,1]==1 & ram[,4]!=0, 4], sum)
								grad[sort(unique.free.2)] <- tapply(t(dfdP)[myarrows.2.free],ram[ram[,1]==2 & ram[,4]!=0, 4], sum)

								attributes(grad) <- list(C=C, A=A, P=P, gradient=grad)
								grad

				}
								)
						}
				class(result) <- "semObjective"
				result
}


#logLik2.objectiveFIML <- function(object, saturated=FALSE, intercept="Intercept", iterlim=1000, ...){
#    logLikSaturated <- function(object, iterlim, ...){
#        objective <- function(par){
#            C <- matrix(0, n, n)
#            C[posn.intercept, posn.intercept] <- 1
#            C[tri] <- par
#            C <- C + t(C) - diag(diag(C))
#            f <- 0
#            for (i in 1:n.pat){
#                sel <- valid.data.patterns[i, ]
#                X <- data[pattern.number == i, sel, drop=FALSE]
#                f <- f + sum(diag(X %*% solve(C[sel, sel]) %*% t(X))) + nrow(X)*(log.2pi + log(det(C[sel, sel])))
#            }
#            f
#        }
#        data <- object$data
#        valid <- !is.na(data)
#        valid.pattern <- apply(valid, 1, function(row) paste(row, collapse="."))
#        unique.patterns <- unique(valid.pattern)
#        pattern.number <- apply(outer(valid.pattern, unique.patterns, `==`), 1, which)
#        valid.data.patterns <- t(sapply(strsplit(unique.patterns, "\\."), as.logical))
#        n.pat <- nrow(valid.data.patterns) 
#        log.2pi <- log(2*pi)
#        n <- ncol(data)
#        N <- nrow(data)
#        C <- object$C
#        tri <- lower.tri(C, diag=TRUE)
#        posn.intercept <- which(rownames(C) == intercept)
#        tri[posn.intercept, posn.intercept] <- FALSE
#        start <- C[tri]
#        opt <- options(warn=-1)
#        on.exit(options(opt))
#        res <- nlm(objective, start, iterlim=iterlim)
#        logL <- - res$minimum/2
#        C <- matrix(0, n, n)
#        C[tri] <- res$estimate
#        C <- C + t(C) - diag(diag(C))
#        C[posn.intercept, posn.intercept] <- 1
#        list(logL=logL, C=C, code=res$code)
#    }
#    if (saturated) {
#        res <- logLikSaturated(object, iterlim=iterlim)
#        if (res$code > 3) warning("nlm return code = ", res$code)
#        logL <- res$logL
#        attr(logL, "C") <- res$C
#        return(logL)
#    }
#    else return(- object$criterion*object$N/2)
#}

residuals.objectiveFIML <- function(object, S, ...){
    if (missing(S)) S <- attr(logLik(object, saturated=TRUE), "C")
    S - object$C
}

normalizedResiduals.objectiveFIML <- function(object, S, ...){
    if (missing(S)) S <- attr(logLik(object, saturated=TRUE), "C")
    res <- residuals(object, S=S)
    N <- object$N - (!object$raw)
    C <- object$C
    c <- diag(C)
    res/sqrt((outer(c, c) + C^2)/N)
}

standardizedResiduals.objectiveFIML <- function(object, S, ...){
    if (missing(S)) S <- attr(logLik(object, saturated=TRUE), "C")
    res <- residuals(object, S=S)
    s <- diag(S)
    res/sqrt(outer(s, s))
}

deviance.objectiveFIML <- function(object, saturated.logLik, ...){
    if (missing(saturated.logLik)) saturated.logLik <- logLik(object, saturated=TRUE)
    2*(as.vector(saturated.logLik) - logLik(object))
}

AIC.objectiveFIML <- function(object, saturated.logLik, ..., k) {
    if (missing(saturated.logLik)) saturated.logLik <- logLik(object, saturated=TRUE)
    deviance(object, saturated.logLik) + 2*object$t
}

AICc.objectiveFIML <- function(object, saturated.logLik, ...) {
    if (missing(saturated.logLik)) saturated.logLik <- logLik(object, saturated=TRUE)
    deviance(object, saturated.logLik) + 2*object$t*(object$t + 1)/(object$N - object$t - 1)
}

CAIC.objectiveFIML <- function(object, saturated.logLik, ...) {
    props <- semProps(object)
    if (missing(saturated.logLik)) saturated.logLik <- logLik(object, saturated=TRUE)
    deviance(object, saturated.logLik) - props$df*(1 + log(object$N))
}

BIC.objectiveFIML <- function(object, saturated.logLik, ...) {
    n <- object$n
    n.fix <- object$n.fix
    N <- object$N
    t <- object$t
    df <- n*(n + 1)/2 - t - n.fix*(n.fix + 1)/2
    if (missing(saturated.logLik)) saturated.logLik <- logLik(object, saturated=TRUE)
    #    deviance(object, saturated.logLik) + object$t*log(object$N)
    deviance(object, saturated.logLik) - df*log(N)
}

summary.objectiveFIML <- function(object, digits=getOption("digits"), conf.level=.90,
                                  fit.indices=c("AIC", "AICc", "BIC", "CAIC"),
                                  saturated=FALSE, intercept="Intercept", saturated.logLik, ...) {
    fit.indices <- if (missing(fit.indices)){
        if (is.null(opt <- getOption("fit.indices"))) c("AIC", "BIC") else opt
    }
    else match.arg(fit.indices, several.ok=TRUE)
    vcov <- vcov(object, robust=FALSE, analytic=FALSE)
    if (any(is.na(vcov))) stop("coefficient covariances cannot be computed")
    if (missing(saturated.logLik)) saturated.logLik <- if (saturated) 
        logLik(object, saturated=TRUE, intercept=intercept) else NULL
    S <- attr(saturated.logLik, "C")
    norm.res <- if (saturated) normalizedResiduals(object, S) else NA
    se <- sqrt(diag(vcov))
    z <- object$coeff/se
    n.fix <- object$n.fix
    n <- object$n
    t <- object$t
    C <- object$C
    N <- object$N
    df <- n*(n + 1)/2 - t - n.fix*(n.fix + 1)/2
    if (saturated){
        invC <- solve(C)
        CSC <- invC %*% (S - C)
        CSC <- CSC %*% CSC
        CS <- invC %*% S
        CS <- CS %*% CS
        chisq <- 2*(saturated.logLik - logLik(object))
        logLik <- NULL
    }
    else {
        chisq <- NULL
        logLik <- logLik(object)
    }
    Rsq <- RMSEA.U <- RMSEA.L <- RMSEA <- NFI <- NNFI <- CFI <- AGFI <- SRMR <- NA
    RMSEA <- c(RMSEA, RMSEA.L, RMSEA.U, conf.level)
    if (!is.null(object$coeff)){
        var.names <- rownames(object$A)
        ram <- object$ram[object$par.posn, , drop=FALSE]
        par.code <- paste(var.names[ram[,2]], c('<---', '<-->')[ram[,1]],
                          var.names[ram[,3]])
        coeff <- data.frame(object$coeff, se, z, 2*pnorm(abs(z), lower.tail=FALSE), par.code)
        names(coeff) <- c("Estimate", "Std Error", "z value", "Pr(>|z|)", " ")
        row.names(coeff) <- names(object$coeff)
    }
    else coeff <- NULL
    if(saturated){
        AIC <- if ("AIC" %in% fit.indices) AIC(object, saturated.logLik=saturated.logLik) else NA
        AICc <- if ("AICc" %in% fit.indices) AICc(object, saturated.logLik=saturated.logLik) else NA
        BIC <- if ("BIC" %in% fit.indices) BIC(object, saturated.logLik=saturated.logLik) else NA
        CAIC <- if ("CAIC" %in% fit.indices)  CAIC(object, saturated.logLik=saturated.logLik) else NA
        #         SRMR <- if ("SRMR" %in% fit.indices  && !object$raw) sqrt(sum(standardizedResiduals(object, S=S)^2 * 
        #             upper.tri(diag(n), diag=TRUE))/(n*(n + 1)/2)) else NA
    }
    else AIC <- AICc <- BIC <- CAIC <- NA
    #   if (robust) { 
    #     chisq.adjusted <- object$adj.obj$chisq.scaled
    #     chisqNull.adjusted <- chisqNull/object$adj.obj$c 
    #     NFI.adjusted <- (chisqNull.adjusted - chisq)/chisqNull.adjusted
    #     NNFI.adjusted <- (chisqNull.adjusted/dfNull - chisq.adjusted/df)/(chisqNull.adjusted/dfNull - 1)
    #     L1 <- max(chisq.adjusted - df, 0)
    #     L0 <- max(L1, chisqNull.adjusted - dfNull)
    #     CFI.adjusted <- 1 - L1/L0
    #   }
    #   else{
    chisq.adjusted <- chisqNull.adjusted <- NFI.adjusted <- NNFI.adjusted <- CFI.adjusted <- NULL
    #   }
    ans <- list(chisq=chisq, logLik=logLik, df=df, chisqNull=chisqNull, dfNull=NA,
                GFI=NULL, AGFI=AGFI, RMSEA=RMSEA, NFI=NFI, NNFI=NNFI, CFI=CFI, BIC=BIC, SRMR=SRMR, 
                AIC=AIC, AICc=AICc, CAIC=CAIC, Rsq=Rsq,
                chisq.adjusted=chisq.adjusted, chisqNull.adjusted=chisqNull.adjusted, NFI.adjusted=NFI.adjusted,
                NNFI.adjusted=NNFI.adjusted, CFI.adjusted=CFI.adjusted,
                norm.res=norm.res, coeff=coeff, digits=digits, 
                iterations=object$iterations, aliased=object$aliased, raw=object$raw,
                robust=FALSE, robust.vcov=NULL, adj.obj=NULL)
    class(ans) <- "summary.objectiveML"
    ans
}

print.objectiveFIML <- function(x, saturated=FALSE, ...) {
    n <- x$n
    t <- x$t
    n.fix <- x$n.fix
    df <- n*(n + 1)/2 - t - n.fix*(n.fix + 1)/2
    if (saturated){
        cat("\n Model Chisquare = ", 2*(logLik(x, saturated=TRUE) - logLik(x)), 
            "  Df = ", df, "\n\n")
    }
    else{
        cat("\n Model log-likelihood = ", logLik(x), "  Df = ", df, "\n\n")
    }
    if (!is.null(x$coef)){
        print(x$coeff)
        if (!is.na(x$iterations)) cat("\n Iterations = ", x$iterations, "\n")
        if (!is.null(x$aliased)) cat("\n Aliased parameters:", x$aliased, "\n")
    }
    invisible(x)
}

# The following functions were originally in the matrixcalc package
# written by Frederick Novomestky
# and distributed under the GPL license version 2 or higher

# last modified 2021-06-02

duplication.matrix <- function (n = 1) {
  return(D.matrix(n))
}

D.matrix <- function (n) {
  if (missing(n)) 
    stop("argument n is missing")
  if (!is.numeric(n)) 
    stop("argument n is not numeric")
  if (n != trunc(n)) 
    stop("argument n is not an integer")
  if (n < 2) 
    stop("argument n is less than 2")
  p <- n * (n + 1)/2
  nsq <- n * n
  Dt <- matrix(0, nrow = p, ncol = nsq)
  T <- T.matrices(n)
  u <- u.vectors(n)
  k <- u$k
  I <- u$I
  for (j in 1:n) {
    for (i in j:n) {
      Dt <- Dt + I[, k[i, j]] %*% t(vec(T[[i]][[j]]))
    }
  }
  return(t(Dt))
}

vech <- function (x) {
  if (!is.square.matrix(x)) 
    stop("argument x is not a square numeric matrix")
  return(t(t(x[!upper.tri(x)])))
}

is.square.matrix <- function (x) {
  if (!is.matrix(x)) 
    stop("argument x is not a matrix")
  return(nrow(x) == ncol(x))
}

T.matrices <- function (n) {
  if (missing(n)) 
    stop("argument n is missing")
  if (!is.numeric(n)) 
    stop("argument n is not numeric")
  if (n != trunc(n)) 
    stop("argument n is not an integer")
  if (n < 2) 
    stop("argument n is less than 2")
  E <- E.matrices(n)
  T <- list()
  for (i in 1:n) {
    T[[i]] <- list()
    for (j in 1:n) {
      if (i == j) {
        T[[i]][[j]] <- E[[i]][[j]]
      }
      else {
        T[[i]][[j]] <- E[[i]][[j]] + E[[j]][[i]]
      }
    }
  }
  return(T)
}

u.vectors <- function (n) {
  if (n != trunc(n)) 
    stop("argument n is not an integer")
  if (n < 2) 
    stop("argument n is less than 2")
  p <- n * (n + 1)/2
  I <- diag(rep(1, p))
  k <- matrix(0, nrow = n, ncol = n)
  for (j in 1:n) {
    for (i in j:n) {
      k[i, j] <- (j - 1) * n + i - 0.5 * j * (j - 1)
    }
  }
  return(list(k = k, I = I))
}

vec <- function (x) {
  if (!is.matrix(x)) {
    stop("argument x is not a matrix")
  }
  if (!is.numeric(x)) {
    stop("argument x is not a numeric matrix")
  }
  return(t(t(as.vector(x))))
}

E.matrices <- function (n) {
  if (missing(n)) 
    stop("argument n is missing")
  if (!is.numeric(n)) 
    stop("argument n is not numeric")
  if (n != trunc(n)) 
    stop("argument n is not an integer")
  if (n < 2) 
    stop("argument n is less than 2")
  I <- diag(rep(1, n))
  E <- list()
  for (i in 1:n) {
    E[[i]] <- list()
    for (j in 1:n) {
      E[[i]][[j]] <- I[i, ] %o% I[j, ]
    }
  }
  return(E)
}

# these functions originally by Jarrett Byrnes

# methods to generate 
# various information criteria from
# sem or adjchisq objects
# as well as generate and AIC table

# last modified 2012-10-02 by J. Fox

logLik.objectiveML <- function(object, ...){
	-0.5*deviance(object)
	}

# generics

AICc <- function(object, ...) UseMethod("AICc")

CAIC <- function(object, ...) UseMethod ("CAIC")


# methods for sem objects

AIC.objectiveML <- function(object, ..., k) {
	deviance(object) + 2*object$t
}

# small sample second order corrected aic
AICc.objectiveML <- function(object, ...) {
	deviance(object) + 2*object$t*(object$t + 1)/(object$N - object$t - 1)
}

# Consistent Akaike Information Criterion
CAIC.objectiveML <- function(object, ...) {
	props <- semProps(object)
	props$chisq - props$df*(1 + log(object$N))
}

BIC.objectiveML <- function(object, ...) {
    n <- object$n
    n.fix <- object$n.fix
    N <- object$N
    t <- object$t
    df <- n*(n + 1)/2 - t - n.fix*(n.fix + 1)/2
#	deviance(object) + object$t*log(object$N)
    deviance(object) - df*log(N)
}

# the following are not exported and are just place-keepers for the summary method

BIC.objectiveGLS <- function(object, ...) NULL

AIC.objectiveGLS <- function(object, ...) NULL

AICc.objectiveGLS <- function(object, ...) NULL

CAIC.objectiveGLS <- function(object, ...) NULL

# weights

aicW <- function(a.list, func=AICc){
	aiclist <- sapply(a.list, function(x) eval(func(x)), simplify=TRUE)			
	delta.i <- aiclist - min(aiclist)
	aicw <- exp(-0.5*delta.i)/sum(exp(-0.5*delta.i))
	return.matrix <- matrix(c(aiclist,delta.i, aicw), ncol=3)
	colnames(return.matrix) <- c("IC", "delta.i", "weight")
	rownames(return.matrix) <- 1:length(return.matrix[,1])
	return(return.matrix)
}

# last modified: 2015-05-14 by J. Fox

cfa <- function(file="", text, covs=paste(factors, collapse=","), reference.indicators=TRUE, raw=FALSE, 
    subscript=c("name", "number"), ...){
    Lines <- if (!missing(text)) scan(text=text, what="", sep=";", strip.white=TRUE, comment.char="#")
    else scan(file=file, what="", sep=";", strip.white=TRUE, comment.char="#")
    lines <- character(0)
    current.line <- ""
    for (line in Lines){
        if (current.line != "") line <- paste(current.line, line)
        if (length(grep(",$", line)) > 0){
            current.line <- line
            next
        }
        current.line <- ""
        lines <- c(lines, line)
    }
    subscript <- match.arg(subscript)
    variance.lines <- grepl("^[Vv]ar.*:", lines)
    variances <- lines[variance.lines]
    lines <- lines[!variance.lines]
    nfactor <- length(lines)
    factors <- rep("", nfactor)
    all.obs.vars <- ram <- character(0)
    equality.sets <- list()
    for (i in 1:nfactor){
        par.number <- 0
        Line <- line <- lines[[i]]
        line <- gsub(" ", "", line)
        line <- strsplit(line, ":")[[1]]
        if (length(line) == 1){
            factors[i] <- paste("Factor.", i, sep="")
            variables <- strsplit(line, ",")[[1]]
            all.obs.vars <- c(all.obs.vars, variables)
        }
        else if (length(line) == 2){
            factors[i] <- line[1]
            variables <- strsplit(line[2], ",")[[1]]
            all.obs.vars <- c(all.obs.vars, unlist(strsplit(variables, "=")))
        }
        else stop("Parse error in ", Line)
        if (reference.indicators){
            if (!grepl("=", variables[1])){
                ram <- c(ram, paste(factors[i], " -> ", variables[1], ", NA, 1", sep=""))}
            else{
                vars <- strsplit(variables[1], "=")[[1]]
                equality.sets[[length(equality.sets) + 1]] <- vars
                for (var in vars){
                    ram <- c(ram, paste(factors[i], " -> ", var, ", NA, 1", sep=""))
                }
            }
            variables <- variables[-1]
        }
        for (variable in variables){
            if (length(grep("\\(", variable)) > 0){
                if (length(grep("\\)", variable)) == 0) stop ("Parse error in ", Line)
                variable <- sub("\\)", "", variable)   
                var.start <- strsplit(variable, "\\(")[[1]]
                if (length(var.start) != 2) stop("Parse error in ", Line)
                variable <- var.start[1]
                start <- var.start[2]
                if (not.number(start)) stop ("Bad start value ", start, " in ", Line)
            }
            else start <- "NA"
            if (!grepl("=", variable)){
                par.number <- par.number + 1
                par.name <- if (subscript == "name") variable else as.character(par.number)
                ram <- c(ram, paste(factors[i], " -> ", variable, ", lam[", par.name, ":", factors[i], "], ", start, sep=""))
            }
            else {
                vars <- strsplit(variable, "=")[[1]]
                equality.sets[[length(equality.sets) + 1]] <- vars
                par.number <- par.number + 1
                lam <- if (subscript == "name") paste(vars, collapse=".") else as.character(par.number)
                for (var in vars){
                    ram <- c(ram, paste(factors[i], " -> ", var, ", lam[", lam, ":", factors[i], "], ", start, sep=""))
                }
            }
        }
    }
    ram <- if (reference.indicators) {
        c(ram, sapply(factors, function(factor) paste(factor, " <-> ", factor, ", ", paste("V[", factor, "]", sep="") , ", NA", sep="")))
    }
    else{
        c(ram, sapply(factors, function(factor) paste(factor, " <-> ", factor, ", NA, 1", sep="")))
    }
    if (raw){
        all.obs.vars <- unique(all.obs.vars)
        if (length(equality.sets) == 0){
            int <- if (subscript == "name") all.obs.vars else as.character(seq(1, length(all.obs.vars)))
            names(int) <- all.obs.vars
            ram <- c(ram, sapply(all.obs.vars, function(var) paste("Intercept -> ", var, ", intercept(", int[var], "), NA", sep="")))
        }
        else{
            par.number <- 0
            for (set in equality.sets){
                par.number <- par.number + 1
                int <- if (subscript == "name") paste(set, collapse=".") else as.character(par.number)
                ram <- c(ram, sapply(set, function(var) 
                    paste("Intercept -> ", var, ", intercept(", int, "), NA", sep="")))
                all.obs.vars <- setdiff(all.obs.vars, set)
            }
            if (length(all.obs.vars) > 0) {
                int <- if (subscript == "name") all.obs.vars else as.character(seq(par.number + 1, par.number + length(all.obs.vars)))
                names(int) <- all.obs.vars
                ram <- c(ram, sapply(all.obs.vars, function(var) paste("Intercept -> ", var, ", intercept(", int[var], "), NA", sep="")))
            }
        }
        message('NOTE: specify fixed.x="Intercept" in call to sem')
    }
    if (length(variances) > 0){
        var.number <- 0
        variances <- sub("^[Vv]ar.*:", "", variances)
        variances <- gsub(" ", "", variances)
        variances <- strsplit(variances, ",")
        for (vars in variances){
            var <- strsplit(vars, "=")
            sub <- if (subscript == "name") sapply(var, function(x) paste(x, collapse=".")) else as.character(seq(var.number + 1, var.number + length(var) + 1))
            var.number <- var.number + length(var)
            for (i in 1:length(var)){
                ram <- c(ram, sapply(var[i], function(x) paste(x, " <-> ", x, ", V[", sub[i], "]", sep="")))
            }
        }
    }
    specifyModel(text=ram, covs=covs, ..., quiet=TRUE)
}

# last modified 2012-09-17 by J. Fox

ram <- function(object, digits=getOption("digits"), startvalues=FALSE){
    var.names <- rownames(object$A)
    ram <- object$ram
    if (!startvalues) colnames(ram) <- c(colnames(ram)[1:4], "estimate")
    par <- object$coeff
    par.names <- rep(" ", nrow(ram))
    t <- object$t
    for (i in 1:t) {
        which.par <- ram[,4] == i
        if (!startvalues)  ram[which.par, 5] <- par[i]
        par.names[which.par] <- names(par)[i]
        }
    par.code <- paste(var.names[ram[,2]], c('<---', '<-->')[ram[,1]],
                    var.names[ram[,3]])
    ram <- data.frame(ram, arrow = par.code)
    print(ram, rowlab=par.names, digits=digits)
    }

### multigroup SEMs  
# last modified J. Fox 2013-06-14

## model definition

multigroupModel <- function(..., groups=names(models), allEqual=FALSE){
	models <- list(...)
	if (length (models) == 1){
		if (missing(groups)) stop("group names are missing, yet only 1 model is defined")
		ngroups <- length(groups)
		model <- models[[1]]
		models <- vector(ngroups, mode="list")
		for (g in 1:ngroups) models[[g]] <- model
		names(models) <- groups
		if (!allEqual){
			for (g in 1:ngroups){
				model <- models[[g]]
				models[[g]][!is.na(model[, 2]), 2] <- paste(model[!is.na(model[, 2]), 2], ".", groups[g], sep="")
			}
		}
	}
	else {
		if (is.null(names(models)) && missing(groups)) groups <- paste("Group.", 1:length(models), sep="")
		names(models) <- groups
	}
	class(models) <- "semmodList"
	models
}

print.semmodList <- function(x, ...){
	cat("\nMultigroup Structural Equation Model\n")
	groups <- names(x)
	for (group in groups){
		cat("\n Group: ", group, "\n\n")
		print(x[[group]])
	}
	invisible(x)
}


## sem() method for semmodList objects

sem.semmodList <- function(model, S, N, data, raw=FALSE, fixed.x=NULL, robust=!missing(data), formula, group="Group", debug=FALSE, ...){
    data.out <- NULL
    if (missing(S)){
        if (missing(data)) stop("S and data cannot both be missing")
        data.df <- inherits(data, "data.frame")
        if (data.df && missing(group)) stop("S and group cannot both be missing")
        if (data.df){
            if (!is.factor(data[, group])) stop("Groups variable, ", group, ", is not a factor")
            levels <- levels(data[, group])
            if (missing(formula)) formula <- as.formula(paste("~ . -", group))
        }
        else {
            if (!all(sapply(data, function(d) inherits(d, "data.frame")))) stop("data must be a data frame or list of data frames")
            levels <- names(data)
            if (is.null(levels)) levels <- paste("Group", seq(along=data), sep=".")
            if (missing(formula)) formula <- ~ .
        }
        G <- length(levels)
        if (is.list(formula) && length(formula) != G) stop("number of formulas, ", length(formula), ", not equal to number of groups, ", G, sep="")
        if (is.list(formula)){    
            if (!all(names(model) == names(formula))) warning("names of groups (", paste(names(model), collapse=", "), 
                ") is not the same as names of formulas in formula argument (", 
                paste(names(formula), collapse=", "), ")")
        }
        S <- vector(G, mode="list")
        names(S) <- levels
        N <- numeric(G)
        data.out <- vector(G, mode="list")
        for (g in 1:G){
            data.group <- if (data.df) subset(data, subset = data[, group] == levels[g]) else data[[g]]
            N.all <- nrow(data.group)
            form <- if (is.list(formula)) formula[[g]] else formula
            data.group <- model.matrix(form, data=data.group)
            colnames(data.group)[colnames(data.group) == "(Intercept)"] <- "Intercept"
            N[g] <- nrow(data.group)
            if (N[g] < N.all) warning(N.all - N[g]," observations removed due to missingness in group ", levels[g])
            S[[g]] <- if (raw) rawMoments(data.group) else{
                data.group <- data.group[, colnames(data.group) != "Intercept"]
                cov(data.group)
            }
            data.out[[g]] <- data.group
        }
    }
    else G <- length(S)
    if (length(model) != G) stop("number of group models, ", length(model), ", not equal to number of moment/data matrices, ", G, sep="")
    pars <-  unique(na.omit(unlist(lapply(model, function(mod) mod[, 2]))))
    vars <- rams <- vector(length(model), mode="list")
    all.par.names <- character(0)
    all.pars <- numeric(0)
    for (i in 1:G){
        obs.variables <- colnames(S[[i]])
        mod <- model[[i]]
        if ((!is.matrix(mod)) | ncol(mod) != 3) stop("model argument must be a 3-column matrix")
        startvalues <- as.numeric(mod[, 3])
        par.names <- mod[, 2]
        n.paths <- length(par.names)
        heads <- from <- to <- rep(0, n.paths)
        for (p in 1:n.paths){
            path <- parse.path(mod[p, 1])
            heads[p] <- abs(path$direction)
            to[p] <- path$second
            from[p] <- path$first
            if (path$direction == -1) {
                to[p] <- path$first
                from[p] <- path$second
            }
        }
        ram <- matrix(0, n.paths, 5)
        all.vars <- unique(c(to, from))
        latent.vars <- setdiff(all.vars, obs.variables)
        not.used <- setdiff(obs.variables, all.vars)
        if (length(not.used) > 0){
            rownames(S[[i]]) <- colnames(S[[i]]) <- obs.variables
            obs.variables <- setdiff(obs.variables, not.used)
            S[[i]] <- S[[i]][obs.variables, obs.variables]
            data.out[[i]] <- data.out[[i]][, obs.variables]
            warning("The following observed variables are in the input covariance or raw-moment matrix for group ", i,
                " but do not appear in the model:\n",
                paste(not.used, collapse=", "), "\n")
        }
        vars[[i]] <- c(obs.variables, latent.vars)
        ram[,1] <- heads
        ram[,2] <- apply(outer(vars[[i]], to, "=="), 2, which)
        ram[,3] <- apply(outer(vars[[i]], from, "=="), 2, which)   
        par.nos <- apply(outer(pars, par.names, "=="), 2, which)
        if (length(par.nos) > 0)
            ram[,4] <- unlist(lapply(par.nos, function(x) if (length(x) == 0) 0 else x))
        ram[,5]<- startvalues
        colnames(ram) <- c("heads", "to", "from", "parameter", "start value")
        rams[[i]] <- ram
        all.pars <- c(all.pars, par.nos)
        all.par.names <- c(all.par.names, par.names)
    }
    all.pars <- unique(unlist(all.pars))
    all.par.names <- unique(na.omit(all.par.names))	
    class(rams) <- "msemmod"
    result <- sem(rams, S, N, group=group, groups=names(model), raw=raw, fixed.x=fixed.x, param.names=all.par.names[all.pars], var.names=vars, debug=debug, ...)
    result$semmodList <- model
    result$data <- if(missing(data)) NULL else data.out
    if (robust && !missing(data) && inherits(result, "msemObjectiveML")){
        res <- robustVcovMsem(result)
        result$robust.vcov <- res$vcov
        result$chisq.scaled <- res$chisq.scaled
        result$adj.objects <- res$adj.objects
    }
    result
}

parse.path <- function(path) {                                           
	path.1 <- gsub("-", "", gsub(" ","", path))
	direction <- if (regexpr("<>", path.1) > 0) 2 
			else if (regexpr("<", path.1) > 0) -1
			else if (regexpr(">", path.1) > 0) 1
			else stop(paste("ill-formed path:", path))
	path.1 <- strsplit(path.1, "[<>]")[[1]]
	list(first=path.1[1], second=path.1[length(path.1)], direction=direction)
}


## sem() method for msemmod objects

sem.msemmod <- function(model, S, N, start.fn=startvalues, group="Group", groups=names(model), raw=FALSE, fixed.x, param.names, var.names, debug=FALSE, analytic.gradient=TRUE, warn=FALSE,
    maxiter=5000, par.size = c("ones", "startvalues"), start.tol = 1e-06, start=c("initial.fit", "startvalues"), initial.maxiter=1000,
    optimizer = optimizerMsem, objective = msemObjectiveML, ...){
    par.size <- match.arg(par.size)
    start <- match.arg(start)
    G <- length(groups)
    if (length(model) != G || length(N) != G) 
        stop("inconsistent number of groups in model (", length(model), "), S (", G, "), and N (", length(N), ") arguments")
    if (is.null(names(S))) names(S) <- groups
    if (is.null(names(N))) names(N) <- groups
    if (is.null(names(model))) names(model) <- groups
    if (!all(groups == names(model))) warning("names of groups (", paste(groups, collapse=", "), 
        ") is not the same as names of models in model argument (", 
        paste(names(model), collapse=", "), ")")
    if (!all(groups == names(S))) warning("names of groups (", paste(groups, collapse=", "),
        ") is not the same as names of moment matrices in S argument (", 
        paste(names(S), collapse=", "), ")")
    if (!all(groups == names(N))) warning("names of groups (", paste(groups, collapse=", "),
        ") is not the same as names of sample sizes in N argument (", 
        paste(names(N), collapse=", "), ")")
    if (length(fixed.x) == 1) fixed.x <- lapply(1:G, function(g) fixed.x)
    n.fix <- 0 
    if (!is.null(fixed.x)){
        n.fix <- numeric(G)
        for (g in 1:G){
            fx <- fixed.x[[g]]
            n.fix[g] <- length(fx)
            if (n.fix[g] == 0) next
            fx <- which(rownames(S[[g]]) %in% fx)
            mod <- model[[g]]
            for (i in 1:n.fix[g]){
                for (j in 1:i){
                    mod <- rbind(mod, c(2, fx[i], fx[j],
                        0, S[[g]][fx[i], fx[j]]))
                }
            }
            model[[g]] <- mod
        }
    }
    t <- max(sapply(model, function(r) max(r[, 4])))
    if(missing(param.names)) param.names <- paste("Parameter", 1:t, sep=".")
    if (missing(var.names)) var.names <- lapply(model, function(mod) paste("Variable", 1:max(mod[, c(2,3)]), sep="."))
    n <- sapply(S, nrow)
    m <- sapply(model, function(r)  max(r[, c(2, 3)]))
    logdetS <- sapply(S, function(s) log(det(unclass(s))))
    sel.free.2 <- sel.free.1 <- arrows.2.free <- arrows.1.free <- arrows.2t <- arrows.2 <- arrows.1 <- 
        two.free <- one.free <- one.head <- sel.free <- fixed <- par.posn <- correct <- J <- vector(mode="list", length=G)  
    initial.iterations <- if (start == "initial.fit") numeric(G) else NULL
    for (g in 1:G){
        mod <- model[[g]]
        #         tt <- sum(mod[, 4] != 0)
        #         mod[mod[, 4] != 0, 4] <- 1:tt
        initial.pars <- mod[, 4]
        unique.pars <- unique(initial.pars)
        unique.pars <- unique.pars[unique.pars != 0]
        tt <- length(unique.pars)
        if (tt > 0) for (i in 1:tt) mod[initial.pars == unique.pars[i], 4] <- i
        startvals <- if (start == "initial.fit"){
            prelim.fit <- sem(mod, S[[g]], N=N[[g]], raw=raw, param.names=if(tt > 0) as.character(1:tt) else character(0), var.names=as.character(1:m[[g]]), 
                maxiter=initial.maxiter)
            initial.iterations[g] <- prelim.fit$iterations
            coef(prelim.fit)
        }
        else start.fn(S[[g]], mod)
        model[[g]][mod[, 4] != 0, 5] <- ifelse(is.na(mod[mod[, 4] != 0, 5]), startvals, mod[mod[, 4] != 0, 5])
        J[[g]] <- matrix(0, n[g], m[g])
        correct[[g]] <- matrix(2, m[g], m[g])
        diag(correct[[g]]) <- 1
        observed <- 1:n[g]
        J[[g]][cbind(observed, observed)] <- 1
        par.posn[[g]] <-  sapply(1:t, function(i) which(model[[g]][,4] == i)[1])
        colnames(model[[g]]) <- c("heads", "to", "from", "parameter", "start value")
        rownames(model[[g]]) <- rep("", nrow(model[[g]]))
        fixed[[g]] <- model[[g]][, 4] == 0
        sel.free[[g]] <- model[[g]][, 4]
        sel.free[[g]][fixed[[g]]] <- 1
        one.head[[g]] <- model[[g]][, 1] == 1
        one.free[[g]] <- which( (!fixed[[g]]) & one.head[[g]] )
        two.free[[g]] <- which( (!fixed[[g]]) & (!one.head[[g]]) )
        arrows.1[[g]] <- model[[g]][one.head[[g]], c(2, 3), drop=FALSE]
        arrows.2[[g]] <- model[[g]][!one.head[[g]], c(2, 3), drop=FALSE]
        arrows.2t[[g]] <- model[[g]][!one.head[[g]], c(3 ,2), drop=FALSE]
        arrows.1.free[[g]] <- model[[g]][one.free[[g]], c(2, 3), drop=FALSE]
        arrows.2.free[[g]] <- model[[g]][two.free[[g]], c(2, 3), drop=FALSE]
        sel.free.1[[g]] <- sel.free[[g]][one.free[[g]]]
        sel.free.2[[g]] <- sel.free[[g]][two.free[[g]]]
    }
    unique.free.1 <- lapply(sel.free.1, unique)
    unique.free.2 <- lapply(sel.free.2, unique)
    startvals <- numeric(t)
    for (j in 1:t) startvals[j] <- mean(unlist(sapply(model, function(r) r[r[, 4] == j, 5])), na.rm=TRUE)
    model.description <- list(G=G, m=m, n=n, t=t, fixed=fixed, ram=model, sel.free=sel.free, arrows.1=arrows.1, 
        one.head=one.head, arrows.2=arrows.2, arrows.2t=arrows.2t, J=J, S=S, logdetS=logdetS, 
        N=N, raw=raw, correct=correct, unique.free.1=unique.free.1, unique.free.2=unique.free.2, 
        arrows.1.free=arrows.1.free, arrows.2.free=arrows.2.free, param.names=param.names, 
        var.names=var.names)
    result <- optimizer(start=startvals, objective=objective, gradient=analytic.gradient,
        maxiter=maxiter, debug=debug, par.size=par.size, model.description=model.description, warn=warn, ...)
    if (!is.na(result$iterations)) if(result$iterations >= maxiter) warning("maximum iterations exceeded")
    result <- c(result, list(ram=model, param.names=param.names, var.names=var.names, group=group, groups=groups,
        S=S, N=N, J=J, n=n, m=m, t=t, raw=raw, optimizer=optimizer, objective=objective, fixed.x=fixed.x, n.fix=n.fix,
        initial.iterations=initial.iterations))
    cls <- gsub("\\.", "", deparse(substitute(objective)))
    cls <- gsub("2", "", cls)
    class(result) <- c(cls, "msem")
    result
}




## ML objective function for multigroup SEMs

msemObjectiveML2 <- function(gradient=TRUE){
	result <- list(
			objective = function(par, model.description){
				with(model.description, {
							f <- numeric(G)
							AA <- PP <- CC <- vector(G, mode="list")
							grad.all <- if (gradient) rep(0, t) else NULL
							for (g in 1:G){
								A <- P <- matrix(0, m[g], m[g])
								val <- ifelse (fixed[[g]], ram[[g]][, 5], par[sel.free[[g]]])
								A[arrows.1[[g]]] <- val[one.head[[g]]]
								P[arrows.2t[[g]]] <- P[arrows.2[[g]]] <- val[!one.head[[g]]]
								I.Ainv <- solve(diag(m[g]) - A)
								C <- J[[g]] %*% I.Ainv %*% P %*% t(I.Ainv) %*% t(J[[g]])
								Cinv <- solve(C)
								f[g] <- sum(diag(S[[g]] %*% Cinv)) + log(det(C)) - n[[g]] - logdetS[g]
								CC[[g]] <- C
								AA[[g]] <- A
								PP[[g]] <- P
								if (gradient){
									grad.P <- correct[[g]] * t(I.Ainv) %*% t(J[[g]]) %*% Cinv %*% (C - S[[g]]) %*% Cinv %*% J[[g]] %*% I.Ainv
									grad.A <- grad.P %*% P %*% t(I.Ainv)        
									grad <- rep(0, t)
									grad[sort(unique.free.1[[g]])] <- tapply(grad.A[arrows.1.free[[g]]], ram[[g]][ram[[g]][,1]==1 & ram[[g]][,4]!=0, 4], sum)
									grad[sort(unique.free.2[[g]])] <- tapply(grad.P[arrows.2.free[[g]]], ram[[g]][ram[[g]][,1]==2 & ram[[g]][,4]!=0, 4], sum)
									grad.all <- grad.all + ((N[g] - (!raw))/(sum(N) - (!raw)*G))*grad
								}
							}
							ff <- f
							f <- sum((N - (!raw))*f)/(sum(N) - (!raw)*G)
							attributes(f) <- list(gradient=grad.all, A=AA, P=PP, C=CC, f=ff)
							f
						})
			}
	)
	if (gradient)
		result$gradient <- function(par, model.description){
			with(model.description, {
						grad.total <- rep(0, t)
						for (g in 1:G){
							A <- P <- matrix(0, m[g], m[g])
							val <- ifelse (fixed[[g]], ram[[g]][,5], par[sel.free[[g]]])
							A[arrows.1[[g]]] <- val[one.head[[g]]]
							P[arrows.2t[[g]]] <- P[arrows.2[[g]]] <- val[!one.head[[g]]]
							I.Ainv <- solve(diag(m[g]) - A)
							C <- J[[g]] %*% I.Ainv %*% P %*% t(I.Ainv) %*% t(J[[g]])
							Cinv <- solve(C)
							grad.P <- correct[[g]] * t(I.Ainv) %*% t(J[[g]]) %*% Cinv %*% (C - S[[g]]) %*% Cinv %*% J[[g]] %*% I.Ainv
							grad.A <- grad.P %*% P %*% t(I.Ainv)        
							grad <- rep(0, t)
							grad[sort(unique.free.1[[g]])] <- tapply(grad.A[arrows.1.free[[g]]], ram[[g]][ram[[g]][,1]==1 & ram[[g]][,4]!=0, 4], sum)
							grad[sort(unique.free.2[[g]])] <- tapply(grad.P[arrows.2.free[[g]]], ram[[g]][ram[[g]][,1]==2 & ram[[g]][,4]!=0, 4], sum)
							grad.total <- grad.total + ((N[g] - (!raw))/(sum(N) - (!raw)*G))*grad
						}
						grad.total
					})
		}
	class(result) <- "msemObjective"
	result
}

msemObjectiveML <- function(gradient=TRUE){
	result <- list(
			objective = function(par, model.description){
				with(model.description, {
							res <- msemCompiledObjective(par=par, model.description=model.description, objective="objectiveML")
							AA <- PP <- CC <- vector(G,  mode="list")
							for(g in 1:model.description$G)
							{
								AA[[g]] <- res$A[[g]]
								PP[[g]] <- res$P[[g]]
								CC[[g]] <- res$C[[g]]
							}
							
							f <- res$f
							attributes(f) <- list(gradient=res$gradient, A=AA, P=PP, C=CC, f=res$ff)
							f
						})
			}
	)
	if (gradient)
		result$gradient <- function(par, model.description){
			with(model.description, {
						
						res <- msemCompiledObjective(par=par, model.description=model.description, objective="objectiveML")
						res$gradient
					})
		}
	class(result) <- "msemObjective"
	result
}

msemObjectiveGLS <- function(gradient=FALSE){
	result <- list(
			objective = function(par, model.description){
				with(model.description, {
							
							res <- msemCompiledObjective(par=par, model.description=model.description, objective="objectiveGLS")
							AA <- PP <- CC <- vector(G,  mode="list")
							for(g in 1:model.description$G)
							{
								AA[[g]] <- res$A[[g]]
								PP[[g]] <- res$P[[g]]
								CC[[g]] <- res$C[[g]]
							}
							
							f <- res$f
							attributes(f) <- list(A=AA, P=PP, C=CC, f=res$ff)
							f
						})
			}
	)
	
	class(result) <- "msemObjective"
	result
}

msemObjectiveFIML <- function(gradient=FALSE){
	result <- list(
			objective = function(par, model.description){
				with(model.description, {
							
							res <- msemCompiledObjective(par=par, model.description=model.description, objective="objectiveFIML")
							AA <- PP <- CC <- vector(G,  mode="list")
							for(g in 1:model.description$G)
							{
								AA[[g]] <- res$A[[g]]
								PP[[g]] <- res$P[[g]]
								CC[[g]] <- res$C[[g]]
							}
							
							f <- res$f
							attributes(f) <- list(A=AA, P=PP, C=CC, f=res$ff)
							f
						})
			}
	)
	
	class(result) <- "msemObjective"
	result
}

##  nlm()-based optimizer for multigroup SEMs

optimizerMsem <- function(start, objective=msemObjectiveML, gradient=TRUE,
		maxiter, debug, par.size, model.description, warn=FALSE, ...){
	with(model.description, {
				obj <- objective(gradient=gradient)$objective
				typsize <- if (par.size == 'startvalues') abs(start) else rep(1, t)
				
				if(identical(objective, msemObjectiveML)) objectiveCompiled <- "objectiveML"
				else if (identical(objective, msemObjectiveGLS)) {
					objectiveCompiled <- "objectiveGLS"
					gradient <- FALSE
				}
				else if (identical(objective, msemObjectiveFIML)) {
					objectiveCompiled <- "objectiveFIML"
					gradient <- FALSE
				}
				else stop("optimizerMsem requires the msemObjectiveML, msemObjectiveGLS or msemObjectiveFIML objective function")
				
				if (!warn) save.warn <- options(warn=-1)
				
				res <- msemCompiledSolve(model.description=model.description, start=start, objective=objectiveCompiled, 
						typsize=typsize, debug=debug, maxiter=maxiter)
				
				if (!warn) options(save.warn)
				result <- list(covergence=NULL, iterations=NULL, coeff=NULL, vcov=NULL, criterion=NULL, C=NULL, A=NULL, P=NULL)
				result$convergence <- res$code <= 2
				result$iterations <- res$iterations
				par <- res$estimate
				names(par) <- param.names
				result$coeff <- par
				if (!result$convergence)
					warning(paste('Optimization may not have converged; nlm return code = ',
									res$code, '. Consult ?nlm.\n', sep=""))
				vcov <- matrix(NA, t, t)
				qr.hess <- try(qr(res$hessian), silent=TRUE)
				if (inherits(qr.hess, "try-error")){
					warning("Could not compute QR decomposition of Hessian.\nOptimization probably did not converge.\n")
				}
				else if (qr.hess$rank < t){
					warning(' singular Hessian: model is probably underidentified.\n')
					which.aliased <- qr.hess$pivot[-(1:qr.hess$rank)]
					result$aliased <- param.names[which.aliased]
				}
				else {
					df <- sum(N) - (!raw)*G
					vcov <-  (2/df) * solve(res$hessian)
					if (any(diag(vcov) < 0)) {
						result$aliased <- param.names[diag(vcov) < 0]
						warning("Negative parameter variances.\nModel may be underidentified.\n")
					}
				}
				colnames(vcov) <- rownames(vcov) <- param.names
				result$vcov <- vcov
				result$criterion <- res$minimum
				C <- res$C
				A <- res$A
				P <- res$P
				for (g in 1:G){
					rownames(C[[g]]) <- colnames(C[[g]]) <-rownames(S[[g]])
					rownames(A[[g]]) <- colnames(A[[g]]) <- var.names[[g]]
					rownames(P[[g]]) <- colnames(P[[g]]) <- var.names[[g]]
				}
				result$C <- C
				result$A <- A
				result$P <- P
				result$group.criteria <- res$ff
				class(result) <- "msemResult"
				result
			})
}

msemOptimizerNlm <- function(start, objective=msemObjectiveML, gradient=TRUE,
		maxiter, debug, par.size, model.description, warn=FALSE, ...){
	with(model.description, {
				obj <- objective(gradient=gradient)$objective
				typsize <- if (par.size == 'startvalues') abs(start) else rep(1, t)
				if (!warn) save.warn <- options(warn=-1)
				res <- nlm(obj, start, iterlim=maxiter, print.level=if(debug) 2 else 0,
						typsize=typsize, hessian=TRUE, model.description=model.description, ...)
				if (!warn) options(save.warn)
				result <- list(covergence=NULL, iterations=NULL, coeff=NULL, vcov=NULL, criterion=NULL, C=NULL, A=NULL, P=NULL)
				result$convergence <- res$code <= 2
				result$iterations <- res$iterations
				par <- res$estimate
				names(par) <- param.names
				result$coeff <- par
				if (!result$convergence)
					warning(paste('Optimization may not have converged; nlm return code = ',
									res$code, '. Consult ?nlm.\n', sep=""))
				vcov <- matrix(NA, t, t)
				qr.hess <- try(qr(res$hessian), silent=TRUE)
				if (inherits(qr.hess, "try-error")){
					warning("Could not compute QR decomposition of Hessian.\nOptimization probably did not converge.\n")
				}
				else if (qr.hess$rank < t){
					warning(' singular Hessian: model is probably underidentified.\n')
					which.aliased <- qr.hess$pivot[-(1:qr.hess$rank)]
					result$aliased <- param.names[which.aliased]
				}
				else {
					df <- sum(N) - (!raw)*G
					vcov <-  (2/df) * solve(res$hessian)
					if (any(diag(vcov) < 0)) {
						result$aliased <- param.names[diag(vcov) < 0]
						warning("Negative parameter variances.\nModel may be underidentified.\n")
					}
				}
				colnames(vcov) <- rownames(vcov) <- param.names
				result$vcov <- vcov
				result$criterion <- res$minimum
				obj <- obj(par, model.description)
				C <- attr(obj, "C")
				A <- attr(obj, "A")
				P <- attr(obj, "P")
				for (g in 1:G){
					rownames(C[[g]]) <- colnames(C[[g]]) <-rownames(S[[g]])
					rownames(A[[g]]) <- colnames(A[[g]]) <- var.names[[g]]
					rownames(P[[g]]) <- colnames(P[[g]]) <- var.names[[g]]
				}
				result$C <- C
				result$A <- A
				result$P <- P
				result$group.criteria <- attr(obj, "f")
				class(result) <- "msemResult"
				result
			})
}

# methods for msem and msemObjectiveML objects


print.msemObjectiveML <- function(x, ...){
	n <- x$n
	n.fix <- x$n.fix
	df <- sum(n*(n + 1)/2) - x$t - sum(n.fix*(n.fix + 1)/2)
	chisq <- sum(x$N - !x$raw)*x$criterion
	cat("\n Model Chisquare =", chisq, " Df =", df, "\n\n")
	print(x$coeff)
	invisible(x)
}

print.msemObjectiveGLS <- function(x, ...) print.msemObjectiveML(x, ...)
print.msemObjectiveFIML <- function(x, ...) print.msemObjectiveML(x, ...)

summary.msemObjectiveML <- function(object, digits=getOption("digits"), conf.level=.90, robust=FALSE, 
		analytic.se=object$t <= 500,
        fit.indices=c("GFI", "AGFI", "RMSEA", "NFI", "NNFI", "CFI", "RNI", "IFI", "SRMR", "AIC", "AICc", "BIC"),
        ...){
    fit.indices <- if (is.null(fit.indices)) ""
    else {
        if (missing(fit.indices)){
            if (is.null(opt <- getOption("fit.indices"))) c("AIC", "BIC") else opt
        }
        else match.arg(fit.indices, several.ok=TRUE)
    }
	if(inherits(object, "msemObjectiveGLS")) analytic.se <- FALSE
	else if(inherits(object, "msemObjectiveFIML")) analytic.se <- FALSE
	groups <- object$groups
	G <- length(groups)
	par <- object$coeff
	vcov <- vcov(object, robust=robust, analytic.se=analytic.se)
	n <- object$n
	m <- object$m
	S <- object$S
	C <- object$C
	A <- object$A
	P <- object$P
	N <- object$N
	J <- object$J
	n.fix <- object$n.fix
	if (length(n.fix) == 1) n.fix <- rep(n.fix, G)
	group.criteria <- object$group.criteria
	var.names <- object$var.names
	param.names <- object$param.names
	semmod <- object$semmodList
	ram <- object$ram
	group.summaries <- list(G, model="list")
	for (g in 1:G){
		par.names <- param.names[ram[[g]][, 4]]
		par.gr <- par[par.names]
		group <- list(coeff=par.gr, vcov=vcov[par.names, par.names], n.fix=n.fix[g], n=n[[g]], m=m[[g]], S=S[[g]], C=C[[g]], N=N[[g]],
				t=length(par.gr), raw=object$raw, var.names=var.names[[g]], ram=ram[[g]],
				J=J[[g]], A=A[[g]], P=P[[g]], criterion=group.criteria[g], par.posn=ram[[g]][, 4] != 0, 
				iterations=object$iterations, semmod=semmod[[g]], adj.obj=object$adj.objects[[g]], 
				robust.vcov=object$robust.vcov[par.names, par.names])
		class(group) <- if(inherits(object, "msemObjectiveGLS")) c("objectiveGLS", "sem") 
				else if(inherits(object, "msemObjectiveFIML")) c("objectiveFIML", "sem")
				else c("objectiveML", "sem")
		group.summaries[[g]] <- if(inherits(object, "msemObjectiveGLS"))
					summary(group, digits=digits, conf.level=conf.level, robust=FALSE, fit.indices=fit.indices, ...)
			else if(inherits(object, "msemObjectiveFIML")) 
					summary(group, digits=digits, conf.level=conf.level, robust=FALSE, fit.indices=fit.indices, ...)
			else summary(group, digits=digits, conf.level=conf.level, robust=robust, analytic.se=FALSE, fit.indices=fit.indices, ...)
		group.summaries[[g]]$iterations <- NA
	}
	df <- sum(n*(n + 1)/2) - object$t - sum(n.fix*(n.fix + 1)/2)
	chisq <- sum(N - !object$raw)*object$criterion
	wt <- (N - object$raw)/(sum(N - object$raw))
	SRMR <-if (!object$raw && "SRMR" %in% fit.indices) sum(wt*sapply(group.summaries, function(s) s$SRMR)) else NA
	GFI <- if (!object$raw && "GFI" %in% fit.indices) sum(wt*sapply(group.summaries, function(s) s$GFI)) else NA
	chisqNull <- if(!object$raw) sum(sapply(group.summaries, function(s) s$chisqNull)) else NA
	dfNull <- sum(n*(n - 1)/2)
	if (!object$raw && df > 0){
		AGFI <- if ("AGFI" %in% fit.indices) 1 - (sum(n*(n * 1))/(2*df))*(1 - GFI) else NA
		NFI <- if ("NFI" %in% fit.indices) (chisqNull - chisq)/chisqNull else NA
		NNFI <- if ("NNFI" %in% fit.indices) (chisqNull/dfNull - chisq/df)/(chisqNull/dfNull -1) else NA
        L1 <- max(chisq - df, 0)
    	L0 <- max(L1, chisqNull - dfNull)
        CFI <- if ("CFI" %in% fit.indices) 1 - L1/L0 else NA
        RNI <- if ("RNI" %in% fit.indices) 1 - (chisq - df)/(chisqNull - dfNull) else NA
        IFI <- if ("IFI" %in% fit.indices) (chisqNull - chisq)/(chisqNull - df) else NA
        if ("RMSEA" %in% fit.indices){
    		RMSEA <- sqrt(G*max(object$criterion/df - 1/(sum(N - 1)), 0))
    		tail <- (1 - conf.level)/2
    		max <- sum(N)
    		while (max > 1) {
    			res <- optimize(function(lam) (tail - pchisq(chisq, 
    											df, ncp = lam))^2, interval = c(0, max))
    			if (is.na(res$objective) || res$objective < 0) {
    				max <- 0
    				warning("cannot find upper bound of RMSEA")
    				break
    			}
    			if (sqrt(res$objective) < tail/100) 
    				break
    			max <- max/2
    		}
    		lam.U <- if (max <= 1) 
    					NA
    				else res$minimum
    		max <- max(max, 1)
    		while (max > 1) {
    			res <- optimize(function(lam) (1 - tail - pchisq(chisq, 
    											df, ncp = lam))^2, interval = c(0, max))
    			if (sqrt(res$objective) < tail/100) 
    				break
    			max <- max/2
    			if (is.na(res$objective) || res$objective < 0) {
    				max <- 0
    				warning("cannot find lower bound of RMSEA")
    				break
    			}
    		}
    		lam.L <- if (max <= 1) 
    					NA
    				else res$minimum
    		RMSEA.U <- sqrt(G*lam.U/(sum(N - 1) * df))
    		RMSEA.L <- sqrt(G*lam.L/(sum(N - 1) * df))
            }
        else RMSEA.U <- RMSEA.L <- RMSEA <- NA
	}
	else RMSEA.U <- RMSEA.L <- RMSEA <- NFI <- NNFI <-IFI <- RNI <- CFI <- AGFI <- NA
	if (robust){
		chisq.adjusted <- sum(sapply(group.summaries, function(x) x$chisq.adjusted))
		if (!object$raw && df > 0){
			chisqNull.adjusted <- sum(sapply(group.summaries, function(x) x$chisqNull.adjusted))
			NFI.adjusted <- if ("NFI" %in% fit.indices) (chisqNull.adjusted - chisq.adjusted)/chisqNull.adjusted else NA
			NNFI.adjusted <- if ("NNFI" %in% fit.indices) (chisqNull.adjusted/dfNull - chisq.adjusted/df)/(chisqNull.adjusted/dfNull - 1) else NA
			L1 <- max(chisq.adjusted - df, 0)
			L0 <- max(L1, chisqNull.adjusted - dfNull)
			CFI.adjusted <- if ("CFI" %in% fit.indices) 1 - L1/L0 else NA
            RNI.adjusted <- if ("RNI" %in% fit.indices) 1 - (chisq.adjusted - df)/(chisqNull.adjusted - dfNull) else NA
            IFI.adjusted <- if ("IFI" %in% fit.indices) (chisqNull.adjusted - chisq.adjusted)/(chisqNull.adjusted - df) else NA
		}
	}
	if (object$raw) cat("\nModel fit to raw moment matrix.\n")	
	if (robust && !is.null(object$robust.vcov)){
		cat("\nSatorra-Bentler Corrected Fit Statistics:\n")
		cat("\n Corrected Model Chisquare = ", chisq.adjusted, "  Df = ", df, 
				"Pr(>Chisq) =", if (df > 0) pchisq(chisq.adjusted, df, lower.tail=FALSE)
						else NA)
		if (!object$raw) {		
			cat("\n Corrected Chisquare (null model) = ", chisqNull.adjusted,  "  Df = ", dfNull)
		}
		if (df > 0 && !object$raw){
			if (!is.na(NFI.adjusted)) cat("\n Corrected Bentler-Bonett NFI = ", NFI.adjusted)
			if (!is.na(NNFI.adjusted)) cat("\n Corrected Tucker-Lewis NNFI = ", NNFI.adjusted)
			if (!is.na(CFI.adjusted)) cat("\n Corrected Bentler CFI = ", CFI.adjusted)
            if (!is.na(RNI.adjusted)) cat("\n Corrected Bentler RNI = ", RNI.adjusted)
            if (!is.na(IFI.adjusted)) cat("\n Corrected Bollen IFI = ", IFI.adjusted)
		}
		cat("\n\nUncorrected Fit Statistics:\n")
	}
	if (inherits(object, "msemObjectiveGLS") || inherits(object, "msemObjectiveFIML")) {
		AIC <- AICc <- BIC <- NA
	}
	else {
		AIC <- if ("AIC" %in% fit.indices) AIC(object) else NA
		AICc <- if ("AICc" %in% fit.indices) AICc(object) else NA
		BIC <- if ("BIC" %in% fit.indices) BIC(object) else NA
	}
	cat("\n Model Chisquare =", chisq, " Df =", df, " Pr(>Chisq) =", pchisq(chisq, df, lower.tail=FALSE))
	if (!is.na(chisqNull)) cat("\n Chisquare (null model) =", chisqNull, " Df =", dfNull)
	if (!is.na(GFI)) cat("\n Goodness-of-fit index =", GFI)
	if (!is.na(AGFI)) cat("\n Adjusted goodness-of-fit index =", AGFI)
	if (!is.na(RMSEA)) cat("\n RMSEA index = ", RMSEA, " ", 100*conf.level, "% CI: (", RMSEA.L, ", ", RMSEA.U, ")", sep="")
	if (!is.na(NFI)) cat("\n Bentler-Bonett NFI =", NFI)
	if (!is.na(NNFI)) cat("\n Tucker-Lewis NNFI =", NNFI)
	if (!is.na(CFI)) cat("\n Bentler CFI =", CFI)
    if (!is.na(RNI)) cat("\n Bentler RNI = ", RNI)
    if (!is.na(IFI)) cat("\n Bollen IFI = ", IFI)
	if (!is.na(SRMR)) cat("\n SRMR =", SRMR)
	if (!is.na(AIC)) cat("\n AIC =", AIC)
	if (!is.na(AICc)) cat("\n AICc =", AICc)
	if (!is.na(BIC)) cat("\n BIC =", BIC)
	cat("\n\n")
	if (is.null(object$initial.iterations)) cat("Iterations:", object$iterations, "\n\n")
	else cat("Iterations: initial fits,", object$initial.iterations, "  final fit,", object$iterations, "\n\n")
	for (g in 1:G){
		cat("\n  ", object$group, ": ", groups[g], "\n", sep="")
		print(group.summaries[[g]])
	}
	invisible(object)
}

summary.msemObjectiveGLS <- function(object, digits=getOption("digits"), conf.level=.90,
    fit.indices=c("GFI", "AGFI", "RMSEA", "NFI", "NNFI", "CFI", "RNI", "IFI", "SRMR"), ...){
    fit.indices <- if (missing(fit.indices)){
       getOption("fit.indices")
    }
    else match.arg(fit.indices, several.ok=TRUE)
    summary.msemObjectiveML(object, digits=digits, conf.level=conf.level, robust=FALSE, fit.indices=fit.indices, ...)
	invisible(object)
}

summary.msemObjectiveFIML <- function(object, digits=getOption("digits"), conf.level=.90, ...){
	summary.msemObjectiveML(object, digits=digits, conf.level=conf.level, 
			robust=FALSE, ...)
	invisible(object)
}

deviance.msemObjectiveML <- function(object, ...) 
	object$criterion * sum(object$N - (!object$raw))

AIC.msemObjectiveML <- function(object, ..., k) {
	deviance(object) + 2*object$t
}

AICc.msemObjectiveML <- function(object, ...) {
	deviance(object) + 2*object$t*(object$t + 1)/(sum(object$N) - object$t - 1)
}

BIC.msemObjectiveML <- function(object, ...) {
	# deviance(object) + object$t*log(sum(object$N))
    deviance(object) - df.residual(object)*log(sum(object$N))
}

residuals.msem <- function(object, ...){
	result <- mapply(function(S, C) S - C, S=object$S, C=object$C, SIMPLIFY=FALSE)
	result <- lapply(result, function(res) {attr(res, "N") <- NULL; res})
	result
}

coef.msem <- function(object, ...) object$coeff

df.residual.msem <- function (object, ...){
	n <- object$n
	n.fix <- object$n.fix
	sum(n*(n + 1)/2) - object$t - sum(n.fix*(n.fix + 1)/2)
}

anova.msemObjectiveML <- function(object, model.2, ...) {
	dev.1 <- deviance(object)
	df.1 <- df.residual(object)
	dev.2 <- deviance(model.2)
	df.2 <- df.residual(model.2)
	name.1 <- deparse(substitute(object))
	name.2 <- deparse(substitute(model.2))
	df <- abs(df.1 - df.2)
	if (df == 0) 
		stop("the models have the same Df")
	if (all(object$N != model.2$N))
		stop("the models are fit to different numbers of observations")
	if (any(sapply(object$S, nrow) != sapply(model.2$S, nrow)) || !all.equal(object$S, model.2$S))
		stop("the models are fit to different moment matrices")
	chisq <- abs(dev.1 - dev.2)
	table <- data.frame(c(df.1, df.2), c(dev.1, dev.2), 
			c(NA, df), c(NA, chisq), c(NA, pchisq(chisq, df, lower.tail = FALSE)))
	names(table) <- c("Model Df", "Model Chisq", "Df", "LR Chisq", "Pr(>Chisq)")
	rownames(table) <- c(name.1, name.2)
	structure(table, heading = c("LR Test for Difference Between Models", ""), class = c("anova", "data.frame"))
}

logLik.msemObjectiveML <- function(object, ...) -0.5*deviance(object)

effects.msem <- function(object, ...){
	eff <- function(A, m, semmod){
		I <- diag(m)
		endog <- classifyVariables(semmod)$endogenous
		AA <- -A
		diag(AA) <- 1
		Total <- solve(AA) - I
		Indirect <- Total - A
		result <- list(Total = Total[endog, ], Direct = A[endog, ], Indirect = Indirect[endog, ])
		class(result) <- "semeffects"
		result
	}
	G <- length(object$groups)
	A <- object$A
	m <- object$m
	semmod <- object$semmodList
	result <- vector(G, mode="list")
	for (g in 1:G){
		result[[g]] <- eff(A[[g]], m[g], semmod[[g]])
	}
	names(result) <- object$groups
	class(result) <- "semeffectsList"
	result
}

print.semeffectsList <- function (x, digits = getOption("digits"), ...) {
	groups <- names(x)
	for (group in groups){
		cat("\n\n Group: ", group, "\n")
		print(x[[group]], digits=digits)
	}
	invisible(x)
}

standardizedCoefficients.msem <- function (object, ...){
	groups <- object$groups
	G <- length(groups)
	param.names <- object$param.names
	ram <- object$ram
	A <- object$A
	P <- object$P
	par <- coef(object)
	for (g in 1:G){
		par.names <- param.names[ram[[g]][, 4]]
		par.gr <- par[par.names]
		t <- length(par.gr)
		par.posn <- ram[[g]][, 4] != 0
		ram[[g]][par.posn, 4] <- 1:t
		group <- list(coeff=par.gr, t=t, ram=ram[[g]], A=A[[g]], P=P[[g]], par.posn=par.posn, param.names=par.names)
		class(group) <- "sem"
		cat("\n\n Group: ", groups[g], "\n")
		print(standardizedCoefficients(group, ...))
	}
}

standardizedResiduals.msem <- function (object, ...) {
	res <- residuals(object)
	S <- object$S
	for (g in 1:length(S)){
		s <- diag(S[[g]])
		res[[g]] <- res[[g]]/sqrt(outer(s, s))
	}
	res
}

normalizedResiduals.msemObjectiveML <- function (object, ...) {
	res <- residuals(object)
	N <- object$N - (!object$raw)
	C <- object$C
	for (g in 1:length(res)){
		c <- diag(C[[g]])
		res[[g]] <- res[[g]]/sqrt((outer(c, c) + C[[g]]^2)/N[g])
	}
	res
}

fscores.msem <- function (model, data = model$data, center = TRUE, scale = FALSE, ...) {
	m <- model$m
	P <- model$P
	A <- model$A
	var.names <- model$var.names
	C <- model$C
	group <- model$group
	groups <- model$groups
	G <- length(groups)
	scores <- B <- vector(G, mode = "list")
	names(scores) <- names(B) <- groups
	for (g in 1:G) {
		observed <- var.names[[g]] %in% rownames(C[[g]])
		if (all(observed)) {
			warning("there are no latent variables in group ", 
					groups[g])
		}
		IAinv <- solve(diag(m[g]) - A[[g]])
		Sigma <- IAinv %*% P[[g]] %*% t(IAinv)
		B[[g]] <- solve(Sigma[observed, observed]) %*% Sigma[observed, 
				!observed]
		rownames(B[[g]]) <- var.names[[g]][observed]
		colnames(B[[g]]) <- var.names[[g]][!observed]
		if (!is.null(data)) {
			X <- data[[g]][, var.names[[g]][observed]]
			if (center || scale) 
				X <- scale(X, center = center, scale = scale)
			scores[[g]] <- X %*% B[[g]]
		}
	}
	if (is.null(data)) 
		return(B)
	else return(scores)
}

vcov.msem <- function (object, robust=FALSE, analytic = inherits(object, "msemObjectiveML") && object$t <= 500, ...) { 
	if(robust){
		if (is.null(object$robust.vcov)) stop("robust coefficient covariance matrix not available")
		return(object$robust.vcov)
	}
	if (!analytic) 
		return(object$vcov)
	if (!inherits(object, "msemObjectiveML")) 
		stop("analytic coefficient covariance matrix unavailable")
	hessian <- function(model) {
		A <- model$A
		P <- model$P
		S <- model$S
		C <- model$C
		J <- model$J
		m <- model$m
		N <- model$N
		rams <- model$ram
		groups <- model$groups
		G <- length(groups)
		nms <- names(coef(model))
		raw <- model$raw
		Hessian <- matrix(0, nrow=length(nms), ncol=length(nms))
		rownames(Hessian) <- colnames(Hessian) <- nms
		wts <- (N - !raw)/(sum(N) - G*!raw)
		for (g in 1:G){
			I.Ainv <- solve(diag(m[g]) - A[[g]])
			Cinv <- solve(C[[g]])
			AA <- t(I.Ainv) %*% t(J[[g]])
			BB <- J[[g]] %*% I.Ainv %*% P[[g]] %*% t(I.Ainv)
			CC <- t(I.Ainv) %*% t(J[[g]])
			DD <- J[[g]] %*% I.Ainv
			dF.dBdB <- accumulate(AA %*% Cinv %*% t(AA), t(BB) %*% 
							Cinv %*% BB, AA %*% Cinv %*% BB, t(BB) %*% Cinv %*% 
							t(AA), m[g])
			dF.dPdP <- accumulate(CC %*% Cinv %*% t(CC), t(DD) %*% 
							Cinv %*% DD, CC %*% Cinv %*% DD, t(DD) %*% Cinv %*% 
							t(CC), m[g])
			dF.dBdP <- accumulate(AA %*% Cinv %*% t(CC), t(BB) %*% 
							Cinv %*% DD, AA %*% Cinv %*% DD, t(BB) %*% Cinv %*% 
							t(CC), m[g])
			ram <- rams[[g]]
			fixed <- ram[, 4] == 0
			sel.free <- ram[, 4]
			sel.free[fixed] <- 0
			one.head <- ram[, 1] == 1
			one.free <- which((!fixed) & one.head)
			two.free <- which((!fixed) & (!one.head))
			two.free.cov <- which((!fixed) & (!one.head) & (ram[, 2] != ram[, 3]))
			arrows.1 <- ram[one.head, c(2, 3), drop = FALSE]
			arrows.2 <- ram[!one.head, c(2, 3), drop = FALSE]
			arrows.2t <- ram[!one.head, c(3, 2), drop = FALSE]
			arrows.1.free <- ram[one.free, c(2, 3), drop = FALSE]
			arrows.2.free <- ram[two.free, c(2, 3), drop = FALSE]
			sel.free.1 <- sel.free[one.free]
			sel.free.2 <- sel.free[two.free]
			unique.free.1 <- unique(sel.free.1)
			unique.free.2 <- unique(sel.free.2)
			posn.matrix <- matrix(1:(m[g]^2), m[g], m[g])
			posn.free <- c(posn.matrix[arrows.1.free], (m[g]^2) + posn.matrix[arrows.2.free])
			DBB <- dF.dBdB[posn.matrix[arrows.1.free], posn.matrix[arrows.1.free], 
					drop = FALSE]
			DPP <- dF.dPdP[posn.matrix[arrows.2.free], posn.matrix[arrows.2.free], 
					drop = FALSE]
			DBP <- dF.dBdP[posn.matrix[arrows.1.free], posn.matrix[arrows.2.free], 
					drop = FALSE]
			hessian <- rbind(cbind(DBB, DBP), cbind(t(DBP), DPP))
			n1 <- length(one.free)
			n2 <- length(two.free)
			nn <- rep(c(sqrt(2), sqrt(2)/2), c(n1, n2))
			nn[c(one.free, two.free) %in% two.free.cov] <- sqrt(2)
			hessian <- hessian * outer(nn, nn)
			pars <- ram[, 4][!fixed]
			all.pars <- ram[, 4]
			t <- length(pars)
			Z <- outer(sort(unique(pars)), pars, function(x, y) as.numeric(x == y))
			hessian <- Z %*% hessian %*% t(Z)
			par.names <- c(nms[all.pars[one.free]], nms[all.pars[two.free]])
			rownames(hessian) <- colnames(hessian) <- par.names
			Hessian[par.names, par.names] <- Hessian[par.names, par.names] + wts[g]*hessian
		}
		Hessian
	}
	h <- hessian(object)
	t <- object$t
	N <- sum(object$N)
	raw <- object$raw
	G <- length(object$groups)
	param.names <- rownames(h)
	vcov <- matrix(NA, t, t)
	qr.hess <- try(qr(h), silent = TRUE)
	if (inherits(qr.hess, "try-error")){
	  warning("Could not compute QR decomposition of Hessian.\nOptimization probably did not converge.\n")
	}
	else if (qr.hess$rank < t) {
		warning(" singular Hessian: model is probably underidentified.\n")
		which.aliased <- qr.hess$pivot[-(1:qr.hess$rank)]
		attr(vcov, "aliased") <- param.names[which.aliased]
	}
	else {
		vcov <- (2/(N - G*(!raw))) * solve(h)
		if (any(diag(vcov) < 0)) {
			attr(vcov, "aliased") <- param.names[diag(vcov) <  0]
			warning("Negative parameter variances.\nModel may be underidentified.\n")
		}
	}
	vcov
}

robustVcovMsem <- function(model){
	G <- length(model$groups)
	t <- model$t  
	N <- model$N
	raw <- model$raw
	wts <- (N - !raw)/(sum(N) - !raw*G)
	hessian <- matrix(0, t, t)
	rownames(hessian) <- colnames(hessian) <- model$param.names
	chisq <- 0
	adj.objects <- vector(G, mode="list")
	for (g in 1:G){
		ram <- model$ram[[g]]
		parameters <- ram[, 4]
		unique.pars <- unique(parameters[parameters != 0])
		par.posn <- sapply(unique.pars, function(x) which(x == parameters)[1])
		unique.posn <- which(parameters %in% unique.pars)
		rownames(ram)[unique.posn] <- unique(model$param.names[ram[, 4]])
		ram[unique.posn, 4] <- unlist(apply(outer(unique.pars, parameters, "=="), 2, which))
		mod.g <- list(var.names=model$var.names[[g]], ram=ram,J=model$J[[g]], n.fix=model$n.fix, n=model$n[[g]], 
				N=model$N[g], m=model$m[[g]], t=length(unique.pars),
				coeff=model$coeff[parameters],  criterion=model$group.criteria[g],  S=model$S[[g]], raw=model$raw,
				C=model$C[[g]], A=model$A[[g]], P=model$P[[g]])
		adj.objects[[g]] <- sbchisq(sem.obj=mod.g, sem.data=model$data[[g]])
		hess <- solve((N[g] - 1)*robustVcov(mod.g, adj.obj=adj.objects[[g]]))
		pars <- rownames(hess)
		hessian[pars, pars] <- hessian[pars, pars] + wts[g]*hess
		chisq <- chisq + adj.objects$chisq.scaled
	}
	return(list(vcov=solve(hessian)/(sum(N) - !raw*G), chisq.scaled=chisq, adj.objects=adj.objects))
}

# added 2011-11-04 by J. Fox

effects.sem <- function(object, ...) {
	A <- object$A
	m <- object$m
	I <- diag(m)
	endog <- classifyVariables(object$semmod)$endogenous  
	AA <- - A
	diag(AA) <- 1
	Total <- solve(AA) - I
	Indirect <-  Total - A
	result <- list(Total=Total[endog, ], Direct=A[endog, ], Indirect=Indirect[endog, ])
	class(result) <- "semeffects"
	result
}

print.semeffects <- function(x, digits=getOption("digits"), ...){
	cat("\n Total Effects (column on row)\n")
	Total <- x$Total
	Direct <- x$Direct
	Indirect <- x$Indirect
	select <- !(apply(Total, 2, function(x) all( x == 0)) & 
				apply(Direct, 2, function(x) all( x == 0)) & 
				apply(Indirect, 2, function(x) all( x == 0)))
	print(Total[, select], digits=digits)
	cat("\n Direct Effects\n")
	print(Direct[, select], digits=digits)
	cat("\n Indirect Effects\n")
	print(Indirect[, select], digits=digits)
	invisible(x)
}

# last modified 2014-10-27 by J. Fox

combineModels <- function (...){
	UseMethod("combineModels")
}

combineModels.semmod <- function(..., warn=TRUE){
	model <- rbind(...)
	removeRedundantPaths(model, warn=warn)
	model
}

update.semmod <- function (object, file = "", text, ...) {
	regularizePaths <- function(p) sub(" *-*> *", "->", sub(" *<-* *", "<-", p))
	delete.model.element <- function(delete.text, old.model, type = "path") {
		type <- match.arg(type, c("path", "variable", "coefficient"))
		if (type == "path") delete.text <- regularizePaths(delete.text)
		col.index <- c(path = 1, variable = 1, coefficient = 2)[type]
		delete.text <- strip.white(delete.text)
		old.model <- old.model[-grep(delete.text, strip.white(old.model[, col.index])),]
		class(old.model) <- "semmod"
		return(old.model)
	}
	object[, 1] <- regularizePaths(object[, 1])
	modmat <- if (!missing(text)) scan(text = text, what = list(change = "", var1 = "", 
	                                         var2 = "", var3 = "", var4 = ""), sep = ",", strip.white = TRUE, 
	                                        comment.char = "#", fill = TRUE)
    else scan(file = file, what = list(change = "", var1 = "", 
					var2 = "", var3 = "", var4 = ""), sep = ",", strip.white = TRUE, 
			comment.char = "#", fill = TRUE)
	modmat <- cbind(modmat$change, modmat$var1, modmat$var2, 
			modmat$var3)
	if ("delete" %in% modmat[, 1]) {
		deletemat <- modmat[which(modmat[, 1] == "delete"), 2:3, drop=FALSE]
		deletemat[which(deletemat[, 2] == ""), 2] <- "path"
		for (i in 1:nrow(deletemat)) 
			object <- delete.model.element(deletemat[i, 1], object, deletemat[i, 2])
	}
	if ("add" %in% modmat[, 1]) {
		addmat <- modmat[which(modmat[, 1] == "add"), 2:4, drop=FALSE]
		addmat[addmat[, 3] == "", 3] <- NA
		addmat[addmat[, 2] == "", 2] <- NA
		class(addmat) <- "semmod"
		object <- combineModels(object, addmat, warn=FALSE)
	}
	if ("replace" %in% modmat[, 1]) {
		submat <- modmat[which(modmat[, 1] == "replace"), 2:3, drop=FALSE]
		for (i in 1:nrow(submat)){
			object[, 1:2] <- mapply(function(x, y) gsub(x, y, object[, 1:2]), submat[i, 1], submat[i, 2])
		}
	}
	if ("fix" %in% modmat[, 1]){
		fixmat <- modmat[which(modmat[, 1] == "fix"), 2:3, drop=FALSE]
		fixmat[, 1] <- regularizePaths(fixmat[, 1])
		for (i in 1:nrow(fixmat)){
			object[which(object[, 1] == fixmat[i, 1]), 2:3] <- c(NA, fixmat[i, 2])
		}
	}
	if ("free" %in% modmat[, 1]){
		freemat <- modmat[which(modmat[, 1] == "free"), 2:4, drop=FALSE]
		freemat[, 1] <- regularizePaths(freemat[, 1])
		for (i in 1:nrow(freemat)){
			object[which(object[, 1] == freemat[i, 1]), 2:3] <- freemat[i, 2:3]
		}
	}
	removeRedundantPaths(object)
}

# the next function is adapted from code contributed by Michael Friendly

edit.semmod <- function(name, ...) {
	name <- unclass(name)
	colnames(name) <- c("Path", "Parameter", "StartValue")
	name <- edit(name)
	class(name) <- "semmod"
	name
}

# last modified 2012-08-01 by J. Fox

anova.objectiveML <- function(object, model.2, robust=FALSE, ...){
	anovaAdjchisq <- function(adjobj0, adjobj1){
		# this subfunction oringally by Jarrett Byrnes
		#from http://www.statmodel.com/chidiff.shtml
		# Satorra-bentler adjusted chi sq
		#switching to get order right
		sbs.nested <- adjobj0
		sbs.full <- adjobj1
		t0 <- sbs.nested$chisq
		tr0 <- sbs.nested$chisq.scaled
		t1 <- sbs.full$chisq
		tr1 <- sbs.full$chisq.scaled		
		c0 <- sbs.nested$c
		c1 <- sbs.full$c		
		d0 <- sbs.nested$df
		d1 <- sbs.full$df		
		cd <- (d0 * c0 - d1*c1)/(d0 - d1)
		trd <- abs((t0 - t1)/cd) 		
		df <- abs(d0 - d1)		
		table <- data.frame(c(d0, d1), c(tr0, tr1), c(NA, df), c(NA, trd),
				c(NA, pchisq(trd, df, lower.tail=FALSE)))		
		return(table)
	}
	dev.1 <- deviance(object)
	df.1 <- df.residual(object)
	dev.2 <- deviance(model.2)
	df.2 <- df.residual(model.2)
	name.1 <- deparse(substitute(object))
	name.2 <- deparse(substitute(model.2))
	df <- abs(df.1 - df.2)
	if (df == 0) stop("the models have the same Df")
	if (object$N != model.2$N)
		stop("the models are fit to different numbers of observations")
	if ((nrow(object$S) != nrow(model.2$S)) || !all.equal(object$S, model.2$S))
		stop("the models are fit to different moment matrices")
	if(!robust){
		chisq <- abs(dev.1 - dev.2)
		table <- data.frame(c(df.1, df.2), c(dev.1, dev.2), c(NA, df), c(NA, chisq),
				c(NA, pchisq(chisq, df, lower.tail=FALSE)))
	}
	else{
		cat("Adjusted Using Satorra-Bentler Correction\n");
		table <- anovaAdjchisq(object$adj.obj, model.2$adj.obj)
	}
	names(table) <- c("Model Df", "Model Chisq", "Df", "LR Chisq", "Pr(>Chisq)")
	rownames(table) <- c(name.1, name.2)
	structure(table, heading = c("LR Test for Difference Between Models", ""),
			class = c("anova", "data.frame"))
}

anova.objectiveFIML <- function(object, model.2, ...){
    logLik.1 <- logLik(object)
    df.1 <- df.residual(object)
    logLik.2 <- logLik(model.2)
    df.2 <- df.residual(model.2)
    name.1 <- deparse(substitute(object))
    name.2 <- deparse(substitute(model.2))
    df <- abs(df.1 - df.2)
    if (df == 0) stop("the models have the same Df")
    if (object$N != model.2$N)
        stop("the models are fit to different numbers of observations")
    if ((nrow(object$S) != nrow(model.2$S)) || !all.equal(object$S, model.2$S))
        stop("the models are fit to different data sets")
    chisq <- 2*(abs(logLik.1 - logLik.2))
    table <- data.frame(c(df.1, df.2), c(logLik.1, logLik.2), c(NA, df), c(NA, chisq),
                        c(NA, pchisq(chisq, df, lower.tail=FALSE)))
    names(table) <- c("Model Df", "Model Log-Likelihood", "Df", "LR Chisq", "Pr(>Chisq)")
    rownames(table) <- c(name.1, name.2)
    structure(table, heading = c("LR Test for Difference Between Models", ""),
              class = c("anova", "data.frame"))
}

# last modified 2011-08-06 by J. Fox

fscores <- function(model, ...){
	UseMethod("fscores")
}

fscores.sem <- function(model, data=model$data, center=TRUE, scale=FALSE, ...){
	m <- model$m
	P <- model$P
	A <- model$A
	var.names <- model$var.names
	observed <- var.names %in% rownames(model$C)
	if (all(observed)) stop("there are no latent variables")
	IAinv <- solve(diag(m) - A)
	Sigma <- IAinv %*% P %*% t(IAinv)
	B <- solve(Sigma[observed, observed]) %*% Sigma[observed, !observed]
	rownames(B) <- var.names[observed]
	colnames(B) <- var.names[!observed]
	if (is.null(data)) return(B)
	X <- as.matrix(data[,var.names[observed]])
	if (center || scale) X <- scale(X, center=center, scale=scale)
	X %*% B
}

# last modified 2019-11-15 by J. Fox

optimizerNlm <- function(start, objective=objectiveML, 
	gradient=TRUE, maxiter, debug, par.size, model.description, warn, ...){
	with(model.description, {
			obj <- objective(gradient=gradient)$objective
			typsize <- if (par.size == 'startvalues') abs(start) else rep(1, t)
			if (!warn) save.warn <- options(warn=-1)
			res <- nlm(obj, start, iterlim=maxiter, print.level=if(debug) 2 else 0,
				typsize=typsize, hessian=TRUE, model.description, ...)
			if (!warn) options(save.warn)
			result <- list()
			result$convergence <- res$code <= 2
			result$iterations <- res$iterations
			par <- res$estimate
			names(par) <- param.names
			result$par <- par
			if (!result$convergence)
				warning(paste('Optimization may not have converged; nlm return code = ',
						res$code, '. Consult ?nlm.\n', sep=""))
			vcov <- matrix(NA, t, t)
			qr.hess <- try(qr(res$hessian), silent=TRUE)
			if (inherits(qr.hess, "try-error")){
				warning("Could not compute QR decomposition of Hessian.\nOptimization probably did not converge.\n")
			}
			else if (qr.hess$rank < t){
				warning(' singular Hessian: model is probably underidentified.\n')
				which.aliased <- qr.hess$pivot[-(1:qr.hess$rank)]
				result$aliased <- param.names[which.aliased]
			}
			else {
				vcov <- (2/(N - (!raw))) * solve(res$hessian)
				if (any(diag(vcov) < 0)) {
					result$aliased <- param.names[diag(vcov) < 0]
					warning("Negative parameter variances.\nModel may be underidentified.\n")
				}
			}
			colnames(vcov) <- rownames(vcov) <- param.names
			result$vcov <- vcov
			result$criterion <- res$minimum # c(result$obj) - n - log(det(S))
			obj <- obj(par, model.description)
			C <- attr(obj, "C")
			rownames(C) <- colnames(C) <- var.names[observed]
			result$C <- C
			A <- attr(obj, "A")
			rownames(A) <- colnames(A) <- var.names
			result$A <- A
			P <- attr(obj, "P")
			rownames(P) <- colnames(P) <- var.names
			result$P <- P
			class(result) <- "semResult"
			result
		}
	)
}

# bootstrapped standard errors and confidence intervals for sem

# last modified 2015-06-09 by J. Fox

boot.sem <- function(...) {
	.Deprecated("bootSem", package="sem")
	bootSem(...)
}

bootSem <- function (model, ...){
    UseMethod("bootSem")
}

bootSem.sem <- function(model, R=100, Cov=cov, data=model$data,  max.failures=10, show.progress=TRUE, ...){
    refit <- function(){
        indices <- sample(N, N, replace=TRUE)
        S <- Cov(data[indices,])
        refitted.model <- sem(ram, S, N, param.names=coef.names, var.names=var.names,
            	optimizer=model$optimizer, objective=model$objective, ...)
        refitted.model$coeff
        }
#    if (!require("boot")) stop("package boot not available")
#	has.tcltk <- require("tcltk")
#	  pb <- tkProgressBar("Bootstrap Sampling", "Bootstrap sample: ", 0, R)
    if (show.progress){
        cat("\n", R, "bootstrap replications\n")
        pb <- txtProgressBar(min=0, max=R, style=3)
    }
    # the following 2 lines borrowed from boot in package boot
    if (!exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)) runif(1)
    seed <- get(".Random.seed", envir = .GlobalEnv, inherits = FALSE)
    warn <- options(warn=-2)
    on.exit(options(warn)) # insure restore even in event of error
    nErrors <- 0
	if (is.null(data)) stop("the model object doesn't contain a data matrix")
    N <- nrow(data)
    coefficients <- model$coeff
    coef.names <- names(coefficients)
    var.names <- model$var.names 
    ram <- model$ram 
    ram[coef.names, "start value"] <- coefficients 
    coefs <- matrix(numeric(0), R, length(coefficients))
    colnames(coefs) <- coef.names
    for (b in 1:R){
#		setTkProgressBar(pb, b, label=sprintf("Bootstrap sample: %d", b))
        if (show.progress) setTxtProgressBar(pb, b)
        for (try in 1:(max.failures + 1)){
            if (try >  max.failures) stop("more than ",  max.failures, " consecutive convergence failures")
            res <- try(refit(), silent=TRUE)
            if (inherits(res, "try-error")) nErrors <- nErrors + 1
            else {
                coefs[b,] <- res
                break()
                }
            }
        }
    options(warn)
    if (nErrors > 0) warning("there were", nErrors, 
        "apparent convergence failures;\nthese are discarded from the",
        R, "bootstrap replications returned")
    res <- list(t0=coefficients, t=coefs, R=R, data=data, seed=seed,
        statistic=refit, sim="ordinary", stype="i", call=match.call(),
        strata=rep(1, N), weights=rep(1/N, N))
    res$call[[1]] <- as.name("bootSem")
	if (show.progress) close(pb)
    class(res) <- c("bootsem", "boot")
    res
    }

bootSem.msem <- function(model, R=100, Cov=cov, data=model$data,  max.failures=10, show.progress=TRUE, ...){
    refit <- function(){
        for (g in 1:G){
            indices <- sample(N[g], N[g], replace=TRUE)
            S[[g]] <- Cov(data[[g]][indices, ])
        }
        refitted.model <- sem(ram, S, N, param.names=coef.names, var.names=var.names,
        		optimizer=model$optimizer, objective=model$objective, fixed.x=model$fixed.x, ...)
        refitted.model$coeff
        }
#    if (!require("boot")) stop("package boot not available")
#	has.tcltk <- require("tcltk")
#	  pb <- tkProgressBar("Bootstrap Sampling", "Bootstrap sample: ", 0, R)
    if (show.progress){
        cat("\n", R, "bootstrap replications\n")
        pb <- txtProgressBar(min=0, max=R, style=3)
    }
    # the following 2 lines borrowed from boot in package boot
    if (!exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)) runif(1)
    seed <- get(".Random.seed", envir = .GlobalEnv, inherits = FALSE)
    warn <- options(warn=-2)
    on.exit(options(warn)) # insure restore even in event of error
    nErrors <- 0
	if (is.null(data)) stop("the model object doesn't contain data matrices")
    N <- sapply(data, nrow)
    coefficients <- model$coeff
    coef.names <- names(coefficients)
    var.names <- model$var.names 
    ram <- model$ram 
    groups <- model$groups
    group <- model$group
    G <- length(groups)
    S <- vector(G, mode="list")
    for (g in 1:G){
        pars <- ram[[g]][, "parameter"]
        free <- pars != 0
        ram[[g]][free, "start value"] <- coefficients[pars[free]]
    }
    coefs <- matrix(numeric(0), R, length(coefficients))
    colnames(coefs) <- coef.names
    for (b in 1:R){
#		setTkProgressBar(pb, b, label=sprintf("Bootstrap sample: %d", b))
        if (show.progress) setTxtProgressBar(pb, b)
        for (try in 1:(max.failures + 1)){
            if (try >  max.failures) stop("more than ",  max.failures, " consecutive convergence failures")
            res <- try(refit(), silent=TRUE)
            if (inherits(res, "try-error")) nErrors <- nErrors + 1
            else {
                coefs[b,] <- res
                break()
                }
            }
        }
    options(warn)
    if (nErrors > 0) warning("there were", nErrors, 
        "apparent convergence failures;\nthese are discarded from the",
        R, "bootstrap replications returned")
    res <- list(t0=coefficients, t=coefs, R=R, data=data, seed=seed,
        statistic=refit, sim="ordinary", stype="i", call=match.call(),
        strata=rep(1:G, N), weights=rep(1/N, N))
    res$call[[1]] <- as.name("bootSem")
	if (show.progress) close(pb)
    class(res) <- c("bootsem", "boot")
    res
    }

        
print.bootsem <- function(x, digits=getOption("digits"), ...){
    t <- x$t
    t0 <- x$t0
    result <- data.frame("Estimate"=t0, "Bias"=colMeans(t) - t0, 
        "Std.Error"=apply(t, 2, sd))
    rownames(result) <- names(t0)
    cat("Call: ")
    dput(x$call)
    cat("\n")
    print(result, digits=digits)
    invisible(x)
    }

summary.bootsem <- function(object,
    type=c("perc", "bca", "norm", "basic", "none"), level=0.95, ...){
    type <- match.arg(type)
    t <- object$t
    t0 <- object$t0
    result <- data.frame("Estimate"=t0, "Bias"=colMeans(t) - t0, 
        "Std.Error"=apply(t, 2, sd))
    if (type != "none"){
        p <- length(t0)
        lower <- upper <- rep(0, p)
        low <- if (type == "norm") 2 else 4
        up  <- if (type == "norm") 3 else 5 
        for (i in 1:p){
            ci <- as.vector(boot.ci(object, type=type, index=i, 
                conf=level)[[type, exact=FALSE]])
            lower[i] <- ci[low]
            upper[i] <- ci[up]
            }
        result$Lower <- lower
        result$Upper <- upper
        }    
    rownames(result) <- names(t0)
    result <- list(table=result, call=object$call, level=level, type=type)
    class(result) <- "summary.bootsem"
    result
    }

print.summary.bootsem <- function(x, digits=getOption("digits"), ...){
    cat("Call: ")
    dput(x$call)
    cat("\n")
    if (x$type != "none") {
        cat(paste("Lower and upper limits are for the", 100*x$level, 
            "percent", x$type, "confidence interval\n\n"))
        }
    print(x$table, digits=digits)
    invisible(return(x))
    }

# last modified 2012-01-06 by J. Fox
# Modified for Compiled code in C/C++ by Zhenghua Nie.

objectiveGLS <- function (gradient = FALSE) 
{
    result <- list(objective = function(par, model.description) {
        with(model.description, {
						 
						 res <- CompiledObjective(par=par, model.description=model.description, objective="objectiveGLS", gradient=gradient) 
				
						 f <- res$f
				     C <- res$C
				     A <- res$A
				     P <- res$P

            attributes(f) <- list(C = C, A = A, P = P)
            f
        }
				)
    }
		)
    class(result) <- "semObjective"
    result
}

# last modified 2013-01-24 by J. Fox

startvalues2 <- function(S, ram, debug=FALSE, tol=1E-6){
	ram <- ram[!(ram[, "parameter"] == 0 & ram[, "start value"] == 0), , drop=FALSE] # eliminate fixed parameters set to 0
    n <- nrow(S) 
    observed <- 1:n       
    m <- max(ram[,c(2,3)])            
    t <- max(ram[,4])   
    s <- sqrt(diag(S))
    R <- S/outer(s,s)
    latent<-(1:m)[-observed]
    par.posn <-  sapply(1:t, function(i) which(ram[,4] == i)[1])    
    one.head <- ram[,1] == 1
    start <- (ram[,5])[par.posn]
    A.pat <-matrix(FALSE, m, m)
    A.pat[ram[one.head, c(2,3), drop=FALSE]] <- TRUE
    P.pat <- C <- matrix(0, m, m)
    P.pat[ram[!one.head, c(2,3), drop=FALSE]] <- P.pat[ram[!one.head, c(3,2), drop=FALSE]] <- 1
    C[observed, observed] <- R
    for (l in latent) {
        indicators <- A.pat[observed, l]
        for (j in observed){
            C[j, l] <- C[l, j] <- if (!any(indicators)) runif(1, .3, .5)
                else {   
                        numerator <- sum(R[j, observed[indicators]])
                        denominator <- sqrt(sum(R[observed[indicators], observed[indicators]]))
                        numerator/denominator
                    }
            }
        }
    for (l in latent){
        for (k in latent){
            C[l, k] <- C[k,l] <- if (l==k) 1 else {
                                indicators.l <- A.pat[observed, l]
                                indicators.k <- A.pat[observed, k]
                                if ((!any(indicators.l)) | (!any(indicators.k))) runif(1, .3, .5) else {
                                    numerator <- sum(R[observed[indicators.l], observed[indicators.k]])
                                    denominator <- sqrt( sum(R[observed[indicators.l], observed[indicators.l]])
                                        * sum(R[observed[indicators.k], observed[indicators.k]]))
                                    numerator/denominator}
                                    }
            }
        }
    A <- matrix(0, m, m)
    for (j in 1:m){
        ind <- A.pat[j,]
        if (!any(ind)) next
        A[j, ind] <- solve(C[ind, ind]) %*% C[ind, j]
        }
    A[observed,] <- A[observed,]*matrix(s, n, m)
    A[,observed] <- A[,observed]/matrix(s, m, n, byrow=TRUE)
    C[observed,] <- C[observed,]*matrix(s, n, m)
    C[,observed] <- C[,observed]*matrix(s, m, n, byrow=TRUE)
    P <- (diag(m) - A) %*% C %*% t(diag(m) - A)
    P <- P.pat * P
    for (par in 1:t){
        if (!is.na(start[par])) next
        posn <- par.posn[par]
        if (ram[posn, 1] == 1) start[par] <- A[ram[posn, 2], ram[posn, 3]]
            else start[par] <- P[ram[posn, 2], ram[posn, 3]]
        if (abs(start[par]) < tol) start[par] <- tol
        }
    if (debug){
        cat('\nStart values:\n')
        print(start)
        cat('\n')
        }
    start
    }

startvalues <- function(S, ram, debug=FALSE, tol=1E-6){
    ram <- ram[!(ram[, "parameter"] == 0 & ram[, "start value"] == 0), , drop=FALSE] # eliminate fixed parameters set to 0
    n <- nrow(S) 
    observed <- 1:n       
    m <- max(ram[,c(2,3)])            
    t <- max(ram[,4])   
    s <- sqrt(diag(S))
    R <- S/outer(s,s)
    latent<-(1:m)[-observed]
    par.posn <-  sapply(1:t, function(i) which(ram[,4] == i)[1])    
    one.head <- ram[,1] == 1
    start <- (ram[,5])[par.posn]
    A.pat <-matrix(FALSE, m, m)
    A.pat[ram[one.head, c(2,3), drop=FALSE]] <- TRUE
    P.pat <- C <- matrix(0, m, m)
    P.pat[ram[!one.head, c(2,3), drop=FALSE]] <- P.pat[ram[!one.head, c(3,2), drop=FALSE]] <- 1
    C[observed, observed] <- R
    ref.indicators <- numeric(m)
    for (l in latent) {
        indicators <- A.pat[observed, l]
        if (any(indicators)){
            ref <- which.max(rowMeans(abs(R[observed[indicators], observed[indicators], drop=FALSE])))
            ref.indicators[l] <- observed[indicators][ref]
        }
        for (j in observed){
            C[j, l] <- C[l, j] <- if (!any(indicators)) runif(1, .3, .5)
            else {   
                numerator <- mean(abs(R[j, observed[indicators]]))
                denominator <- sqrt(mean(abs(R[observed[indicators], observed[indicators]])))
                c <- numerator/denominator
                sign(R[j, ref.indicators[l]])*abs(c)
            }
        }
    }
    for (l in latent){
        indicators.l <- A.pat[observed, l]
        for (k in latent){
            C[l, k] <- C[k, l] <-  if (l==k) 1 else { 
                #                 if (l == k){
                #                     if (!any(indicators.l)) runif(1, .3, .5)
                #                     else mean(abs(R[indicators.l, ref.indicators[l]]))
                #                 }
                #             else {
                indicators.k <- A.pat[observed, k]
                if ((!any(indicators.l)) | (!any(indicators.k))) runif(1, .3, .5) else {
                    numerator <- mean(abs(R[observed[indicators.l], observed[indicators.k]]))
                    denominator <- sqrt( mean(abs(R[observed[indicators.l], observed[indicators.l]]))
                                         * mean(abs(R[observed[indicators.k], observed[indicators.k]])))
                    c <- numerator/denominator
                    sign(R[ref.indicators[l], ref.indicators[k]])*abs(c)
                }
            }
        }
    }
    A <- matrix(0, m, m)
    for (j in 1:m){
        ind <- A.pat[j,]
        if (!any(ind)) next
        A[j, ind] <- solve(C[ind, ind]) %*% C[ind, j]
    }
    A[observed,] <- A[observed,]*matrix(s, n, m)
    A[,observed] <- A[,observed]/matrix(s, m, n, byrow=TRUE)
    C[observed,] <- C[observed,]*matrix(s, n, m)
    C[,observed] <- C[,observed]*matrix(s, m, n, byrow=TRUE)
    P <- (diag(m) - A) %*% C %*% t(diag(m) - A)
    P <- P.pat * P
    diag(P) <- abs(diag(P))
    for (par in 1:t){
        if (!is.na(start[par])) next
        posn <- par.posn[par]
        if (ram[posn, 1] == 1) start[par] <- A[ram[posn, 2], ram[posn, 3]]
        else start[par] <- P[ram[posn, 2], ram[posn, 3]]
        if (abs(start[par]) < tol) start[par] <- tol
    }
    if (debug){
        cat('\nStart values:\n')
        print(start)
        cat('\n')
    }
    start
}

# last modified 2012-01-11 by J. Fox

objectiveML2 <- function(gradient=TRUE){
	result <- list(
		objective = function(par, model.description){
			with(model.description, {
					A <- P <- matrix(0, m, m)
					val <- ifelse (fixed, ram[,5], par[sel.free])
					A[arrows.1] <- val[one.head]
					P[arrows.2t] <- P[arrows.2] <- val[!one.head]
					I.Ainv <- solve(diag(m) - A)
					C <- J %*% I.Ainv %*% P %*% t(I.Ainv) %*% t(J)
					Cinv <- solve(C)
					f <- sum(diag(S %*% Cinv)) + log(det(C)) - n - logdetS
					grad <- NULL
					if (gradient){
						grad.P <- correct * t(I.Ainv) %*% t(J) %*% Cinv %*% (C - S) %*% Cinv %*% J %*% I.Ainv
						grad.A <- grad.P %*% P %*% t(I.Ainv)        
						grad <- rep(0, t)
						grad[sort(unique.free.1)] <- tapply(grad.A[arrows.1.free],ram[ram[,1]==1 & ram[,4]!=0, 4], sum)
						grad[sort(unique.free.2)] <- tapply(grad.P[arrows.2.free],ram[ram[,1]==2 & ram[,4]!=0, 4], sum)
					}
					attributes(f) <- list(C=C, A=A, P=P, gradient=grad)
					f
				}
			)
		}
	)
	if (gradient)
		result$gradient <- function(par, model.description){
			with(model.description, {
					A <- P <- matrix(0, m, m)
					val <- ifelse (fixed, ram[,5], par[sel.free])
					A[arrows.1] <- val[one.head]
					P[arrows.2t] <- P[arrows.2] <- val[!one.head]
					I.Ainv <- solve(diag(m) - A)
					C <- J %*% I.Ainv %*% P %*% t(I.Ainv) %*% t(J)
					Cinv <- solve(C)
					grad.P <- correct * t(I.Ainv) %*% t(J) %*% Cinv %*% (C - S) %*% Cinv %*% J %*% I.Ainv
					grad.A <- grad.P %*% P %*% t(I.Ainv)        
					grad <- rep(0, t)
					grad[sort(unique.free.1)] <- tapply(grad.A[arrows.1.free],ram[ram[,1]==1 & ram[,4]!=0, 4], sum)
					grad[sort(unique.free.2)] <- tapply(grad.P[arrows.2.free],ram[ram[,1]==2 & ram[,4]!=0, 4], sum)
					attributes(grad) <- list(C=C, A=A, P=P, gradient=grad)
					grad
				}
			)
		}
	class(result) <- "semObjective"
	result
}

# last modified 2013-12-19 by J. Fox


summary.objectiveML <- function(object, digits=getOption("digits"), conf.level=.90, robust=FALSE, analytic.se=object$t <= 500, 
                                fit.indices=c("GFI", "AGFI", "RMSEA", "NFI", "NNFI", "CFI", "RNI", "IFI", "SRMR", "AIC", "AICc", "BIC", "CAIC"), ...) {
    fit.indices <- if (is.null(fit.indices)) ""
    else {
        if (missing(fit.indices)){
            if (is.null(opt <- getOption("fit.indices"))) c("AIC", "BIC") else opt
        }
        else match.arg(fit.indices, several.ok=TRUE)
    }
	vcov <- vcov(object, robust=robust, analytic=analytic.se)
	if (any(is.na(vcov))) stop("coefficient covariances cannot be computed")
	norm.res <- normalizedResiduals(object)
	se <- sqrt(diag(vcov))
	z <- object$coeff/se
	n.fix <- object$n.fix
	n <- object$n
	t <- object$t
	S <- object$S
	C <- object$C
	N <- object$N
	df <- n*(n + 1)/2 - t - n.fix*(n.fix + 1)/2
	dfNull <- n*(n - 1)/2
	invC <- solve(C)
	CSC <- invC %*% (S - C)
	CSC <- CSC %*% CSC
	CS <- invC %*% S
	CS <- CS %*% CS
	chisqNull <- chisqNull(object)
	chisq <- object$criterion * (N - (!object$raw))
	GFI <- if (!"GFI" %in% fit.indices) NA else if (!object$raw) 1 - sum(diag(CSC))/sum(diag(CS)) else NA
    Rsq <- if (!object$raw) Rsq(object) else NA
	if ((!object$raw) && df > 0){
		AGFI <- if (!"AGFI" %in% fit.indices) NA else 1 - (n*(n + 1)/(2*df))*(1 - GFI)
		NFI <- if (!"NFI" %in% fit.indices) NA else (chisqNull - chisq)/chisqNull
		NNFI <- if (!"NNFI" %in% fit.indices) NA else (chisqNull/dfNull - chisq/df)/(chisqNull/dfNull - 1)
		L1 <- max(chisq - df, 0)
		L0 <- max(L1, chisqNull - dfNull)
		CFI <- if (!"CFI" %in% fit.indices) NA else 1 - L1/L0
        RNI <- if (!"RNI" %in% fit.indices) NA else 1 - (chisq - df)/(chisqNull - dfNull)
        IFI <- if (!"IFI" %in% fit.indices) NA else (chisqNull - chisq)/(chisqNull - df)
        if (!"RMSEA" %in% fit.indices) {
            RMSEA <- NA
            }
		else {
		    RMSEA <- sqrt(max(object$criterion/df - 1/(N - (!object$raw)), 0))
		    tail <- (1 - conf.level)/2 
		    max <- N
		    while (max > 1){
		        res <- optimize(function(lam) (tail - pchisq(chisq, df, ncp=lam))^2, interval=c(0, max))
		        if (is.na(res$objective) || res$objective < 0){
		            max <- 0
		            warning("cannot find upper bound of RMSEA")
		            break
		        }				
		        if (sqrt(res$objective) < tail/100) break
		        max <- max/2
		    }
		    lam.U <- if (max <= 1) NA else res$minimum
		    max <- max(max, 1)
		    while (max > 1){
		        res <- optimize(function(lam) (1 - tail - pchisq(chisq, df, ncp=lam))^2, interval=c(0, max))
		        if (sqrt(res$objective) < tail/100) break
		        max <- max/2
		        if (is.na(res$objective) || res$objective < 0){
		            max <- 0
		            warning("cannot find lower bound of RMSEA")
		            break
		        }				
		    }
		    lam.L <- if (max <= 1) NA else res$minimum
		    RMSEA.U <- sqrt(lam.U/((N - (!object$raw))*df))
		    RMSEA.L <- sqrt(lam.L/((N - (!object$raw))*df))
		}
	}
	else RMSEA.U <- RMSEA.L <- RMSEA <- NFI <- NNFI <- CFI <- AGFI <- RNI <- IFI <- NA
	if (!is.na(RMSEA)) RMSEA <- c(RMSEA, RMSEA.L, RMSEA.U, conf.level)
	if (!is.null(object$coeff)){
		var.names <- rownames(object$A)
		ram <- object$ram[object$par.posn, , drop=FALSE]
		par.code <- paste(var.names[ram[,2]], c('<---', '<-->')[ram[,1]],
				var.names[ram[,3]])
		coeff <- data.frame(object$coeff, se, z, 2*pnorm(abs(z), lower.tail=FALSE), par.code)
		names(coeff) <- c("Estimate", "Std Error", "z value", "Pr(>|z|)", " ")
		row.names(coeff) <- names(object$coeff)
	}
	else coeff <- NULL
	AIC <- if (!"AIC" %in% fit.indices) NULL else AIC(object)
	AICc <- if (!"AICc" %in% fit.indices) NULL else AICc(object)
	BIC <- if (!"BIC" %in% fit.indices) NULL else BIC(object)
	CAIC <- if (!"CAIC" %in% fit.indices) NULL else CAIC(object)
	SRMR <- if (!"SRMR" %in% fit.indices) NA else sqrt(sum(standardizedResiduals(object)^2 * 
							upper.tri(diag(n), diag=TRUE))/(n*(n + 1)/2))
	if (robust) { 
		chisq.adjusted <- object$adj.obj$chisq.scaled
		chisqNull.adjusted <- chisqNull/object$adj.obj$c 
		NFI.adjusted <- if (!"NFI" %in% fit.indices) NULL else (chisqNull.adjusted - chisq)/chisqNull.adjusted
		NNFI.adjusted <- if (!"NNFI" %in% fit.indices) NULL else (chisqNull.adjusted/dfNull - chisq.adjusted/df)/(chisqNull.adjusted/dfNull - 1)
		L1 <- max(chisq.adjusted - df, 0)
		L0 <- max(L1, chisqNull.adjusted - dfNull)
		CFI.adjusted <- if (!"CFI" %in% fit.indices) NULL else 1 - L1/L0
        RNI.adjusted <- if (!"RNI" %in% fit.indices) NULL else 1 - (chisq.adjusted - df)/(chisqNull.adjusted - dfNull)
        IFI.adjusted <- if (!"IFI" %in% fit.indices) NULL else (chisqNull.adjusted - chisq.adjusted)/(chisqNull.adjusted - df)
	}
	else{
		chisq.adjusted <- chisqNull.adjusted <- NFI.adjusted <- NNFI.adjusted <- CFI.adjusted <- RNI.adjusted <- IFI.adjusted <- NULL
	}
	ans <- list(chisq=chisq, df=df, chisqNull=chisqNull, dfNull=dfNull,
			GFI=GFI, AGFI=AGFI, RMSEA=RMSEA, NFI=NFI, NNFI=NNFI, CFI=CFI, RNI=RNI, IFI=IFI, BIC=BIC, SRMR=SRMR, 
			AIC=AIC, AICc=AICc, CAIC=CAIC, Rsq=Rsq,
			chisq.adjusted=chisq.adjusted, chisqNull.adjusted=chisqNull.adjusted, NFI.adjusted=NFI.adjusted,
			NNFI.adjusted=NNFI.adjusted, CFI.adjusted=CFI.adjusted, RNI.adjusted=RNI.adjusted, IFI.adjusted=IFI.adjusted,
			norm.res=norm.res, coeff=coeff, digits=digits, 
			iterations=object$iterations, aliased=object$aliased, raw=object$raw,
			robust=robust, robust.vcov=object$robust.vcov, adj.obj=object$adj.obj)
	class(ans) <- "summary.objectiveML"
	ans
}

print.summary.objectiveML <- function(x, digits=getOption("digits"), ...){
	old.digits <- options(digits=digits)
	on.exit(options(old.digits))
	if (x$raw) cat("\nModel fit to raw moment matrix.\n")	
	if (x$robust && !is.null(x$robust.vcov)){
		cat("\nSatorra-Bentler Corrected Fit Statistics:\n")
		cat("\n Corrected Model Chisquare = ", x$chisq.adjusted, "  Df = ", x$df, 
				"Pr(>Chisq) =", if (x$df > 0) pchisq(x$chisq.adjusted, x$df, lower.tail=FALSE)
						else NA)
		if (!x$raw) {		
			cat("\n Corrected Chisquare (null model) = ", x$chisqNull.adjusted,  "  Df = ", x$dfNull)
		}
		if (x$df > 0 && !x$raw){
			if (!is.null(x$NFI.adjusted)) cat("\n Corrected Bentler-Bonett NFI = ", x$NFI.adjusted)
			if (!is.null(x$NNFI.adjusted)) cat("\n Corrected Tucker-Lewis NNFI = ", x$NNFI.adjusted)
			if (!is.null(x$CFI.adjusted)) cat("\n Corrected Bentler CFI = ", x$CFI.adjusted)
            if (!is.null(x$RNI.adjusted)) cat("\n Corrected Bentler RNI = ", x$RNI.adjusted)
            if (!is.null(x$IFI.adjusted)) cat("\n Corrected Bollen IFI = ", x$IFI.adjusted)
		}
		cat("\n\nUncorrected Fit Statistics:\n")
		x$coeff[,2] <- sqrt(diag(x$robust.vcov))
		x$coeff[,3] <- x$coeff[,1]/x$coeff[,2]
		x$coeff[,4] <- 2*pnorm(abs(x$coeff[,3]), lower.tail=FALSE)
		colnames(x$coeff)[2] <- "Corrected SE"
	}
	if (!is.null(x$chisq)) cat("\n Model Chisquare = ", x$chisq, "  Df = ", x$df, 
				"Pr(>Chisq) =", if (x$df > 0) pchisq(x$chisq, x$df, lower.tail=FALSE)
						else NA)
	else if (!is.null(x$logLik)) cat("\n Model log-likelihood = ", x$logLik, "  Df = ", x$df, "\n")
	if (!x$raw) {		
		if ((!is.null(x$chisqNULL)) && (!is.na(x$chisqNULL))) cat("\n Chisquare (null model) = ", x$chisqNull,  "  Df = ", x$dfNull)
		if (!is.na(x$GFI)) cat("\n Goodness-of-fit index = ", x$GFI)
	}
	if (x$df > 0 && !x$raw){
		if (!is.na(x$AGFI)) cat("\n Adjusted goodness-of-fit index = ", x$AGFI)
		if (length(x$RMSEA) > 1 || !is.na(x$RMSEA)) cat("\n RMSEA index =  ", x$RMSEA[1],
					"   ", 100*x$RMSEA[4], "% CI: (", x$RMSEA[2], ", ", x$RMSEA[3],")", sep="")
		if (!is.na(x$NFI)) cat("\n Bentler-Bonett NFI = ", x$NFI)
		if (!is.na(x$NNFI)) cat("\n Tucker-Lewis NNFI = ", x$NNFI)
		if (!is.na(x$CFI)) cat("\n Bentler CFI = ", x$CFI)
        if (!is.na(x$RNI)) cat("\n Bentler RNI = ", x$RNI)
        if (!is.na(x$IFI)) cat("\n Bollen IFI = ", x$IFI)
		if (!is.na(x$SRMR)) cat("\n SRMR = ", x$SRMR)
	}
	if (!is.null(x$AIC) && !is.na(x$AIC)) cat("\n AIC = ", x$AIC)
	if (!is.null(x$AICc) && !is.na(x$AICc)) cat("\n AICc = ", x$AICc)
	if (!is.null(x$BIC) && !is.na(x$BIC)) cat("\n BIC = ", x$BIC)
	if (!is.null(x$CAIC) && !is.na(x$CAIC)) cat("\n CAIC = ", x$CAIC)
	if (length(x$norm.res) > 1 || !is.na(x$norm.res)){
		cat("\n\n Normalized Residuals\n")
		print(summary(as.vector(x$norm.res)))
	}
	if (!is.na(x$Rsq[1])){
		cat("\n R-square for Endogenous Variables\n")
		print(round(x$Rsq, 4))
	}
	if (!is.null(x$coeff)){
		if (x$robust && !is.null(x$robust.vcov)) cat("\n Parameter Estimates (with Robust Standard Errors)\n") else cat("\n Parameter Estimates\n")
		print(x$coeff, right=FALSE, digits=digits)
		if (!is.na(x$iterations)) cat("\n Iterations = ", x$iterations, "\n")
		if (!is.null(x$aliased)) cat("\n Aliased parameters:", x$aliased, "\n")
	}
	invisible(x)
}

summary.objectiveGLS <- function(object, digits=getOption("digits"), conf.level=.90, 
                                 fit.indices=c("GFI", "AGFI", "RMSEA", "NFI", "NNFI", "CFI", "RNI", "IFI", "SRMR"), ...){
    fit.indices <- if (missing(fit.indices)){
        getOption("fit.indices")
    }
    else if (fit.indices[1] != "") match.arg(fit.indices, several.ok=TRUE)
	summary <-  summary.objectiveML(object, digits=digits, conf.level=conf.level, analytic.se=FALSE, fit.indices=fit.indices, ...)
	S <- object$S
	Sinv <- solve(S)
	C <- object$C
	SinvSmC <- Sinv %*% (S - C)
	SinvS <- Sinv %*% S
	n <- object$n
	if ("GFI" %in% fit.indices) summary$GFI <- 1 - sum(diag(SinvSmC %*% SinvSmC))/sum(diag(SinvS %*% SinvS))
	if ("AGFI" %in% fit.indices) summary$AGFI <-  1 - (n*(n + 1)/(2*summary$df))*(1 - summary$GFI)
	summary
}

deviance.objectiveML <- function(object, ...) object$criterion * (object$N - (!object$raw))

df.residual.sem <- function(object, ...) {
	n.fix <- object$n.fix
	n <- object$n
	t <- object$t
	n*(n + 1)/2 - t - n.fix*(n.fix + 1)/2
}

Rsq <- function(model){
	A <- model$A
	P <- model$P
	IAinv <- solve(diag(nrow(A)) - A)
	C <- IAinv %*% P %*% t(IAinv)
	R2 <- 1 - diag(P)/diag(C)
	R2 <- R2[classifyVariables(model$semmod)$endogenous]
	R2
}

# Two-Stage Least Squares
#   John Fox

# last modified 2019-11-15 by J. Fox

tsls <- function (y, ...) {
	UseMethod("tsls")
}

tsls.default <- function (y, X, Z, w, names = NULL, ...) {
	if (is.null(w)) w <- 1
	if (any(w < 0 | is.na(w))) 
		stop("missing or negative weights not allowed")
	n <- length(y)
	p <- ncol(X)
	sqrt.w <- sqrt(w)
	invZtZ <- solve(crossprod(Z*sqrt.w))
	XtZ <- crossprod(X*w, Z)
	V <- chol2inv(chol(XtZ %*% invZtZ %*% t(XtZ)))
	b <- V %*% XtZ %*% invZtZ %*% crossprod(Z*w, y)
	residuals <- (y - X %*% b)*sqrt.w
	s2 <- sum(residuals^2)/(n - p)
	V <- s2 * V
	result <- list()
	result$n <- n
	result$p <- p
	b <- as.vector(b)
	names(b) <- names
	result$coefficients <- b
	rownames(V) <- colnames(V) <- names
	result$V <- V
	result$s <- sqrt(s2)
	result$residuals <- as.vector(residuals)
	result$response <- y
	result$model.matrix <- X
	result$instruments <- Z
	result$weights <- w
	result
}

tsls.formula <- function (formula, instruments, data, subset, weights, na.action, contrasts = NULL, ...) {
	if (missing(na.action)) 
		na.action <- options()$na.action
	m <- match.call(expand.dots = FALSE)
	if (is.matrix(eval(m$data, sys.frame(sys.parent())))) 
		m$data <- as.data.frame(data)
	response.name <- deparse(formula[[2]])
	form <- as.formula(paste(paste(response.name, collapse = ""), 
					"~", paste(deparse(formula[[3]]), collapse = ""), "+", 
					paste(deparse(instruments[[2]]), collapse = "")))
	m$formula <- form
	m$instruments <- m$contrasts <- NULL
	m[[1]] <- as.name("model.frame")
	mf <- eval(m, sys.frame(sys.parent()))
	na.act <- attr(mf, "na.action")
	w <- as.vector(model.weights(mf))
	wt.var <- if(!is.null(w)) deparse(substitute(weights)) else NULL
	Z <- model.matrix(instruments, data = mf, contrasts)
	y <- mf[, response.name]
	X <- model.matrix(formula, data = mf, contrasts)
	result <- tsls(y, X, Z, w, colnames(X))
	result$response.name <- response.name
	result$formula <- formula
	result$instruments <- instruments
	result$wt.var <- wt.var
	if (!is.null(na.act)) 
		result$na.action <- na.act
	class(result) <- "tsls"
	result
}

print.tsls <- function (x, ...) {
	cat("\nModel Formula: ")
	print(x$formula)
	cat("\nInstruments: ")
	print(x$instruments)
	if (!is.null(x$wt.var)){
		cat("\nWeights: ", x$wt.var, "\n")
	}
	cat("\nCoefficients:\n")
	print(x$coefficients)
	cat("\n")
	invisible(x)
}

summary.tsls <- function (object, digits=getOption("digits"), ...) {
    save.digits <- options(digits = digits)
    on.exit(options(save.digits))
    df <- object$n - object$p
    std.errors <- sqrt(diag(object$V))
    b <- object$coefficients
    t <- b/std.errors
    p <- 2 * (1 - pt(abs(t), df))
    table <- cbind(b, std.errors, t, p)
    rownames(table) <- names(b)
    colnames(table) <- c("Estimate", "Std. Error", "t value", 
        "Pr(>|t|)")
    
    result <- list(formula=object$formula, instruments=object$instruments, wt.var=object$wt.var,
        residuals=summary(residuals(object)), coefficients=table, digits=digits, s=object$s, df=df)
    class(result) <- "summary.tsls"
    result
}

print.summary.tsls <- function(x, ...){
    cat("\n 2SLS Estimates\n")
    cat("\nModel Formula: ")
    print(x$formula)
    cat("\nInstruments: ")
    print(x$instruments)
    if (!is.null(x$wt.var)){
        cat("\nWeights: ", x$wt.var, "\n")
    }
    cat("\nResiduals:\n")
    print(x$residuals)
    cat("\n")
    printCoefmat(x$coefficients, digits=x$digits)
    cat(paste("\nResidual standard error:", round(x$s, x$digits), 
        "on", x$df, "degrees of freedom\n\n"))
    invisible(x)
}

residuals.tsls <- function(object, ...){
    res <- object$residuals
    if (is.null(object$na.action)) 
        res
    else naresid(object$na.action, res)
    }

coef.tsls <- function(object, ...){
    object$coefficients
    }
    
fitted.tsls <- function(object, ...){
    yhat <- as.vector(object$model.matrix %*% object$coefficients)
    if (is.null(object$na.action)) 
        yhat
    else napredict(object$na.action, yhat)
    }
    
vcov.tsls <- function(object, ...) object$V
    
anova.tsls <- function(object, model.2, s2, dfe, ...){
    if(!inherits(model.2, "tsls")) stop('requires two models of class tsls')
    s2.1 <- object$s^2
    n.1 <- object$n 
    p.1 <- object$p
    dfe.1 <- n.1 - p.1
    s2.2 <- model.2$s^2
    n.2 <- model.2$n
    p.2 <- model.2$p
    dfe.2 <- n.2 - p.2
    SS.1 <- s2.1 * dfe.1
    SS.2 <- s2.2 * dfe.2
    SS <- abs(SS.1 - SS.2)
    Df <- abs(dfe.2 - dfe.1)
    if (missing(s2)){
        s2 <- if (dfe.1 > dfe.2) s2.2 else s2.1
        f <- (SS/Df) / s2
        RSS <- c(SS.1, SS.2)
        Res.Df <- c(dfe.1, dfe.2)
        SS <- c(NA, SS)
        P <- c(NA, 1 - pf(f, Df, min(dfe.1, dfe.2)))
        Df <- c(NA, Df)
        f <- c(NA, f)
        rows <- c("Model 1", "Model 2")
        }
    else{
        f <- (SS/Df) / s2
        RSS <- c(SS.1, SS.2, s2*dfe)
        Res.Df <- c(dfe.1, dfe.2, dfe)
        SS <- c(NA, SS, NA)
        P <- c(NA, 1 - pf(f, Df, dfe), NA)
        Df <- c(NA, Df, NA)
        f <- c(NA, f, NA)
        rows <- c("Model 1", "Model 2", "Error")
        }
    table <- data.frame(Res.Df, RSS, Df, SS, f, P)
    head.1 <- paste("Model 1: ",format(object$formula), "  Instruments:", 
        format(object$instruments))
    head.2 <- paste("Model 2: ",format(model.2$formula), "  Instruments:", 
        format(model.2$instruments))
    names(table) <- c("Res.Df", "RSS", "Df", "Sum of Sq", "F", "Pr(>F)")
    row.names(table) <- rows
    structure(table, heading = c("Analysis of Variance", "", head.1, head.2, ""), 
        class = c("anova", "data.frame"))
    }

# last modified 2011-08-04 by J. Fox

print.objectiveML <- function(x, ...) {
    n <- x$n
    t <- x$t
    n.fix <- x$n.fix
    df <- n*(n + 1)/2 - t - n.fix*(n.fix + 1)/2
    cat("\n Model Chisquare = ", x$criterion * (x$N - (!x$raw)), 
        "  Df = ", df, "\n\n")
    if (!is.null(x$coef)){
        print(x$coeff)
        if (!is.na(x$iterations)) cat("\n Iterations = ", x$iterations, "\n")
        if (!is.null(x$aliased)) cat("\n Aliased parameters:", x$aliased, "\n")
        }
    invisible(x)
    }

print.objectiveGLS <- function(x, ...) print.objectiveML(x, ...)

# last modified 2011-07-30

optimizerNlminb <- function(start, objective=objectiveML, 
		gradient=TRUE, maxiter, debug, par.size, model.description, warn, ...){
	with(model.description, {
				obj <- objective(gradient=gradient)
				objective <- obj$objective
				grad <- if (gradient) obj$gradient else NULL
				if (!warn) save.warn <- options(warn=-1)
				res <- nlminb(start, objective, gradient=grad, model.description=model.description, 
						control=list(trace=if(debug) 1 else 0, iter.max=maxiter, ...))
				if (!warn) options(save.warn)
				result <- list()
				result$convergence <- res$convergence == 0
				result$iterations <- res$iterations
				par <- res$par
				names(par) <- param.names
				result$par <- par
				if (!result$convergence)
					warning(paste('Optimization may not have converged; nlminb return code = ',
									res$convergence, '. Consult ?nlminb.\n', sep=""))
				result$criterion <- res$objective
				obj <- objective(par, model.description)
				C <- attr(obj, "C")
				rownames(C) <- colnames(C) <- var.names[observed]
				result$C <- C
				A <- attr(obj, "A")
				rownames(A) <- colnames(A) <- var.names
				result$A <- A
				P <- attr(obj, "P")
				rownames(P) <- colnames(P) <- var.names
				result$P <- P
				class(result) <- "semResult"
				result
			}
	)
}

# modified 24 May 2009 by Adam Kramer (original by J. Fox)
# last modified 2011-08-10 by J. Fox

standardized.coefficients <- function(...){
	.Deprecated("standardizedCoefficients", package="sem")
	standardizedCoefficients(...)
}

standardizedCoefficients <- function(object, ...){
	UseMethod("standardizedCoefficients")
}

standardizedCoefficients.sem <- function (object, digits = getOption("digits"), oneheaded = TRUE, twoheaded = TRUE, ...) {
	if (!oneheaded && !twoheaded) {
		stop("No coefficients requested.")
	}
	old.digits <- options(digits = digits)
	on.exit(options(old.digits))
	P <- object$P
	A <- object$A
	t <- object$t
	par <- object$coeff
	par.posn <- object$par.posn
	IAinv <- solve(diag(nrow(A)) - A)
	C <- IAinv %*% P %*% t(IAinv)
	ram <- object$ram
	par.names <- rep(" ", nrow(ram))
	for (i in 1:t) {
		which.par <- ram[, 4] == i
		ram[which.par, 5] <- par[i]
		par.names[which.par] <- names(par)[i]
	}
	coeff <- ram[, 5]
	if (oneheaded) {
		one.head <- ram[, 1] == 1
		coeff[one.head] <- coeff[one.head] * sqrt(diag(C[ram[one.head, 
												3], ram[one.head, 3], drop = FALSE])/diag(C[ram[one.head, 
												2], ram[one.head, 2], drop = FALSE]))
	}
	if (twoheaded) {
		two.head <- ram[, 1] == 2
		coeff[two.head] <- coeff[two.head]/sqrt(diag(C[ram[two.head, 
												3], ram[two.head, 3], drop = FALSE]) * diag(C[ram[two.head, 
												2], ram[two.head, 2], drop = FALSE]))
	}
	var.names <- rownames(A)
	par.code <- paste(var.names[ram[, 2]], c("<---", "<-->")[ram[, 
							1]], var.names[ram[, 3]])
	coeff <- data.frame(par.names, coeff, par.code)
	colnames(coeff) <- c(" ", "Std. Estimate", " ")
	if (oneheaded && twoheaded) {
		coeff
	}
	else if (oneheaded) {
		coeff[one.head, ]
	}
	else {
		coeff[two.head, ]
	}
}

std.coef <- function(...){
	.Deprecated("stdCoef", package="sem")
	standardizedCoefficients(...)
}

stdCoef <- function (...){
    standardizedCoefficients(...)
    }

chisqNull <- function(object){
	UseMethod("chisqNull")
}

chisqNull.objectiveML <- function(object){
	chisq <- if (!object$raw) {
			S <- object$S
			CC <- diag(diag(S))
			(object$N - 1) * 
				(sum(diag(S %*% solve(CC))) + log(det(CC)) - log(det(S)) - object$n)
		}
		else NULL
	chisq
}

chisqNull.objectiveGLS <- function(object){
	chisq <- if (!object$raw) {
			S <- object$S
			CC <- diag(diag(S))
			SS <- solve(S) %*% (S - CC)
			(object$N - 1)*0.5*sum(diag(SS %*% SS))
		}
		else NULL
	chisq
}

# last modified 2015-06-09 by J. Fox

mod.indices <- function(...){
	.Deprecated("modIndices", package="sem")
	modIndices(...)
}

modIndices <- function(model, ...){
    UseMethod("modIndices")
    }

# incorporates corrections by Michael Culbertson:

modIndices.objectiveML <- function(model, duplicated, deviance=NULL, ...){
	Duplicated <- function(x){
		X <- outer(x, x, FUN="==")
		diag(X) <- NA
		apply(X, 2, any, na.rm=TRUE)
	}
	if (is.null(deviance)) deviance <- deviance(model)
	A <- model$A
	P <- model$P
	S <- model$S
	C <- model$C
	J <- model$J
	m <- model$m
	t <- model$t
	NM <- model$N - (!model$raw)
	vars <- model$var.names    
	I.Ainv <- solve(diag(m) - A) 
	Cinv <- solve(C)    
	AA <- t(I.Ainv) %*% t(J)
	BB <- J %*% I.Ainv %*% P %*% t(I.Ainv)
	correct <- matrix(2, m, m)
	diag(correct) <- 1
	grad.P <- correct * AA %*% Cinv %*% (C - S) %*% Cinv %*% t(AA)
	grad.A <- correct * AA %*% Cinv %*% (C - S) %*% Cinv %*% BB 
	grad <- c(grad.A, grad.P) * NM
	dF.dBdB <- accumulate(AA %*% Cinv %*% t(AA), t(BB) %*% Cinv %*% BB,
			AA %*% Cinv %*% BB, t(BB) %*% Cinv %*% t(AA), m)                
	dF.dPdP <- accumulate(AA %*% Cinv %*% t(AA), AA %*% Cinv %*% t(AA),
			AA %*% Cinv %*% t(AA), AA %*% Cinv %*% t(AA), m)                
	dF.dBdP <- accumulate(AA %*% Cinv %*% t(AA), t(BB) %*% Cinv %*% t(AA),
			AA %*% Cinv %*% t(AA), t(BB) %*% Cinv %*% t(AA), m)    
	correct.BB <- correct.PP <- correct.BP <- matrix(1, m^2, m^2)
	d0 <- as.vector(diag(m) ==0 )
	d1 <- as.vector(diag(m) == 1)
	correct.BB[d0, d0] <- 2
	correct.PP[d1, d1] <- 0.5
	correct.PP[d0, d0] <- 2
	correct.BP[d0, d0] <- 2
	Hessian <- NM*rbind(cbind(dF.dBdB * correct.BB,    dF.dBdP * correct.BP),
			cbind(t(dF.dBdP * correct.BP), dF.dPdP * correct.PP))
	ram <- model$ram   
	fixed <- ram[,4] == 0
	one.head <- ram[,1] == 1
	one.free <- which( (!fixed) & one.head )
	two.free <- which( (!fixed) & (!one.head) )
	arrows.1.free <- ram[one.free,c(2,3)]
	arrows.2.free <- ram[two.free,c(2,3)]
	duplicated <- if (missing(duplicated)) (Duplicated(ram[, 4]) & (!fixed)) else (duplicated & (!fixed))
	one.free.duplicated <- which( (!fixed) & one.head & duplicated)
	two.free.duplicated <- which( (!fixed) & (!one.head) & duplicated)
	arrows.1.free.duplicated <- ram[one.free.duplicated, c(2,3)]
	arrows.2.free.duplicated <- ram[two.free.duplicated, c(2,3)]
	posn.matrix <- matrix(1:(m^2), m, m)
	posn.free <- c(posn.matrix[arrows.1.free], 
			(m^2) + posn.matrix[arrows.2.free]) 
	posn.free.duplicated <- c(posn.matrix[arrows.1.free.duplicated], 
			(m^2) + posn.matrix[arrows.2.free.duplicated])
	hessian <- Hessian[posn.free, posn.free]
	par.no <- ram[ram[, 4] > 0, 4]
	pars <- ram[, 4][!fixed]
	Z <- outer(1:t, pars, function(x, y) as.numeric(x == y))
	hessian <- Z %*% hessian %*% t(Z)
	E.inv <- solve(hessian)                      
	par.change <- mod.indices <- rep(0, 2*(m^2))       
	n.posn.free.1 <- length(posn.free) + 1
	for (i in 1:(2*(m^2))) {
		if ((! i %in% posn.free.duplicated) && (i %in% posn.free || qr(as.matrix(Hessian[c(posn.free, i), c(posn.free, i)]))$rank < n.posn.free.1)){  
			par.change[i] <- mod.indices[i] <- NA  
		} 
		else {
			k <- Hessian[i, i]
			d <- Hessian[i, posn.free]
			d <- sapply(1:t, function(i) sum(d[which(par.no==i)]))
			par.change[i] <- -grad[i]/ (k - d %*% E.inv %*% d)
			mod.indices[i] <- -0.5 * grad[i] * par.change[i]
		}
	}
	par.change[mod.indices > 1.1*deviance] <- NA
	mod.indices[mod.indices > 1.1*deviance] <- NA
	mod.A <- matrix(mod.indices[1:(m^2)], m, m)
	mod.P <- matrix(mod.indices[-(1:(m^2))], m, m)
	par.A <- matrix(par.change[1:(m^2)], m, m)
	par.P <- matrix(par.change[-(1:(m^2))], m, m)
	diag(mod.A) <- diag(par.A) <- NA
	rownames(mod.A) <- colnames(mod.A) <- vars
	rownames(mod.P) <- colnames(mod.P) <- vars
	rownames(par.A) <- colnames(par.A) <- vars
	rownames(par.P) <- colnames(par.P) <- vars
	result <- list(mod.A=mod.A, mod.P=mod.P, par.A=par.A, par.P=par.P)
	class(result) <- "modIndices"
	result
}

summary.modIndices <- function(object, round=2, 
    print.matrices=c('both', 'par.change', 'mod.indices'), ...) {
    print.matrices <- match.arg(print.matrices)
    if (print.matrices != "mod.indices"){ 
        cat("\n Parameter change: A matrix (regression coefficients, row <- column)\n")
        print(object$par.A)
        }
    if (print.matrices != "par.change"){ 
        cat("\n Modification indices: A matrix (regression coefficients, row <- column)\n")
        print(round(object$mod.A, round))
        }
    if (print.matrices != "mod.indices"){ 
        cat("\n Parameter change: P matrix (variances/covariances)\n")
        print(object$par.P)
        }
    if (print.matrices != "par.change"){ 
        cat("\n Modification indices: P matrix (variances/covariances)\n")
        print(round(object$mod.P, round))
        }
    invisible(NULL)
    }

print.modIndices <- function(x, n.largest=5, ...){
    cat("\n", n.largest, "largest modification indices, A matrix (regression coefficients):\n")
    mod.A <- as.vector(x$mod.A)
    names <- rownames(x$mod.A)
    names(mod.A) <- outer(names, names, paste, sep="<-")
    print(rev(sort(mod.A))[1:n.largest])
    cat("\n ", n.largest, "largest modification indices, P matrix (variances/covariances):\n")
    mod.P <- as.vector(x$mod.P)
    names(mod.P) <- outer(names, names, paste, sep="<->")
    print(rev(sort(mod.P[lower.tri(x$mod.P, diag=TRUE)]))[1:n.largest])
    }

modIndices.msemObjectiveML <- function(model, ...){
	deviance <- deviance(model)
	G <- length(model$groups)
	t <- model$t  
	N <- model$N
	raw <- model$raw
	wts <- (N - !raw)/(sum(N) - !raw*G)
	hessian <- matrix(0, t, t)
	rownames(hessian) <- colnames(hessian) <- model$param.names
	result <- vector(G, mode="list")
	names(result) <- paste(model$group, model$groups, sep=": ")
	all.pars <- unlist(lapply(model$ram, function(ram) ram[, 4]))
	unique.pars <- unique(all.pars)
	pars.count <- table(all.pars)
	duplicated.pars <- as.numeric(names(pars.count)[pars.count > 1])
	for (g in 1:G){
		ram <- model$ram[[g]]
		parameters <- ram[, 4]
		duplicated <- parameters %in% duplicated.pars
		unique.pars <- unique(parameters[parameters != 0])
		par.posn <- sapply(unique.pars, function(x) which(x == parameters)[1])
		unique.posn <- which(parameters %in% unique.pars)
		rownames(ram)[unique.posn] <- unique(model$param.names[ram[, 4]])
		ram[unique.posn, 4] <- unlist(apply(outer(unique.pars, parameters, "=="), 2, which))
		mod.g <- list(var.names=model$var.names[[g]], ram=ram,J=model$J[[g]], n.fix=model$n.fix, n=model$n[[g]], 
				N=model$N[g], m=model$m[[g]], t=length(unique.pars),
				coeff=model$coeff[parameters],  criterion=model$group.criteria[g],  S=model$S[[g]], raw=model$raw,
				C=model$C[[g]], A=model$A[[g]], P=model$P[[g]])
		class(mod.g) <- c("objectiveML", "sem")
		result[[g]] <- modIndices(mod.g, duplicated=duplicated, deviance=deviance, ...)
	}
	class(result) <- "msemModIndices"
	result
}

print.msemModIndices <- function(x, ...){
	G <- length(x)
	groups <- names(x)
	for (g in 1:G) {
		cat("\n\n", groups[g], "\n")
		print(x[[g]], ...)
	}
}

summary.msemModIndices <- function(object, ...){
	G <- length(object)
	groups <- names(object)
	for (g in 1:G) {
		cat("\n\n", groups[g], "\n")
		print(summary(object[[g]], ...))
	}
}

# last modified 2014-10-27 by J. Fox

read.moments <- function(...){
	.Deprecated("readMoments", package="sem")
	readMoments(...)
}

readMoments <- function(file="", text, diag=TRUE, 
        names=as.character(paste("X", 1:n, sep=""))){
    elements <- if (!missing(text)) scan(text=text) else scan(file=file)
    m <- length(elements)
    d <- if (diag) 1 else -1
    n <- floor((sqrt(1 + 8*m) - d)/2)
    if (m != n*(n + d)/2) 
        stop("wrong number of elements (cannot make square matrix)")
    if (length(names) != n) stop("wrong number of variable names")
    X <- diag(n)
    X[upper.tri(X, diag=diag)] <- elements
    rownames(X) <- colnames(X) <- names
    t(X)
    }
    

# File:   csem.R
# Author: Zhenghua Nie 
# Date:   Mon 26 Dec 2011 23:54:22 EST
#
#
#
# Copyright (C) 2011 Zhenghua Nie. All Rights Reserved.
# This code is published under GNU GENERAL PUBLIC LICENSE.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License,  or
# (at your option) any later version.
#      
# This program is distributed WITHOUT ANY WARRANTY. See the
# GNU General Public License for more details.
#           
# If you do not have a copy of the GNU General Public License,  
# write to the Free Software Foundation, Inc., 
# 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


# The following function is a wrapper to compute the objective function and its gradient.
# If hessian=TRUE,  csem will return Hessian computed by the numerical method,  but
# is flexible to return Hessian computed by the analytical solution.
CompiledObjective <- function(par, model.description, gradient=TRUE, hessian=FALSE, objective=c("objectiveML", "objectiveGLS", "objectiveFIML", "objectivelogLik"), ...)
{
		if(missing(objective)) objective <- "objectiveML"
		objective <- match.arg(objective)

		res <- csem(model=model.description,  start=par, objective=objective,  opt.flag=0,  gradient=gradient,  opts=list("hessian"=hessian, "check.analyticals"=FALSE), ...)
		ret <- list();
		ret$f <- res$minimum
		ret$parameters <- res$estimate
		ret$C <- res$C
		ret$A <- res$A
		ret$P <- res$P
		ret$gradient <- res$gradient
		ret$hessian <- res$hessian

		return(ret)
}

msemCompiledObjective <- function(par, model.description, gradient=TRUE, hessian=FALSE, objective=c("objectiveML", "objectiveGLS", "objectiveFIML"), ...)
{
		if(missing(objective)) objective <- "objectiveML"
		objective <- match.arg(objective)

		res <- cmsem(model=model.description,  start=par, objective=objective,  opt.flag=0, gradient=gradient,   opts=list("hessian"=hessian, "check.analyticals"=FALSE), ...)
		AA <- PP <- CC <- vector(model.description$G,  mode="list")
		indAP <- 1
		indC <- 1
		ff <- numeric(model.description$G)
		for(g in 1:model.description$G)
		{
				m <- model.description$m[g]
				n <- model.description$n[g]
				AA[[g]] <- matrix(res$A[indAP:(indAP+m*m-1)], m, m);
				PP[[g]] <- matrix(res$P[indAP:(indAP+m*m-1)], m, m);
				indAP <- indAP + m*m;
				CC[[g]] <- matrix(res$C[indC:(indC+n*n-1)], n, n);
				indC <- indC + n*n;
				ff[g] <- as.numeric(res$f[g])
		}
		ret <- list();
		ret$f <- res$minimum
		ret$parameters <- res$estimate
		ret$C <- CC
		ret$A <- AA
		ret$P <- PP
		ret$ff <- ff
		ret$gradient <- res$gradient
		ret$hessian <- res$hessian

		return(ret)
}

# The wrapper function for solving optimization problems. Please note that the objective function is written in C/C++,  we need to know the name.
CompiledSolve <- function(model.description, start, objective=c("objectiveML", "objectiveGLS", "objectiveFIML", "objectivelogLik"),  gradient=TRUE, typsize=rep(1.0, length(start)), debug=FALSE, maxiter=100,...)
{
		if(missing(objective)) objective <- "objectiveML"
		objective <- match.arg(objective)

		stepmax=max(1000.0 * sqrt(sum((start/typsize)^2)),  1000.0)

		res <- csem(model=model.description, start, opt.flag=1, typsize=typsize,objective=objective,  
								gradient=gradient, 
								opts=list("iterlim"=maxiter, "print.level"=if(debug) 2 else 0,
													"hessian"=TRUE, "check.analyticals"=FALSE, "stepmax"=stepmax), ...)

		return(res)
}

# The wrapper function for solving optimization problems. Please note that the objective function is written in C/C++,  we need to know the name.
msemCompiledSolve <- function(model.description, start, objective=c("objectiveML", "objectiveGLS", "objectiveFIML"),  
															gradient=TRUE, 
															typsize=rep(1.0, length(start)), debug=FALSE, maxiter=100,gradtol=1e-6, ...)
{
		if(missing(objective)) objective <- "objectiveML"
		objective <- match.arg(objective)

		stepmax=max(1000.0 * sqrt(sum((start/typsize)^2)),  1000.0)

		res <- cmsem(model=model.description, start, opt.flag=1, typsize=typsize,objective=objective,  
								 gradient=gradient, 
								opts=list("iterlim"=maxiter, "print.level"=if(debug) 2 else 0,"gradtol"=gradtol, 
													"hessian"=TRUE, "check.analyticals"=FALSE, "stepmax"=stepmax), ...)

#reoraginize the matrix A,  P,  C 
		AA <- PP <- CC <- vector(model.description$G,  mode="list")
		indAP <- 1
		indC <- 1
		ff <- numeric(model.description$G)
		for(g in 1:model.description$G)
		{
				m <- as.integer(model.description$m[g])
				n <- as.integer(model.description$n[g])
				AA[[g]] <- matrix(as.numeric(res$A)[indAP:(indAP+m*m-1)], m, m);
				PP[[g]] <- matrix(as.numeric(res$P)[indAP:(indAP+m*m-1)], m, m);
				indAP <- indAP + m*m;
				CC[[g]] <- matrix(as.numeric(res$C)[indC:(indC+n*n-1)], n, n);
				indC <- indC + n*n;
				ff[g] <- as.numeric(res$f[g])
		}

		ret <- list();
		ret$minimum <- res$minimum
		ret$estimate <- res$estimate
		ret$gradient <- res$gradient
		ret$hessian <- res$hessian
		ret$code <- res$code
		ret$iterations <- res$iterations
		ret$C <- CC
		ret$A <- AA
		ret$P <- PP
		ret$ff <- ff

		return(ret)
}

print.f <- function(input)
{
		print(input);   # call R function "print" 
}

#optimze:0 we only compute the objective function,  gradients or hessian and return them.
# 
csem <- function(model=NULL, start=NULL,opt.flag=1,  typsize=rep(1, model$t), objective=c("objectiveML", "objectiveGLS", "objectiveFIML", "objectivelogLik", "test_objective"),  
								 gradient=TRUE, 
								 opts=list("hessian"=1, "fscale"=1, "gradtol"=1e-6, "steptol"=1e-6, "stepmax"=max(1000 * sqrt(sum((start/typsize)^2)),  1000), "iterlim"=100, 
													 "ndigit"=12,"print.level"=0, "check.analyticals"=1), 
								 csem.environment = new.env(), ...){

		environment(print.f) <- csem.environment; 
 ## Write wrappers around user-defined functions to pass additional
  ## arguments
  print.f.wrapper <- function(x){ print.f(x,...) }

		if(missing(model)) stop("Must provide the model.")
		if(missing(objective)) objective <- "objectiveML"
		objective <- match.arg(objective)
		if(missing(typsize) || is.null(typsize)) typsize <- rep(1, model$t)
		if(missing(start)) start <- rep(0.10, model$t)
		if(length(opts$print.level)==0) 
				print.level <- 0
		else 
				print.level <- as.integer(opts$print.level)
		if(print.level < 0 || print.level > 2) stop("'print.level' must be in {0, 1, 2}")


		## the following is for generating gradient.
		if(objective != "objectivelogLik")
		{
				arrows.1.seq <- model$ram[model$ram[, 1]==1 & model$ram[, 4]!=0,  4] 
				arrows.2.seq <- model$ram[model$ram[, 1]==2 & model$ram[, 4]!=0,  4]
		}

		# this function is modfied from ipoptr developed by Jelmer Ypma (http://www.ucl.ac.uk/~uctpjyy/ipoptr.html).
		# Please reference the license of ipoptr.
		get.option.types <- function(opts) {
				# define types of nlm options,  we should add all options here.
				nlm.option.types <- list(
																 "fscale"="numeric", 
																 "gradtol"="numeric", 
																 "steptol"="numeric", 
																 "stepmax"="numeric", 
																 "hessian"="integer", 
																 "iterlim"="integer", 
																 "ndigit"="integer", 
																 "print.level"="integer", 
																 "check.analyticals"="integer"
																 )


				# initialize list with options sorted by type
				converted.opts <- list( "integer"=list(), "string"=list(), "numeric"=list() )

				is.wholenumber <- function(x, tol = .Machine$double.eps^0.5)  abs(x - round(x)) < tol

				# check if we have at least 1 element in the list, otherwise the 
				# loop runs from 1 to down 0 and we get errors
				if ( length( opts ) > 0 ) {

						# loop over all options and give them the correct type
						for ( i in 1:length( opts ) ) {
								tmp.type <- nlm.option.types[[match( names(opts)[i], names(nlm.option.types) )]]
								if ( is.null( tmp.type ) ) {
										# determine type
										if ( is.character(opts[[i]]) ) {
												tmp.type <- "string"
										} else if ( is.wholenumber(opts[[i]]) ) {
												tmp.type <- "integer"
										} else {
												tmp.type <- "numeric"
										}
										cat( paste( "Warning: ", names(opts)[i], " is not a recognized option, we try to pass it to nlm as ", tmp.type, "\n" ) )
								}

								if ( tmp.type=="string" ) {
										converted.opts$string[[ names(opts)[i] ]] <- as.character(opts[[i]])
								} else if ( tmp.type=="integer" ) {
										converted.opts$integer[[ names(opts)[i] ]] <- as.integer(opts[[i]])
								} else if ( tmp.type=="numeric" ) {
										converted.opts$numeric[[ names(opts)[i] ]] <- as.numeric(opts[[i]])
								} else {
										stop(paste("Type of option ", names(opts)[i], " not recognized"))
								}
						}
				}

				return ( converted.opts )
		}

		if(objective != "objectivelogLik")
		{
				ret <- list( 
										"objective" = objective, 
										"gradient" = as.integer(gradient), 
										"opt.flg" = as.integer(opt.flag), 
										"start" = start, 
										"options" = get.option.types(opts), 
										"data" = model$data, 
										"pattern.number" = model$pattern.number, 
										"valid.data.patterns" = model$valid.data.patterns, 
										"S" = model$S, 
										"logdetS" = as.numeric(model$logdetS), 
										"invS" = model$invS, 
										"N" = as.integer(model$N), 
										"m" = as.integer(model$m), 
										"n" = as.integer(model$n), 
										"t" = as.integer(model$t), 
										"fixed" = model$fixed, 
										"ram" = model$ram, 
										"sel.free" = model$sel.free, 
										"arrows.1" = model$arrows.1, 
										"arrows.1.free" = model$arrows.1.free, 
										"one.head" = model$one.head, 
										"arrows.2t" = model$arrows.2t, 
										"arrows.2" = model$arrows.2, 
										"arrows.2.free" = model$arrows.2.free, 
										"unique.free.1" = model$unique.free.1, 
										"unique.free.2" = model$unique.free.2, 
										"J" = model$J, 
										"correct" = model$correct, 
										"param.names" = model$param.names, 
										"var.names" = model$var.names, 
										"one.free" = model$one.free, 
										"two.free" = model$two.free, 
										"raw" = as.integer(model$raw), 
										"arrows.1.seq" = arrows.1.seq, 
										"arrows.2.seq" = arrows.2.seq, 
										"typsize" = typsize, 
										"print.f" = print.f.wrapper, 
										"csem.environment"=csem.environment)
				attr(ret, "class") <- "csem"
		}
		else
		{
				ret <- list(
										"objective" = objective, 
										"gradient" = as.integer(gradient), 
										"opt.flg" = as.integer(opt.flag), 
										"start" = start, 
										"t" = length(start), 
										"options" = get.option.types(opts), 
										"data" = model$data, 
										"pattern.number" = model$pattern.number, 
										"valid.data.patterns" = model$valid.data.patterns, 
										"tri" = model$tri, 
										"posn.intercept" = model$posn.intercept, 
										"typsize" = typsize, 
										"print.f" = print.f.wrapper, 
										"csem.environment"=csem.environment
										)
		}
		# add the current call to the list
		# ret$call <- match.call()

		solution <- .Call("csemSolve", ret)

		# ret$environment <- NULL
		# ret$solution <- solution

		ret <- solution   #this is for simplifing the interface.
		# add solution variables to object
		#ret$status <- solution$status

		return(ret)
}

cmsem <- function(model=NULL, start=NULL,opt.flag=1,  typsize=rep(1, model$t), objective=c("objectiveML", "objectiveGLS", "objectiveFIML", "test_objective"),  
									gradient=TRUE, 
									opts=list("hessian"=1, "fscale"=1, "gradtol"=1e-6, "steptol"=1e-6, "stepmax"=max(1000 * sqrt(sum((start/typsize)^2)),  1000), "iterlim"=100, 
														"ndigit"=12,"print.level"=0, "check.analyticals"=1), 
									csem.environment = new.env(), ...){

		environment(print.f) <- csem.environment; 
 ## Write wrappers around user-defined functions to pass additional
  ## arguments
  print.f.wrapper <- function(x){ print.f(x,...) }

		if(missing(model)) stop("Must provide the model.")
		if(missing(objective)) objective <- "objectiveML"
		objective <- match.arg(objective)
		if(missing(typsize) || is.null(typsize)) typsize <- rep(1, model$t)
		if(missing(start)) start <- rep(0.10, model$t)
		if(length(opts$print.level)==0) 
				print.level <- 0
		else 
				print.level <- as.integer(opts$print.level)
		if(print.level < 0 || print.level > 2) stop("'print.level' must be in {0, 1, 2}")


		## the following is for generating gradient.
		G <- model$G
		arrows.1.seq <- arrows.2.seq <- vector(G, mode="list")
		for(g in 1:G)
		{
				arrows.1.seq[[g]] <- model$ram[[g]][model$ram[[g]][, 1]==1 & model$ram[[g]][, 4]!=0,  4] 
				arrows.2.seq[[g]] <- model$ram[[g]][model$ram[[g]][, 1]==2 & model$ram[[g]][, 4]!=0,  4]
		}

		get.option.types <- function(opts) {
				# define types of nlm options,  we should add all options here.
				nlm.option.types <- list(
																 "fscale"="numeric", 
																 "gradtol"="numeric", 
																 "steptol"="numeric", 
																 "stepmax"="numeric", 
																 "hessian"="integer", 
																 "iterlim"="integer", 
																 "ndigit"="integer", 
																 "print.level"="integer", 
																 "check.analyticals"="integer"
																 )


				# initialize list with options sorted by type
				converted.opts <- list( "integer"=list(), "string"=list(), "numeric"=list() )

				is.wholenumber <- function(x, tol = .Machine$double.eps^0.5)  abs(x - round(x)) < tol

				# check if we have at least 1 element in the list, otherwise the 
				# loop runs from 1 to down 0 and we get errors
				if ( length( opts ) > 0 ) {

						# loop over all options and give them the correct type
						for ( i in 1:length( opts ) ) {
								tmp.type <- nlm.option.types[[match( names(opts)[i], names(nlm.option.types) )]]
								if ( is.null( tmp.type ) ) {
										# determine type
										if ( is.character(opts[[i]]) ) {
												tmp.type <- "string"
										} else if ( is.wholenumber(opts[[i]]) ) {
												tmp.type <- "integer"
										} else {
												tmp.type <- "numeric"
										}
										cat( paste( "Warning: ", names(opts)[i], " is not a recognized option, we try to pass it to nlm as ", tmp.type, "\n" ) )
								}

								if ( tmp.type=="string" ) {
										converted.opts$string[[ names(opts)[i] ]] <- as.character(opts[[i]])
								} else if ( tmp.type=="integer" ) {
										converted.opts$integer[[ names(opts)[i] ]] <- as.integer(opts[[i]])
								} else if ( tmp.type=="numeric" ) {
										converted.opts$numeric[[ names(opts)[i] ]] <- as.numeric(opts[[i]])
								} else {
										stop(paste("Type of option ", names(opts)[i], " not recognized"))
								}
						}
				}

				return ( converted.opts )
		}

		ret <- list( 
								"objective" = objective, 
								"gradient" = as.integer(gradient), 
								"opt.flg" = as.integer(opt.flag), 
								"start" = start, 
								"options" = get.option.types(opts), 
								"G" = as.integer(model$G), 
								"data" = model$data, 
								"pattern.number" = model$pattern.number, 
								"valid.data.patterns" = model$valid.data.patterns, 
								"S" = model$S, 
								"logdetS" = model$logdetS, 
								"invS" = model$invS, 
								"N" = model$N, 
								"m" = model$m, 
								"n" = model$n, 
								"t" = as.integer(model$t), 
								"fixed" = model$fixed, 
								"ram" = model$ram, 
								"sel.free" = model$sel.free, 
								"arrows.1" = model$arrows.1, 
								"arrows.1.free" = model$arrows.1.free, 
								"one.head" = model$one.head, 
								"arrows.2t" = model$arrows.2t, 
								"arrows.2" = model$arrows.2, 
								"arrows.2.free" = model$arrows.2.free, 
								"unique.free.1" = model$unique.free.1, 
								"unique.free.2" = model$unique.free.2, 
								"J" = model$J, 
								"correct" = model$correct, 
								"param.names" = model$param.names, 
								"var.names" = model$var.names, 
								"one.free" = model$one.free, 
								"two.free" = model$two.free, 
								"raw" = as.integer(model$raw), 
								"arrows.1.seq" = arrows.1.seq, 
								"arrows.2.seq" = arrows.2.seq, 
								"typsize" = typsize, 
								"print.f" = print.f.wrapper, 
								"csem.environment"=csem.environment)
		attr(ret, "class") <- "cmsem"

		# add the current call to the list
		# ret$call <- match.call()

		solution <- .Call("cmsemSolve", ret)

		# ret$environment <- NULL
		# ret$solution <- solution

		ret <- solution   #this is for simplifing the interface.
		# add solution variables to object
		#ret$status <- solution$status

		return(ret)
}

\name{ram}
\alias{ram}

\title{RAM Matrix for a Structural-Equation Model}
\description{
  Print the labelled RAM definition matrix for a structural-equation
  model fit by \code{sem}.
}

\usage{
ram(object, digits=getOption("digits"), startvalues=FALSE)
}


\arguments{
  \item{object}{an object of class \code{sem} returned by the \code{sem} function.}
  \item{digits}{number of digits for printed output.}
  \item{startvalues}{if \code{TRUE}, start values for parameters are printed;
    otherwise, the parameter estimates are printed; the default is \code{FALSE}.}
}

\value{
  A data frame containing the labelled RAM definition matrix, which is normally
  just printed.
}

\author{John Fox \email{jfox@mcmaster.ca}}

\seealso{\code{\link{sem}}}

\examples{

# ------------- assumes that Duncan, Haller and Portes peer-influences model
# -------------     has been fit and is in sem.dhp

    \dontrun{
ram(sem.dhp)
    }
}
\keyword{models}

\name{standardizedCoefficients}
\alias{standardizedCoefficients}
\alias{standardizedCoefficients.sem}
\alias{standardizedCoefficients.msem}
\alias{stdCoef}

\title{Standardized Coefficients for Structural Equation Models}
\description{
  These functions calculate standardized regression coefficients
  for structural equation models. The function \code{stdCoef} is
  simply an abbreviation for \code{standardizedCoefficients}.
}

\usage{
standardizedCoefficients(object, ...)
\method{standardizedCoefficients}{sem}(object, 
    digits = getOption("digits"), oneheaded = TRUE, twoheaded = TRUE, ...)
\method{standardizedCoefficients}{msem}(object, ...)

stdCoef(...)
}

\arguments{
  \item{object}{an object of class \code{sem} or \code{msem} returned by the 
    \code{\link{sem}} function.}
  \item{digits}{number of digits for printed output.}
  \item{oneheaded}{standardize path coefficients? Default is \code{TRUE}.}
  \item{twoheaded}{standardize variances and covariances? Default is \code{TRUE}.}
  \item{...}{arguments to pass down.}
}

\value{
  Returns a data frame with the coefficients, labelled 
  both by parameter names and by arrows in the path diagram for the
  model. The \code{msem} (multigroup) method computes and prints the
  standardized coefficients for each group; it does not return a useful result.
}

\references{
    Bollen, K. A. (1989) 
    \emph{Structural Equations With Latent Variables.} Wiley.
}

\author{John Fox \email{jfox@mcmaster.ca} and Adam Kramer}

\seealso{\code{\link{sem}}}

\examples{
# In the first example, readMoments() and specifyModel() read from the
# input stream. This example cannot be executed via example() but can be entered
# at the command prompt. The example is repeated using file input;
# this example can be executed via example(). 
    \dontrun{
# Duncan, Haller, and Portes peer-influences model

R.DHP <- readMoments(diag=FALSE, names=c("ROccAsp", "REdAsp", "FOccAsp", 
                "FEdAsp", "RParAsp", "RIQ", "RSES", "FSES", "FIQ", "FParAsp"))
    .6247     
    .3269  .3669       
    .4216  .3275  .6404
    .2137  .2742  .1124  .0839
    .4105  .4043  .2903  .2598  .1839
    .3240  .4047  .3054  .2786  .0489  .2220
    .2930  .2407  .4105  .3607  .0186  .1861  .2707
    .2995  .2863  .5191  .5007  .0782  .3355  .2302  .2950
    .0760  .0702  .2784  .1988  .1147  .1021  .0931 -.0438  .2087
            
model.dhp <- specifyModel()
    RParAsp  -> RGenAsp, gam11,  NA
    RIQ      -> RGenAsp, gam12,  NA
    RSES     -> RGenAsp, gam13,  NA
    FSES     -> RGenAsp, gam14,  NA
    RSES     -> FGenAsp, gam23,  NA
    FSES     -> FGenAsp, gam24,  NA
    FIQ      -> FGenAsp, gam25,  NA
    FParAsp  -> FGenAsp, gam26,  NA
    FGenAsp  -> RGenAsp, beta12, NA
    RGenAsp  -> FGenAsp, beta21, NA
    RGenAsp  -> ROccAsp,  NA,     1
    RGenAsp  -> REdAsp,  lam21,  NA
    FGenAsp  -> FOccAsp,  NA,     1
    FGenAsp  -> FEdAsp,  lam42,  NA
    RGenAsp <-> RGenAsp, ps11,   NA
    FGenAsp <-> FGenAsp, ps22,   NA
    RGenAsp <-> FGenAsp, ps12,   NA
    ROccAsp <-> ROccAsp, theta1, NA
    REdAsp  <-> REdAsp,  theta2, NA
    FOccAsp <-> FOccAsp, theta3, NA
    FEdAsp  <-> FEdAsp,  theta4, NA

sem.dhp <- sem(model.dhp, R.DHP, 329,
    fixed.x=c('RParAsp', 'RIQ', 'RSES', 'FSES', 'FIQ', 'FParAsp'))
standardizedCoefficients(sem.dhp) 
	}
# The following example can be executed via example():

etc <- system.file(package="sem", "etc") # path to data and model files
   
(R.DHP <- readMoments(file=file.path(etc, "R-DHP.txt"),
				diag=FALSE, names=c("ROccAsp", "REdAsp", "FOccAsp", 
                "FEdAsp", "RParAsp", "RIQ", "RSES", "FSES", "FIQ", "FParAsp")))
(model.dhp <- specifyModel(file=file.path(etc, "model-DHP.txt")))
(sem.dhp <- sem(model.dhp, R.DHP, 329,
    fixed.x=c('RParAsp', 'RIQ', 'RSES', 'FSES', 'FIQ', 'FParAsp')))
standardizedCoefficients(sem.dhp) 
}

\keyword{models}

\name{effects.sem}
\alias{effects.sem}
\alias{effects.msem}
\alias{print.semeffects}
\alias{print.semeffectsList}

\title{
Total, Direct, and Indirect Effects for Structural Equation Models
}

\description{
The \code{sem} method for the standard generic function \code{effects} computes total, direct, 
and indirect effects for a fitted structural equation model according to the method described in Fox (1980).
}

\usage{
\method{effects}{sem}(object, ...)
\method{effects}{msem}(object, ...)

\method{print}{semeffects}(x, digits = getOption("digits"), ...)
\method{print}{semeffectsList}(x, digits = getOption("digits"), ...)
}

\arguments{
  \item{object}{a fitted structural-equation model object produced by the \code{\link{sem}} function.}
  \item{x}{an object of class \code{semeffects} or \code{semeffectsList}, produced by \code{effects}.}
  \item{digits}{digits to print.}
  \item{\dots}{not used.}
}

\value{
\code{effect.sem} returns an object of class \code{semeffects} with \code{Total}, \code{Direct}, and \code{Indirect} elements.
}

\references{
  Fox, J. (1980)
  Effect analysis in structural equation models: Extensions and simplified methods of computation.
  \emph{Sociological Methods and Research}
  \bold{9}, 3--28.
}

\author{John Fox \email{jfox@mcmaster.ca}}

\seealso{\code{\link{sem}}}

\examples{
    \dontrun{

# These examples are from Fox (1980)

# In the first pair of examples, readMoments() and specifyModel() read from the
# input stream. These examples cannot be executed via example() but can be entered
# at the command prompt. The Blau and Duncan example is repeated using file input;
# this example can be executed via example(). 

# The recursive Blau and Duncan basic stratification model:
#  x1 is father's education, x2 father's SES, y3 respondent's education,
#  y4 SES of respondent's first job, y5 respondent's SES in 1962

R.bd <- readMoments(names=c("x1", "x2", "y3", "y4", "y5"))
1
.516 1
.453 .438 1
.332 .417 .538 1
.322 .405 .596 .541 1

mod.bd <- specifyModel()
y3 <- x1, gam31
y3 <- x2, gam32
y4 <- x2, gam42
y4 <- y3, beta43
y5 <- x2, gam52
y5 <- y3, beta53
y5 <- y4, beta54

sem.bd <- sem(mod.bd, R.bd, N=20700, fixed.x=c("x1", "x2"))
summary(sem.bd)
effects(sem.bd)


# The nonrecursive Duncan, Haller, and Portes peer-influences model for observed variables:

R.DHP <- readMoments(diag=FALSE, names=c("ROccAsp", "REdAsp", "FOccAsp", 
"FEdAsp", "RParAsp", "RIQ", "RSES", "FSES", "FIQ", "FParAsp"))
.6247     
.3269  .3669       
.4216  .3275  .6404
.2137  .2742  .1124  .0839
.4105  .4043  .2903  .2598  .1839
.3240  .4047  .3054  .2786  .0489  .2220
.2930  .2407  .4105  .3607  .0186  .1861  .2707
.2995  .2863  .5191  .5007  .0782  .3355  .2302  .2950
.0760  .0702  .2784  .1988  .1147  .1021  .0931 -.0438  .2087

model.dhp <- specifyModel()
RIQ      -> ROccAsp, gam51,  NA
RSES     -> ROccAsp, gam52,  NA
FSES     -> FOccAsp, gam63,  NA
FIQ      -> FOccAsp, gam64,  NA
FOccAsp  -> ROccAsp, beta56, NA
ROccAsp  -> FOccAsp, beta65, NA
ROccAsp <-> ROccAsp, ps55,   NA
FOccAsp <-> FOccAsp, ps66,   NA
ROccAsp <-> FOccAsp, ps56,   NA


# Note: The following generates a warning because not all of the variables
#       in the correlation matrix are used
sem.dhp <- sem(model.dhp, R.DHP, 329,
                fixed.x=c('RIQ', 'RSES', 'FSES', 'FIQ'))
summary(sem.dhp)
effects(sem.dhp)
    }
    
## the following example may be executed via example()

etc <- system.file(package="sem", "etc") # path to data and model files

# The recursive Blau and Duncan basic stratification model:
#  x1 is father's education, x2 father's SES, y3 respondent's education,
#  y4 SES of respondent's first job, y5 respondent's SES in 1962

(R.bd <- readMoments(file=file.path(etc, "R-Blau-Duncan.txt"),
					names=c("x1", "x2", "y3", "y4", "y5")))
(mod.bd <- specifyModel(file=file.path(etc, "model-Blau-Duncan.txt")))
sem.bd <- sem(mod.bd, R.bd, N=20700, fixed.x=c("x1", "x2"))
summary(sem.bd)
effects(sem.bd)
}


\keyword{models}

\name{rawMoments}
\alias{rawMoments}
\alias{rawMoments.formula}
\alias{rawMoments.default}
\alias{cov2raw}
\alias{print.rawmoments}

\title{Compute Raw Moments Matrix}
\description{
 Computes the \dQuote{uncorrected} sum-of-squares-and-products matrix divided by the
 number of observations.
}
\usage{
\method{rawMoments}{formula}(formula, data, subset, na.action, 
    contrasts=NULL, ...)

\method{rawMoments}{default}(object, na.rm=FALSE, ...)

cov2raw(cov, mean, N, sd)

\method{print}{rawmoments}(x, ...)
}

\arguments{
  \item{object}{a one-sided model formula or an object coercible to a
    numeric matrix.}
  \item{formula}{a one-sided model formula specifying the model matrix for
    which raw moments are to be computed; note that a constant is included
    if it is not explicitly suppressed by putting \code{-1} in the formula.}
  \item{data}{an optional data frame containing the variables in the formula. 
    By default the variables are taken from the environment from which 
    \code{rawMoments} is called.}
  \item{subset}{an optional vector specifying a subset of observations to be 
    used in computing moments.}
  \item{na.action}{a function that indicates what should happen when the data 
    contain \code{NA}s. The default is set by the \code{na.action} option.}
  \item{contrasts}{an optional list. See the \code{contrasts.arg} argument of 
    \code{\link[stats:model.matrix]{model.matrix.default}}}.
  \item{na.rm}{if \code{TRUE}, any data rows with missing data will be removed.}
  \item{cov}{a covariance or correlation matrix.}
  \item{mean}{a vector of means.}
  \item{N}{the number of observations on which the covariances or correlations 
    are based.}
  \item{sd}{an optional vector of standard deviations, to be given if \code{cov} is a
    correlation matrix.}
  \item{x}{an object of class \code{rawmoments} to print.}
  \item{\dots}{arguments passed down.}
}

\value{
  \code{rawMoments} and \code{cov2raw} return an object of class \code{rawmoments}, 
  which is simply a matrix
  with an attribute \code{"N"} that contains the number of observations on
  which the moments are based.
}

\author{John Fox \email{jfox@mcmaster.ca}}

\seealso{\code{\link{sem}}}

\examples{
# the following are all equivalent (with the exception of the name of the intercept):

rawMoments(cbind(1, Kmenta))

rawMoments(~ Q + P + D + F + A, data=Kmenta)

Cov <- with(Kmenta, cov(cbind(Q, P, D, F, A)))
cov2raw(Cov, colMeans(Kmenta), nrow(Kmenta))
}

\keyword{manip}

\name{Kmenta}
\alias{Kmenta}
\docType{data}
\title{Partly Artificial Data on the U. S. Economy}
\usage{Kmenta}
\description{
These are partly contrived data from Kmenta (1986), constructed
to illustrate estimation of a simultaneous-equation model.

The \code{Kmenta} data frame has 20 rows and 5 columns.
}
\format{
This data frame contains the following columns:
\describe{
\item{Q}{
food consumption per capita.
}
\item{P}{
ratio of food prices to general consumer prices.
}
\item{D}{
disposable income in constant dollars.
}
\item{F}{
ratio of preceding year's prices received by farmers
to general consumer prices.
}
\item{A}{
time in years.
}
}
}

\details{
 The exogenous variables \code{D}, \code{F}, and \code{A} are based on
 real data; the endogenous variables \code{P} and \code{Q} were generated
 by simulation.
}
\source{
 Kmenta, J. (1986)
 \emph{Elements of Econometrics, Second Edition}, Macmillan.
}

\keyword{datasets}

\name{ML.methods}
\alias{ML.methods}
\alias{GLS.methods}
\alias{anova.objectiveML}
\alias{anova.objectiveFIML}
\alias{logLik.objectiveML}
\alias{logLik.objectiveFIML}
\alias{AIC.objectiveML}
\alias{AIC.objectiveFIML}
\alias{AICc.objectiveML}
\alias{AICc.objectiveFIML}
\alias{BIC.objectiveML}
\alias{BIC.objectiveFIML}
\alias{CAIC.objectiveML}
\alias{CAIC.objectiveFIML}
\alias{deviance.objectiveML}
\alias{deviance.objectiveFIML}
\alias{print.objectiveML}
\alias{print.objectiveGLS}
\alias{print.objectiveFIML}
\alias{summary.objectiveML}
\alias{summary.objectiveGLS}
\alias{summary.objectiveFIML}
\alias{print.summary.objectiveML}
\alias{print.msemObjectiveML}
\alias{print.msemObjectiveGLS}
\alias{summary.msemObjectiveML}
\alias{summary.msemObjectiveGLS}
\alias{deviance.msemObjectiveML}
\alias{AIC.msemObjectiveML}
\alias{AICc.msemObjectiveML}
\alias{BIC.msemObjectiveML}
\alias{deviance.msemObjectiveML}
\alias{anova.msemObjectiveML}
\alias{logLik.msemObjectiveML}


\title{
Methods for sem Objects Fit Using the \code{objectiveML}, \code{objectiveGLS}, \code{objectiveFIML}, \code{msemObjectiveML},
and \code{msemObjectiveGLS} Objective Functions
}

\description{
These functions are for objects fit by \code{\link{sem}} using the \code{\link{objectiveML}} (multivariate-normal full-information maximum-likelihood), \code{link{objectiveFIML}} (multivariate-normal full-information maximum-likihood in
the presence of missing data),
\code{\link{objectiveGLS}} (generalized least squares), and \code{\link{msemObjectiveML}} (multigroup multivariate-normal FIML) objective functions.
}

\usage{
\method{anova}{objectiveML}(object, model.2, robust=FALSE, ...)
\method{anova}{objectiveFIML}(object, model.2, ...)

\method{logLik}{objectiveML}(object, ...)
\method{logLik}{objectiveFIML}(object, saturated=FALSE, 
    intercept="Intercept", iterlim=1000, ...)
\method{deviance}{objectiveML}(object, ...)
\method{deviance}{objectiveFIML}(object, saturated.logLik, ...)
\method{deviance}{msemObjectiveML}(object, ...) 
\method{AIC}{objectiveML}(object, ..., k)
\method{AIC}{objectiveFIML}(object, saturated.logLik, ..., k)
\method{AIC}{msemObjectiveML}(object, ..., k)
\method{AICc}{objectiveML}(object, ...)
\method{AICc}{objectiveFIML}(object, saturated.logLik, ...)
\method{AICc}{msemObjectiveML}(object, ...)
\method{BIC}{objectiveML}(object, ...)
\method{BIC}{objectiveFIML}(object, saturated.logLik, ...)
\method{BIC}{msemObjectiveML}(object, ...)
\method{CAIC}{objectiveML}(object, ...)
\method{CAIC}{objectiveFIML}(object, saturated.logLik, ...)

\method{print}{objectiveML}(x, ...)
\method{print}{objectiveGLS}(x, ...)
\method{print}{objectiveFIML}(x, saturated=FALSE, ...)
\method{print}{msemObjectiveML}(x, ...)
\method{print}{msemObjectiveGLS}(x, ...)

\method{summary}{objectiveML}(object, digits=getOption("digits"), 
    conf.level=.90, robust=FALSE, analytic.se=object$t <= 500, 
    fit.indices=c("GFI", "AGFI", "RMSEA", "NFI", "NNFI", "CFI", "RNI",
      "IFI", "SRMR", "AIC", "AICc", "BIC", "CAIC"), ...)
\method{summary}{objectiveFIML}(object, digits=getOption("digits"), conf.level=.90,
    fit.indices=c("AIC", "AICc", "BIC", "CAIC"),
    saturated=FALSE, intercept="Intercept", saturated.logLik, ...)
\method{summary}{objectiveGLS}(object, digits=getOption("digits"), conf.level=.90, 
    fit.indices=c("GFI", "AGFI", "RMSEA", "NFI", "NNFI", "CFI", "RNI", "IFI", "SRMR"),
    ...)
\method{summary}{msemObjectiveML}(object, digits=getOption("digits"), 
    conf.level=.90, robust=FALSE, 
    analytic.se=object$t <= 500,
    fit.indices=c("GFI", "AGFI", "RMSEA", "NFI", "NNFI", "CFI", "RNI", 
      "IFI", "SRMR", "AIC", "AICc", "BIC"), ...)
\method{summary}{msemObjectiveGLS}(object, digits=getOption("digits"), 
    conf.level=.90,
    fit.indices=c("GFI", "AGFI", "RMSEA", "NFI", "NNFI", 
      "CFI", "RNI", "IFI", "SRMR"), ...)
}

\arguments{
  \item{object, model.2, x}{an object inheriting from class \code{objectiveML}, \code{objectiveGLS}, 
    \code{objectiveFIML}, \code{msemObjectiveML}, or \code{msemObjectiveGLS}.}
  \item{robust}{if \code{TRUE}, compute robust standard errors or test.}
  \item{fit.indices}{a character vector of ``fit indices'' to report; the allowable values are those given in \bold{Usage}
    above, and vary by the objective function. If the argument isn't given then the fit indices reported are taken
    from the R \code{fit.indices} option; if this option isn't set, then only the AIC and BIC are reported for models fit
    with \code{objectiveML}, \code{objectiveFIML}, or \code{msemObjectiveML}, and no fit indices are reported for
    models fit with \code{objectiveGLS} or \code{msemObjectiveGLS}.}
  \item{k, \ldots}{ignored.}
  \item{digits}{digits to be printed.}
  \item{conf.level}{level for confidence interval for the RMSEA index (default is .9).}
  \item{analytic.se}{use analytic (as opposed to numeric) coefficient standard errors; default is \code{TRUE}
    where analytic standard errors are available if there are no more than
    100 parameters in the model and \code{FALSE} otherwise.}
  \item{saturated}{if \code{TRUE} (the default is \code{FALSE}); compute the log-likelihood (and statistics that
    depend on it) for the saturated model when the objective function is FIML in the presence of missing data. 
    This can be computationally costly.}
  \item{intercept}{the name of the intercept regressor in the raw data, to be used in calculating the
    saturated log-likelihood for the FIML estimator; the default is \code{"Intercept"}.}
  \item{saturated.logLik}{the log-likelihood for the saturated model, as returned by \code{logLik}
    with \code{saturated=TRUE}; if absent, this will be computed and the computation can be time-consuming.}
  \item{iterlim}{iteration limit used by the \code{nlm} optimizer to compute the saturated log-likelihood for
    the FIML estimator with missing data; defaults to \code{1000}.}
}

\author{John Fox \email{jfox@mcmaster.ca} and Jarrett Byrnes}

\references{
See \code{\link{sem}}.
}

\seealso{
\code{\link{sem}}, \code{\link{objective.functions}}, \code{\link{modIndices.objectiveML}} 
}

\keyword{models}

\name{objective.functions}

\alias{objective.functions}
\alias{objectiveML}
\alias{objectiveGLS}
\alias{objectiveFIML}
\alias{objectiveML2}
\alias{objectiveGLS2}
\alias{objectiveFIML2}
\alias{msemObjectiveML}
\alias{msemObjectiveML2}
\alias{msemObjectiveGLS}
%% \alias{msemObjectiveFIML}


\title{
sem Objective-Function Builders
}

\description{
These functions return objective functions suitable for use with \code{\link{optimizers}} called by \code{\link{sem}}. The user would not
normally call these functions directly, but rather supply one of them in the \code{objective} argument to
\code{sem}. Users may also write their own objective functions. \code{objectiveML} and \code{objectiveML2} are for multinormal maximum-likelihood
estimation; \code{objectiveGLS} and \code{objectiveGLS2} are for generalized least squares; and \code{objectiveFIML2}
is for so-called ``full-information maximum-likelihood'' estimation in the presence of missing data. The FIML estimator
provides the same estimates as the ML estimator when there is no missing data; it can be slow because it iterates over
the unique patterns of missing data that occur in the data set.
\code{objectiveML} and \code{objectiveGLS} use
compiled code and are therefore substantially faster. \code{objectiveML2} and \code{objectiveGLS2} are provided primarily to illustrate
how to write \code{sem} objective functions in R. \code{msemObjectiveML} uses compiled code is for fitting multi-group models by 
multinormal maximum likelihood; \code{msemObjectiveML2} is similar but doesn't use compiled code. \code{msemObjectiveGLS} uses compiled
code and is for fitting multi-group models by generalized least squares.
}

\usage{
objectiveML(gradient=TRUE, hessian=FALSE)
objectiveML2(gradient=TRUE)

objectiveGLS(gradient=FALSE)
objectiveGLS2(gradient=FALSE)

objectiveFIML(gradient=TRUE, hessian=FALSE)
objectiveFIML2(gradient=TRUE, hessian=FALSE)

msemObjectiveML(gradient=TRUE)
msemObjectiveML2(gradient=TRUE)

msemObjectiveGLS(gradient=FALSE)

%% msemObjectiveFIML(gradient=FALSE)
}

\arguments{
  \item{gradient}{If \code{TRUE}, the object that's returned includes a function for computing an analytic gradient; there is at present no
  analytic gradient available for \code{objectiveFIML}, \code{objectiveGLS}, \code{objectiveGLS2}, or \code{msemObjectiveGL}.}
  \item{hessian}{If \code{TRUE}, the objected returned includes a function to compute an analytic Hessian; only avaiable for \code{objectiveML}
  and not generally recommended.}
}

\value{
These functions return an object of class \code{"semObjective"}, with up to two elements:
\item{objective}{an objective function.}
\item{gradient}{a gradient function.}
}

\author{John Fox \email{jfox@mcmaster.ca}}

\references{
See \code{\link{sem}}.
}

\seealso{
\code{\link{sem}}, \code{\link{optimizers}}
}

\keyword{models}

\name{tsls}
\alias{tsls}
\alias{tsls.formula}
\alias{tsls.default}
\alias{fitted.tsls}
\alias{residuals.tsls}
\alias{coef.tsls}
\alias{vcov.tsls}
\alias{anova.tsls}
\alias{print.tsls}
\alias{summary.tsls}
\alias{print.summary.tsls}

\title{Two-Stage Least Squares}
\description{
  Fits a regression equation, such as an equation in a structural-equation model, by two-stage least squares. This
  is equivalent to direct instrumental-variables estimation when the number of
  instruments is equal to the number of predictors.
}
\usage{
\method{tsls}{formula}(formula, instruments, data, subset, weights, 
	na.action, contrasts=NULL, ...)
\method{tsls}{default}(y, X, Z, w, names=NULL, ...)

\method{print}{tsls}(x, ...)
\method{summary}{tsls}(object, digits=getOption("digits"), ...)
\method{print}{summary.tsls}(x, ...)
\method{anova}{tsls}(object, model.2, s2, dfe, ...)

\method{fitted}{tsls}(object, ...)
\method{residuals}{tsls}(object, ...)
\method{coef}{tsls}(object, ...)
\method{vcov}{tsls}(object, ...)
}

\arguments{
  \item{formula}{model formula for structural equation to be estimated; a regression
    constant is implied if not explicitly omitted.}
  \item{instruments}{one-sided model formula specifying instrumental variables.}
  \item{data}{an optional data frame containing the variables in the model. 
    By default the variables are taken from the environment from which \code{tsls} is
    called.}
  \item{subset}{an optional vector specifying a subset of observations to be used in 
    fitting the model.}
  \item{weights, w}{an optional vector of weights to be used in the fitting process; 
    if specified should be a non-negative numeric vector with one entry for each
    observation, to be used to compute weighted 2SLS estimates.}
  \item{na.action}{a function that indicates what should happen when the 
    data contain \code{NA}s. 
    The default is set by the \code{na.action} option.}
  \item{contrasts}{an optional list. See the \code{contrasts.arg} argument of 
    \code{\link[stats:model.matrix]{model.matrix.default}}.}
  \item{y}{Response-variable vector.}
  \item{X}{Matrix of predictors, including a constant (if one is in the model).}
  \item{Z}{Matrix of instrumental variables, including a constant (if one is in the model).}
  \item{names}{optional character vector of names for the columns of the \code{X} matrix.}
  \item{x, object, model.2}{objects of class \code{tsls} returned by \code{tsls.formula} 
    (or of class \code{summary.tsls}), for \code{anova} 
    containing nested models
    to be compared by an incremental \eqn{F}-test. One model should be nested in the other; the
    order of models is immaterial.}
  \item{s2}{an optional estimate of error variance for the denominator of the \eqn{F}-test. 
    If missing, the error-variance estimate is taken from the larger model.}
  \item{dfe}{optional error degrees of freedom, to be specified when an estimate of error
    variance is given.}
  \item{digits}{number of digits for summary output.}
  \item{...}{arguments to be passed down.}
}

\value{
  \code{tsls.formula} returns an object of class \code{tsls}, with the following components:
  \item{n}{number of observations.}
  \item{p}{number of parameters.}
  \item{coefficients}{parameter estimates.}
  \item{V}{estimated covariance matrix of coefficients.}
  \item{s}{residual standard error.}
  \item{residuals}{vector of residuals.}
  \item{response}{vector of response values.}
  \item{X}{model matrix.}
  \item{Z}{instrumental-variables matrix.}
  \item{response.name}{name of response variable, or expression evaluating to response.}
  \item{formula}{model formula.}
  \item{instruments}{one-sided formula for instrumental variables.}
}

\references{    
    Fox, J. (1979)
    Simultaneous equation models and two-stage least-squares.
    In Schuessler, K. F. (ed.)
    \emph{Sociological Methodology 1979}, Jossey-Bass.
    
    Greene, W. H. (1993)
    \emph{Econometric Analysis, Second Edition}, Macmillan.
}

\author{John Fox \email{jfox@mcmaster.ca}}

\seealso{\code{\link{sem}}}

\examples{
summary(tsls(Q ~ P + D, ~ D + F + A, data=Kmenta))     # demand equation

summary(tsls(Q ~ P + F + A, ~ D + F + A, data=Kmenta)) # supply equation

anova(tsls(Q ~ P + F + A, ~ D + F + A, data=Kmenta),
    tsls(Q ~ 1, ~ D + F + A, data=Kmenta))
}
\keyword{models}

\name{optimizers}

\alias{optimizers}
\alias{optimizerSem}
\alias{optimizerNlm}
\alias{optimizerOptim}
\alias{optimizerNlminb}
\alias{optimizerMsem}
\alias{msemOptimizerNlm}

\title{sem Optimizers}

\description{
The default optimizer used by \code{\link{sem}} is \code{optimizerSem}, which employs compiled code and is integrated with
the \code{\link{objectiveML}} and \code{\link{objectiveGLS}} objective functions; 
\code{optimizerSem}, written by Zhenghua Nie, is a modified
version of the standard R \code{nlm} optimizer, which was written by Saikat DebRoy, R-core, and Richard H. Jones.
The other functions call optimizers (\code{\link[stats]{nlm}}, \code{\link[stats]{optim}}, or \code{\link[stats]{nlminb}}), 
to fit structural equation models, and are called by the \code{\link{sem}} function. 
The user would not normally call these functions directly, but rather supply one of them in the \code{optimizer} argument to
\code{sem}. Users may also write them own optimizer functions. \code{msemOptimizerNlm} is for fitting multigroup models, and also adapts the \code{nlm} code.
}

\usage{
optimizerSem(start, objective=objectiveML,  
	gradient=TRUE, maxiter, debug, par.size, model.description, warn, ...)
	
optimizerMsem(start, objective=msemObjectiveML, gradient=TRUE,
	maxiter, debug, par.size, model.description, warn=FALSE, ...)
	
optimizerNlm(start, objective=objectiveML, gradient=TRUE, 
	maxiter, debug, par.size, model.description, warn, ...)
	
optimizerOptim(start, objective=objectiveML, gradient=TRUE, 
	maxiter, debug, par.size, model.description, warn, method="CG", ...)

optimizerNlminb(start, objective=objectiveML, gradient=TRUE, maxiter, 
	debug, par.size, model.description, warn, ...)

msemOptimizerNlm(start, objective=msemObjectiveML, gradient=TRUE,
		maxiter, debug, par.size, model.description, warn=FALSE, ...)
}

\arguments{

  \item{start}{a vector of start values for the parameters.}
  
  \item{objective}{the objective function to be optimized; see \link{objective.functions}.}
  
  \item{gradient}{\code{TRUE} if an analytic gradient is to be used (if one is available).}
  
  \item{maxiter}{the maximum number of iterations allowed.}
  
  \item{debug}{\code{TRUE} to show the iteration history and other available information about the optimization.}
  
  \item{par.size}{\code{"startvalues"} to have the optimizer scale the problem according to the magitudes of the start values (ignored by \code{optimizerNlminb}).}
  
  \item{model.description}{a list with elements describing the structural-equation model (see the code for details).}
  
  \item{warn}{if \code{FALSE}, suppress warnings during the optimization.}
  
  \item{method}{the method to be employed by the \code{\link[stats]{optim}} optimizer; the default is \code{"CG"} (conjugate-gradient).}
  
  \item{\dots}{additional arguments for the \code{nlm}, \code{optim}, or \code{nlminb} optimizer.}

}

\value{
An object of class \code{"semResult"}, with elements:

\item{convergence}{\code{TRUE} if the optimization apparently converged.}

\item{iterations}{the number of iterations required.}

\item{par}{the vector of parameter estimates.}

\item{vcov}{the estimated covariance matrix of the parameter estimates, based on a numeric Hessian; not supplied by \code{optimizerNlminb}.}

\item{criterion}{the optimized value of the objective function.}

\item{C}{the model-implied covariance or moment matrix at the parameter estimates.}

\item{A}{the estimated \eqn{A} matrix.}

\item{P}{the estimated \eqn{P} matrix.}

}

\author{John Fox \email{jfox@mcmaster.ca}, and Zhenghua Nie, in part adapting work by
Saikat DebRoy, R-core, and Richard H. Jones.}

\seealso{\code{\link{sem}}, \code{\link{objective.functions}}, \code{\link[stats]{nlm}}, \code{\link[stats]{optim}}, \code{\link[stats]{nlminb}}}

\keyword{models}

\name{readMoments}
\alias{readMoments}

\title{Input a Covariance, Correlation, or Raw Moment Matrix}
    
\description{
This functions makes it simpler to input covariance, correlation, and raw-moment 
matrices to be analyzed by the \code{\link{sem}} function. The matrix
is input in lower-triangular form on as many lines as is convenient, 
omitting the above-diagonal elements. The
elements on the diagonal may also optionally be omitted, in which case they
are taken to be 1.
}

\usage{
readMoments(file="", text, diag=TRUE, 
    names=as.character(paste("X", 1:n, sep = "")))
}

\arguments{
  \item{file}{The (quoted) file from which to read the moment matrix,
    including the path to the file if it is not in the current directory. If
    \code{""} (the default) and the \code{text} argument is absent, 
    then the moment matrix is read from the standard
    input stream, and is terminated by a blank line.}
  \item{text}{The moment matrix given as a character string, as an alternative
    to specifying the \code{file} argument or reading the moments from
    the input stream --- e.g., when the session is not interactive and there is no
    standard input.}
  \item{diag}{If \code{TRUE} (the default), then the input matrix includes
    diagonal elements.}
  \item{names}{a character vector containing the names of the variables, to
    label the rows and columns of the moment matrix.}
}

\value{
 Returns a lower-triangular matrix (i.e., with zeroes above the main diagonal)
 suitable for input to \code{sem}.
}

\author{John Fox \email{jfox@mcmaster.ca}}

\seealso{\code{\link{sem}}}

\examples{

R.DHP <- readMoments(diag=FALSE, names=c("ROccAsp", "REdAsp", "FOccAsp", 
                "FEdAsp", "RParAsp", "RIQ", "RSES", "FSES", "FIQ", "FParAsp"),
                text="
    .6247     
    .3269  .3669       
    .4216  .3275  .6404
    .2137  .2742  .1124  .0839
    .4105  .4043  .2903  .2598  .1839
    .3240  .4047  .3054  .2786  .0489  .2220
    .2930  .2407  .4105  .3607  .0186  .1861  .2707
    .2995  .2863  .5191  .5007  .0782  .3355  .2302  .2950
    .0760  .0702  .2784  .1988  .1147  .1021  .0931 -.0438  .2087
")   
R.DHP

#the following will work only in an interactive sessions:
    \dontrun{
R.DHP <- readMoments(diag=FALSE, names=c("ROccAsp", "REdAsp", "FOccAsp", 
                "FEdAsp", "RParAsp", "RIQ", "RSES", "FSES", "FIQ", "FParAsp"))
    .6247     
    .3269  .3669       
    .4216  .3275  .6404
    .2137  .2742  .1124  .0839
    .4105  .4043  .2903  .2598  .1839
    .3240  .4047  .3054  .2786  .0489  .2220
    .2930  .2407  .4105  .3607  .0186  .1861  .2707
    .2995  .2863  .5191  .5007  .0782  .3355  .2302  .2950
    .0760  .0702  .2784  .1988  .1147  .1021  .0931 -.0438  .2087
    
R.DHP
    }
}

\keyword{manip}

\name{sem-deprecated}
\alias{sem-deprecated}

\alias{boot.sem}
\alias{mod.indices}
\alias{normalized.residuals}
\alias{path.diagram}
\alias{raw.moments}
\alias{read.moments}
\alias{specify.model}
\alias{standardized.coefficients}
\alias{standardized.residuals}
\alias{std.coef}

\title{Deprecated Functions in the sem Package}

\description{
  These functions are provided for compatibility with older versions of the \pkg{sem} package only, 
  and may be removed eventually. Although an effort has been made to insure backwards-compatibility,
  commands that worked in versions of the \pkg{sem} package prior to version 2.0-0
  will not necessarily work in version 2.0-0 and beyond, or may not work in the same manner.
}

\usage{
boot.sem(...)
mod.indices(...)
normalized.residuals(...)
path.diagram(...)
raw.moments(...)
read.moments(...)
specify.model(...)
standardized.coefficients(...)
standardized.residuals(...)
std.coef(...)
}

\arguments{
  \item{\dots}{pass arguments down to replacements for deprecated functions.}
}

\details{

\code{boot.sem} is now a synonym for the \code{\link{bootSem}} function.

\code{mod.indices} is now a synonym for \code{\link{modIndices}}.

\code{normalized.residuals} is now a synonym for \code{\link{normalizedResiduals}}.

\code{path.diagram} is now a synonym for \code{\link{pathDiagram}}.

\code{raw.moments} is now a synonym for \code{\link{rawMoments}}.

\code{read.moments} is now a synonym for \code{\link{readMoments}}.

\code{specify.model} is now a synonym for \code{\link{specifyModel}}.

\code{standardized.coefficients} and \code{std.coef} are now synonyms for the \code{\link{standardizedCoefficients}} and \code{\link{stdCoef}} 
functions.

\code{standardized.residuals} is now a synonym for \code{\link{standardizedResiduals}}.

}

\name{modIndices}
\alias{modIndices}
\alias{modIndices.objectiveML}
\alias{modIndices.msemObjectiveML}
\alias{print.modIndices}
\alias{print.msemModIndices}
\alias{summary.modIndices}
\alias{summary.msemModIndices}

\title{Modification Indices for Structural Equation Models}
\description{
  \code{mod.indices} calculates modification indices (score tests) 
  and estimated parameter changes for the fixed and constrained 
  parameters in a structural equation model fit by multinormal maximum likelihood.
}

\usage{
\method{modIndices}{objectiveML}(model, duplicated, deviance=NULL, ...)
\method{modIndices}{msemObjectiveML}(model, ...)

\method{print}{modIndices}(x, n.largest=5, ...)
\method{print}{msemModIndices}(x, ...)

\method{summary}{modIndices}(object, round=2, 
    print.matrices=c("both", "par.change", "mod.indices"), ...)
\method{summary}{msemModIndices}(object, ...)
}
    
\arguments{
  \item{model}{an object of class \code{objectiveML} or \code{msemObjectiveML},
    produced by the \code{\link{sem}} function.}
  \item{object, x}{an object of class \code{modIndices} or \code{msemModIndices}, produced by the 
    \code{modIndices} function.}
  \item{n.largest}{number of modification indices to print in each of the \eqn{A} and
    \eqn{P} matrices of the RAM model.}
  \item{round}{number of places to the right of the decimal point in printing 
    modification indices.}
  \item{print.matrices}{which matrices to print: estimated changes in the fixed parameters,
    modification indices, or both (the default).}
  \item{duplicated, deviance}{for internal use.}
  \item{\dots}{arguments to be passed down.}
}

\details{
  Modification indices are one-df chi-square score (\dQuote{Lagrange-multiplier}) test statistics for the fixed and constrained
  parameters in a structural equation model. They may be regarded as an estimate of the improvement
  in the likelihood-ratio chi-square statistic for the model if the corresponding parameter is
  respecified as a free parameter. The \code{modIndices} function also estimates the change in the
  value of a fixed or constrained parameter if the parameter is respecified as free. When several
  parameters are set equal, modification indices and estimated changes are given for all but the first.
  Modification indices and estimated parameter changes for currently free parameters are given as
  \code{NA}.
  
  The method employed is described in Saris, Satorra, and Sorbom (1987) and Sorbom (1989).
}

\value{
  \code{modIndices} returns an object of class \code{modIndices} with the following elements:
  \item{mod.A}{modification indices for the elements of the \eqn{A} matrix.}
  \item{mod.P}{modification indices for the elements of the \eqn{P} matrix.}
  \item{par.A}{estimated parameter changes for the elements of the \eqn{A} matrix.}
  \item{par.P}{estimated parameter changes for the elements of the \eqn{P} matrix.}
}

\references{
    Sarris, W. E., Satorra, A., and Sorbom, D. (1987)
    The detection and correction of specification errors in structural
    equation models. Pp. 105--129 in Clogg, C. C. (ed.), \emph{Sociological Methodology 1987.}
    American Sociological Association.
    
    Sorbom, D. (1989)
    Model modification.
    \emph{Psychometrika} \bold{54}, 371--384.
}

\author{John Fox \email{jfox@mcmaster.ca} and Michael Culbertson}

\seealso{\code{\link{sem}}}

\examples{
# In the first example, readMoments() and specifyModel() read from the
# input stream. This example cannot be executed via example() but can be entered
# at the command prompt. The example is repeated using file input;
# this example can be executed via example(). 
	\dontrun{
# This example is adapted from the SAS manual

S.wh <- readMoments(names=c('Anomia67','Powerless67','Anomia71',
                                    'Powerless71','Education','SEI'))
   11.834                                    
    6.947    9.364                            
    6.819    5.091   12.532                    
    4.783    5.028    7.495    9.986            
   -3.839   -3.889   -3.841   -3.625   9.610     
  -21.899  -18.831  -21.748  -18.775  35.522  450.288

model.wh <- specifyModel()
    Alienation67   ->  Anomia67,      NA,   1
    Alienation67   ->  Powerless67,   NA,   0.833
    Alienation71   ->  Anomia71,      NA,   1
    Alienation71   ->  Powerless71,   NA,   0.833
    SES            ->  Education,     NA,   1     
    SES            ->  SEI,           lamb, NA
    SES            ->  Alienation67,  gam1, NA
    Alienation67   ->  Alienation71,  beta, NA
    SES            ->  Alienation71,  gam2, NA
    Anomia67       <-> Anomia67,      the1, NA
    Anomia71       <-> Anomia71,      the1, NA
    Powerless67    <-> Powerless67,   the2, NA
    Powerless71    <-> Powerless71,   the2, NA
    Education      <-> Education,     the3, NA
    SEI            <-> SEI,           the4, NA
    Anomia67       <-> Anomia71,      the5, NA
    Powerless67    <-> Powerless71,   the5, NA
    Alienation67   <-> Alienation67,  psi1, NA
    Alienation71   <-> Alienation71,  psi2, NA
    SES            <-> SES,           phi,  NA

sem.wh <- sem(model.wh, S.wh, 932)
modIndices(sem.wh)
	}
	
# The following example can be executed via example():

etc <- system.file(package="sem", "etc") # path to data and model files

(S.wh <- readMoments(file=file.path(etc, "S-Wheaton.txt"),
					names=c('Anomia67','Powerless67','Anomia71',
                            'Powerless71','Education','SEI')))
(model.wh <- specifyModel(file=file.path(etc, "model-Wheaton-1.txt")))                    
(sem.wh <- sem(model.wh, S.wh, 932))
modIndices(sem.wh)
}

\keyword{models}

\name{HS.data}
\alias{HS.data}
\docType{data}
\title{
Holizinger and Swineford's Data
}
\description{
This data set, for scores on a variety of tests, was originally in the MBESS package. A new version of the data set in that package doesn't appear to be identical to this one.
}
\usage{HS.data}
\format{
  A data frame with 301 observations on the following 32 variables.
  \describe{
    \item{\code{id}}{a numeric vector}
    \item{\code{Gender}}{a factor with levels \code{Female} \code{Male}}
    \item{\code{grade}}{a numeric vector}
    \item{\code{agey}}{a numeric vector}
    \item{\code{agem}}{a numeric vector}
    \item{\code{school}}{a factor with levels \code{Grant-White} \code{Pasteur}}
    \item{\code{visual}}{a numeric vector}
    \item{\code{cubes}}{a numeric vector}
    \item{\code{paper}}{a numeric vector}
    \item{\code{flags}}{a numeric vector}
    \item{\code{general}}{a numeric vector}
    \item{\code{paragrap}}{a numeric vector}
    \item{\code{sentence}}{a numeric vector}
    \item{\code{wordc}}{a numeric vector}
    \item{\code{wordm}}{a numeric vector}
    \item{\code{addition}}{a numeric vector}
    \item{\code{code}}{a numeric vector}
    \item{\code{counting}}{a numeric vector}
    \item{\code{straight}}{a numeric vector}
    \item{\code{wordr}}{a numeric vector}
    \item{\code{numberr}}{a numeric vector}
    \item{\code{figurer}}{a numeric vector}
    \item{\code{object}}{a numeric vector}
    \item{\code{numberf}}{a numeric vector}
    \item{\code{figurew}}{a numeric vector}
    \item{\code{deduct}}{a numeric vector}
    \item{\code{numeric}}{a numeric vector}
    \item{\code{problemr}}{a numeric vector}
    \item{\code{series}}{a numeric vector}
    \item{\code{arithmet}}{a numeric vector}
    \item{\code{paperrev}}{a numeric vector}
    \item{\code{flagssub}}{a numeric vector}
  }
}

\source{
Originally from Holzinger and Swineford (1939). This copy is originally from version 4.6.0 of the MBESS package.
}

\references{
Holzinger, K. J. and Swineford, F. A. (1939). 
A study in factor analysis: The stability of a bi-factor solution. 
Supplementary Education Monographs, 48. University of Chicago.
}

\examples{
summary(HS.data)
}
\keyword{datasets}

\name{pathDiagram}

\alias{pathDiagram}
\alias{pathDiagram.sem}
\alias{pathDiagram.semmod}
\alias{math}

\title{Draw Path Diagram}
\description{
  \code{pathDiagram} creates a description of the path diagram
  for a structural-equation-model or SEM-specification object to be processed by the
  graph-drawing program \emph{dot}.
}

\usage{
pathDiagram(model, ...)

\method{pathDiagram}{sem}(model, file = "pathDiagram", 
    style = c("ram", "traditional"),
    output.type = c("html", "graphics", "dot"), graphics.fmt = "pdf", 
    dot.options = NULL,
    size = c(8, 8), node.font = c("Helvetica", 14),
    edge.font = c("Helvetica", 10), digits = 2, 
    rank.direction = c("LR", "TB"),
    min.rank = NULL, max.rank = NULL, same.rank = NULL,
    variables = model$var.names, var.labels, parameters, par.labels,
    ignore.double = TRUE, ignore.self = FALSE, error.nodes = TRUE,
    edge.labels = c("names", "values", "both"),  
    edge.colors = c("black", "black"),
    edge.weight = c("fixed", "proportional"),
    node.colors = c("transparent", "transparent", "transparent"),
    standardize = FALSE, ...)

\method{pathDiagram}{semmod}(model, obs.variables, ...)

math(text, html.only=FALSE, hat=FALSE)
}

\arguments{
  \item{model}{a structural-equation-model or SEM-specification object produced by \code{sem}, 
    or, respectively, \code{specifyEquations}, \code{specifyModel}, or \code{cfa}.}
  \item{\dots}{arguments passed down, e.g., from the \code{semmod} method to the \code{sem} method.}
  \item{file}{a file name, by default \code{"pathDiagram"}, 
    given \emph{without} an extension, to which to write the \emph{dot} description of the
    path diagram if \code{output.type} \code{"graphics"} or \code{"dot"} is selected, and for the
    graphics output file (with appropriate extension) if \code{"graphics"} output is selected, in which case
    a "\code{.dot}" file and a graphics file of type specified by the \code{graphics.fmt} argument (below);
    \code{file} may include a path specification.}
  \item{style}{\code{"ram"} (the default) for a RAM path diagram including self-directed double-headed arrows
    representing variances, including error variances; or \code{"traditional"} for a path diagram 
    including nodes representing error variables.}
  \item{output.type}{if \code{"html"} (the default), the path diagram will open in the user"s default 
    web browser; if \code{"dot"}, a file containing \emph{dot} commands will be written; if \code{"graphics"},
    both \code{.dot} and graphics files will be written.}
  \item{graphics.fmt}{a graphics format recognized by the \emph{dot} program; the default is \code{"pdf"};
  	\code{graphics.fmt} is also used for the extension of the graphics file that is created.}
  \item{dot.options}{options to be passed to the \emph{dot} program, given as a character string.}
  \item{size}{the size of the graph, in inches.}
  \item{node.font}{font name and point-size for printing variable names.}
  \item{edge.font}{font name and point-size for printing arrow names or values.}
  \item{digits}{number of digits after the decimal point (default, 2) to which 
    to round parameter estimates.}
  \item{rank.direction}{draw graph left-to-right, \code{"LR"}, the default, 
    or top-to-bottom, \code{"TB"}.}
  \item{min.rank}{a character string listing names of variables to be assigned
    minimum rank (order) in the graph; the names should be separated by commas.}
  \item{max.rank}{a character string listing names of variables to be assigned
    maximum rank in the graph; the names should be separated by commas.}
  \item{same.rank}{a character string or vector of character strings of variables
    to be assigned equivalent rank in the graph; names in each string should be
    separated by commas.}
  \item{variables}{variable names; defaults to the variable names in \code{model}.
    If specified, the variable names should be in the same order as in \code{model}.}
  \item{var.labels}{a character vector with labels to be used 
    in lieu of (some of) the variables names, for greater flexibility
    in labelling nodes in the graph --- e.g., the labels can be created with the \code{math}
    function. The elements of the vector must have names corresponding to variables in the model.}
  \item{parameters}{parameter names; defaults to the parameter names in 
    \code{model}. If specified, the parameter names should be in the same order 
    as in \code{model}.}
  \item{par.labels}{a character vector with labels to be used 
    in lieu of (some of) the parameter names, for greater flexibility
    in labelling edges in the graph --- e.g., the labels can be created with the \code{math}
    function. The elements of the vector must have names corresponding to parameters in the model.}
  \item{ignore.double}{if \code{TRUE}, the default, double-headed arrows, representing
    variances and covariances, are not graphed.}
  \item{ignore.self}{if \code{TRUE} (the default is \code{FALSE}), and \code{ignore.double=FALSE},
    self-directed double-headed arrows representing error variances are suppressed; note that
    if \code{ignore.double=TRUE}, \emph{all} double-headed arrows, including self-directed arrows,
    are suppressed.}
  \item{error.nodes}{if \code{TRUE} (the default) and \code{style="traditional"}, show the nodes
    representing error variables.}
  \item{edge.labels}{\code{"names"} to label arrows with parameter names; 
    \code{"values"} to label arrows with parameter estimates, or \code{"both"}.}
  \item{edge.colors}{two-element character vector giving colors of positive
    and negative arrows respectively; the default is \code{c("black", "black")}.}
  \item{edge.weight}{if \code{"proportional"} (the default is \code{"fixed"}), the thickness of
    edges is proportional to the absolute size of the corresponding parameter estimate;
    this is generally sensible only if \code{standardize=TRUE}.}
  \item{node.colors}{a two- or three-element character vector giving colors of nodes representing
    exogenous, endogenous, and error variables (for traditional path diagrams) consecutively; 
    the default is \code{"transparent"} for all three; if a two colors are given, error variables
    are colored as exogenous (the first color.}
  \item{standardize}{if \code{TRUE}, display standardized coefficients; default is \code{FALSE}.}

  \item{obs.variables}{a character vector with the names of the observed variables in the model.}
  
  \item{text}{a character string or vector of character strings to be translated into node or edge label symbols.
    If a vector of character strings is supplied, then the elements of the vector should be named with the
    corresponding variable (node) or parameter (edge) name.}
  \item{html.only}{If \code{TRUE} (the default is \code{FALSE}), the
    character strings in \code{text} are to be treated as an HTML character codes, in
    which case the prefix \code{"#"} and suffix \code{";"} are appended to each. Otherwise, \code{text} should
    only contain the names of lowercase or uppercase Greek letters, such as \code{"alpha"} or \code{"Alpha"}.
    The full set of Greek letters recognized is given in the file \code{Greek.txt} in the package's
    \code{etc} subdirectory -- or type \code{sem:::Greek} at the R command prompt.
    In either case, the symbols may be followed by numeric subscripts in curly braces consisting of numerals 
    (e.g., \code{"beta_{12}"}),
    and/or numeric superscripts (e.g., \code{"sigma^{2}"},
    \code{"sigma_{1}^{2}"}). Depending upon your OS, subscripts and superscripts may only work properly with HTML output
    from \code{pathDiagram}, not with graphics output produced by \emph{dot}.}
  \item{hat}{If \code{TRUE} (the default is \code{FALSE}), a hat (circumflex) is placed over the
    symbols in \code{text}; this feature doesn't produce a visually appealing result.}
}

\details{
\code{pathDiagram} creates a description of the path diagram
  for a structural-equation-model or SEM-specification object to be processed by the
  graph-drawing program \emph{dot}, which can be called
  automatically; see Koutsofios and North (2002)
  and \url{http://www.graphviz.org/}. To obtain graphics output
  directly, the \emph{dot} program must be on the system search path. 
  
  Alternatively, \emph{HTML} output can be created in a web browser without an independent installation
  of \code{dot} using facilities in the \pkg{DiagrammeR} package.
  
  The \code{math} function can be used to create node (variable) and edge (arrow) labels with
  symbols such as Greek letters, subscripts, and superscripts.
  
  The \code{semmod} method of \code{pathDiagram} sets up a call to the \code{sem} method. 
  
  The various
  arguments to \code{pathDiagram} can be used to customize the diagram, but if there are too many constraints
  on node placement, \emph{dot} may fail to produce a graph or may produce a distorted graph. 
  \code{pathDiagram} can create both RAM-style diagrams, in which variances are represented as self-directed
  arrows, and traditional path diagrams, in which error variables appear explicitly as nodes. As is conventional,
  latent variables (including error variables) are represented as ellipses and observed variables as 
  rectangles; double-headed arrows represent covariances (and in RAM diagrams, variances) and single-headed
  arrows represent structural coefficients.
}

\value{
  \code{pathDiagram} invisibly returns a character vector containing \emph{dot} commands.
  \code{math} returns a character vector containing suitable \code{HTML} markup.
}

\references{
    Koutsofios, E., and North, S. C. (2002)
    Drawing graphs with \emph{dot}.
    \url{https://graphviz.org/documentation/}.    
}

\author{John Fox \email{jfox@mcmaster.ca}, Adam Kramer, and Michael Friendly}

\seealso{\code{\link{sem}}, \code{\link{specifyEquations}}, \code{\link{specifyModel}}, \code{\link{cfa}}}

\examples{
	if (interactive()) {
# The Duncan, Haller, and Portes Peer-Influences Model

R.DHP <- readMoments(diag=FALSE, names=c("ROccAsp", "REdAsp", "FOccAsp", 
                "FEdAsp", "RParAsp", "RIQ", "RSES", "FSES", "FIQ", "FParAsp"),
                text="
    .6247                                                              
    .3269  .3669                                                        
    .4216  .3275  .6404                                      
    .2137  .2742  .1124  .0839                                
    .4105  .4043  .2903  .2598  .1839                          
    .3240  .4047  .3054  .2786  .0489  .2220                    
    .2930  .2407  .4105  .3607  .0186  .1861  .2707              
    .2995  .2863  .5191  .5007  .0782  .3355  .2302  .2950        
    .0760  .0702  .2784  .1988  .1147  .1021  .0931 -.0438  .2087  
")

model.dhp <- specifyModel(text="
    RParAsp  -> RGenAsp, gam11,  NA
    RIQ      -> RGenAsp, gam12,  NA
    RSES     -> RGenAsp, gam13,  NA
    FSES     -> RGenAsp, gam14,  NA
    RSES     -> FGenAsp, gam23,  NA
    FSES     -> FGenAsp, gam24,  NA
    FIQ      -> FGenAsp, gam25,  NA
    FParAsp  -> FGenAsp, gam26,  NA
    FGenAsp  -> RGenAsp, beta12, NA
    RGenAsp  -> FGenAsp, beta21, NA
    RGenAsp  -> ROccAsp,  NA,       1
    RGenAsp  -> REdAsp,  lam21,  NA
    FGenAsp  -> FOccAsp,  NA,       1
    FGenAsp  -> FEdAsp,  lam42,  NA
    RGenAsp <-> RGenAsp, ps11,   NA
    FGenAsp <-> FGenAsp, ps22,   NA
    RGenAsp <-> FGenAsp, ps12,   NA
    ROccAsp <-> ROccAsp, theta1, NA
    REdAsp  <-> REdAsp,  theta2, NA
    FOccAsp <-> FOccAsp, theta3, NA
    FEdAsp  <-> FEdAsp,  theta4, NA
")

sem.dhp <- sem(model.dhp, R.DHP, 329,
    fixed.x=c("RParAsp", "RIQ", "RSES", "FSES", "FIQ", "FParAsp"))
    
pathDiagram(sem.dhp, min.rank="RIQ, RSES, RParAsp, FParAsp, FSES, FIQ", 
                     max.rank="ROccAsp, REdAsp, FEdAsp, FOccAsp",
                     same.rank="RGenAsp, FGenAsp",
                     edge.labels="values")
    
pathDiagram(model.dhp,
    obs.variables=c("RParAsp", "RIQ", "RSES", "FSES", "FIQ", 
        "FParAsp", "ROccAsp", "REdAsp", "FOccAsp", "FEdAsp"),
    style="traditional", 
    node.colors=c("pink", "lightblue", "lightgreen"),
    min.rank="RIQ, RSES, RParAsp, FParAsp, FSES, FIQ",
    max.rank="ROccAsp, REdAsp, FEdAsp, FOccAsp",
    same.rank="RGenAsp, FGenAsp",
    var.labels=c(RParAsp="Respondent Parental Aspiration", 
        RIQ="Respondent IQ",
        RSES="Respondent SES",
        FSES="Friend SES",
        FIQ="Friend IQ",
        FParAsp="Friend Parental Aspiration",
        ROccAsp="Respondent Occupational Aspiration",
        REdAsp="Respondent Educational Aspiration",
        RGenAsp="Respondent General Aspiration",
        FOccAsp="Friend Occupational Aspiration",
        FEdAsp="Friend Educational Aspiration",
        FGenAsp="Friend General Aspiration",
        math(c(RGenAsp.error="xi_{1}",
        FGenAsp.error="xi_{2}",
        ROccAsp.error="epsilon_{1}",
        REdAsp.error="epsilon_{2}",
        FOccAsp.error="epsilon_{3}",
        FEdAsp.error="epsilon_{4}"))),
    par.labels=math(c(gam11="gamma_{11}",
        gam12="gamma_{12}",
        gam13="gamma_{13}",
        gam14="gamma_{14}",
        gam23="gamma_{23}",
        gam24="gamma_{24}",
        gam25="gamma_{25}",
        gam26="gamma_{26}",
        beta12="beta_{12}",
        beta21="beta_{21}",
        lam21="lambda_{21}",
        lam42="lambda_{42}",
        ps11="psi_{11}",
        ps22="psi_{22}",
        ps12="psi_{12}",
        theta1="theta_{1}",
        theta2="theta_{2}",
        theta3="theta_{3}",
        theta4="theta_{4}")))
        
    # the following example contributed by Michael Friendly:
    
union <- readMoments(diag=TRUE,
    names=c('y1', 'y2', 'y3', 'x1', 'x2'), text="
14.610
-5.250  11.017
-8.057  11.087   31.971
-0.482   0.677    1.559   1.021
-18.857  17.861   28.250   7.139  215.662
")

union.mod <- specifyEquations(covs=c("x1, x2"), text="
y1 = gam12*x2
y2 = beta21*y1 + gam22*x2
y3 = beta31*y1 + beta32*y2 + gam31*x1
")

union.sem <- sem(union.mod, union, N=173)

dot <- pathDiagram(union.sem, style="traditional",  
    ignore.double=FALSE, error.nodes=FALSE,
    edge.labels="values", 
    min.rank=c("Years", "Age"), 
    max.rank=c("Sentiment", "Sentiment.error"),
    same.rank=c("Deference, Deference.error", "Activism, Activism.error"),
    variables=c("Deference", "Activism", "Sentiment", "Years", "Age"),
    edge.colors=c("black", "red"),
    node.colors = c("pink", "lightblue"))

cat(paste(dot, collapse="\n")) # dot commands

    }
}

\keyword{dplot}
\keyword{models}

\name{fscores}
\alias{fscores}
\alias{fscores.sem}
\alias{fscores.msem}

\title{Factor Scores for Latent Variables}
\description{
  Calculate factor scores or factor-score coefficients for the latent variables
  in a structural-equation model.
}
\usage{
\method{fscores}{sem}(model, data=model$data, center=TRUE, scale=FALSE, ...)
\method{fscores}{msem}(model, data=model$data, center=TRUE, scale=FALSE, ...)
}

\arguments{
  \item{model}{an object of class \code{"sem"} or \code{"msem"}, produced by the \code{\link{sem}}
    function.}
  \item{data}{an optional numeric data frame or matrix containing the observed variables
    in the model; if not \code{NULL}, the estimated factor scores are returned; if \code{NULL}, the
    factor-score \emph{coefficients} are returned. The default is the \code{data} element of \code{model},
    which is non-\code{NULL} if the model was fit to a data set rather than a covariance or moment matrix.}
  \item{center}{if \code{TRUE}, the default, the means of the observed variables are
    subtracted prior to computing factor scores. One would normally use this option
    if the model is estimated from a covariance or correlation matrix among the
    observed variables.}
  \item{scale}{if \code{TRUE}, the possibly centered variables are divided by their
    root-mean-squares; the default is \code{FALSE}.
    One would normally use this option if the model is estimated
    from a correlation matrix among the observed variables. Centering and scaling 
    are performed by the \code{\link[base]{scale}} function.}
  \item{\dots}{arguments to pass down.}
}
\details{
  Factor-score coefficients are computed by the \dQuote{regression} method as
  \eqn{B = C^{-1} C^{*}}{C^-1 C*}, where \eqn{C} is the model-implied covariance or
  moment matrix among the observed variables and  \eqn{C^{*}}{C*} is the matrix
  of model-implied covariances or moments between the observed and latent variables.
}
\value{
  Either a matrix of estimated factor scores (if the \code{data} argument is
  supplied) or a matrix of factor-score coefficients (otherwise). In the case of an \code{"msem"}
  argument, a list of matrices is returned.
}

\references{
    Bollen, K. A. (1989) 
    \emph{Structural Equations With Latent Variables.} Wiley.
}

\author{John Fox \email{jfox@mcmaster.ca}}

\seealso{\code{\link{sem}}, \code{\link[base]{scale}}}

\examples{

# In the first example, readMoments() and specifyModel() read from the
# input stream. This example cannot be executed via example() but can be entered
# at the command prompt. The example is repeated using file input;
# this example can be executed via example(). 
	   \dontrun{

S.wh <- readMoments(names=c('Anomia67','Powerless67','Anomia71',
                                    'Powerless71','Education','SEI'))
   11.834                                    
    6.947    9.364                            
    6.819    5.091   12.532                    
    4.783    5.028    7.495    9.986            
   -3.839   -3.889   -3.841   -3.625   9.610     
  -21.899  -18.831  -21.748  -18.775  35.522  450.288

# This model in the SAS manual for PROC CALIS

model.wh.1 <- specifyModel()
    Alienation67   ->  Anomia67,      NA,     1
    Alienation67   ->  Powerless67,   NA,     0.833
    Alienation71   ->  Anomia71,      NA,     1
    Alienation71   ->  Powerless71,   NA,     0.833 
    SES            ->  Education,     NA,     1     
    SES            ->  SEI,           lamb,   NA
    SES            ->  Alienation67,  gam1,   NA
    Alienation67   ->  Alienation71,  beta,   NA
    SES            ->  Alienation71,  gam2,   NA
    Anomia67       <-> Anomia67,      the1,   NA
    Anomia71       <-> Anomia71,      the1,   NA
    Powerless67    <-> Powerless67,   the2,   NA
    Powerless71    <-> Powerless71,   the2,   NA
    Education      <-> Education,     the3,   NA
    SEI            <-> SEI,           the4,   NA
    Anomia67       <-> Anomia71,      the5,   NA
    Powerless67    <-> Powerless71,   the5,   NA
    Alienation67   <-> Alienation67,  psi1,   NA
    Alienation71   <-> Alienation71,  psi2,   NA
    SES            <-> SES,           phi,    NA
    
                        
sem.wh.1 <- sem(model.wh.1, S.wh, 932)

fscores(sem.wh.1)
   }

# The following example can be executed via example():

etc <- system.file(package="sem", "etc") # path to data and model files
   
(S.wh <- readMoments(file=file.path(etc, "S-Wheaton.txt"),
					names=c('Anomia67','Powerless67','Anomia71',
                            'Powerless71','Education','SEI')))
(model.wh.1 <- specifyModel(file=file.path(etc, "model-Wheaton-1.txt")))        
(sem.wh.1 <- sem(model.wh.1, S.wh, 932))
fscores(sem.wh.1)
}

\keyword{models}
\keyword{manip}

\name{specifyModel}
\alias{specifyModel}
\alias{specifyEquations}
\alias{cfa}
\alias{multigroupModel}
\alias{print.semmod}
\alias{print.semmodList}
\alias{classifyVariables}
\alias{removeRedundantPaths}
\alias{combineModels}
\alias{combineModels.semmod}
\alias{update.semmod}
\alias{edit.semmod}

\title{Specify a Structural Equation Model}
\description{
  Create the RAM specification of a structural equation model.
}
\usage{
specifyModel(file="", text, exog.variances=FALSE, endog.variances=TRUE, covs, 
	suffix="", quiet=FALSE)

specifyEquations(file="", text, ...)

cfa(file="", text, covs=paste(factors, collapse=","), 
    reference.indicators=TRUE, raw=FALSE, 
    subscript=c("name", "number"), ...)

multigroupModel(..., groups=names(models), allEqual=FALSE)

classifyVariables(model)

removeRedundantPaths(model, warn=TRUE)
\method{combineModels}{semmod}(..., warn=TRUE)
\method{update}{semmod}(object, file = "", text, ...)
\method{edit}{semmod}(name, ...)

\method{print}{semmod}(x, ...)
\method{print}{semmodList}(x, ...)
}

\arguments{
  \item{file}{The (quoted) file from which to read the model specification,
    including the path to the file if it is not in the current directory. If
    \code{""} (the default) and the \code{text} argument is not supplied,
    then the specification is read from the standard
    input stream, and is terminated by a blank line.}
  \item{text}{The model specification given as a character string, as an alternative
    to specifying the ]code{file} argument or reading the model specification from
    the input stream --- e.g., when the session is not interactive and there is no
    standard input.}
  \item{exog.variances}{If \code{TRUE} (the default is \code{FALSE}), free variance
  	parameters are added for the exogenous variables that lack them.}
  \item{endog.variances}{If \code{TRUE} (the default), free error-variance parameters
    are added for the endogenous variables that lack them.}
  \item{covs}{optional: a character vector of one or more elements, with each element
  	giving a string of variable names, separated by commas. Variances and covariances
  	among all variables in each such string are added to the model. For confirmatory
  	factor analysis models specified via \code{cfa}, \code{covs} defaults to all of
  	the factors in the model, thus specifying all variances and covariances among these factors.
  	\emph{Warning}: \code{covs="x1, x2"} and \code{covs=c("x1", "x2")} are \emph{not}
  	equivalent: \code{covs="x1, x2"} specifies the variance of \code{x1}, the variance
  	of \code{x2}, \emph{and} their covariance, while \code{covs=c("x1", "x2")} specifies
  	the variance of \code{x1} and the variance of \code{x2} \emph{but not} their covariance.}
  \item{suffix}{a character string (defaulting to an empty string) to be 
    appended to each parameter name; this can be convenient for specifying multiple-group models.}
  \item{reference.indicators}{if \code{FALSE}, the default, variances of factors are
    set to 1 by \code{cfa}; if \code{TRUE}, variances of factors are free parameters
    to estimate from the data, and instead the first factor loading for each factor is set
    to 1 to identify the model.}
  \item{raw}{if \code{TRUE} (the default is \code{FALSE}), a path from \code{Intercept} to each
    observed variable is added to the model, and the raw second moment for \code{Intercept} is
    fixed to \code{1}. The \code{sem} function should then be called with \code{raw=TRUE}, and either
    supplied with a data set (via the \code{data} argument) or a raw-moment matrix (via the \code{S}
    argument).}
  \item{subscript}{The \dQuote{subscripts} to be appended to \code{lam} to name factor-loading parameters,
    either \code{"name"} (the default) to use the names of observed variables, or \code{"number"} to
    number the parameters serially within each factor. Using \code{"number"} produces shorter parameter
    names.}
  \item{quiet}{if \code{FALSE}, the default, then the number of input lines is reported and
    a message is printed suggesting that \code{specifyEquations} or \code{cfa} be used.}
  \item{x, model, object, name}{An object of class \code{semmod} or \code{semmodList}, as produced by
    \code{specifyModel} or \code{multigroupModel}.}
  \item{warn}{print a warning if redundant paths are detected.}
  \item{...}{For \code{multigroupModel}, one or more optionally named arguments each of
     which is a \code{semmod} object produced, e.g., by \code{specifyModel}, \code{specifyEquations},
     or \code{cfa}; if only one such model is given, then it will be used for all groups defined by
     the \code{groups} argument. If parameters have the same name in different groups, then they will
     be constrained to be equal.
     For \code{specifyEquations} and \code{cfa}, arguments (such as \code{covs},
  	 in the case of \code{specifyEquations}) to be 
     passed to \code{specifyModel}; for \code{combineModels}, \code{sem} objects; 
     ignored in the \code{update} and \code{print} methods.}
  \item{groups}{a character vector of names for the groups in a multigroup model; taken by default from
  	 the names of the \code{...} arguments.}
  \item{allEqual}{if \code{FALSE} (the default), then if only one model object is given for
  	 a multigroup model, all corresponding parameters in the groups will be distinct; if \code{TRUE},
  	 all corresponding parameters will be constrained to be equal.}
}

\details{
  The principal functions for model specification are \code{specifyModel},
  to specify a model in RAM (path) format via single- and double-headed arrows;
  \code{specifyEquations}, to specify a model in equation format, which is then
  translated by the function into RAM format; and \code{cfa}, for compact
  specification of simple confirmatory factor analysis models.
  
  \code{specifyModel}:
  
  Each line of the RAM specification for \code{specifyModel} consists of three (unquoted) entries, 
  separated by commas:
  
    \describe{
      \item{1. Arrow specification:}{This is a simple formula, of the form
        \code{A -> B} or, equivalently, \code{B <- A} for a regression
        coefficient (i.e., a single-headed or directional arrow); 
        \code{A <-> A} for a variance or \code{A <-> B} for a covariance
        (i.e., a double-headed or bidirectional arrow). Here, \code{A} and
        \code{B} are variable names in the model. If a name does not correspond
        to an observed variable, then it is assumed to be a latent variable.
        Spaces can appear freely in an arrow specification, and
        there can be any number of hyphens in the arrows, including zero: Thus,
        e.g., \code{A->B}, \code{A --> B}, and \code{A>B} are all legitimate
        and equivalent.}
      \item{2. Parameter name:}{The name of the regression coefficient, variance,
        or covariance specified by the arrow. Assigning the same name to two or
        more arrows results in an equality constraint. Specifying the parameter name
        as \code{NA} produces a fixed parameter.}
      \item{3. Value:}{start value for a free parameter or value of a fixed parameter.
        If given as \code{NA} (or simply omitted), \code{sem} will compute the start value.}
      }
      
  Lines may end in a comment following \code{#}.
  
  \code{specifyEquations}:

  For \code{specifyEquations}, each input line is either a regression equation or the specification
  of a variance or covariance. Regression equations are of the form
  
  \code{y = par1*x1 + par2*x2 + ... + park*xk}
  
  where \code{y} and the \code{x}s are variables in the model (either observed or latent),
  and the \code{par}s are parameters. If a parameter is given as a numeric value (e.g.,
  \code{1}) then it is treated as fixed. Note that no \dQuote{error} variable is included in
  the equation; \dQuote{error variances} are specified via either the \code{covs} argument,
  via \code{V(y) = par} (see immediately below), or are added automatically to the model
  when, as by default, \code{endog.variances=TRUE}. A regression equation may be split over more
  than one input by breaking at a \code{+}, so that \code{+} is either the last non-blank character
  on a line or the first non-blank character on the subsequent line.
  
  Variances are specified in the form \code{V(var) = par} and covariances in the form
  \code{C(var1, var2) = par}, where the \code{var}s are variables (observed or unobserved) in
  the model. The symbols \code{V} and \code{C} may be in either lower- or upper-case. If \code{par}
  is a numeric value (e.g., \code{1}) then it is treated as fixed. In conformity with the RAM model,
  a variance or covariance for an endogenous variable in the model is an \dQuote{error} variance or
  covariance.
  
  \emph{Warning}: If the \code{covs} argument to \code{specifyEquations} is used to specify
  variances and covariances, please be aware that
  \code{covs="x1, x2"} and \code{covs=c("x1", "x2")} are \emph{not}
  equivalent: \code{covs="x1, x2"} specifies the variance of \code{x1}, the variance
  of \code{x2}, \emph{and} their covariance, while \code{covs=c("x1", "x2")} specifies
  the variance of \code{x1} and the variance of \code{x2} \emph{but not} their covariance.
  
  To set a start value for a free parameter, enclose the numeric start value in parentheses after the
  parameter name, as \code{parameter(value)}.
  
  \code{cfa}:
  
  For \code{cfa}, each input line includes the names of the variables, separated by commas, 
  that load on the corresponding factor; the name of the factor is given optionally at the beginning
  of the line, followed by a colon. If necessary, the variables that load on a factor may be continued
  across two or more input lines; in this case, each such line but the last must end in a comma. A
  variable may load on more than one factor (as long as the resulting model is identified, of course),
  but each factor may appear in only one input line (or set of input lines, if the variable list
  is continued onto the next line). 
  
  Equality constraints for factor loadings can be set by using equal-signs (\code{=}) rather than commas 
  to separate observed variable names. For example, \code{fac1: x1=x2=x3, x4=x5} sets the loadings
  for \code{x1}, \code{x2}, and \code{x3} equal to each other, and the loadings for \code{x4} and \code{x5}
  equal to each other. 
  
  Equality constraints among error variances can similarly be specified by using \code{var:} or \code{variance:}
  at the beginning of a line (actually, any character string beginning with \code{var} will do, and thus
  no factor name may begin with the characters \code{var}). For example, \code{var: x1=x2=x3, x4=x5} sets the
  error variances for \code{x1}, \code{x2}, and \code{x3} equal to each other, and the 
  error variances for \code{x4} and \code{x5} equal to each other. There may be several lines beginning with
  \code{var:}.
  
  If the argument \code{reference.indicators=FALSE}, the default,
  \code{cfa} will fix the variance of each factor to 1, and by
  default include covariances (i.e., correlations) among all pairs of factors. Alternatively,
  if \code{reference.indicators=TRUE}, then the factor variances are free parameters to be estimated
  from the data, and the first loading for each factor is set to 1 to identify the model. These two
  approaches produce equivalent models, with the same fit to the data, but alternative parametrizations.
  Specifying the argument \code{covs=NULL} implicitly fixes the factor intercorrelations to 0.
  
  See \code{\link{sem}} and the examples for further details on model specification.
  
  Other Functions:
  
  \code{classifyVariables} classifies the variables in a model as endogenous or exogenous.
  
  \code{combineModels} and \code{removeRedundantPaths} take \code{semmod} objects as arguments and do what their names imply.
  
  The \code{file} input argument to the \code{update} method for \code{semmod} objects, which by default comes from
  standard input, is a set of update directives, one per line. There are five kinds of directives. In each case
  the directive begins with the directive name, followed by one or more fields separated by commas.
  
  \describe{
  
  	\item{1. delete:}{Remove a path from the model. Example: \code{delete, RSES -> FGenAsp}}
  	
  	\item{2. add:}{Add a path to the model. Example (the \code{NA} for the start value is optional): \code{add, RSES -> FGenAsp, gam14,  NA}}
  	
  	\item{3. replace:}{Replace every occurrence of the first string with the second in the variables and
  		parameters of the model. This directive may be used, for example, to change one variable to
  		another or to rename a parameter. Example: \code{replace, gam, gamma}, substitutes the string \code{"gamma"}
  		for \code{"gam"} wherever the latter appears, presumably in parameter names.}
  		
  	\item{4. fix:}{Fix a parameter that was formerly free. Example: \code{fix, RGenAsp  -> REdAsp, 1}}
  	
  	\item{5. free:}{Free a parameter that was formerly fixed. Example (the \code{NA} for the start value is optional): 
  	\code{free, RGenAsp  -> ROccAsp, lam11, NA}}
  }
  
  The \code{edit} method for \code{semmod} objects opens the model in the R editor.
}

\value{
  \code{specifyModel}, \code{specifyEquations}, \code{cfa}, \code{removeRedundantPaths}, \code{combineModels}, 
  \code{update}, and \code{edit} return an object of class \code{semmod}, suitable as input for \code{\link{sem}}.
  
  \code{multigroupModel} returns an object of class \code{semmodList}, also suitable as input for \code{\link{sem}}.
  
  \code{classifyVariables} returns a list with two character vectors: \code{endogenous}, containing the names of endogenous
  variables in the model; and \code{exogenous}, containing the names of exogenous variables.
}

\author{John Fox \email{jfox@mcmaster.ca} and Jarrett Byrnes}

\seealso{\code{\link{sem}}}

\examples{

# example using the text argument:

model.dhp <- specifyModel(text="
    RParAsp  -> RGenAsp, gam11,  NA
    RIQ      -> RGenAsp, gam12,  NA
    RSES     -> RGenAsp, gam13,  NA
    FSES     -> RGenAsp, gam14,  NA
    RSES     -> FGenAsp, gam23,  NA
    FSES     -> FGenAsp, gam24,  NA
    FIQ      -> FGenAsp, gam25,  NA
    FParAsp  -> FGenAsp, gam26,  NA
    FGenAsp  -> RGenAsp, beta12, NA
    RGenAsp  -> FGenAsp, beta21, NA
    RGenAsp  -> ROccAsp,  NA,     1
    RGenAsp  -> REdAsp,  lam21,  NA
    FGenAsp  -> FOccAsp,  NA,     1
    FGenAsp  -> FEdAsp,  lam42,  NA
    RGenAsp <-> RGenAsp, ps11,   NA
    FGenAsp <-> FGenAsp, ps22,   NA
    RGenAsp <-> FGenAsp, ps12,   NA
    ROccAsp <-> ROccAsp, theta1, NA
    REdAsp  <-> REdAsp,  theta2, NA
    FOccAsp <-> FOccAsp, theta3, NA
    FEdAsp  <-> FEdAsp,  theta4, NA
")    
model.dhp

   # same model in equation form:
model.dhp.1 <- specifyEquations(covs="RGenAsp, FGenAsp", text="
RGenAsp = gam11*RParAsp + gam12*RIQ + gam13*RSES + gam14*FSES + beta12*FGenAsp
FGenAsp = gam23*RSES + gam24*FSES + gam25*FIQ + gam26*FParAsp + beta21*RGenAsp
ROccAsp = 1*RGenAsp
REdAsp = lam21(1)*RGenAsp  # to illustrate setting start values
FOccAsp = 1*FGenAsp
FEdAsp = lam42(1)*FGenAsp
")
model.dhp

# Note: The following examples can't be run via example() because the 
#  default file argument requires that the model specification be entered
#  at the command prompt. The examples can be copied and run in an interactive 
#  session in the R console, however.

    \dontrun{
model.dhp <- specifyModel()
    RParAsp  -> RGenAsp, gam11,  NA
    RIQ      -> RGenAsp, gam12,  NA
    RSES     -> RGenAsp, gam13,  NA
    FSES     -> RGenAsp, gam14,  NA
    RSES     -> FGenAsp, gam23,  NA
    FSES     -> FGenAsp, gam24,  NA
    FIQ      -> FGenAsp, gam25,  NA
    FParAsp  -> FGenAsp, gam26,  NA
    FGenAsp  -> RGenAsp, beta12, NA
    RGenAsp  -> FGenAsp, beta21, NA
    RGenAsp  -> ROccAsp,  NA,     1
    RGenAsp  -> REdAsp,  lam21,  NA
    FGenAsp  -> FOccAsp,  NA,     1
    FGenAsp  -> FEdAsp,  lam42,  NA
    RGenAsp <-> RGenAsp, ps11,   NA
    FGenAsp <-> FGenAsp, ps22,   NA
    RGenAsp <-> FGenAsp, ps12,   NA
    ROccAsp <-> ROccAsp, theta1, NA
    REdAsp  <-> REdAsp,  theta2, NA
    FOccAsp <-> FOccAsp, theta3, NA
    FEdAsp  <-> FEdAsp,  theta4, NA
    
model.dhp
    
# an equivalent specification, allowing specifyModel() to generate
#  variance parameters for endogenous variables (and suppressing
#  the unnecessary trailing NAs):
 
model.dhp <- specifyModel()
RParAsp  -> RGenAsp, gam11
RIQ      -> RGenAsp, gam12
RSES     -> RGenAsp, gam13
FSES     -> RGenAsp, gam14
RSES     -> FGenAsp, gam23
FSES     -> FGenAsp, gam24
FIQ      -> FGenAsp, gam25
FParAsp  -> FGenAsp, gam26
FGenAsp  -> RGenAsp, beta12
RGenAsp  -> FGenAsp, beta21
RGenAsp  -> ROccAsp,  NA,     1
RGenAsp  -> REdAsp,  lam21
FGenAsp  -> FOccAsp,  NA,     1
FGenAsp  -> FEdAsp,  lam42
RGenAsp <-> FGenAsp, ps12

model.dhp

# Another equivalent specification, telling specifyModel to add paths for 
#   variances and covariance of RGenAsp and FGenAsp:
 
model.dhp <- specifyModel(covs="RGenAsp, FGenAsp")
RParAsp  -> RGenAsp, gam11
RIQ      -> RGenAsp, gam12
RSES     -> RGenAsp, gam13
FSES     -> RGenAsp, gam14
RSES     -> FGenAsp, gam23
FSES     -> FGenAsp, gam24
FIQ      -> FGenAsp, gam25
FParAsp  -> FGenAsp, gam26
FGenAsp  -> RGenAsp, beta12
RGenAsp  -> FGenAsp, beta21
RGenAsp  -> ROccAsp,  NA,     1
RGenAsp  -> REdAsp,  lam21
FGenAsp  -> FOccAsp,  NA,     1
FGenAsp  -> FEdAsp,  lam42

model.dhp

# The same model in equation format:

model.dhp.1 <- specifyEquations(covs="RGenAsp, FGenAsp")
RGenAsp = gam11*RParAsp + gam12*RIQ + gam13*RSES + gam14*FSES + beta12*FGenAsp
FGenAsp = gam23*RSES + gam24*FSES + gam25*FIQ + gam26*FParAsp + beta21*RGenAsp
ROccAsp = 1*RGenAsp
REdAsp = lam21(1)*RGenAsp  # to illustrate setting start values
FOccAsp = 1*FGenAsp
FEdAsp = lam42(1)*FGenAsp

model.dhp

classifyVariables(model.dhp)

# updating the model to impose equality constraints
#  and to rename the latent variables and gamma parameters

model.dhp.eq <- update(model.dhp)
delete, RSES -> FGenAsp
delete, FSES -> FGenAsp
delete, FIQ  -> FGenAsp
delete, FParAsp -> FGenAs
delete, RGenAsp  -> FGenAsp
add, RSES     -> FGenAsp, gam14,  NA
add, FSES     -> FGenAsp, gam13,  NA
add, FIQ      -> FGenAsp, gam12,  NA
add, FParAsp  -> FGenAsp, gam26,  NA
add, RGenAsp  -> FGenAsp, beta12, NA
replace, gam, gamma
replace, Gen, General

model.dhp.eq

# A three-factor CFA model for the Thurstone mental-tests data, 
#    specified three equivalent ways:

R.thur <- readMoments(diag=FALSE, 
    names=c('Sentences','Vocabulary',
            'Sent.Completion','First.Letters','4.Letter.Words','Suffixes',
            'Letter.Series','Pedigrees', 'Letter.Group'))
.828                                              
.776   .779                                        
.439   .493    .46                                 
.432   .464    .425   .674                           
.447   .489    .443   .59    .541                    
.447   .432    .401   .381    .402   .288              
.541   .537    .534   .35    .367   .32   .555        
.38   .358    .359   .424    .446   .325   .598   .452

	#  (1a) in CFA format:

mod.cfa.thur.c <- cfa(reference.indicators=FALSE)
FA: Sentences, Vocabulary, Sent.Completion
FB: First.Letters, 4.Letter.Words, Suffixes
FC: Letter.Series, Pedigrees, Letter.Group

cfa.thur.c <- sem(mod.cfa.thur.c, R.thur, 213)
summary(cfa.thur.c)

	#  (1b) in CFA format, using reference indicators:
	
mod.cfa.thur.r <- cfa()
FA: Sentences, Vocabulary, Sent.Completion
FB: First.Letters, 4.Letter.Words, Suffixes
FC: Letter.Series, Pedigrees, Letter.Group

cfa.thur.r <- sem(mod.cfa.thur.r, R.thur, 213)
summary(cfa.thur.r)

	#  (2) in equation format:

mod.cfa.thur.e <- specifyEquations(covs="F1, F2, F3")
Sentences = lam11*F1
Vocabulary = lam21*F1
Sent.Completion = lam31*F1
First.Letters = lam42*F2
4.Letter.Words = lam52*F2
Suffixes = lam62*F2
Letter.Series = lam73*F3
Pedigrees = lam83*F3
Letter.Group = lam93*F3
V(F1) = 1
V(F2) = 1
V(F3) = 1

cfa.thur.e <- sem(mod.cfa.thur.e, R.thur, 213)
summary(cfa.thur.e)

	#  (3) in path format:

mod.cfa.thur.p <- specifyModel(covs="F1, F2, F3")
F1 -> Sentences,                      lam11
F1 -> Vocabulary,                     lam21
F1 -> Sent.Completion,                lam31
F2 -> First.Letters,                  lam41
F2 -> 4.Letter.Words,                 lam52
F2 -> Suffixes,                       lam62
F3 -> Letter.Series,                  lam73
F3 -> Pedigrees,                      lam83
F3 -> Letter.Group,                   lam93
F1 <-> F1,                            NA,     1
F2 <-> F2,                            NA,     1
F3 <-> F3,                            NA,     1

cfa.thur.p <- sem(mod.cfa.thur.p, R.thur, 213)
summary(cfa.thur.p)

# The Thursstone CFA model with equality constraints on the
#  factor loadings and error variances

mod.cfa.thur.ceq <- cfa(reference.indicators=FALSE)
FA: Sentences = Vocabulary = Sent.Completion
FB: First.Letters = 4.Letter.Words = Suffixes
FC: Letter.Series = Pedigrees = Letter.Group
var: Sentences = Vocabulary = Sent.Completion
var: First.Letters = 4.Letter.Words = Suffixes
var: Letter.Series = Pedigrees = Letter.Group

cfa.thur.ceq <- sem(mod.cfa.thur.ceq, R.thur, 213)
summary(cfa.thur.ceq)
anova(cfa.thur.c, cfa.thur.ceq)
pathDiagram(cfa.thur.ceq, ignore.double=FALSE, ignore.self=TRUE,
    min.rank="FA, FB, FC", edge.labels="values")

# a multigroup CFA model fit to the Holzinger-Swineford
#   mental-tests data 

mod.hs <- cfa()
spatial: visual, cubes, paper, flags
verbal: general, paragrap, sentence, wordc, wordm
memory: wordr, numberr, figurer, object, numberf, figurew
math: deduct, numeric, problemr, series, arithmet

mod.mg <- multigroupModel(mod.hs, groups=c("Female", "Male")) 

sem.mg <- sem(mod.mg, data=HS.data, group="Gender",
              formula = ~ visual + cubes + paper + flags +
              general + paragrap + sentence + wordc + wordm +
              wordr + numberr + figurer + object + numberf + figurew +
              deduct + numeric + problemr + series + arithmet
              )
summary(sem.mg)

	# with cross-group equality constraints:
	
mod.mg.eq <- multigroupModel(mod.hs, groups=c("Female", "Male"), allEqual=TRUE)

sem.mg.eq <- sem(mod.mg.eq, data=HS.data, group="Gender",
              formula = ~ visual + cubes + paper + flags +
                general + paragrap + sentence + wordc + wordm +
                wordr + numberr + figurer + object + numberf + figurew +
                deduct + numeric + problemr + series + arithmet
              )
summary(sem.mg.eq)
    }
}

\keyword{models}

\name{residuals.sem}
\alias{residuals.sem}
\alias{residuals.msem}
\alias{standardizedResiduals}
\alias{standardizedResiduals.sem}
\alias{standardizedResiduals.msem}
\alias{normalizedResiduals}
\alias{normalizedResiduals.objectiveML}
\alias{normalizedResiduals.objectiveGLS}
\alias{normalizedResiduals.msemObjectiveML}

\title{Residual Covariances for a Structural Equation Model}
\description{
  These functions compute residual covariances, variance-standardized
  residual covariances, and normalized residual covariances
  for the observed variables in a structural-equation
  model fit by \code{sem}.
}
\usage{
\method{residuals}{sem}(object, ...)
\method{residuals}{msem}(object, ...)

\method{standardizedResiduals}{sem}(object, ...)
\method{standardizedResiduals}{msem}(object, ...)

\method{normalizedResiduals}{objectiveML}(object, ...)
\method{normalizedResiduals}{objectiveGLS}(object, ...)
\method{normalizedResiduals}{msemObjectiveML}(object, ...)
}

\arguments{
  \item{object}{an object inheriting from class \code{sem} or \code{msem} returned by the \code{\link{sem}} function.}
  \item{...}{not for the user.}
}

\details{
  Residuals are defined as \eqn{S - C}, where \eqn{S} is the sample covariance matrix
  of the observed variables and \eqn{C} is the model-reproduced covariance matrix.
  The \emph{standardized} residual covariance for a pair of variables divides the
  residual covariance by the product of the sample standard deviations of the 
  two variables, \eqn{(s_{ij} - c_{ij})/(s_{ii}s_{jj})^{1/2}}. The \emph{normalized} residual
  is given by
  \deqn{\frac{s_{ij}-c_{ij}} {[(c_{ii}c_{jj}-c_{ij}^2)/N^{*}]^{1/2}}}{%
    (s[ij] - c[ij])/[(c[ii]c[ii] + c[ij]^2)/N*]^[1/2]}
  where \eqn{N^{*}} is the number of observations minus one if the model is fit to a
  covariance matrix, or the number of observations if it is fit to a raw moment matrix.
}

\value{
  Each function returns a matrix of residuals.
}

\references{
    Bollen, K. A. (1989) 
    \emph{Structural Equations With Latent Variables.} Wiley.
    }
    
\author{John Fox \email{jfox@mcmaster.ca}}


\seealso{\code{\link{sem}}}

\examples{
# In the first example, readMoments() and specifyModel() read from the
# input stream. This example cannot be executed via example() but can be entered
# at the command prompt. The example is repeated using file input;
# this example can be executed via example(). 
    \dontrun{
# Duncan, Haller, and Portes peer-influences model

R.DHP <- readMoments(diag=FALSE, names=c("ROccAsp", "REdAsp", "FOccAsp", 
                "FEdAsp", "RParAsp", "RIQ", "RSES", "FSES", "FIQ", "FParAsp"))
    .6247     
    .3269  .3669       
    .4216  .3275  .6404
    .2137  .2742  .1124  .0839
    .4105  .4043  .2903  .2598  .1839
    .3240  .4047  .3054  .2786  .0489  .2220
    .2930  .2407  .4105  .3607  .0186  .1861  .2707
    .2995  .2863  .5191  .5007  .0782  .3355  .2302  .2950
    .0760  .0702  .2784  .1988  .1147  .1021  .0931 -.0438  .2087
            
model.dhp <- specifyModel()
    RParAsp  -> RGenAsp, gam11,  NA
    RIQ      -> RGenAsp, gam12,  NA
    RSES     -> RGenAsp, gam13,  NA
    FSES     -> RGenAsp, gam14,  NA
    RSES     -> FGenAsp, gam23,  NA
    FSES     -> FGenAsp, gam24,  NA
    FIQ      -> FGenAsp, gam25,  NA
    FParAsp  -> FGenAsp, gam26,  NA
    FGenAsp  -> RGenAsp, beta12, NA
    RGenAsp  -> FGenAsp, beta21, NA
    RGenAsp  -> ROccAsp,  NA,     1
    RGenAsp  -> REdAsp,  lam21,  NA
    FGenAsp  -> FOccAsp,  NA,     1
    FGenAsp  -> FEdAsp,  lam42,  NA
    RGenAsp <-> RGenAsp, ps11,   NA
    FGenAsp <-> FGenAsp, ps22,   NA
    RGenAsp <-> FGenAsp, ps12,   NA
    ROccAsp <-> ROccAsp, theta1, NA
    REdAsp  <-> REdAsp,  theta2, NA
    FOccAsp <-> FOccAsp, theta3, NA
    FEdAsp  <-> FEdAsp,  theta4, NA

sem.dhp <- sem(model.dhp, R.DHP, 329,
    fixed.x=c('RParAsp', 'RIQ', 'RSES', 'FSES', 'FIQ', 'FParAsp'))
residuals(sem.dhp)
normalizedResiduals(sem.dhp) 
standardizedResiduals(sem.dhp) # same as residuals because model is fit to correlations
    }
# The following example can be executed via example():

etc <- system.file(package="sem", "etc") # path to data and model files
   
(R.DHP <- readMoments(file=file.path(etc, "R-DHP.txt"),
				diag=FALSE, names=c("ROccAsp", "REdAsp", "FOccAsp", 
                "FEdAsp", "RParAsp", "RIQ", "RSES", "FSES", "FIQ", "FParAsp")))
(model.dhp <- specifyModel(file=file.path(etc, "model-DHP.txt")))
(sem.dhp <- sem(model.dhp, R.DHP, 329,
    fixed.x=c('RParAsp', 'RIQ', 'RSES', 'FSES', 'FIQ', 'FParAsp')))

residuals(sem.dhp)

normalizedResiduals(sem.dhp) 

standardizedResiduals(sem.dhp)  # same as residuals because model is fit to correlations
}

\keyword{models}

\name{Tests}
\alias{Tests}
\docType{data}
\title{
Six Mental Tests
}
\description{
These data are from the SAS manual and consist of six mental tests for 32 students,
with some missing data. The three \code{x} variables are intended to load on a verbal
factor, and the three \code{y} variables on a math factor. The data can be used to
illustrate the estimation of a confirmatory factor analysis model by multinormal full-information
maximum-likelihood in the presence of missing data.
}

\usage{Tests}
\format{
  A data frame with 32 observations on the following 6 variables.
  \describe{
    \item{\code{x1}}{score on verbal test 1.}
    \item{\code{x2}}{score on verbal test 2.}
    \item{\code{x3}}{score on verbal test 3.}
    \item{\code{y1}}{score on math test 1.}
    \item{\code{y2}}{score on math test 2.}
    \item{\code{y3}}{score on math test 3.}
  }
}

\source{
Example 25.13 from \emph{SAS/STAT 9.22 User's Guide}, SAS Institute, 2010.
}

\keyword{datasets}

\name{sem} 

\alias{sem}
\alias{sem.semmod}
\alias{sem.default}
\alias{sem.semmodList}
\alias{sem.msemmod}
\alias{startvalues}
\alias{startvalues2}
\alias{coef.sem}
\alias{vcov.sem}
\alias{df.residual.sem}
\alias{coef.msem}
\alias{vcov.msem}
\alias{df.residual.msem}

\title{General Structural Equation Models}
\description{
  \code{sem} fits general structural equation models (with both observed and
  unobserved variables). Observed variables are also called \emph{indicators} or 
  \emph{manifest variables}; unobserved variables are also called \emph{factors}
  or \emph{latent variables}. Normally, the generic function (\code{sem}) is
  called directly with a \code{semmod} first argument produced by \code{\link{specifyModel}},
  \code{\link{specifyEquations}}, or \code{\link{cfa}}, invoking the \code{sem.semmod}
  method, which in turn sets up a call to the \code{sem.default} method; thus, the user
  may wish to specify arguments accepted by the \code{semmod} and \code{default} methods.
  Similarly, for a multigroup model, \code{sem} would normally be called with a
  \code{semmodList} object produced by \code{\link{multigroupModel}} as its first argument,
  and would then generate a call to the code \code{msemmod} method.
}

\usage{
\method{sem}{semmod}(model, S, N, data, 
    raw=identical(na.action, na.pass), obs.variables=rownames(S), 
  	fixed.x=NULL, formula= ~ ., na.action=na.omit, 
    robust=!missing(data), debug=FALSE, 
    optimizer=optimizerSem, objective=objectiveML, ...)
    
\method{sem}{default}(model, S, N, raw=FALSE, data=NULL, start.fn=startvalues,
    pattern.number=NULL, valid.data.patterns=NULL,
    use.means=TRUE, param.names, 
  	var.names, fixed.x=NULL, robust=!is.null(data), semmod=NULL, debug=FALSE,
		analytic.gradient=!identical(objective, objectiveFIML), 
        warn=FALSE, maxiter=1000, par.size=c("ones", "startvalues"), 
		start.tol=1E-6, optimizer=optimizerSem, objective=objectiveML, cls, ...)
		
\method{sem}{semmodList}(model, S, N, data, raw=FALSE, fixed.x=NULL, 
		robust=!missing(data), formula, group="Group", debug=FALSE, ...)
		
\method{sem}{msemmod}(model, S, N, start.fn=startvalues,
        group="Group", groups=names(model), raw=FALSE, fixed.x, 
		param.names, var.names, debug=FALSE, analytic.gradient=TRUE, warn=FALSE,
		maxiter=5000, par.size = c("ones", "startvalues"), start.tol = 1e-06, 
		start=c("initial.fit", "startvalues"), initial.maxiter=1000,
		optimizer = optimizerMsem, objective = msemObjectiveML, ...)
    
startvalues(S, ram, debug=FALSE, tol=1E-6)
startvalues2(S, ram, debug=FALSE, tol=1E-6)

\method{coef}{sem}(object, standardized=FALSE, ...)
\method{coef}{msem}(object, ...)
\method{vcov}{sem}(object, robust=FALSE, 
	analytic=inherits(object, "objectiveML") && object$t <= 500, ...)
\method{vcov}{msem}(object, robust=FALSE, 
    analytic=inherits(object, "msemObjectiveML") && object$t <= 500, ...)
\method{df.residual}{sem}(object, ...)
\method{df.residual}{msem}(object, ...)
}

\arguments{
  \item{model}{RAM specification, which is a simple encoding of the path
    diagram for the model. The model may be given either in symbolic 
    form (as a \code{semmod} object, as returned by the \code{\link{specifyModel}},  
    \code{\link{specifyEquations}}, or \code{\link{cfa}} function,
    or as a character matrix), invoking \code{sem.semmod}, which calls \code{sem.default} 
    after setting up the model,
    or (less conveniently) in numeric form, invoking \code{sem.default} directly, which is
    not recommended (see \bold{Details} below). The \code{model} argument may also be a
    multigroup-model specification, as produced by \code{\link{multigroupModel}}.}
  \item{S}{covariance matrix among observed variables; may be input as a symmetric matrix,
    or as a lower- or upper-triangular matrix. \code{S} may also be a raw (i.e., \dQuote{uncorrected})
    moment matrix --- that is, a sum-of-squares-and-products matrix divided by \code{N}. This
    form of input is useful for fitting models with intercepts, in which case the moment matrix
    should include the mean square and cross-products for a unit variable all of whose entries are 1;
    of course, the raw mean square for the unit variable is 1. Raw-moment matrices may be computed
    by \code{\link{rawMoments}}. If the \code{ram} argument is given in symbolic form, then
    the observed-variable covariance or raw-moment matrix may contain variables that do not appear in the model,
    in which case a warning is printed. \code{S} may also be a list of covariance or moment matrices for each
    group in a multigroup model.
    As an alternative to specifying \code{S} the user may
    supply a data frame containing the data for the model (see the argument \code{data}).}
  \item{N}{number of observations on which the covariance matrix is based; for a multigroup model, a vector
    of group \eqn{N}s.}
  \item{data}{As a generally preferable alternative to specifying \code{S} and 
 	  \code{N}, the user may supply a data frame containing the data to which the model is to be fit.
 	  In a multigroup model, the \code{data} argument may be a list of data frames or a single data frame;
 	  in the later event, the factor given as the \code{group} argument is used to split the data into groups.}
  \item{start.fn}{a function to compute startvalues for the free parameters of the model;
  two functions are supplied, \code{startvalues} and a older version, \code{startvalues2},
  the first of which is the default.}
  \item{na.action}{a function to process missing data, if raw data are supplied in the \code{data} argument.
    The default is \code{na.omit}, which returns only complete cases; specify \code{na.action=na.pass} to
    get FIML estimates in the presence of missing data from the \code{objectiveFIML} 
    and \code{objectiveFIML2} objective functions.}
  \item{raw}{\code{TRUE} if \code{S} is a raw moment matrix or if a raw moment matrix ---  as
    opposed to a covariance matrix --- is to be
  	computed from \code{data}; the default is \code{FALSE} unless the \code{na.action} argument
    is set to \code{na.pass}.}
  \item{pattern.number, valid.data.patterns}{these arguments pass information about valid (i.e.,
    non-missing) data patterns and normally would not be specified directly by the user.}
  \item{use.means}{When raw data are supplied and intercepts are included in the model, use the
    observed-variable means as start values for the intercepts; the default is \code{TRUE}.}
  \item{obs.variables}{names of observed variables, by default taken from the row names of
    the covariance or moment matrix \code{S}, which may be given directly or generated according to the
    \code{data} and \code{formula} arguments.}
  \item{fixed.x}{names (if the \code{ram} matrix is given in symbolic form) or indices 
    (if it is in numeric form) of fixed exogenous variables. Specifying these obviates
    the necessity of having to fix the variances and covariances among these 
    variables (and produces correct degrees of freedom for the model chisquare).}
  \item{formula}{a one-sided formula, to be applied to \code{data} to generate the variables for
  	which covariances or raw moments are computed. The default formula is \code{~.}, i.e., all of the
  	variables in the data, including an implied intercept; if a covariance matrix is to be computed,
  	the constant is suppressed. In a multigroup model, alternatively a list one one-sided formulas as be
  	given, to be applied individually to the groups.}
  \item{robust}{In \code{sem}: if \code{TRUE}, then quantities are calculated that can be used to compute robust
  	estimates of coefficient standard errors and robust tests when the model is fit by multinormal maximum likelihood;
  	the default is \code{TRUE} when the \code{data} argument is \code{TRUE}, and this option is only
  	available when the \code{data} argument is given. In \code{vcov}: if \code{TRUE}, return a robust coefficient covariance matrix
  	(if \code{object} contains the requisite information).}
  \item{semmod}{a \code{semmod} object containing the description of the model; optional, and normally supplied
  	not directly by the user but via the \code{semmod} method for \code{sem}.}
  \item{debug}{if \code{TRUE}, some information is printed to help you debug the symbolic
    model specification; for example, if a variable name is misspelled, \code{sem} will
    assume that the variable is a (new) latent variable. Information about
    the optimization will also be printed, but details will vary with the optimizer employed. 
     The default is \code{FALSE}.}
  \item{...}{arguments to be passed down, including from \code{sem.default} to the 
    optimizer.}
  \item{param.names}{names of the \eqn{t} free parameters, given in their numerical order;
    default names are \code{Param1}, \ldots, \code{Param}\emph{t}. Note: Should not be
    specified when the model is given in symbolic form.}
  \item{var.names}{names of the \eqn{m} entries of the \eqn{v} vector
    (typically the observed and latent variables --- see below), given in their
    numerical order; default names are \code{Var1}, \ldots, \code{Var}\emph{m}.
    Note: Should not be specified when the model is given in symbolic form.}
  \item{analytic.gradient}{if \code{TRUE} (the default, except for the \code{objectiveFIML} objective function,
    where, at present, an analytic gradient slows down the computation), then analytic first derivatives are
    used in the maximization of the likelihood if the optimzer employed will accept them; 
    otherwise numeric derivatives are used, again if the optimizer will compute them.}
  \item{warn}{if \code{TRUE}, warnings produced by the optimization function will be printed.
    This should generally not be necessary, since \code{sem} prints its own warning, and saves
    information about convergence. The default is \code{FALSE}.}
  \item{maxiter}{the maximum number of iterations for the optimization of the objective function,
    to be passed to the optimizer.}
  \item{par.size}{the anticipated size of the free parameters; if \code{"ones"},
    a vector of ones is used; if \code{"startvalues"}, taken from the start values.
    You can try changing this argument if you encounter convergence problems.
    The default is \code{"startvalues"} if the largest input variance is at
    least 100 times the smallest, and \code{"ones"} otherwise. Whether this argument is actually
    used depends upon the optimizer employed.}
  \item{start.tol, tol}{if the magnitude of an automatic start value is less than \code{start.tol}, then
    it is set to \code{start.tol}; defaults to 1E-6.}
  \item{optimizer}{a function to be used to minimize the objective function; the default for single-group models is 
  	\code{\link{optimizerSem}}.
  	Alternatives are \code{\link[stats]{nlm}}, which employs the standard R optimizer \code{\link[stats]{nlm}};
  	\code{\link{optimizerOptim}}, which employs \code{\link[stats]{optim}};
  	and \code{\link{optimizerNlminb}}, which uses \code{\link[stats]{nlminb}} --- or
  	the user can supply an optimizer. For multigroup model, the default is \code{\link{optimizerMsem}}, and
  	\code{\link{msemOptimizerNlm}}, based on \code{nlm}, is provided as an alternative.}
  \item{objective}{An objective function to be minimized, sometimes called a \dQuote{fit} function 
  	in the SEM literature. The default for single-group models is \code{\link{objectiveML}},
  	which produces maximum-likelihood estimates assuming multinormality. An alternative is 
  	\code{\link{objectiveGLS}}, which produced generalized least squares estimates --- or the user
  	can supply an objective function to be minimized. For multigroup models, the default is
  	available is \code{\link{msemObjectiveML}} for ML estimates and an alternative is \code{\link{msemObjectiveGLS}}
  	for GLS estiamtes.}
  \item{cls}{primary class to be assigned to the result; normally this is not specified directly, but
    raither is inferred from the objective function.}
  \item{ram}{numeric RAM matrix.}
  \item{object}{an object of class \code{"sem"} or \code{"msem"}, returned by \code{sem}.}
  \item{standardized}{if \code{TRUE}, return standardized coefficients.}
  \item{analytic}{return an analytic (as opposed to numeric) estimate of the coefficient covariance matrix;
  	at present only available for the \code{\link{objectiveML}} objective function. The default is \code{FALSE}
  	for this objective function if there are no more than 100 parameters and \code{FALSE} otherwise.}
  \item{group}{for a multigroup model, the quoted name of the group variable; if the \code{data} argument is given,
    snd is a single data frame, then this should be a factor in the data set or a variable coercible to a factor,
    to be used to split the data into groups; otherwise, the name is arbitrary.}
  \item{groups}{a character vector giving the names of the groups; will be ignored if \code{group} is a factor
    in \code{data}.}
  \item{start}{if \code{"initial.fit"} (the default), start values 
    for a multi-group model are computed by first fitting the intra-group models separately by group;
    if \code{"startvalues"}, then start values are computed as for a single-group model.
    In some cases, the intra-group models may not be identified even if the multi-group model is, and
    then \code{start="initial.fit"} should not be used.}
  \item{initial.maxiter}{if \code{start="initial.fit"} for a multi-group model, then
    \code{initial.maxiter} gives the maximum number of iterations for each initial intra-group fit.}
} 

\details{
  The model is set up using either RAM (\dQuote{reticular action model} -- don't ask!) 
  notation -- a simple format
  for specifying general structural equation models by coding the
  \dQuote{arrows} in the path diagram for the model (see, e.g., McArdle and McDonald, 1984) --
  typically using the \code{\link{specifyModel}} function; in equation format using the
  \code{\link{specifyEquations}} function; or, for a simple confirmatory factor analysis model,
  via the \code{\link{cfa}} function. In any case, the model is represented internally in RAM format.
  
  The variables in the \eqn{v} vector in the model (typically, the observed and
  unobserved variables, but not error variables) are numbered from 1 to \eqn{m}.  
  the RAM matrix contains one row for each (free or constrained) parameter of the model, and
  may be specified either in symbolic format or in numeric format.
  
  A symbolic \code{ram} matrix consists of three columns, as follows:
  
    \describe{
      \item{1. Arrow specification:}{This is a simple formula, of the form
        \code{"A -> B"} or, equivalently, \code{"B <- A"} for a regression
        coefficient (i.e., a single-headed or directional arrow); 
        \code{"A <-> A"} for a variance or \code{"A <-> B"} for a covariance
        (i.e., a double-headed or bidirectional arrow). Here, \code{A} and
        \code{B} are variable names in the model. If a name does not correspond
        to an observed variable, then it is assumed to be a latent variable.
        Spaces can appear freely in an arrow specification, and
        there can be any number of hyphens in the arrows, including zero: Thus,
        e.g., \code{"A->B"}, \code{"A --> B"}, and \code{"A>B"} are all legitimate
        and equivalent.}
      \item{2. Parameter name:}{The name of the regression coefficient, variance,
        or covariance specified by the arrow. Assigning the same name to two or
        more arrows results in an equality constraint. Specifying the parameter name
        as \code{NA} produces a fixed parameter.}
      \item{3. Value:}{start value for a free parameter or value of a fixed parameter.
        If given as \code{NA}, \code{sem} will compute the start value.}
      }
      
  It is simplest to construct the RAM matrix with the \code{\link{specifyModel}}, \code{\link{specifyEquations}},
  or \code{\link{cfa}} function,
  all of which return an object of class \code{semmod}, and also incorporate some model-specification
  convenience shortcuts. This process is illustrated in the examples below. 
    
  A numeric \code{ram} matrix consists of five columns, as follows:
    
    \describe{
        \item{1. Number of arrow heads:}{1 (directed arrow) or 2 (covariance).}
        \item{2. Arrow \emph{to}:}{index of the variable at the head of
            a directional arrow, or at one end of a bidirectional arrow.
            Observed variables should be assigned the numbers 1 to \eqn{n}, where
            \eqn{n} is the number of rows/columns in the covariance matrix \code{S},
            with the indices corresponding to the variables' positions in \code{S}.
            Variable indices above \eqn{n} represent latent variables.}
        \item{3. Arrow \emph{from}:}{the index of the variable at the tail of 
            a directional arrow, or at the other end of a bidirectional arrow.}
        \item{4. Parameter number:}{free parameters are numbered from 1 to \eqn{t},
            but do not necessarily appear in consecutive order. Fixed parameters are given
            the number 0. Equality contraints are specified by assigning two or more 
            parameters the same number.}
        \item{5. Value:}{start value for a free parameter, or value of a fixed parameter. If given
            as \code{NA}, the program will compute a start value, by a slight modification of the
            method described by McDonald and Hartmann (1992). \emph{Note:} In some circumstances,
            some start values are selected randomly; this might produce small differences in
            the parameter estimates when the program is rerun.}
        }
        
  The numeric \code{ram} matrix is normally generated automatically, not specified directly by the user.
    
  For \code{specifyEquations}, each input line is either a regression equation or the specification
  of a variance or covariance. Regression equations are of the form
  
  \code{y = par1*x1 + par2*x2 + ... + park*xk}
  
  where \code{y} and the \code{x}s are variables in the model (either observed or latent),
  and the \code{par}s are parameters. If a parameter is given as a numeric value (e.g.,
  \code{1}) then it is treated as fixed. Note that no \dQuote{error} variable is included in
  the equation; \dQuote{error variances} are specified via either the \code{covs} argument,
  via \code{V(y) = par} (see immediately below), or are added automatically to the model
  when, as by default, \code{endog.variances=TRUE}.
  
  Variances are specified in the form \code{V(var) = par} and covariances in the form
  \code{C(var1, var2) = par}, where the \code{var}s are variables (observed or unobserved) in
  the model. The symbols \code{V} and \code{C} may be in either lower- or upper-case. If \code{par}
  is a numeric value (e.g., \code{1}) then it is treated as fixed. In conformity with the RAM model,
  a variance or covariance for an endogenous variable in the model is an \dQuote{error} variance or
  covariance.
  
  To set a start value for a free parameter, enclose the numeric start value in parentheses after the
  parameter name, as \code{parameter(value)}.
        
  \code{sem} fits the model by calling the optimizer specified in the \code{optimizer} argument
  to minimize the objective function specified in the \code{objective} argument.
  If the optimization fails to converge, a warning message is printed.
  
  The RAM formulation of the general structural equation model is given by the basic equation
  \deqn{v = Av + u}
  where \eqn{v} and \eqn{u} are vectors of random variables (observed or unobserved), and
  the parameter matrix \eqn{A} contains regression coefficients, symbolized by single-headed arrows
  in a path diagram. Another parameter matrix, 
  \deqn{P = E(uu')}
  contains covariances among the elements of \eqn{u} (assuming that the elements of \eqn{u} have zero
  means). Usually \eqn{v} contains endogenous and exogenous observed and unobserved variables, but not
  error variables (see the examples below).
  
  The \code{startvalues} function may be called directly, but is usually called by \code{sem.default}; \code{startvalues2} is an older version of this function that may be used alternatively; see the \code{startvalues} argument to \code{sem}.
}

\value{
  \code{sem} returns an object of class \code{c(}\emph{objective}\code{, "sem")}, where \emph{objective}
  	is the name of the objective function that was optimized (e.g., \code{"objectiveML"}), with the following elements:
  \item{var.names}{vector of variable names.}
  \item{ram}{RAM matrix, including any rows generated for covariances
    among fixed exogenous variables; column 5 includes computed start values.}
  \item{S}{observed covariance matrix.}
  \item{J}{RAM selection matrix, \eqn{J}, which picks out observed variables.}
  \item{n.fix}{number of fixed exogenous variables.}
  \item{n}{number of observed variables.}
  \item{N}{number of observations.}
  \item{m}{number of variables (observed plus unobserved).}
  \item{t}{number of free parameters.}
  \item{raw}{\code{TRUE} if the model is fit to a raw moment matrix, \code{FALSE} otherwise.}
  \item{data}{the observed-variable data matrix, or \code{NULL} if data are not supplied.}
  \item{semmod}{the \code{semmod} specification object for the model, if one was supplied; otherwise \code{NULL}.}
  \item{optimizer}{the optimizer function.}
  \item{objective}{the objective function.}
  \item{coeff}{estimates of free parameters.}
  \item{vcov}{estimated asymptotic covariance matrix of parameter estimates, based on a numeric Hessian,
  	if supplied by the optimizer; otherwise \code{NULL}.}
  \item{par.posn}{indices of free parameters.}
  \item{convergence}{\code{TRUE} or \code{FALSE}, depending upon whether the optimization apparently converged.}
  \item{iterations}{number of iterations performed.}
  \item{criterion}{value of the objective function at the minimum.}
  \item{C}{model-reproduced covariance matrix.}
  \item{A}{RAM \eqn{A} matrix.}
  \item{P}{RAM \eqn{P} matrix.}
  \item{adj.obj}{robust adjusted value of the objective function; \code{NULL} if
  	\code{robust} is \code{FALSE}.}
  \item{robust.vcov}{robust estimated coefficient covariance matrix; \code{NULL} if
  	\code{robust} is \code{FALSE}.}
  	
  For multigroup models, \code{sem} returns an object of class \code{c("msemObjectiveML", "msem")}.
}

\section{Warning}{
    A common error is to fail to specify variance or covariance terms in the model, which are denoted 
    by double-headed arrows, \code{<->}.
    
    In general, every observed or latent variable in the model should be associated 
    with a variance or error variance. 
    This may be
    a free parameter to estimate or a fixed constant (as in the case of a latent exogenous 
    variable for which you wish to fix the variance, e.g.,
    to 1). Again in general, there will be an \emph{error variance} associated with each 
    endogenous variable in the model (i.e., each variable
    to which at least one single-headed arrow points --- including observed indicators of latent variables), 
    and a \emph{variance} associated with each exogenous variable (i.e., each variable that
    appears only at the tail of single-headed arrows, never at the head).
    
    To my knowledge, the only \emph{apparent} exception to this rule is for observed variables that 
    are declared to be fixed exogenous variables.
    In this case, the program generates the necessary (fixed-constant) variances and covariances automatically.
    
    If there are missing variances, a warning message will be printed, and estimation will almost surely 
    fail in some manner. Missing
    variances might well indicate that there are missing covariances too, but it is not possible 
    to deduce this in a mechanical manner. The \code{\link{specifyModel}} funciton will by default supply
    error-variance parameters if these are missing.
}

\references{

	Fox, J. (2006)
	Structural equation modeling with the sem package in R.
	\emph{Structural Equation Modeling} 
	\bold{13}:465--486.
	
    Bollen, K. A. (1989) 
    \emph{Structural Equations With Latent Variables.} Wiley.
    
    Bollen, K. A. and Long, J. S. (eds.)
    \emph{Testing Structural Equation Models}, Sage.
    
    McArdle, J. J. and Epstein, D. (1987)
    Latent growth curves within developmental structural equation models.
    \emph{Child Development}
    \bold{58}, 110--133.
    
    McArdle, J. J. and McDonald, R. P. (1984)
    Some algebraic properties of the reticular action model.
    \emph{British Journal of Mathematical and Statistical Psychology}
    \bold{37}, 234--251.
    
    McDonald, R. P. and Hartmann, W. M. (1992)
    A procedure for obtaining initial values of parameters in
    the RAM model. \emph{Multivariate Behavioral Research}
    \bold{27}, 57--76.
    
    Raftery, A. E. (1993)
    Bayesian model selection in structural equation models.
    In Bollen, K. A. and Long, J. S. (eds.)
    \emph{Testing Structural Equation Models}, Sage.
    
    Raftery, A. E. (1995)
    Bayesian model selection in social research (with discussion).
    \emph{Sociological Methodology} \bold{25}, 111--196.
    
    Satorra, A. (2000)
	Scaled and adjusted restricted tests in multi-sample analysis of moment structures. 
	pp. 233--247 in Heijmans, R.D.H., Pollock, D.S.G. & Satorra, A. (eds.)
	\emph{Innovations in Multivariate Statistical Analysis. A Festschrift for Heinz Neudecker }, Kluwer. 
}

\author{John Fox \email{jfox@mcmaster.ca}, Zhenghua Nie, and Jarrett Byrnes}

\seealso{\code{\link{rawMoments}}, \code{\link{startvalues}}, 
	\code{\link{objectiveML}}, \code{\link{objectiveGLS}},  
	\code{\link{optimizerNlm}}, \code{\link{optimizerOptim}}, \code{\link{optimizerNlminb}},
	\code{\link[stats]{nlm}}, \code{\link[stats]{optim}}, \code{\link[stats]{nlminb}},
	\code{\link{specifyModel}}, \code{\link{specifyEquations}}, \code{\link{cfa}}
}

\examples{

# The following example illustrates the use the text argument to 
#   readMoments() and specifyEquations():

R.DHP <- readMoments(diag=FALSE, names=c("ROccAsp", "REdAsp", "FOccAsp", 
                "FEdAsp", "RParAsp", "RIQ", "RSES", "FSES", "FIQ", "FParAsp"),
                text="
    .6247     
    .3269  .3669       
    .4216  .3275  .6404
    .2137  .2742  .1124  .0839
    .4105  .4043  .2903  .2598  .1839
    .3240  .4047  .3054  .2786  .0489  .2220
    .2930  .2407  .4105  .3607  .0186  .1861  .2707
    .2995  .2863  .5191  .5007  .0782  .3355  .2302  .2950
    .0760  .0702  .2784  .1988  .1147  .1021  .0931 -.0438  .2087
")

model.dhp.1 <- specifyEquations(covs="RGenAsp, FGenAsp", text="
RGenAsp = gam11*RParAsp + gam12*RIQ + gam13*RSES + gam14*FSES + beta12*FGenAsp
FGenAsp = gam23*RSES + gam24*FSES + gam25*FIQ + gam26*FParAsp + beta21*RGenAsp
ROccAsp = 1*RGenAsp
REdAsp = lam21(1)*RGenAsp  # to illustrate setting start values
FOccAsp = 1*FGenAsp
FEdAsp = lam42(1)*FGenAsp
")

sem.dhp.1 <- sem(model.dhp.1, R.DHP, 329,
    fixed.x=c('RParAsp', 'RIQ', 'RSES', 'FSES', 'FIQ', 'FParAsp'))
summary(sem.dhp.1)

# Note: The following set of examples can't be run via example() because the default file
#  argument of specifyeEquations, specifyModel(), and readMoments() requires that the model 
#  specification and covariances, correlations, or raw moments be entered in an interactive
#  session at the command prompt. The examples can be copied and run in the R console,
#  however. See ?specifyModel and ?readMoments for further information.
#  These examples are repeated below using file input to specifyModel() and
#  readMoments(). The second version of the examples may be executed through example().

    \dontrun{

# ------------- Duncan, Haller and Portes peer-influences model ----------------------
# A nonrecursive SEM with unobserved endogenous variables and fixed exogenous variables

R.DHP <- readMoments(diag=FALSE, names=c("ROccAsp", "REdAsp", "FOccAsp", 
                "FEdAsp", "RParAsp", "RIQ", "RSES", "FSES", "FIQ", "FParAsp"))
    .6247     
    .3269  .3669       
    .4216  .3275  .6404
    .2137  .2742  .1124  .0839
    .4105  .4043  .2903  .2598  .1839
    .3240  .4047  .3054  .2786  .0489  .2220
    .2930  .2407  .4105  .3607  .0186  .1861  .2707
    .2995  .2863  .5191  .5007  .0782  .3355  .2302  .2950
    .0760  .0702  .2784  .1988  .1147  .1021  .0931 -.0438  .2087
            
# Fit the model using a symbolic ram specification

model.dhp <- specifyModel()
    RParAsp  -> RGenAsp, gam11,  NA
    RIQ      -> RGenAsp, gam12,  NA
    RSES     -> RGenAsp, gam13,  NA
    FSES     -> RGenAsp, gam14,  NA
    RSES     -> FGenAsp, gam23,  NA
    FSES     -> FGenAsp, gam24,  NA
    FIQ      -> FGenAsp, gam25,  NA
    FParAsp  -> FGenAsp, gam26,  NA
    FGenAsp  -> RGenAsp, beta12, NA
    RGenAsp  -> FGenAsp, beta21, NA
    RGenAsp  -> ROccAsp,  NA,     1
    RGenAsp  -> REdAsp,  lam21,  NA
    FGenAsp  -> FOccAsp,  NA,     1
    FGenAsp  -> FEdAsp,  lam42,  NA
    RGenAsp <-> RGenAsp, ps11,   NA
    FGenAsp <-> FGenAsp, ps22,   NA
    RGenAsp <-> FGenAsp, ps12,   NA
    ROccAsp <-> ROccAsp, theta1, NA
    REdAsp  <-> REdAsp,  theta2, NA
    FOccAsp <-> FOccAsp, theta3, NA
    FEdAsp  <-> FEdAsp,  theta4, NA
    
# an equivalent specification, allowing specifyModel() to generate
#  variance parameters for endogenous variables (and suppressing the
#  unnecessary NAs):
 
model.dhp <- specifyModel()
RParAsp  -> RGenAsp, gam11
RIQ      -> RGenAsp, gam12
RSES     -> RGenAsp, gam13
FSES     -> RGenAsp, gam14
RSES     -> FGenAsp, gam23
FSES     -> FGenAsp, gam24
FIQ      -> FGenAsp, gam25
FParAsp  -> FGenAsp, gam26
FGenAsp  -> RGenAsp, beta12
RGenAsp  -> FGenAsp, beta21
RGenAsp  -> ROccAsp,  NA,     1
RGenAsp  -> REdAsp,  lam21
FGenAsp  -> FOccAsp,  NA,     1
FGenAsp  -> FEdAsp,  lam42
RGenAsp <-> FGenAsp, ps12

# Another equivalent specification, telling specifyModel to add paths for 
#   variances and covariance of RGenAsp and FGenAsp:
 
model.dhp <- specifyModel(covs="RGenAsp, FGenAsp")
RParAsp  -> RGenAsp, gam11
RIQ      -> RGenAsp, gam12
RSES     -> RGenAsp, gam13
FSES     -> RGenAsp, gam14
RSES     -> FGenAsp, gam23
FSES     -> FGenAsp, gam24
FIQ      -> FGenAsp, gam25
FParAsp  -> FGenAsp, gam26
FGenAsp  -> RGenAsp, beta12
RGenAsp  -> FGenAsp, beta21
RGenAsp  -> ROccAsp,  NA,     1
RGenAsp  -> REdAsp,  lam21
FGenAsp  -> FOccAsp,  NA,     1
FGenAsp  -> FEdAsp,  lam42

# Yet another equivalent specification using specifyEquations():

model.dhp.1 <- specifyEquations(covs="RGenAsp, FGenAsp")
RGenAsp = gam11*RParAsp + gam12*RIQ + gam13*RSES + gam14*FSES + beta12*FGenAsp
FGenAsp = gam23*RSES + gam24*FSES + gam25*FIQ + gam26*FParAsp + beta21*RGenAsp
ROccAsp = 1*RGenAsp
REdAsp = lam21(1)*RGenAsp  # to illustrate setting start values
FOccAsp = 1*FGenAsp
FEdAsp = lam42(1)*FGenAsp
 
sem.dhp.1 <- sem(model.dhp.1, R.DHP, 329,
    fixed.x=c('RParAsp', 'RIQ', 'RSES', 'FSES', 'FIQ', 'FParAsp'))
summary(sem.dhp.1)

# Fit the model using a numerical ram specification (not recommended!)

ram.dhp <- matrix(c(
#               heads   to      from    param  start
                1,       1,     11,      0,     1,
                1,       2,     11,      1,     NA, # lam21
                1,       3,     12,      0,     1,
                1,       4,     12,      2,     NA, # lam42
                1,      11,      5,      3,     NA, # gam11
                1,      11,      6,      4,     NA, # gam12
                1,      11,      7,      5,     NA, # gam13
                1,      11,      8,      6,     NA, # gam14
                1,      12,      7,      7,     NA, # gam23
                1,      12,      8,      8,     NA, # gam24
                1,      12,      9,      9,     NA, # gam25
                1,      12,     10,     10,     NA, # gam26
                1,      11,     12,     11,     NA, # beta12
                1,      12,     11,     12,     NA, # beta21
                2,       1,      1,     13,     NA, # theta1
                2,       2,      2,     14,     NA, # theta2
                2,       3,      3,     15,     NA, # theta3
                2,       4,      4,     16,     NA, # theta4
                2,      11,     11,     17,     NA, # psi11
                2,      12,     12,     18,     NA, # psi22
                2,      11,     12,     19,     NA  # psi12
                ), ncol=5, byrow=TRUE)

params.dhp <- c('lam21', 'lam42', 'gam11', 'gam12', 'gam13', 'gam14',
                 'gam23',  'gam24',  'gam25',  'gam26',
                 'beta12', 'beta21', 'theta1', 'theta2', 'theta3', 'theta4',
                 'psi11', 'psi22', 'psi12')
                 
vars.dhp <- c('ROccAsp', 'REdAsp', 'FOccAsp', 'FEdAsp', 'RParAsp', 'RIQ',
                'RSES', 'FSES', 'FIQ', 'FParAsp', 'RGenAsp', 'FGenAsp')
                
sem.dhp.2 <- sem(ram.dhp, R.DHP, 329, param.names=params.dhp, var.names=vars.dhp, 
	fixed.x=5:10)
summary(sem.dhp.2)


# -------------------- Wheaton et al. alienation data ----------------------
    

S.wh <- readMoments(names=c('Anomia67','Powerless67','Anomia71',
                                    'Powerless71','Education','SEI'))
   11.834                                    
    6.947    9.364                            
    6.819    5.091   12.532                    
    4.783    5.028    7.495    9.986            
   -3.839   -3.889   -3.841   -3.625   9.610     
  -21.899  -18.831  -21.748  -18.775  35.522  450.288

# This is the model in the SAS manual for PROC CALIS: A Recursive SEM with
# latent endogenous and exogenous variables.
# Curiously, both factor loadings for two of the latent variables are fixed.

model.wh.1 <- specifyModel()
    Alienation67   ->  Anomia67,      NA,     1
    Alienation67   ->  Powerless67,   NA,     0.833
    Alienation71   ->  Anomia71,      NA,     1
    Alienation71   ->  Powerless71,   NA,     0.833 
    SES            ->  Education,     NA,     1     
    SES            ->  SEI,           lamb,   NA
    SES            ->  Alienation67,  gam1,   NA
    Alienation67   ->  Alienation71,  beta,   NA
    SES            ->  Alienation71,  gam2,   NA
    Anomia67       <-> Anomia67,      the1,   NA
    Anomia71       <-> Anomia71,      the1,   NA
    Powerless67    <-> Powerless67,   the2,   NA
    Powerless71    <-> Powerless71,   the2,   NA
    Education      <-> Education,     the3,   NA
    SEI            <-> SEI,           the4,   NA
    Anomia67       <-> Anomia71,      the5,   NA
    Powerless67    <-> Powerless71,   the5,   NA
    Alienation67   <-> Alienation67,  psi1,   NA
    Alienation71   <-> Alienation71,  psi2,   NA
    SES            <-> SES,           phi,    NA
                           
sem.wh.1 <- sem(model.wh.1, S.wh, 932)
summary(sem.wh.1)

# The same model in equation format:

model.wh.1 <- specifyEquations()
Anomia67 = 1*Alienation67
Powerless67 = 0.833*Alienation67
Anomia71 = 1*Alienation71
Powerless71 = 0.833*Alienation71
Education = 1*SES
SEI = lamb*SES
Alienation67 = gam1*SES
Alienation71 = gam2*SES + beta*Alienation67
V(Anomia67) = the1
V(Anomia71) = the1
V(Powerless67) = the2
V(Powerless71) = the2
V(SES) = phi
C(Anomia67, Anomia71) = the5
C(Powerless67, Powerless71) = the5

# The same model, but treating one loading for each latent variable as free
# (and equal to each other).

model.wh.2 <- specifyModel()
    Alienation67   ->  Anomia67,      NA,        1
    Alienation67   ->  Powerless67,   lamby,    NA
    Alienation71   ->  Anomia71,      NA,        1
    Alienation71   ->  Powerless71,   lamby,    NA 
    SES            ->  Education,     NA,        1     
    SES            ->  SEI,           lambx,    NA
    SES            ->  Alienation67,  gam1,     NA
    Alienation67   ->  Alienation71,  beta,     NA
    SES            ->  Alienation71,  gam2,     NA
    Anomia67       <-> Anomia67,      the1,     NA
    Anomia71       <-> Anomia71,      the1,     NA
    Powerless67    <-> Powerless67,   the2,     NA
    Powerless71    <-> Powerless71,   the2,     NA
    Education      <-> Education,     the3,     NA
    SEI            <-> SEI,           the4,     NA
    Anomia67       <-> Anomia71,      the5,     NA
    Powerless67    <-> Powerless71,   the5,     NA
    Alienation67   <-> Alienation67,  psi1,     NA
    Alienation71   <-> Alienation71,  psi2,     NA
    SES            <-> SES,           phi,      NA 


sem.wh.2 <- sem(model.wh.2, S.wh, 932)
summary(sem.wh.2)

# And again, in equation format:

model.wh <- specifyEquations()
Anomia67 = 1*Alienation67
Powerless67 = lamby*Alienation67
Anomia71 = 1*Alienation71
Powerless71 = lamby*Alienation71
Education = 1*SES
SEI = lambx*SES
Alienation67 = gam1*SES
Alienation71 = gam2*SES + beta*Alienation67
V(Anomia67) = the1
V(Anomia71) = the1
V(Powerless67) = the2
V(Powerless71) = the2
V(SES) = phi
C(Anomia67, Anomia71) = the5
C(Powerless67, Powerless71) = the5


# Compare the two models by a likelihood-ratio test:

anova(sem.wh.1, sem.wh.2)


# ----------------------- Thurstone data ---------------------------------------
#  Second-order confirmatory factor analysis, from the SAS manual for PROC CALIS

R.thur <- readMoments(diag=FALSE, names=c('Sentences','Vocabulary',
        'Sent.Completion','First.Letters','4.Letter.Words','Suffixes',
        'Letter.Series','Pedigrees', 'Letter.Group'))
    .828                                              
    .776   .779                                        
    .439   .493    .46                                 
    .432   .464    .425   .674                           
    .447   .489    .443   .59    .541                    
    .447   .432    .401   .381    .402   .288              
    .541   .537    .534   .35    .367   .32   .555        
    .38   .358    .359   .424    .446   .325   .598   .452  
            
model.thur <- specifyModel()
    F1 -> Sentences,                      lam11
    F1 -> Vocabulary,                     lam21
    F1 -> Sent.Completion,                lam31
    F2 -> First.Letters,                  lam42
    F2 -> 4.Letter.Words,                 lam52
    F2 -> Suffixes,                       lam62
    F3 -> Letter.Series,                  lam73
    F3 -> Pedigrees,                      lam83
    F3 -> Letter.Group,                   lam93
    F4 -> F1,                             gam1
    F4 -> F2,                             gam2
    F4 -> F3,                             gam3
    F1 <-> F1,                            NA,     1
    F2 <-> F2,                            NA,     1
    F3 <-> F3,                            NA,     1
    F4 <-> F4,                            NA,     1

sem.thur <- sem(model.thur, R.thur, 213)
summary(sem.thur)

# The model in equation format:

model.thur <- specifyEquations()
Sentences = lam11*F1
Vocabulary = lam21*F1
Sent.Completion = lam31*F1
First.Letters = lam42*F2
4.Letter.Words = lam52*F2
Suffixes = lam62*F2
Letter.Series = lam73*F3
Pedigrees = lam83*F3
Letter.Group = lam93*F3
F1 = gam1*F4
F2 = gam2*F4
F3 = gam3*F4
V(F1) = 1
V(F2) = 1
V(F3) = 1
V(F4) = 1


#------------------------- Kerchoff/Kenney path analysis ---------------------
# An observed-variable recursive SEM from the LISREL manual

R.kerch <- readMoments(diag=FALSE, names=c('Intelligence','Siblings',
                        'FatherEd','FatherOcc','Grades','EducExp','OccupAsp'))
    -.100                                
     .277  -.152                          
     .250  -.108  .611                     
     .572  -.105  .294   .248               
     .489  -.213  .446   .410   .597         
     .335  -.153  .303   .331   .478   .651   
    
model.kerch <- specifyModel()
    Intelligence -> Grades,       gam51
    Siblings -> Grades,           gam52
    FatherEd -> Grades,           gam53
    FatherOcc -> Grades,          gam54
    Intelligence -> EducExp,      gam61
    Siblings -> EducExp,          gam62
    FatherEd -> EducExp,          gam63
    FatherOcc -> EducExp,         gam64
    Grades -> EducExp,            beta65
    Intelligence -> OccupAsp,     gam71
    Siblings -> OccupAsp,         gam72
    FatherEd -> OccupAsp,         gam73
    FatherOcc -> OccupAsp,        gam74
    Grades -> OccupAsp,           beta75
    EducExp -> OccupAsp,          beta76
                       
sem.kerch <- sem(model.kerch, R.kerch, 737, 
    fixed.x=c('Intelligence', 'Siblings', 'FatherEd', 'FatherOcc'))
summary(sem.kerch)

# The model in equation format:

model.kerch <- specifyEquations()
Grades = gam51*Intelligence + gam52*Siblings + gam53*FatherEd 
           + gam54*FatherOcc
EducExp = gam61*Intelligence + gam62*Siblings + gam63*FatherEd 
            + gam64*FatherOcc + beta65*Grades
OccupAsp = gam71*Intelligence + gam72*Siblings + gam73*FatherEd 
            + gam74*FatherOcc + beta75*Grades + beta76*EducExp


#------------------- McArdle/Epstein latent-growth-curve model -----------------
# This model, from McArdle and Epstein (1987, p.118), illustrates the use of a 
# raw moment matrix to fit a model with an intercept. (The example was suggested
# by Mike Stoolmiller.)

M.McArdle <- readMoments(
    names=c('WISC1', 'WISC2', 'WISC3', 'WISC4', 'UNIT'))
    365.661                                      
    503.175     719.905                           
    675.656     958.479    1303.392                
    890.680    1265.846    1712.475    2278.257     
     18.034      25.819      35.255      46.593     1.000
 
mod.McArdle <- specifyModel()
    C -> WISC1, NA, 6.07
    C -> WISC2, B2, NA
    C -> WISC3, B3, NA
    C -> WISC4, B4, NA
    UNIT -> C, Mc, NA
    C <-> C, Vc, NA,
    WISC1 <-> WISC1, Vd, NA
    WISC2 <-> WISC2, Vd, NA
    WISC3 <-> WISC3, Vd, NA
    WISC4 <-> WISC4, Vd, NA

sem.McArdle <- sem(mod.McArdle, M.McArdle, 204, fixed.x="UNIT", raw=TRUE)
summary(sem.McArdle)

# The model in equation format:

mod.McArdle <- specifyEquations()
WISC1 = 6.07*C
WISC2 = B2*C
WISC3 = B3*C
WISC4 = b4*C
C = Mc*UNIT
v(C) = Vc
v(WISC1) = Vd
v(WISC2) = Vd
v(WISC3) = Vd
v(WISC4) = Vd

    
#------------ Bollen industrialization and democracy example -----------------
# This model, from Bollen (1989, Ch. 8), illustrates the use in sem() of a
# case-by-variable data (see ?Bollen) set rather than a covariance or moment matrix

model.bollen <- specifyModel()
	Demo60 -> y1, NA, 1
	Demo60 -> y2, lam2, 
	Demo60 -> y3, lam3, 
	Demo60 -> y4, lam4, 
	Demo65 -> y5, NA, 1
	Demo65 -> y6, lam2, 
	Demo65 -> y7, lam3, 
	Demo65 -> y8, lam4, 
	Indust -> x1, NA, 1
	Indust -> x2, lam6, 
	Indust -> x3, lam7, 
	y1 <-> y5, theta15
	y2 <-> y4, theta24
	y2 <-> y6, theta26
	y3 <-> y7, theta37
	y4 <-> y8, theta48
	y6 <-> y8, theta68
	Indust -> Demo60, gamma11, 
	Indust -> Demo65, gamma21, 
	Demo60 -> Demo65, beta21, 
	Indust <-> Indust, phi
	
sem.bollen <- sem(model.bollen, data=Bollen)
summary(sem.bollen)
summary(sem.bollen, robust=TRUE) # robust SEs and tests
summary(sem.bollen, analytic.se=FALSE) # uses numeric rather than analytic Hessian

  # GLS rather than ML estimator:
sem.bollen.gls <- sem(model.bollen, data=Bollen, objective=objectiveGLS) 
summary(sem.bollen.gls)

# The model in equation format:

model.bollen <- specifyEquations()
y1 = 1*Demo60
y2 = lam2*Demo60
y3 = lam3*Demo60
y4 = lam4*Demo60
y5 = 1*Demo65
y6 = lam2*Demo65
y7 = lam3*Demo65
y8 = lam4*Demo65
x1 = 1*Indust
x2 = lam6*Indust
x3 = lam7*Indust
c(y1, y5) = theta15
c(y2, y4) = theta24
c(y2, y6) = theta26
c(y3, y7) = theta37
c(y4, y8) = theta48
c(y6, y8) = theta68
Demo60 = gamma11*Indust
Demo65 = gamma21*Indust + beta21*Demo60
v(Indust) = phi


# -------------- A simple CFA model for the Thurstone mental tests data --------------

R.thur <- readMoments(diag=FALSE, 
  names=c('Sentences','Vocabulary',
          'Sent.Completion','First.Letters','4.Letter.Words','Suffixes',
          'Letter.Series','Pedigrees', 'Letter.Group'))
.828                                              
.776   .779                                        
.439   .493    .46                                 
.432   .464    .425   .674                           
.447   .489    .443   .59    .541                    
.447   .432    .401   .381    .402   .288              
.541   .537    .534   .35    .367   .32   .555        
.38   .358    .359   .424    .446   .325   .598   .452

	#  (1) in CFA format:

mod.cfa.thur.c <- cfa(reference.indicators=FALSE)
FA: Sentences, Vocabulary, Sent.Completion
FB: First.Letters, 4.Letter.Words, Suffixes
FC: Letter.Series, Pedigrees, Letter.Group

cfa.thur.c <- sem(mod.cfa.thur.c, R.thur, 213)
summary(cfa.thur.c)

	#  (2) in equation format:

mod.cfa.thur.e <- specifyEquations(covs="F1, F2, F3")
Sentences = lam11*F1
Vocabulary = lam21*F1
Sent.Completion = lam31*F1
First.Letters = lam42*F2
4.Letter.Words = lam52*F2
Suffixes = lam62*F2
Letter.Series = lam73*F3
Pedigrees = lam83*F3
Letter.Group = lam93*F3
V(F1) = 1
V(F2) = 1
V(F3) = 1

cfa.thur.e <- sem(mod.cfa.thur.e, R.thur, 213)
summary(cfa.thur.e)

	#  (3) in path format:

mod.cfa.thur.p <- specifyModel(covs="F1, F2, F3")
F1 -> Sentences,                      lam11
F1 -> Vocabulary,                     lam21
F1 -> Sent.Completion,                lam31
F2 -> First.Letters,                  lam41
F2 -> 4.Letter.Words,                 lam52
F2 -> Suffixes,                       lam62
F3 -> Letter.Series,                  lam73
F3 -> Pedigrees,                      lam83
F3 -> Letter.Group,                   lam93
F1 <-> F1,                            NA,     1
F2 <-> F2,                            NA,     1
F3 <-> F3,                            NA,     1

cfa.thur.p <- sem(mod.cfa.thur.p, R.thur, 213)
summary(cfa.thur.p)

# -----  a CFA model fit by FIML to the mental-tests dataset with missing data -----

mod.cfa.tests <- cfa(raw=TRUE)
verbal: x1, x2, x3
math: y1, y2, y3

cfa.tests <- sem(mod.cfa.tests, data=Tests, na.action=na.pass, 
                objective=objectiveFIML, fixed.x="Intercept")
summary(cfa.tests)
summary(cfa.tests, saturated=TRUE) # takes time to fit saturated model for comparison


# ---  a multigroup CFA model fit to the Holzinger-Swineford mental-tests data  -----

mod.hs <- cfa()
spatial: visual, cubes, paper, flags
verbal: general, paragrap, sentence, wordc, wordm
memory: wordr, numberr, figurer, object, numberf, figurew
math: deduct, numeric, problemr, series, arithmet

mod.mg <- multigroupModel(mod.hs, groups=c("Female", "Male")) 

sem.mg <- sem(mod.mg, data=HS.data, group="Gender",
              formula = ~ visual + cubes + paper + flags +
              general + paragrap + sentence + wordc + wordm +
              wordr + numberr + figurer + object + numberf + figurew +
              deduct + numeric + problemr + series + arithmet
              )
summary(sem.mg)

	# with cross-group equality constraints:
	
mod.mg.eq <- multigroupModel(mod.hs, groups=c("Female", "Male"), allEqual=TRUE)

sem.mg.eq <- sem(mod.mg.eq, data=HS.data, group="Gender",
              formula = ~ visual + cubes + paper + flags +
                general + paragrap + sentence + wordc + wordm +
                wordr + numberr + figurer + object + numberf + figurew +
                deduct + numeric + problemr + series + arithmet
              )
summary(sem.mg.eq)

anova(sem.mg, sem.mg.eq) # test equality constraints
	}

## ===============================================================================
	
# The following examples use file input and may be executed via example():

etc <- system.file(package="sem", "etc") # path to data and model files

#   to get all fit indices (not recommended, but for illustration):

opt <- options(fit.indices = c("GFI", "AGFI", "RMSEA", "NFI", "NNFI", 
         "CFI", "RNI", "IFI", "SRMR", "AIC", "AICc", "BIC", "CAIC"))

# ------------- Duncan, Haller and Portes peer-influences model ----------------------
# A nonrecursive SEM with unobserved endogenous variables and fixed exogenous variables

(R.DHP <- readMoments(file=file.path(etc, "R-DHP.txt"),
				diag=FALSE, names=c("ROccAsp", "REdAsp", "FOccAsp", 
                "FEdAsp", "RParAsp", "RIQ", "RSES", "FSES", "FIQ", "FParAsp")))
(model.dhp <- specifyModel(file=file.path(etc, "model-DHP.txt")))
sem.dhp.1 <- sem(model.dhp, R.DHP, 329,
    fixed.x=c('RParAsp', 'RIQ', 'RSES', 'FSES', 'FIQ', 'FParAsp'))
summary(sem.dhp.1)


# -------------------- Wheaton et al. alienation data ----------------------

(S.wh <- readMoments(file=file.path(etc, "S-Wheaton.txt"),
					names=c('Anomia67','Powerless67','Anomia71',
                            'Powerless71','Education','SEI')))

# This is the model in the SAS manual for PROC CALIS: A Recursive SEM with
# latent endogenous and exogenous variables.
# Curiously, both factor loadings for two of the latent variables are fixed.

(model.wh.1 <- specifyModel(file=file.path(etc, "model-Wheaton-1.txt")))                    
sem.wh.1 <- sem(model.wh.1, S.wh, 932)
summary(sem.wh.1)

# The same model, but treating one loading for each latent variable as free
# (and equal to each other).

(model.wh.2 <- specifyModel(file=file.path(etc, "model-Wheaton-2.txt")))
sem.wh.2 <- sem(model.wh.2, S.wh, 932)
summary(sem.wh.2)

# Compare the two models by a likelihood-ratio test:

anova(sem.wh.1, sem.wh.2)


# ----------------------- Thurstone data ---------------------------------------

#  Second-order confirmatory factor analysis, from the SAS manual for PROC CALIS

(R.thur <- readMoments(file=file.path(etc, "R-Thurstone.txt"),
		diag=FALSE, names=c('Sentences','Vocabulary',
        'Sent.Completion','First.Letters','4.Letter.Words','Suffixes',
        'Letter.Series','Pedigrees', 'Letter.Group')))
(model.thur <- specifyModel(file=file.path(etc, "model-Thurstone.txt")))
sem.thur <- sem(model.thur, R.thur, 213)
summary(sem.thur)


#------------------------- Kerchoff/Kenney path analysis ---------------------

# An observed-variable recursive SEM from the LISREL manual

(R.kerch <- readMoments(file=file.path(etc, "R-Kerchoff.txt"),
					   diag=FALSE, names=c('Intelligence','Siblings',
                        'FatherEd','FatherOcc','Grades','EducExp','OccupAsp')))
(model.kerch <- specifyModel(file=file.path(etc, "model-Kerchoff.txt")))
sem.kerch <- sem(model.kerch, R.kerch, 737, 
    fixed.x=c('Intelligence', 'Siblings', 'FatherEd', 'FatherOcc'))
summary(sem.kerch)


#------------------- McArdle/Epstein latent-growth-curve model -----------------

# This model, from McArdle and Epstein (1987, p.118), illustrates the use of a 
# raw moment matrix to fit a model with an intercept. (The example was suggested
# by Mike Stoolmiller.)

(M.McArdle <- readMoments(file=file.path(etc, "M-McArdle.txt"),
    names=c('WISC1', 'WISC2', 'WISC3', 'WISC4', 'UNIT')))
(mod.McArdle <- specifyModel(file=file.path(etc, "model-McArdle.txt")))
sem.McArdle <- sem(mod.McArdle, M.McArdle, 204, fixed.x="UNIT", raw=TRUE)
summary(sem.McArdle)


#------------ Bollen industrialization and democracy example -----------------

# This model, from Bollen (1989, Ch. 8), illustrates the use in sem() of a
# case-by-variable data set (see ?Bollen) rather than a covariance or moment matrix

(model.bollen <- specifyModel(file=file.path(etc, "model-Bollen.txt")))
sem.bollen <- sem(model.bollen, data=Bollen)
summary(sem.bollen)
summary(sem.bollen, robust=TRUE) # robust SEs and tests
summary(sem.bollen, analytic.se=FALSE) # uses numeric rather than analytic Hessian

  # GLS rather than ML estimator:
sem.bollen.gls <- sem(model.bollen, data=Bollen, objective=objectiveGLS) 
summary(sem.bollen.gls)

# -----  a CFA model fit by FIML to the mental-tests dataset with missing data -----

(mod.cfa.tests <- cfa(file=file.path(etc, "model-Tests.txt"), raw=TRUE))
cfa.tests <- sem(mod.cfa.tests, data=Tests, na.action=na.pass, 
                optimizer=optimizerNlm, objective=objectiveFIML, fixed.x="Intercept")
summary(cfa.tests)

#------------ Holzinger and Swineford muiltigroup CFA example ----------------

mod.hs <- cfa(file=file.path(etc, "model-HS.txt"))

mod.mg <- multigroupModel(mod.hs, groups=c("Female", "Male")) 

sem.mg <- sem(mod.mg, data=HS.data, group="Gender",
              formula = ~ visual + cubes + paper + flags +
              general + paragrap + sentence + wordc + wordm +
              wordr + numberr + figurer + object + numberf + figurew +
              deduct + numeric + problemr + series + arithmet
              )
summary(sem.mg)

	# with cross-group equality constraints:
	
mod.mg.eq <- multigroupModel(mod.hs, groups=c("Female", "Male"), allEqual=TRUE)

sem.mg.eq <- sem(mod.mg.eq, data=HS.data, group="Gender",
              formula = ~ visual + cubes + paper + flags +
                general + paragrap + sentence + wordc + wordm +
                wordr + numberr + figurer + object + numberf + figurew +
                deduct + numeric + problemr + series + arithmet
              )
summary(sem.mg.eq)

anova(sem.mg, sem.mg.eq) # test equality constraints

options(opt) # restore fit.indices option

}   

\keyword{models}

\name{information.criteria}
\alias{information.criteria}
\alias{AICc}
\alias{CAIC}

\title{
Additional Information Criteria
}

\description{
These are generic functions for computing, respectively, the AICc (second-order corrected Akaike Information Criterion) and CAIC (consistent Akaike Information Criterion).
}

\usage{
AICc(object, ...)

CAIC(object, ...)
}

\arguments{
  \item{object}{an object for which an appropriate \code{AICc} or \code{CAIC} method exists.}
  \item{\ldots}{possible additional arguments for methods.}
}

\author{Jarrett Byrnes and John Fox \email{jfox@mcmaster.ca}}

\references{
     Burnham, K. P., and Anderson, D. R. (1998)
    \emph{Model Selection and Inference: A Practical Information-Theoretical Approach.} Springer.
    
     Bozdogan, H. (1987)
     Model selection and Akaike's information criterion (AIC).
	\emph{Psychometrika}  bold{52}, 345--370.
}

\seealso{
\code{\link{AICc.objectiveML}}, \code{\link{CAIC.objectiveML}}
}

\keyword{models}

\name{Klein}
\alias{Klein}
\docType{data}
\title{Klein's Data on the U. S. Economy}
\usage{Klein}
\description{
Data for Klein's (1950) simple econometric model of the U. S. economy.

The \code{Klein} data frame has 22 rows and 10 columns.

}
\format{
  This data frame contains the following columns:
  \describe{
    \item{Year}{1921--1941}
    \item{C}{consumption.}
    \item{P}{private profits.}
    \item{Wp}{private wages.}
    \item{I}{investment.}
    \item{K.lag}{capital stock, lagged one year.}
    \item{X}{equilibrium demand.}
    \item{Wg}{government wages.}
    \item{G}{government non-wage spending.}
    \item{T}{indirect business taxes and net exports.}
  }
}

\source{
    Greene, W. H. (1993)
    \emph{Econometric Analysis, Second Edition.}
    Macmillan.
}
\references{
    Klein, L. (1950)
    \emph{Economic Fluctuations in the United States 1921--1941.}
    Wiley.
}
\examples{
Klein$P.lag <- c(NA, Klein$P[-22])
Klein$X.lag <- c(NA, Klein$X[-22])

summary(tsls(C ~ P + P.lag + I(Wp + Wg), 
    instruments=~1 + G + T + Wg + I(Year - 1931) + K.lag + P.lag + X.lag,
    data=Klein))
    
summary(tsls(I ~ P + P.lag + K.lag,
    instruments=~1 + G + T + Wg + I(Year - 1931) + K.lag + P.lag + X.lag,
    data=Klein))
    
summary(tsls(Wp ~ X + X.lag + I(Year - 1931),
    instruments=~1 + G + T + Wg + I(Year - 1931) + K.lag + P.lag + X.lag,
    data=Klein))
}
\keyword{datasets}

\name{CNES}
\alias{CNES}
\docType{data}
\title{Variables from the 1997 Canadian National Election Study}
\description{
  These variables are from the mailback questionnaire to the 1997 Canadian
  National Election Study, and are intended to tap attitude towards \dQuote{traditional
  values.}
}
\usage{CNES}
\format{
  A data frame with 1529 observations on the following 4 variables.
  \describe{
    \item{\code{MBSA2}}{an ordered factor with levels \code{StronglyDisagree}, 
    \code{Disagree}, \code{Agree}, and \code{StronglyAgree}, in response to the statement,
    \dQuote{We should be more tolerant of people who choose to live according to their own standards, 
    even if they are very different from our own.}}
    
    \item{\code{MBSA7}}{an ordered factor with levels \code{StronglyDisagree}, 
    \code{Disagree}, \code{Agree}, and \code{StronglyAgree}, in response to the statement,
    \dQuote{Newer lifestyles are contributing to the breakdown of our society.}}
    
    \item{\code{MBSA8}}{an ordered factor with levels \code{StronglyDisagree}, 
    \code{Disagree}, \code{Agree}, and \code{StronglyAgree}, in response to the statement,
    \dQuote{The world is always changing and we should adapt our view of moral behaviour to
    these changes.}}
    
    \item{\code{MBSA9}}{an ordered factor with levels \code{StronglyDisagree}, 
    \code{Disagree}, \code{Agree}, and \code{StronglyAgree}, in response to the statement,
    \dQuote{This country would have many fewer problems if there were more emphasis on
    traditional family values.}}
  }
}


\source{
  York University Institute for Social Research.
}

\keyword{datasets}

\name{Bollen}
\alias{Bollen}
\docType{data}
\title{
Bollen's Data on Industrialization and Political Democracy
}
\description{
This data set includes four measures of democracy at two points in
time, 1960 and 1965, and three measures of industrialization in 1960,
for 75 developing countries.
}
\usage{Bollen}
\format{
  A data frame with 75 observations on the following 11 variables.
  \describe{
    \item{\code{y1}}{freedom of the press, 1960}
    \item{\code{y2}}{freedom of political opposition, 1960}
    \item{\code{y3}}{fairness of elections, 1960}
    \item{\code{y4}}{effectivness of elected legislature, 1960}
    \item{\code{y5}}{freedom of the press, 1965}
    \item{\code{y6}}{freedom of political opposition, 1965}
    \item{\code{y7}}{fairness of elections, 1965}
    \item{\code{y8}}{effectivness of elected legislature, 1965}
    \item{\code{x1}}{GNP per capita, 1960}
    \item{\code{x2}}{energy consumption per capita, 1960}
    \item{\code{x3}}{percentage of labor force in industry, 1960}
  }
}

\details{
Variables \code{y1} through \code{y4} are intended to be indicators of the latent variable \emph{political democracy in 1960};
\code{y5} through \code{y8} indicators of \emph{political democracy in 1965}; and
\code{x1} through \code{x3} indicators of \emph{industrialization in 1960}.
}

\source{
personal communication from Ken Bollen.
}
\references{
Bollen, K. A. (1989) \emph{Structural Equations With Latent Variables}. Wiley. 

}

\keyword{datasets}

\name{miSem}
\alias{miSem}
\alias{miSem.semmod}
\alias{miSem.semmodList}
\alias{print.miSem}
\alias{summary.miSem}

\title{
Estimate a Structural Equation Model By Multiple Imputation
}

\description{
\code{miSem} uses the \code{\link{mi}} function in the \pkg{mi} package to generate multiple imputations of missing
data, fitting the specified model to each completed data set.
}

\usage{
miSem(model, ...)

\method{miSem}{semmod}(model, ..., data, formula = ~., raw=FALSE, 
        fixed.x=NULL, objective=objectiveML,
        n.imp=5, n.chains=n.imp, n.iter=30, seed=sample(1e6, 1), mi.args=list(), 
        show.progress=TRUE)
    
\method{miSem}{semmodList}(model, ..., data, formula = ~., group, raw=FALSE, 
        fixed.x=NULL, objective=msemObjectiveML,
        n.imp=5, n.chains=n.imp, n.iter=30, seed=sample(1e6, 1), mi.args=list(),
        show.progress=TRUE)

\method{print}{miSem}(x, ...)

\method{summary}{miSem}(object, digits=max(3, getOption("digits") - 2), ...)
}

\arguments{
  \item{model}{
    an SEM model-description object of class \code{semmod} or \code{semmodList}, created by \code{\link{specifyEquations}}
    \code{\link{cfa}}, or \code{\link{specifyModel}}, in the case of a multi-group model 
    in combination with \code{\link{multigroupModel}}.
    }
  \item{\dots, formula, raw, fixed.x, objective, group}{
    arguments to be passed to \code{\link{sem}}.
    }
  \item{data}{
    an R data frame, presumably with some missing data (encoded as \code{NA}), containing the data for fitting
    the SEM, possibly along with other variables to use to obtain multiple imputations of missing values. In
    the case of a multi-group model, this must be a \emph{single} data frame.
    }

  \item{n.imp}{
    number of imputations (default \code{5}).
    }
    
  \item{n.chains}{
    number of Markov chains (default is the number of imputations).
  }

  \item{n.iter}{
    number of iterations for the multiple-imputation process (default \code{30}).
    }

  \item{seed}{
    seed for the random-number generator (default is an integer sampled from 1 to 1E6); stored in the resulting object.
    }

  \item{mi.args}{
    other arguments to be passed to \code{\link{mi}}.
    }
  \item{show.progress}{
    show a text progress bar on the console tracking model fitting to the multiple imputations; this is distinct from
    the progress of the multiple-imputation process, which is controlled by the \code{verbose} argument to \code{\link{mi}}
    (and which, although it defaults to \code{TRUE}, \emph{fails} to produce verbose output on Windows system, as of \pkg{mi} version
    1.0).
  }
  \item{x, object}{
    an object of class \code{"miSem"}.
    }
  \item{digits}{
    for printing numbers.
  }
}

\value{
    \code{miSem} returns an object of class \code{"miSem"} with the following components:
    \item{initial.fit}{an \code{sem} model object produced using \code{\link{objectiveFIML}} if \code{raw=TRUE},
    or the objective function given by the \code{objective} argument otherwise.}
    \item{mi.fits}{a list of \code{sem} model objects, one for each imputed data set.}
    \item{imputation}{the object produced by \code{\link{complete}}, containing the completed imputed data sets.}
    \item{seed}{the seed used for the random number generator.}
    \item{mi.data}{the object returned by \code{\link{mi}}, containing the multiple imputations, and useful, e.g., for diagnostic checking of the imputation process.}
}

\references{
    Yu-Sung Su, Andrew Gelman, Jennifer Hill, Masanao Yajima. (2011). 
    ``Multiple imputation with diagnostics (mi) in R: Opening windows into the black box.'' 
    \emph{Journal of Statistical Software} 45(2).
}
\author{
    John Fox \email{jfox@mcmaster.ca}
    }

\seealso{
\code{\link{sem}}, \code{\link{mi}}
}
\examples{
    \donttest{
mod.cfa.tests <- cfa(raw=TRUE, text="
verbal: x1, x2, x3
math: y1, y2, y3
")
imps <- miSem(mod.cfa.tests, data=Tests, fixed.x="Intercept", 
              raw=TRUE, seed=12345)
summary(imps, digits=3) 


# introduce some missing data to the HS.data data set:
HS <- HS.data[, c(2,7:10,11:15,20:25,26:30)]
set.seed(12345)
r <- sample(301, 100, replace=TRUE)
c <- sample(2:21, 100, replace=TRUE)
for (i in 1:100) HS[r[i], c[i]] <- NA

mod.hs <- cfa(text="
spatial: visual, cubes, paper, flags
verbal: general, paragrap, sentence, wordc, wordm
memory: wordr, numberr, figurer, object, numberf, figurew
math: deduct, numeric, problemr, series, arithmet
")

mod.mg <- multigroupModel(mod.hs, groups=c("Female", "Male")) 
system.time( # relatively time-consuming!
  imps.mg <- miSem(mod.mg, data=HS, group="Gender", seed=12345)
)
summary(imps.mg, digits=3)
    }
}

\keyword{models}
